//
//  DSInterval.m
//  DSInterval
//
//  Created by David Sinclair on 2007-04-05.
//  Copyright 2007-2008 Dejal Systems, LLC. All rights reserved.
//
//  Redistribution and use in source and binary forms, with or without modification,
//  are permitted provided that the following conditions are met:
//  
//  Redistributions of source code must retain this list of conditions and the following disclaimer.
//  
//  The name of Dejal Systems, LLC may not be used to endorse or promote products derived from this
//  software without specific prior written permission.
//  
//  THIS SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED,
//  INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
//  PURPOSE AND NONINFRINGEMENT OF THIRD PARTY RIGHTS. IN NO EVENT SHALL THE AUTHORS OR
//  COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN
//  ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE
//  SOFTWARE OR THE USE OR OTHER DEALINGS IN THIS SOFTWARE.
//

#import "DSInterval.h"


#define DS_INTERVAL_NEVER          0
#define DS_INTERVAL_SECOND         1
#define DS_INTERVAL_MINUTE         60
#define DS_INTERVAL_HOUR           (60 * 60)
#define DS_INTERVAL_DAY            (60 * 60 * 24)
#define DS_INTERVAL_WEEK           (60 * 60 * 24 * 7)
#define DS_INTERVAL_MONTH          (60 * 60 * 24 * 365 / 12)
#define DS_INTERVAL_QUARTER        (60 * 60 * 24 * 365 / 4)
#define DS_INTERVAL_YEAR           (60 * 60 * 24 * 365)
#define DS_INTERVAL_FOREVER        INT_MAX


@implementation DSInterval

@synthesize usingRange = _usingRange;
@synthesize firstAmount = _firstAmount;
@synthesize secondAmount = _secondAmount;
@synthesize units = _units;
@synthesize userInfo = _userInfo;

/*
 intervalWithAmount:units:
 
 Convenience class method to return a new DSInterval with the specified amount and units.
 
 Written by DJS 2007-04.
 Changed by DJS 2008-07 to support ranges.
*/

+ (id)intervalWithAmount:(NSInteger)amount units:(DSIntervalUnits)units;
{
    return [[[self alloc] initWithRange:NO firstAmount:amount secondAmount:amount units:units userInfo:nil] autorelease];
}

/*
 intervalWithAmount:units:userInfo:
 
 Convenience class method to return a new DSInterval with the specified amount and units.  It also supports an optional user value.
 
 Written by DJS 2007-04.
 Changed by DJS 2008-07 to support ranges.
*/

+ (id)intervalWithAmount:(NSInteger)amount units:(DSIntervalUnits)units userInfo:(id)userInfo;
{
    return [[[self alloc] initWithRange:NO firstAmount:amount secondAmount:amount units:units userInfo:userInfo] autorelease];
}

/*
 intervalWithRangeFirstAmount:secondAmount:units:
 
 Convenience class method to return a new DSInterval with the specified range of amounts and units.
 
 Written by DJS 2008-07.
*/

+ (id)intervalWithRangeFirstAmount:(NSInteger)firstAmount secondAmount:(NSInteger)secondAmount units:(DSIntervalUnits)units;
{
    return [[[self alloc] initWithRange:YES firstAmount:firstAmount secondAmount:secondAmount units:units userInfo:nil] autorelease];
}

/*
 intervalWithRangeFirstAmount:secondAmount:units:userInfo:
 
 Convenience class method to return a new DSInterval with the specified range of amounts and units.  It also supports an optional user value.
 
 Written by DJS 2008-08.
*/

+ (id)intervalWithRangeFirstAmount:(NSInteger)firstAmount secondAmount:(NSInteger)secondAmount units:(DSIntervalUnits)units userInfo:(id)userInfo;
{
    return [[[self alloc] initWithRange:YES firstAmount:firstAmount secondAmount:secondAmount units:units userInfo:userInfo] autorelease];
}

/*
 intervalWithDictionary:
 
 Convenience class method to return a new DSInterval with the amount, units and optional user value specified in the dictionary representation (as returned by -dictionary).
 
 Written by DJS 2007-04.
 Changed by DJS 2008-07 to support ranges.
*/

+ (id)intervalWithDictionary:(NSDictionary *)dict;
{
    if ([[dict objectForKey:@"UsingRange"] boolValue])
        return [self intervalWithRangeFirstAmount:[[dict objectForKey:@"FirstAmount"] integerValue]
                                     secondAmount:[[dict objectForKey:@"SecondAmount"] integerValue]
                                            units:[[dict objectForKey:@"Units"] integerValue]
                                         userInfo:[dict objectForKey:@"UserInfo"]];
    else
        return [self intervalWithAmount:[[dict objectForKey:@"SecondAmount"] integerValue]
                                  units:[[dict objectForKey:@"Units"] integerValue]
                               userInfo:[dict objectForKey:@"UserInfo"]];
}

/*
 initWithRange:firstAmount:secondAmount:units:userInfo:
 
 Designated initializer.
 
 Written by DJS 2007-04.
 Changed by DJS 2008-07 to use properties and support ranges.
*/

- (id)initWithRange:(BOOL)usingRange firstAmount:(NSInteger)firstAmount secondAmount:(NSInteger)secondAmount units:(DSIntervalUnits)units userInfo:(id)userInfo;
{
    if ((self = [super init]))
    {
        self.usingRange = usingRange;
        self.firstAmount = firstAmount;
        self.secondAmount = secondAmount;
        self.units = units;
        self.userInfo = userInfo;
    }
    
    return self;
}

/*
 copyWithZone:
 
 NSCopying protocol method; returns a new copy of the receiver.
 
 Written by DJS 2007-07.
 Changed by DJS 2008-07 to use properties and support ranges.
*/

- (id)copyWithZone:(NSZone *)zone;
{
    return [[[self class] allocWithZone:zone] initWithRange:self.usingRange firstAmount:self.firstAmount secondAmount:self.secondAmount units:self.units userInfo:self.userInfo];
}

/*
 description
 
 Returns a string that describes the contents of the receiver.
 
 Written by DJS 2007-04.
 Changed by DJS 2008-07 to use properties and support ranges.
*/

- (NSString *)description;
{
    NSString *extras = self.userInfo ? [NSString stringWithFormat:@" (userInfo %@)", self.userInfo] : @"";
    
    if (self.usingRange)
        return [NSString stringWithFormat:@"%@ = %.0f - %.0f seconds%@", [self amountWithFullUnitsName], self.firstTimeInterval, self.secondTimeInterval, extras];
    else
        return [NSString stringWithFormat:@"%@ = %.0f seconds%@", [self amountWithFullUnitsName], self.amountTimeInterval, extras];
}

/*
 dictionary
 
 Returns a dictionary representation of the receiver, suitable for saving in a plist etc.
 
 Written by DJS 2007-04.
 Changed by DJS 2008-07 to use properties and support ranges.
*/

- (NSDictionary *)dictionary;
{
    NSMutableDictionary *dict = [NSMutableDictionary dictionary];
    
    [dict setObject:[NSNumber numberWithBool:self.usingRange] forKey:@"UsingRange"];
    [dict setObject:[NSNumber numberWithInt:self.firstAmount] forKey:@"FirstAmount"];
    [dict setObject:[NSNumber numberWithInt:self.secondAmount] forKey:@"SecondAmount"];
    [dict setObject:[NSNumber numberWithInt:self.units] forKey:@"Units"];
    
    if (self.userInfo)
        [dict setObject:self.userInfo forKey:@"UserInfo"];
    
    return dict;
}

/*
 amountTimeInterval
 
 Returns the receiver represented as a time interval, i.e. in seconds or fractions thereof.
 
 Written by DJS 2008-07.
*/

- (NSTimeInterval)amountTimeInterval;
{
    return self.secondTimeInterval;
}

/*
 firstTimeInterval
 
 Returns the first amount of the receiver represented as a time interval, i.e. in seconds or fractions thereof.
 
 Written by DJS 2008-07.
*/

- (NSTimeInterval)firstTimeInterval;
{
    if (self.units == DSIntervalNever)
        return DS_INTERVAL_NEVER;
    else if (self.units == DSIntervalForever)
        return DS_INTERVAL_FOREVER;
    else
        return self.firstAmount * self.unitsTimeInterval;
}

/*
 secondTimeInterval
 
 Returns the second amount of the receiver represented as a time interval, i.e. in seconds or fractions thereof.
 
 Written by DJS 2008-07.
*/

- (NSTimeInterval)secondTimeInterval;
{
    if (self.units == DSIntervalNever)
        return DS_INTERVAL_NEVER;
    else if (self.units == DSIntervalForever)
        return DS_INTERVAL_FOREVER;
    else
        return self.secondAmount * self.unitsTimeInterval;
}

/*
 rangeDifferenceInterval
 
 Returns the time difference between the first and second amounts, represented as a time interval, i.e. in seconds or fractions thereof.  The result is negative if the first amount is greater than the second one.
 
 Written by DJS 2008-07.
*/

- (NSTimeInterval)rangeDifferenceInterval;
{
    return self.secondTimeInterval - self.firstTimeInterval;
}

/*
 unitsTimeInterval
 
 Returns the units of the receiver represented as a time interval, i.e. in seconds or fractions thereof.  This value can be multiplied by the amounts.
 
 Written by DJS 2008-07.
*/

- (NSTimeInterval)unitsTimeInterval;
{
    switch (self.units)
    {
        case DSIntervalMinutes:
            return DS_INTERVAL_MINUTE;
        case DSIntervalHours:
            return DS_INTERVAL_HOUR;
        case DSIntervalDays:
            return DS_INTERVAL_DAY;
        case DSIntervalWeeks:
            return DS_INTERVAL_WEEK;
        case DSIntervalMonths:
            return DS_INTERVAL_MONTH;
        case DSIntervalYears:
            return DS_INTERVAL_YEAR;
        case DSIntervalNever:
            return DS_INTERVAL_NEVER;
        case DSIntervalForever:
            return DS_INTERVAL_FOREVER;
        default:
            return DS_INTERVAL_SECOND;
    }
}

/*
 amount
 
 Accessor for the amount property.  Returns the second amount; provided as a more intuitive property name when not using a range.
 
 Written by DJS 2008-07.
*/

- (NSInteger)amount;
{
    return self.secondAmount;
}

/*
 setAmount:
 
 Accessor for the amount property.  Sets both amounts; provided as a more intuitive property name when not using a range.
 
 Written by DJS 2008-07.
*/

- (void)setAmount:(NSInteger)amount;
{
    self.firstAmount = amount;
    self.secondAmount = amount;
}

/*
 amountWithBriefUnitsName
 
 Returns a string with the amount and brief units name, e.g. "5 mins" or "1 wk".
 
 Written by DJS 2008-07.
*/

- (NSString *)amountWithBriefUnitsName;
{
    if (self.units == DSIntervalNever || self.units == DSIntervalForever)
        return [self briefUnitsName];
    
    if (self.usingRange)
        return [NSString stringWithFormat:@"%d - %d %@", self.firstAmount, self.secondAmount, [self briefUnitsName]];
    else
        return [NSString stringWithFormat:@"%d %@", self.secondAmount, [self briefUnitsName]];
}

/*
 amountWithFullUnitsName
 
 Returns a string with the amount and full units name, e.g. "5 minutes" or "1 week".
 
 Written by DJS 2008-07.
*/

- (NSString *)amountWithFullUnitsName;
{
    if (self.units == DSIntervalNever || self.units == DSIntervalForever)
        return [self fullUnitsName];
    
    if (self.usingRange)
        return [NSString stringWithFormat:@"%d - %d %@", self.firstAmount, self.secondAmount, [self fullUnitsName]];
    else
        return [NSString stringWithFormat:@"%d %@", self.secondAmount, [self fullUnitsName]];
}

/*
 briefUnitsName
 
 Returns a brief localized string for the units of the receiver, using singular or plural as appropriate for the amount, e.g. "min" or "wks".
 
 Written by DJS 2007-04.
 Changed by DJS 2007-07 to add forever.
 Changed by DJS 2008-07 to use properties.
*/

- (NSString *)briefUnitsName;
{
    switch (self.units)
    {
        case DSIntervalMinutes:
            return self.secondAmount == 1 ? NSLocalizedString(@"min", @"minute singular") : NSLocalizedString(@"mins", @"minute plural");
        case DSIntervalHours:
            return self.secondAmount == 1 ? NSLocalizedString(@"hr", @"hour singular") : NSLocalizedString(@"hrs", @"hour plural");
        case DSIntervalDays:
            return self.secondAmount == 1 ? NSLocalizedString(@"day", @"day singular") : NSLocalizedString(@"days", @"day plural");
        case DSIntervalWeeks:
            return self.secondAmount == 1 ? NSLocalizedString(@"wk", @"week singular") : NSLocalizedString(@"wks", @"week plural");
        case DSIntervalMonths:
            return self.secondAmount == 1 ? NSLocalizedString(@"mth", @"month singular") : NSLocalizedString(@"mths", @"month plural");
        case DSIntervalYears:
            return self.secondAmount == 1 ? NSLocalizedString(@"yr", @"year singular") : NSLocalizedString(@"yrs", @"year plural");
        case DSIntervalNever:
            return NSLocalizedString(@"never", @"never");
        
        case DSIntervalForever:
            return NSLocalizedString(@"forever", @"forever");
        
        default:
            return self.secondAmount == 1 ? NSLocalizedString(@"sec", @"second singular") : NSLocalizedString(@"secs", @"second plural");
    }
}

/*
 fullUnitsName
 
 Returns a longer localized string for the units of the receiver, using singular or plural as appropriate for the amount, e.g. "minute" or "weeks".
 
 Written by DJS 2007-04.
 Changed by DJS 2007-07 to add forever.
 Changed by DJS 2008-07 to use properties.
*/

- (NSString *)fullUnitsName;
{
    switch (self.units)
    {
        case DSIntervalMinutes:
            return self.secondAmount == 1 ? NSLocalizedString(@"minute", @"minute singular") : NSLocalizedString(@"minutes", @"minute plural");
        
        case DSIntervalHours:
            return self.secondAmount == 1 ? NSLocalizedString(@"hour", @"hour singular") : NSLocalizedString(@"hours", @"hour plural");
        case DSIntervalDays:
            return self.secondAmount == 1 ? NSLocalizedString(@"day", @"day singular") : NSLocalizedString(@"days", @"day plural");
        case DSIntervalWeeks:
            return self.secondAmount == 1 ? NSLocalizedString(@"week", @"week singular") : NSLocalizedString(@"weeks", @"week plural");
        case DSIntervalMonths:
            return self.secondAmount == 1 ? NSLocalizedString(@"month", @"month singular") : NSLocalizedString(@"months", @"month plural");
        case DSIntervalYears:
            return self.secondAmount == 1 ? NSLocalizedString(@"year", @"year singular") : NSLocalizedString(@"years", @"year plural");
        case DSIntervalNever:
            return NSLocalizedString(@"never", @"never");
        
        case DSIntervalForever:
            return NSLocalizedString(@"forever", @"forever");
        
        default:
            return self.secondAmount == 1 ? NSLocalizedString(@"second", @"second singular") : NSLocalizedString(@"seconds", @"second plural");
    }
}

@end

