//
//  DSOutlineViewCategories.m
//  Shared categories
//
//  Created by David Sinclair on Fri May 30 2003.
//  Copyright (c) 2003 - 2007 Dejal Systems, LLC. All rights reserved.
//
//  Portions based on code by Wil Shipley, Copyright (c) 2006 Delicious Monster Software, LLC:
//      <http://wilshipley.com/blog/2006/04/pimp-my-code-part-10-whining-about.html>.
//
//  Redistribution and use in source and binary forms, with or without modification,
//  are permitted provided that the following conditions are met:
//  
//  Redistributions of source code must retain this list of conditions and the following disclaimer.
//  
//  The name of Dejal Systems, LLC may not be used to endorse or promote products derived from this
//  software without specific prior written permission.
//  
//  THIS SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED,
//  INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
//  PURPOSE AND NONINFRINGEMENT OF THIRD PARTY RIGHTS. IN NO EVENT SHALL THE AUTHORS OR
//  COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN
//  ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE
//  SOFTWARE OR THE USE OR OTHER DEALINGS IN THIS SOFTWARE.
//


#import "DSOutlineViewCategories.h"
#import "DSTableViewCategories.h"


@interface NSObject (DSTableViewDelegate)

- (NSMenu *)outlineView:(NSOutlineView *)olv menuForTableColumn:(NSTableColumn *)tableColumn byItem:(id)item;

@end


// ----------------------------------------------------------------------------------------
#pragma mark -
// ----------------------------------------------------------------------------------------


@interface NSOutlineView (DSOutlineViewCategories_Private)

- (NSTreeController *)_treeController;
- (id)_realItemForOpaqueItem:(id)findOpaqueItem outlineRowIndex:(NSInteger *)outlineRowIndex
                       items:(NSArray *)items;

@end


// ----------------------------------------------------------------------------------------
#pragma mark -
// ----------------------------------------------------------------------------------------


@implementation NSOutlineView (DSOutlineViewCategories)

/*
 selectedItem

 Returns the object for the selected row (or the first selected row if multiple).
*/

- (id)selectedItem
{
    return [self itemAtRow:[self selectedRow]];
}

/*
 selectedOrAllItems:

 Returns an array containing the objects for the selected rows, or for all rows if none are selected and YES is passed.
*/

- (NSArray *)selectedOrAllItems:(BOOL)allIfNoneSelected
{
    NSMutableArray *items = [NSMutableArray array];
    NSEnumerator *enumerator;
    NSNumber *selRow = nil;

    if (allIfNoneSelected)
        enumerator = [self selectedOrAllRowsEnumerator];
    else
        enumerator = [self selectedRowEnumerator];
    
    while ((selRow = [enumerator nextObject]))
    {
        id item = [self itemAtRow:[selRow intValue]];
        
        if (item)
            [items addObject:item];
    }

    return items;
}

/*
 selectedItems:byExtendingSelection:

 Selects the rows for the objects in the array, optionally extending any existing selection.
*/

- (void)selectItems:(NSArray *)items byExtendingSelection:(BOOL)extendSel
{
    if (!extendSel)
        [self deselectAll:nil];

    id item;

    for (item in items)
    {
        NSInteger row = [self rowForItem:item];

        if (row >= 0)
            [self selectRow:row byExtendingSelection:extendSel];
    }
}

/*
 menuForEvent:
 
 Override of a NSView method, to allow a contextual menu for an individual row and column.  You should implement -outlineView:menuForTableColumn:byItem: in your outline delegate.  If that isn't implemented, this returns the default menu.
 
 Written by DJS 2004-06.
*/

- (NSMenu *)menuForEvent:(NSEvent *)event
{
    NSPoint point = [self convertPoint:[event locationInWindow] fromView:nil];
    NSInteger column = [self columnAtPoint:point];
    NSInteger row = [self rowAtPoint:point];
    id item = [self itemAtRow:row];
    
    if (column >= 0 && item && [[self delegate] respondsToSelector:@selector(outlineView:menuForTableColumn:byItem:)])
        return [[self delegate] outlineView:self menuForTableColumn:[[self tableColumns] objectAtIndex:column] byItem:item];
    else
        return [super menuForEvent:event];
}

/*
 realItemForOpaqueItem:
 
 Given an opaque item from a tree controller, return the corresponding real item.  Note: requires Mac OS X 10.4 or later.
 
 Written by WS 2006-03.
*/

- (id)realItemForOpaqueItem:(id)opaqueItem;
{
    NSInteger outlineRowIndex = 0;
    return [self _realItemForOpaqueItem:opaqueItem outlineRowIndex:&outlineRowIndex
                                  items:[[self _treeController] content]];
}

@end


// ----------------------------------------------------------------------------------------
#pragma mark -
// ----------------------------------------------------------------------------------------


@implementation NSOutlineView (DSOutlineViewCategories_Private)

- (NSTreeController *)_treeController;
{
    return (NSTreeController *)[[self infoForBinding:@"contentAttributeKey"]
      objectForKey:@"NSObservedObject"];
}

- (id)_realItemForOpaqueItem:(id)findOpaqueItem outlineRowIndex:(NSInteger *)outlineRowIndex
                       items:(NSArray *)items;
{
    NSUInteger itemIndex;
    for (itemIndex = 0; itemIndex < [items count] && *outlineRowIndex < [self numberOfRows];
         itemIndex++, (*outlineRowIndex)++) {
        id realItem = [items objectAtIndex:itemIndex];
        id opaqueItem = [self itemAtRow:*outlineRowIndex];
        if (opaqueItem == findOpaqueItem)
            return realItem;
        if ([self isItemExpanded:opaqueItem]) {
            realItem = [self _realItemForOpaqueItem:findOpaqueItem outlineRowIndex:outlineRowIndex
                                              items:[realItem valueForKeyPath:[[self _treeController] childrenKeyPath]]];
            if (realItem)
                return realItem;
        }
    }
    
    return nil;
}

@end

