//
//  DSPopUpButtonCategories.m
//  Shared categories
//
//  Created by David Sinclair on Fri Mar 12 2004.
//  Copyright (c) 2004 - 2007 Dejal Systems, LLC. All rights reserved.
//
//  Redistribution and use in source and binary forms, with or without modification,
//  are permitted provided that the following conditions are met:
//  
//  Redistributions of source code must retain this list of conditions and the following disclaimer.
//  
//  The name of Dejal Systems, LLC may not be used to endorse or promote products derived from this
//  software without specific prior written permission.
//  
//  THIS SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED,
//  INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
//  PURPOSE AND NONINFRINGEMENT OF THIRD PARTY RIGHTS. IN NO EVENT SHALL THE AUTHORS OR
//  COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN
//  ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE
//  SOFTWARE OR THE USE OR OTHER DEALINGS IN THIS SOFTWARE.
//

#import "DSPopUpButtonCategories.h"
#import "DSStringCategories.h"


@implementation NSPopUpButton (DSPopUpButtonCategories)

/*
 addSeparatorItem
 
 Adds a separator item to the menu.
 
 Written by DJS 2006-10.
*/

- (void)addSeparatorItem;
{
    [[self menu] addItem:[NSMenuItem separatorItem]];
    
    [self synchronizeTitleAndSelectedItem];
}

/*
 addItemWithTitle:tag:
 
 Similar to -addItemWithTitle:, but allows specifying a tag.  It does not remove any other items with the same name, however, as -[NSPopUpButton addItemWithTitle:] does.  It also returns the created item.
 
 Written by DJS 2004-03.
*/

- (NSMenuItem *)addItemWithTitle:(NSString *)aString tag:(NSInteger)tag
{
    NSMenuItem *item = [[[NSMenuItem alloc] initWithTitle:aString action:nil keyEquivalent:@""] autorelease];
    
    [item setTag:tag];
    
    [[self menu] addItem:item];
    
    [self synchronizeTitleAndSelectedItem];
    
    return item;
}

/*
 addItemWithTitle:tag:representedObject:
 
 Similar to -addItemWithTitle:, but allows specifying a tag and represented object.  It does not remove any other items with the same name, however, as -[NSPopUpButton addItemWithTitle:] does.  It also returns the created item.
 
 Written by DJS 2004-03.
*/

- (NSMenuItem *)addItemWithTitle:(NSString *)aString tag:(NSInteger)tag representedObject:(id)object
{
    NSMenuItem *item = [[[NSMenuItem alloc] initWithTitle:aString action:nil keyEquivalent:@""] autorelease];
    
    [item setTag:tag];
    [item setRepresentedObject:object];
    
    [[self menu] addItem:item];
    
    [self synchronizeTitleAndSelectedItem];
    
    return item;
}

/*
 addItemsWithPaths:prefixDivider:tag:abbreviatedObject:
 
 Given an array of file paths, adds items to the receiver.  The item titles are the last path components sans extensions, and the item tags are set as requested.  The item represented objects are set to the abbreviated or full paths, based on the abbreviatedObject value.  If prefixDivider is YES, a divider line is added before the other items, iff there are any.  You can call -pathsWithExtension:atPath:deepScan: (in NSFileManagerCategories) to get the paths.
 
 Written by DJS 2005-05.
*/

- (void)addItemsWithPaths:(NSArray *)paths
            prefixDivider:(BOOL)prefixDivider
                      tag:(NSInteger)tag
        abbreviatedObject:(BOOL)abbreviatedObject
{
    NSString *path;
    
    for (path in paths)
    {
        if (prefixDivider)
        {
            [[self menu] addItem:[NSMenuItem separatorItem]];
            prefixDivider = NO;
        }
        
        NSString *title = [path lastPathComponentWithoutExtension];
        NSString *object = path;
        
        if (abbreviatedObject)
            object = [object abbreviatedPath];
        
        [self addItemWithTitle:title tag:tag representedObject:object];
    }
    
    [self synchronizeTitleAndSelectedItem];
}


/*
 itemWithTag:
 
 Convenience method to return the menu item with the given tag.
 
 Written by DJS 2004-03.
*/

- (NSMenuItem *)itemWithTag:(NSInteger)aTag
{
    NSMenu *menu = [self menu];
    NSMenuItem *item = [menu itemWithTag:aTag];
    
    return item;
}

/*
 selectItemWithTagValue:
 
 Convenience method to select the menu item with the given tag.  Note: this method has been deprecated.  It used to be called -selectItemWithTag:, but Apple introduced a method with the same name in Tiger.  If requiring a minimum of 10.4 or later, use that method instead of this.  This method may be removed once 10.4 is a reasonable minimum for all apps.
 
 Written by DJS 2004-03.
 Changed by DJS 2005-05 to rename to avoid Tiger conflict, and deprecate.
*/

- (NSInteger)selectItemWithTagValue:(NSInteger)aTag
{
    NSUInteger i = [self indexOfItemWithTag:aTag];
    
    [self selectItemAtIndex:i];
    
    return i;
}

/*
 selectItemWithRepresentedObject:
 
 Convenience method to select the menu item with the given represented object.
 
 Written by DJS 2005-05.
*/

- (NSInteger)selectItemWithRepresentedObject:(id)anObj
{
    NSUInteger i = [self indexOfItemWithRepresentedObject:anObj];
    
    [self selectItemAtIndex:i];
    
    return i;
}

/*
 tagOfSelectedItem
 
 Convenience method to return the tag of the selected item, similar to indexOfSelectedItem etc.
 
 Written by DJS 2007-07.
 */

- (NSInteger)tagOfSelectedItem;
{
    return [[self selectedItem] tag];
}

@end
