//
//  DSTextFieldCategories.m
//  Shared categories
//
//  Created by David Sinclair on Mon Feb 16 2004.
//  Copyright (c) 2004 - 2007 Dejal Systems, LLC. All rights reserved.
//
//  Redistribution and use in source and binary forms, with or without modification,
//  are permitted provided that the following conditions are met:
//  
//  Redistributions of source code must retain this list of conditions and the following disclaimer.
//  
//  The name of Dejal Systems, LLC may not be used to endorse or promote products derived from this
//  software without specific prior written permission.
//  
//  THIS SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED,
//  INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
//  PURPOSE AND NONINFRINGEMENT OF THIRD PARTY RIGHTS. IN NO EVENT SHALL THE AUTHORS OR
//  COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN
//  ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE
//  SOFTWARE OR THE USE OR OTHER DEALINGS IN THIS SOFTWARE.
//

#import "DSTextFieldCategories.h"
#import "DSViewCategories.h"
#import "DSWindowCategories.h"


@implementation NSTextField (DSTextFieldCategories)

/*
 containsSomething
 
 Returns YES if the field's string value is non-empty, i.e. not nil and not @"".
 
 Written by DJS 2005-03.
*/

- (BOOL)containsSomething
{
    return ([[self stringValue] length] > 0);
}

/*
 setPromptLabel:
 
 Performs the common task of setting a prompt label with a colon suffix.
 
 Written by DJS 2004-02.
*/

- (void)setPromptLabel:(NSString *)label
{
    if ([label length])
    {
        if (![label hasSuffix:@":"])
            label = [label stringByAppendingString:@":"];
        
        [self setStringValue:label];
    }
}

/*
 setFieldValue:
 
 Performs the common task of setting a field's value, making sure that the value isn't nil.
 
 Written by DJS 2004-02.
*/

- (void)setFieldValue:(NSString *)value
{
    if (value)
        [self setStringValue:value];
    else
        [self setStringValue:@""];
}

/*
 syncronizeWithIntegerSlider:setPreference:usingFieldValue:
 
 Validates the field value using the range from the slider, and sets either the receiver or the slider to that value, as appropriate, and updates the preference key, if any.  All this could be done with bindings, but this works too, and works in Mac OS X 10.2.
 
 Written by DJS 2005-03.
 Changed by DJS 2006-01 to divide by 60 if the maximum value is large, indicating minutes expressed in seconds.
*/

- (void)syncronizeWithIntegerSlider:(NSSlider *)slider setPreference:(NSString *)preferenceKey usingFieldValue:(BOOL)isUsingField
{
    NSInteger max = [slider maxValue];
    NSInteger value = 0;
    
    if (isUsingField)
    {
        value = [self intValue];
        
        if (max > 1000)
            value *= 60;
    }
    else
        value = [slider intValue];
    
    if (value < [slider minValue] || value > [slider maxValue])
    {
        isUsingField = NO;
        value = [slider intValue];
    }
    
    if (isUsingField)
        [slider setIntValue:value];
    else
    {
        NSInteger temp = value;
        
        if (max > 1000)
            temp /= 60;
        
        [self setIntValue:temp];
    }
    
    if (preferenceKey)
        [[NSUserDefaults standardUserDefaults] setInteger:value forKey:preferenceKey];
}

/*
 resizeWindowVerticallyToFit
 
 If the field content doesn't currently fit within its frame, this method resizes the window that contains this field, and the field itself, so that it does fit.  The window and field are only resized vertically.  The auto-resizing springs and struts are not needed or affected for this change.  The window size is only increased, never decreased.  Returns YES if a resize was needed, or NO if it was already big enough.
 
 Written by DJS 2006-01.
*/

- (BOOL)resizeWindowVerticallyToFit;
{
    NSRect oldFrame = [self frame];
    NSRect newFrame = oldFrame;
    
    // Determine the ideal height for the field:
    newFrame.size.height = 10000.0;
    newFrame.size = [[self cell] cellSizeForBounds:newFrame];
    
    // If no change, or the new height would be smaller, do nothing:
    if (NSHeight(newFrame) <= NSHeight(oldFrame))
        return NO;
    
    // We need to resize, so adjust the autoresize masks:
    NSArray *masks = [self adjustAutoresizeMasks];
    
    // Resize the window based on the new text field frame size:
    [[self window] setFrameSoView:self hasSize:newFrame.size centerHorizontalPostion:NO];
    
    // Tidy up:
    [self restoreAutoresizeMasks:masks];
    
    return YES;
}

@end
