//
//  DSTextViewCategories.m
//  Shared categories
//
//  Created by David Sinclair on Mon May 24 2004.
//  Copyright (c) 2004 - 2007 Dejal Systems, LLC. All rights reserved.
//
//  Redistribution and use in source and binary forms, with or without modification,
//  are permitted provided that the following conditions are met:
//  
//  Redistributions of source code must retain this list of conditions and the following disclaimer.
//  
//  The name of Dejal Systems, LLC may not be used to endorse or promote products derived from this
//  software without specific prior written permission.
//  
//  THIS SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED,
//  INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
//  PURPOSE AND NONINFRINGEMENT OF THIRD PARTY RIGHTS. IN NO EVENT SHALL THE AUTHORS OR
//  COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN
//  ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE
//  SOFTWARE OR THE USE OR OTHER DEALINGS IN THIS SOFTWARE.
//

#import "DSTextViewCategories.h"
#import "DSAttributedStringCategories.h"


@implementation NSTextView (DSTextViewCategories)

/*
 length
 
 Returns the number of characters in the text view.
 
 Written by DJS 2004-05.
*/

- (NSInteger)length
{
    return [[self string] length];
}

/*
 allRange
 
 Returns a NSRange that includes all characters in the text view.
 
 Written by DJS 2004-05.
*/

- (NSRange)allRange
{
    return NSMakeRange(0, [self length]);
}

/*
 setTypingAttributesToMatchView
 
 After changing the font or foreground/background colors of a NSTextView, call this to apply those changes to zero-length selections too (i.e. the next user typing).  Otherwise, new typing will be inserted in the old style (which can be useful sometimes).
 
 Written by DJS 2006-10.
*/

- (void)setTypingAttributesToMatchView;
{
    NSMutableDictionary *attributes = [NSMutableDictionary dictionary];
    
    [attributes setObject:[self font] forKey:NSFontAttributeName];
    [attributes setObject:[self textColor] forKey:NSForegroundColorAttributeName];
    [attributes setObject:[self backgroundColor] forKey:NSBackgroundColorAttributeName];
    
    [self setTypingAttributes:attributes];
}

/*
 deselectAll:
 
 Action method to select no text; the opposite of -selectAll:.  Makes an insertion point at the start of the current selection, if any.  Does nothing if there is no selection.
 
 Written by DJS 2007-02.
*/

- (IBAction)deselectAll:(id)sender;
{
    NSRange range = [self selectedRange];
    
    if (range.length == 0)
        return;
    
    range.length = 0;
    
    [self setSelectedRange:range];
}


// ----------------------------------------------------------------------------------------
#pragma mark -
// ----------------------------------------------------------------------------------------


/*
 appendStringValue:
 
 Appends the specified plain string to the end of the receiver.
 
 Written by DJS 2004-07.
*/

- (void)appendStringValue:(NSString *)value
{
    [[self textStorage] appendAttributedString:[NSAttributedString attributedStringWithString:value]];
}

/*
 appendAttributedStringValue:
 
 Appends the specified attributed string to the end of the receiver.
 
 Written by DJS 2004-07.
*/

- (void)appendAttributedStringValue:(NSAttributedString *)value
{
    [[self textStorage] appendAttributedString:value];
}


// ----------------------------------------------------------------------------------------
#pragma mark -
// ----------------------------------------------------------------------------------------


/*
 selectedStringValue
 
 Returns an autoreleased copy of the selected text of the receiver as a plain string.
 
 Written by DJS 2004-07.
*/

- (NSString *)selectedStringValue
{
    NSRange range = [self selectedRange];
    
    if (range.length == 0)
        return @"";
    else
        return [[self string] substringWithRange:range];
}

/*
 selectedOrAllStringValue
 
 Returns an autoreleased copy of the selected text of the receiver as a plain string.
 
 Written by DJS 2004-07.
*/

- (NSString *)selectedOrAllStringValue
{
    NSRange range = [self selectedRange];
    
    if (range.length == 0)
        return [self stringValue];
    else
        return [[self string] substringWithRange:range];
}

/*
 selectedAttributedStringValue
 
 Returns an autoreleased copy of the selected text of the receiver as an attributed string.
 
 Written by DJS 2004-07.
*/

- (NSAttributedString *)selectedAttributedStringValue
{
    NSRange range = [self selectedRange];
    
    if (range.length == 0)
        return [NSAttributedString attributedString];
    else
        return [[self textStorage] attributedSubstringFromRange:range];
}

/*
 selectedOrAllAttributedStringValue
 
 Returns an autoreleased copy of the selected text of the receiver as an attributed string.
 
 Written by DJS 2004-07.
*/

- (NSAttributedString *)selectedOrAllAttributedStringValue
{
    NSRange range = [self selectedRange];
    
    if (range.length == 0)
        return [self attributedStringValue];
    else
        return [[self textStorage] attributedSubstringFromRange:range];
}


// ----------------------------------------------------------------------------------------
#pragma mark -
// ----------------------------------------------------------------------------------------


/*
 stringValue
 
 Returns an autoreleased copy of the text as a plain string; see also the standard -string method for a non-copied edition.
 
 Written by DJS 2004-05.
*/

- (NSString *)stringValue
{
    return [[[self string] copy] autorelease];
}

/*
 attributedStringValue
 
 Returns an autoreleased copy of the text as an attributed string.
 
 Written by DJS 2004-05.
*/

- (NSAttributedString *)attributedStringValue
{
    return [[[self textStorage] copy] autorelease];
}

/*
 RTFValue
 
 Returns a RTF edition of the text.
 
 Written by DJS 2004-05.
*/

- (NSData *)RTFValue
{
    NSAttributedString *text = [self attributedStringValue];
    
    return [text RTFFromRange:NSMakeRange(0, [text length]) documentAttributes:nil];
}

/*
 RTFDValue
 
 Returns a RTFD edition of the text.
 
 Written by DJS 2004-05.
*/

- (NSData *)RTFDValue
{
    NSAttributedString *text = [self attributedStringValue];
    
    return [text RTFDFromRange:NSMakeRange(0, [text length]) documentAttributes:nil];
}


// ----------------------------------------------------------------------------------------
#pragma mark -
// ----------------------------------------------------------------------------------------


/*
 setStringValue:
 
 Replaces all of the text with the specified plain string (using the current font etc)
 
 Written by DJS 2004-05.
*/

- (void)setStringValue:(NSString *)value
{
    if (value)
        [self replaceCharactersInRange:[self allRange] withString:value];
}

/*
 setAttributedStringValue:
 
 Replaces all of the text with the specified attributed string.
 
 Written by DJS 2004-05.
*/

- (void)setAttributedStringValue:(NSAttributedString *)value
{
    if (value)
        [[self textStorage] replaceCharactersInRange:[self allRange] withAttributedString:[[value copy] autorelease]];
}

/*
 setRTFValue:
 
 Replaces all of the text with the specified RTF data.
 
 Written by DJS 2004-05.
*/

- (void)setRTFValue:(NSData *)value
{
    [self replaceCharactersInRange:[self allRange] withRTF:value];
}

/*
 setRTFDValue:
 
 Replaces all of the text with the specified RTFD data.
 
 Written by DJS 2004-05.
*/

- (void)setRTFDValue:(NSData *)value
{
    [self replaceCharactersInRange:[self allRange] withRTFD:value];
}


/*
 setRTFDValue:orRTF:orAttributedString:orString:
 
 Replaces all of the text with the richest of the specified values; any or all may be nil.  If none are valid, the text is cleared.
 
 Written by DJS 2004-05.
*/

- (void)setRTFDValue:(NSData *)theRTFD orRTF:(NSData *)theRTF orAttributedString:(NSAttributedString *)attrString orString:(NSString *)string
{
    if ([theRTFD length])
        [self setRTFDValue:theRTFD];
    else if ([theRTF length])
        [self setRTFValue:theRTF];
    else if ([attrString length])
        [self setAttributedStringValue:attrString];
    else if (string)
        [self setStringValue:string];
    else
        [self setStringValue:@""];
}

@end
