//
//  DSViewCategories.m
//  Shared categories
//
//  Created by David Sinclair on Fri Dec 10 2004.
//  Copyright (c) 2004 - 2007 Dejal Systems, LLC. All rights reserved.
//
//  Redistribution and use in source and binary forms, with or without modification,
//  are permitted provided that the following conditions are met:
//  
//  Redistributions of source code must retain this list of conditions and the following disclaimer.
//  
//  The name of Dejal Systems, LLC may not be used to endorse or promote products derived from this
//  software without specific prior written permission.
//  
//  THIS SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED,
//  INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
//  PURPOSE AND NONINFRINGEMENT OF THIRD PARTY RIGHTS. IN NO EVENT SHALL THE AUTHORS OR
//  COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN
//  ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE
//  SOFTWARE OR THE USE OR OTHER DEALINGS IN THIS SOFTWARE.
//


#import "DSViewCategories.h"


@implementation NSView (DSViewSizingCategories)

/*
 adjustAutoresizeMasks
 
 Adjusts the autoresize masks in the window so that when the window is resized, the receiver will expand vertically and other views will remain the same height.  Returns an array of the old masks, to be passed to -restoreAutoresizeMasks, below, to change the masks back to how they were before this method was invoked.
 
 Written by DJS 2006-01.
*/

- (NSArray *)adjustAutoresizeMasks;
{
    return [self adjustAutoresizingAroundPosition:NSMaxY([self frame]) stickPositionToTop:YES];
}

/*
 adjustAutoresizingAroundPosition:stickPositionToTop:
 
 Adjusts the autoresize masks in the window so that when the window is resized, the receiver will expand vertically and other views will remain the same height.  Returns an array of the old masks, to be passed to -restoreAutoresizeMasks, below, to change the masks back to how they were before this method was invoked.
 
 Written by DJS 2006-01, based on code from my DSViewsListView.
*/

- (NSArray *)adjustAutoresizingAroundPosition:(CGFloat)position stickPositionToTop:(BOOL)stickPositionToTop;
{
    NSMutableArray *subviewMasks = [NSMutableArray array];
    NSView *superview = self;
    NSView *oldSuperview = superview;
    
    while (superview)
    {
        // First adjust the superview's mask:
        NSUInteger mask = [superview autoresizingMask];
        [subviewMasks addObject:[NSNumber numberWithUnsignedInt:mask]];
        
        // Make it stick to the top and bottom of the window, and change height:
        mask |= NSViewHeightSizable;
        mask &= ~NSViewMaxYMargin;
        mask &= ~NSViewMinYMargin;
        
        [superview setAutoresizingMask:mask];
        
        NSArray *subviews = [superview subviews];
        
        for (id subview in subviews)
        {
            
            if (subview != oldSuperview)
            {
                mask = [subview autoresizingMask];
                BOOL stickToBottom = NSMaxY([subview frame]) <= position;
                
                if (!stickPositionToTop && (NSMaxY([subview frame]) == position))
                    stickToBottom = YES;
                
                [subviewMasks addObject:[NSNumber numberWithUnsignedInt:mask]];
                
                if (stickToBottom)
                {
                    // This subview is below us.  Make it stick to the bottom of the window and not change height:
                    mask &= ~NSViewHeightSizable;
                    mask |= NSViewMaxYMargin;
                    mask &= ~NSViewMinYMargin;
                }
                else
                {
                    // This subview is above us.  Make it stick to the top of the window, and not change height:
                    mask &= ~NSViewHeightSizable;
                    mask &= ~NSViewMaxYMargin;
                    mask |= NSViewMinYMargin;
                }
                
                [subview setAutoresizingMask:mask];
            }
        }
        
        // Go to this superview's superview and repeat the process; note that the looping algorithm must be replicated exactly in the restore method, below.  Ideally both methods should use another method to get the next subview, but I can't be bothered refactoring it at this stage, so just be aware of the issue:
        oldSuperview = superview;
        position = NSMaxY([superview frame]);
        superview = [superview superview];
    }
    
    return subviewMasks;
}

/*
 restoreAutoresizeMasks:
 
 Changes the window's autoresizing masks back to how they were before -adjustAutoresizeMasks or -adjustAutoresizingAroundPosition:stickPositionToTop: was invoked.
 
 Written by DJS 2006-01, based on code from my DSViewsListView.
*/

- (void)restoreAutoresizeMasks:(NSArray *)masks;
{
    NSView *superview = self;
    NSView *oldSuperview = superview;
    NSEnumerator *enumerator = [masks objectEnumerator];
    
    while (superview)
    {
        // First item is the superview's mask:
        [superview setAutoresizingMask:[[enumerator nextObject] unsignedIntValue]];
        
        // Following items are the subview masks:
        NSArray *subviews = [superview subviews];
        
        for (id subview in subviews)
        {
            
            if (subview != oldSuperview)
                [subview setAutoresizingMask:[[enumerator nextObject] unsignedIntValue]];
        }
        
        // Go to this superview's superview and repeat the process; note that the looping algorithm must be replicated exactly in the adjust method, above.  Ideally both methods should use another method to get the next subview, but I can't be bothered refactoring it at this stage, so just be aware of the issue:
        oldSuperview = superview;
        superview = [superview superview];
    }
}

@end


// ----------------------------------------------------------------------------------------
#pragma mark -
// ----------------------------------------------------------------------------------------


@implementation NSView (DSViewScalingCategories)

const NSSize unitSize = {1.0, 1.0};

/*
 resetScaling
 
 This method makes the scaling of the receiver equal to the window's base coordinate system.
 
 Written by Apple; added by DJS 2004-12.  See <http://developer.apple.com/qa/qa2004/qa1346.html>.
*/

- (void)resetScaling
{
    [self scaleUnitSquareToSize:[self convertSize:unitSize fromView:nil]];
}

/*
 scale
 
 This method returns the scale of the receiver's coordinate system, relative to the window's base coordinate system, expressed in absolute terms.
 
 Written by Apple; added by DJS 2004-12.  See <http://developer.apple.com/qa/qa2004/qa1346.html>.
*/

- (NSSize)scale
{
    return [self convertSize:unitSize fromView:nil];
}

/*
 setScale:
 
 This method sets the scale in absolute terms.
 
 Written by Apple; added by DJS 2004-12.  See <http://developer.apple.com/qa/qa2004/qa1346.html>.
*/

- (void)setScale:(NSSize)newScale
{
    [self resetScaling];  // First, match our scaling to the window's coordinate system
    [self scaleUnitSquareToSize:newScale]; // Then, set the scale.
}

/*
 scalePercent
 
 This method returns the scale of the receiver's coordinate system, relative to the window's base coordinate system, expressed as a percentage.
 
 Written by Apple; added by DJS 2004-12.  See <http://developer.apple.com/qa/qa2004/qa1346.html>.
*/

- (CGFloat)scalePercent
{
    return [self scale].width * 100;
}

/*
 setScalePercent:
 
 This method sets the scale expressed as a percentage.
 
 Written by Apple; added by DJS 2004-12.  See <http://developer.apple.com/qa/qa2004/qa1346.html>.
*/

- (void)setScalePercent:(CGFloat)scale
{
    scale = scale / 100.0;
    [self setScale:NSMakeSize(scale, scale)];
    [self setNeedsDisplay:YES];
}

@end
