//
//  DSArrayCategories.m
//  Shared categories
//
//  Created by David Sinclair on Fri May 30 2003.
//  Copyright (c) 2003 - 2009 Dejal Systems, LLC. All rights reserved.
//
//  Redistribution and use in source and binary forms, with or without modification,
//  are permitted provided that the following conditions are met:
//  
//  Redistributions of source code must retain this list of conditions and the following disclaimer.
//  
//  The name of Dejal Systems, LLC may not be used to endorse or promote products derived from this
//  software without specific prior written permission.
//  
//  THIS SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED,
//  INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
//  PURPOSE AND NONINFRINGEMENT OF THIRD PARTY RIGHTS. IN NO EVENT SHALL THE AUTHORS OR
//  COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN
//  ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE
//  SOFTWARE OR THE USE OR OTHER DEALINGS IN THIS SOFTWARE.
//

#import "DSObjectCategories.h"
#import "DSArrayCategories.h"


int finderSortWithLocale(id string1, id string2, void *locale)
{
    static int comparisonOptions = NSCaseInsensitiveSearch | NSNumericSearch | NSWidthInsensitiveSearch | NSForcedOrderingSearch;
    NSRange string1Range = NSMakeRange(0, [string1 length]);
    
    return [string1 compare:string2
                    options:comparisonOptions
                      range:string1Range
                     locale:(NSLocale *)locale];
}


// ----------------------------------------------------------------------------------------
#pragma mark -
// ----------------------------------------------------------------------------------------


@implementation NSArray (DSArrayCategories)

/*
 isValidIndex:
 
 Returns YES if the index is in the acceptable range for the receiver, or NO if not.  If using the index with -objectAtIndex: or similar when this returns NO, you'll get an exception.
 
 Written by DJS 2003-11.
*/

- (BOOL)isValidIndex:(NSUInteger)i
{
    return (i >= 0 && i < [self count]);
}

// See <http://www.macdevcenter.com/pub/a/mac/2004/07/16/hom.html> for an interesting technique to do something similar to the following methods.

/*
 objectMatching:usingKey:
 
 Returns the first dictionary object where the value for the specified key is equivalent to the match value (see -isEquivalentTo:), or nil if none do.  Useful for the common pattern of finding an object based on a dictionary value.  See also -indexOfObjectMatching:usingKey:, -objectMatching:usingSelector:, and -arrayWithObjectsMatching:usingKey:, below.
 
 Written by DJS 2004-04.
*/

- (id)objectMatching:(id)match usingKey:(NSString *)key
{
    NSEnumerator *enumerator = [self objectEnumerator];
    id object = nil;
    BOOL found = NO;
    
    while (!found && (object = [enumerator nextObject]) && [object respondsToSelector:@selector(objectForKey:)])
    	found = ([[object objectForKey:key] isEquivalentTo:match]);
    
    if (found)
        return object;
    else
        return nil;
}

/*
 indexOfObjectMatching:usingKey:
 
 Returns the index of the first dictionary object where the value for the specified key is equivalent to the match value (see -isEquivalentTo:), or NSNotFound if none do.  Useful for the common pattern of finding the index of an object based on a dictionary value.  See also -objectMatching:usingKey:, above, and -arrayWithObjectsMatching:usingKey:, below.
 
 Written by DJS 2004-04.
*/

- (NSUInteger)indexOfObjectMatching:(id)match usingKey:(NSString *)key
{
    NSUInteger i = 0;
    id object = nil;
    BOOL found = NO;
    
    while (!found && i < [self count])
    {
        if ((object = [self objectAtIndex:i]) && [object respondsToSelector:@selector(objectForKey:)])
            found = ([[object objectForKey:key] isEquivalentTo:match]);
        
        if (!found)
            i++;
    }
    
    if (found)
        return i;
    else
        return NSNotFound;
}

/*
 objectMatching:usingSelector:
 
 Returns the first object where the specified selector's return value is equivalent to the match value (see -isEquivalentTo:), or nil if none do.  Useful for the common pattern of finding an object based on an accessor value.  See also arrayWithObjectsMatching:usingSelector:, below.
 
 Written by DJS 2004-02.
*/

- (id)objectMatching:(id)match usingSelector:(SEL)selector
{
    NSEnumerator *enumerator = [self objectEnumerator];
    id object = nil;
    BOOL found = NO;
    
    while (!found && (object = [enumerator nextObject]))
    	found = ([[object performSelector:selector] isEquivalentTo:match]);
    
    if (found)
        return object;
    else
        return nil;
}

/*
 objectMatching:orDefault:orFirst:usingSelector:
 
 Returns the first object where the specified selector's return value is equivalent to the match value (see -isEquivalentTo:).  If none do, but a default match value was specified and an object matching that was found, it is returned instead.  If not, but firstIfNotFound is YES, the first object is returned.  Otherwise nil is returned.  Useful for the common pattern of finding an object based on an accessor value, and handling default cases.  See also objectMatching:usingSelector:, above.
 
 Written by DJS 2004-06.
*/

- (id)objectMatching:(id)match orDefault:(id)defaultMatch orFirst:(BOOL)firstIfNotFound usingSelector:(SEL)selector
{
    NSEnumerator *enumerator = [self objectEnumerator];
    id object = nil;
    id defaultObject = nil;
    id firstObject = nil;
    BOOL found = NO;
    
    while (!found && (object = [enumerator nextObject]))
    {
        id result = [object performSelector:selector];
        
        // Is it the one we want?
        if ([result isEquivalentTo:match])
            found = YES;
        
        // Is a default match specified, and is it that one?
        if (defaultMatch && [result isEquivalentTo:defaultMatch])
            defaultObject = object;
        
        // Have we not noted the first one yet, and do we care?
        if (!firstObject && firstIfNotFound)
            firstObject = object;
    }
    
    if (found)
        return object;
    else if (defaultObject)
        return defaultObject;
    else if (firstObject)
        return firstObject;
    else
        return nil;
}

/*
 arrayWithObjectsMatching:usingKey:
 
 Returns a new array of all objects that respond to -objectForKey: and that method's return value is equivalent to the match value (see -isEquivalentTo:).  If none do, an empty array is returned.  Useful for the common pattern of finding objects in an array of dictionaries.  See also -objectMatching:usingKey: and -arrayWithObjectsMatching:usingSelector:, above.
 
 Written by DJS 2005-03.
*/

- (NSArray *)arrayWithObjectsMatching:(id)match usingKey:(NSString *)key
{
    NSMutableArray *array = [NSMutableArray array];
    NSEnumerator *enumerator = [self objectEnumerator];
    id object = nil;
    
    while ((object = [enumerator nextObject]))
    {
    	if ([object respondsToSelector:@selector(objectForKey:)] && [[object objectForKey:key] isEquivalentTo:match])
            [array addObject:object];
    }
    
    return array;
}

/*
 arrayWithObjectsMatching:usingSelector:
 
 Returns a new array of all objects where the specified selector's return value is equivalent to the match value (see -isEquivalentTo:).  If none do, an empty array is returned.  Useful for the common pattern of finding objects based on an accessor value.  See also -objectMatching:usingSelector:, above, and -arrayWithObjectsMatching:usingKey:, below.
 
 Written by DJS 2004-02.
*/

- (NSArray *)arrayWithObjectsMatching:(id)match usingSelector:(SEL)selector
{
    NSMutableArray *array = [NSMutableArray array];
    NSEnumerator *enumerator = [self objectEnumerator];
    id object = nil;
    
    while ((object = [enumerator nextObject]))
    {
    	if ([[object performSelector:selector] isEquivalentTo:match])
            [array addObject:object];
    }
    
    return array;
}

/*
 arrayUsingSelector:
 
 Returns a new array containing the values returned by the specified selector for each object in the receiver.  The selector would normally be an accessor, and must take no parameters and return an object value.
 
 Written by DJS 2004-02.
*/

- (NSArray *)arrayUsingSelector:(SEL)selector
{
    NSMutableArray *array = [NSMutableArray array];
    NSEnumerator *enumerator = [self objectEnumerator];
    id object = nil;
    
    while ((object = [enumerator nextObject]))
    {
    	[array addObject:[object performSelector:selector]];
    }
    
    return array;
}

/*
 arrayByRemovingObjectsInArray:
 
 Returns a new array containing only the objects from the receiver that are not also in the other array.
 
 Written by DJS 2004-04.
*/

- (NSArray *)arrayByRemovingObjectsInArray:(NSArray *)otherArray
{
    NSMutableArray *array = [[self mutableCopy] autorelease];
    
    [array removeObjectsInArray:otherArray];
    
    return array;
}

/*
 reverseArray
 
 Returns a reversed edition of the reciever.
 
 Written by DJS 2004-03.
*/

- (NSArray *)reverseArray
{
    NSMutableArray *array = [NSMutableArray arrayWithCapacity:[self count]];
    NSEnumerator *enumerator = [self reverseObjectEnumerator];
    id object;
    
    while ((object = [enumerator nextObject]))
    {
    	[array addObject:object];
    }
    
    return array;
}

/*
 sortedArrayUsingFinderOrder
 
 Returns a copy of the receiver sorted as in the Finder.
 
 Written by DJS 2008-01.
*/

- (NSArray *)sortedArrayUsingFinderOrder;
{
    return [self sortedArrayUsingFunction:finderSortWithLocale context:[NSLocale currentLocale]];
}

/*
 containsObjectIdenticalTo:

 Returns YES if the object is already in the array.
*/

- (BOOL)containsObjectIdenticalTo:(id)obj
{
    return [self indexOfObjectIdenticalTo:obj] != NSNotFound;
}

/*
 containsObjectEquivalentTo:
 
 Returns YES if the receiver contains an object that is equivalent to the specified one.  See -isEquivalentTo:.
 
 Written by DJS 2005-05.
*/

- (BOOL)containsObjectEquivalentTo:(id)obj
{
    return [self indexOfObjectEquivalentTo:obj] != NSNotFound;
}

/*
 indexOfObjectEquivalentTo:
 
 Returns the index of the first object in the receiver that is equivalent to the specified one.  If none is found, returns NSNotFound.  See -isEquivalentTo:.
 
 Written by DJS 2005-05.
*/

- (NSUInteger)indexOfObjectEquivalentTo:(id)obj
{
    NSUInteger i = 0;
    id object = nil;
    BOOL found = NO;
    
    while (!found && i < [self count])
    {
        if ((object = [self objectAtIndex:i]))
            found = [object isEquivalentTo:obj];
        
        if (!found)
            i++;
    }
    
    if (found)
        return i;
    else
        return NSNotFound;
}

/*
 firstObject
 
 Returns the first object in the array, or nil if the array is empty.
 
 Written by DJS 2003-06.
*/

- (id)firstObject
{
    if ([self count] >= 1)
        return [self objectAtIndex:0];
    else
        return nil;
}

/*
 penultimateObject
 
 Returns the second-to-last object in the array, or nil if the array is empty or only contains one object.
 
 Written by DJS 2003-06.
*/

- (id)penultimateObject
{
    NSUInteger numberOfItems = [self count];
    
    if (numberOfItems >= 2)
        return [self objectAtIndex:numberOfItems - 2];
    else
        return nil;
}

/*
 deepCopy
 
 Similar to -copy, but each of the objects in the array are copied too.  Note that like -copy, the array is retained.
 
 Written by DJS 2004-06.
 Changed by DJS 2006-01 to fix memory leak through excessive retaining.
*/

- (id)deepCopy
{
    id array = [[NSMutableArray alloc] init];
    id copy;
    
    for (id object in self)
    {
        if ([object respondsToSelector:@selector(deepCopy)])
            copy = [object deepCopy];
        else
            copy = [object copy];
        
    	[array addObject:copy];
        
        // Both -deepCopy and -copy retain the object, and so does -addObject:, so need to -release:
        [copy release];
    }
    
    return array;
}

/*
 deepMutableCopy
 
 Similar to -deepCopy, above, but makes all of the contents of the dictionary mutable.
 
 Written by DJS 2009-01.
 */

- (id)deepMutableCopy;
{
    id array = [[NSMutableArray alloc] init];
    id mutableCopy;
    
    for (id object in self)
    {
        if ([object respondsToSelector:@selector(deepMutableCopy)])
            mutableCopy = [object deepMutableCopy];
        else if ([object conformsToProtocol:@protocol(NSMutableCopying)])
            mutableCopy = [object mutableCopy];
        else
            mutableCopy = [object copy];
        
    	[array addObject:mutableCopy];
        
        // Both -deepMutableCopy and -copy retain the object, and so does -addObject:, so need to -release:
        [mutableCopy release];
    }
    
    return array;
}

@end


// ----------------------------------------------------------------------------------------
#pragma mark -
// ----------------------------------------------------------------------------------------


@implementation NSMutableArray (DSArrayCategories)

/*
 insertObject:atIndex:orEnd:
 
 Similar to -insertObject:atIndex:, but if the index is out of range, it inserts at the end (if atEnd is YES) or at the start (if atEnd is NO), rather than giving an exception.  Returns the index the object was inserted at.
 
 Written by DJS 2004-01.
*/

- (NSUInteger)insertObject:(id)object atIndex:(NSUInteger)i orEnd:(BOOL)atEnd
{
    NSUInteger count = [self count];
    
    // Remember that the index can equal count for -insertObject:atIndex: -- that just means to insert afterwards, but check for it here in case atEnd is NO:
    if (i < 0 || i > count)
    {
        if (atEnd)
            i = count;
        else
            i = 0;
    }
    
    [self insertObject:object atIndex:i];
    
    return i;
}

/*
 insertObjectsFromArray:atIndex:

 Similar to -insertObject:atIndex:, but inserts multiple objects at the index position.
*/

- (void)insertObjectsFromArray:(NSArray *)array atIndex:(NSUInteger)i
{
    NSEnumerator *enumerator = [array objectEnumerator];
    id object;
    
    while ((object = [enumerator nextObject]))
    {
        [self insertObject:object atIndex:i++];
    }
}

/*
 insertOrMoveObjectsFromArray:atIndex:
 
 Inserts objects from the specified array at the index position, removing them from their old position if already present.  If the index is invalid, the objects are added to the array (instead of causing an exception).  The adjusted index is returned; it is adjusted if some old indexes were before the new index.
 
 Written by DJS 2003-11.
 Changed by DJS 2007-04 to return the adjusted index.
*/

- (NSUInteger)insertOrMoveObjectsFromArray:(NSArray *)array atIndex:(NSUInteger)i;
{
    NSEnumerator *enumerator = [array objectEnumerator];
    id object;
    
    while ((object = [enumerator nextObject]))
    {
        NSUInteger existing = [self indexOfObject:object];
        
        if (existing != NSNotFound)
        {
            if (existing < i)
                i--;
            
            [self removeObjectAtIndex:existing];
        }
    }
    
    if ([self isValidIndex:i])
        [self insertObjectsFromArray:array atIndex:i];
    else
    {
        i = [array count];
        [self addObjectsFromArray:array];
    }
    
    return i;
}

/*
 addOrMoveObjectsFromArray:
 
 Adds objects from the specified array, removing them from their old position if already present.
 
 Written by DJS 2003-11.
*/

- (void)addOrMoveObjectsFromArray:(NSArray *)array
{
    [self insertOrMoveObjectsFromArray:array atIndex:-1];
}

/*
 arrayWithIndexEnumerator:
 
 Given an enumerator for an array of index numbers (e.g. as used in NSTableView's -selectedRowEnumerator), this returns a new mutable array containing the corresponding objects from the receiver.
 
 Written by DJS 2004-01.
*/

- (NSMutableArray *)arrayWithIndexEnumerator:(NSEnumerator *)enumerator
{
    NSMutableArray *array = [NSMutableArray array];
    NSNumber *i;
    
    while ((i = [enumerator nextObject]))
    {
        id object = [self objectAtIndex:[i intValue]];
        
        if (object)
            [array addObject:object];
    }
    
    return array;
}

/*
 removeObjectsFromIndexEnumerator:
 
 Given an enumerator for an array of index numbers (e.g. as returned by -[NSTableView selectedRowEnumerator]), this removes the corresponding objects from the receiver.
 
 Written by DJS 2004-01.
*/

- (void)removeObjectsFromIndexEnumerator:(NSEnumerator *)enumerator
{
    [self removeObjectsInArray:[self arrayWithIndexEnumerator:enumerator]];
}

/*
 removeObjectsMatching:usingKey:
 
 Given a value to match and a key to use to find that value, this removes the corresponding objects from the receiver.  Useful for removing all dictionaries in an array that have the match value.
 
 Written by DJS 2005-03.
*/

- (void)removeObjectsMatching:(id)match usingKey:(NSString *)key
{
    [self removeObjectsInArray:[self arrayWithObjectsMatching:match usingKey:key]];
}

/*
 reverseArray
 
 Returns a mutable reversed edition of the reciever.
 
 Written by DJS 2004-03.
*/

- (NSMutableArray *)reverseArray
{
    return [[[super reverseArray] mutableCopy] autorelease];
}

@end


// ----------------------------------------------------------------------------------------
#pragma mark -
// ----------------------------------------------------------------------------------------


@implementation NSMutableArray (DSToolbarCategories)

/*
 toolbarItemForIdentifier:
 
 Finds and returns the toolbar item with the specified identifier, or nil if it isn't found.
 
 Written by DJS 2004-02.
*/

- (NSToolbarItem *)toolbarItemForIdentifier:(NSString *)identifier
{
    return [self objectMatching:identifier usingSelector:@selector(itemIdentifier)];
}

/*  longhand edition:
- (NSToolbarItem *)toolbarItemForIdentifier:(NSString *)identifier
{
    NSEnumerator *enumerator = [self objectEnumerator];
    NSToolbarItem *item;
    BOOL found = NO;
    
    while (!found && (item = [enumerator nextObject]))
    {
    	found = ([[item itemIdentifier] isEqualToString:identifier]);
    }
    
    if (found)
        return item;
    else
        return nil;
}
*/

/*
 toolbarItemIdentifiers
 
 Returns a new array containing the identifiers for all of the toolbar items.
 
 Written by DJS 2004-02.
*/

- (NSArray *)toolbarItemIdentifiersIncludingStandardItems:(BOOL)inclStd
{
    NSArray *items = [self arrayUsingSelector:@selector(itemIdentifier)];
    
    if (inclStd)
    {
        items = [items arrayByAddingObjectsFromArray:[NSArray arrayWithObjects:NSToolbarCustomizeToolbarItemIdentifier, NSToolbarFlexibleSpaceItemIdentifier, NSToolbarSpaceItemIdentifier, NSToolbarSeparatorItemIdentifier, nil]];
    }
    
    return items;
}

/*
 addToolbarItem:label:paletteLabel:toolTip:view:minWidth:maxWidth:minHeight:maxHeight:
 menuRepresentation:enabled:target:action:tag:
 
 Sets up a view-based toolbar item.  Pass zero for any of the min or max width or height values to use the size of the view frame for that value.  Pass nil for the menu item if a menu representation is not desired.  Pass nil for the submenu if one is not desired.  If a submenu is provided and a menu rep is not, a default menu rep is created.
 
 Written by DJS 2003-10.
 Changed by DJS 2004-02 to add submenu support.
 Changed by DJS 2004-02 to convert from NSMutableDictionary edition.
*/

- (void)addToolbarItem:(NSString *)identifier
                 label:(NSString *)label
          paletteLabel:(NSString *)paletteLabel
               toolTip:(NSString *)toolTip
                  view:(id)view
              minWidth:(CGFloat)minWidth
              maxWidth:(CGFloat)maxWidth
             minHeight:(CGFloat)minHeight
             maxHeight:(CGFloat)maxHeight
    menuRepresentation:(NSMenuItem *)menuItem
               submenu:(NSMenu *)submenu
               enabled:(BOOL)enable
                target:(id)target
                action:(SEL)action
                   tag:(NSInteger)tag
{
    NSToolbarItem *item = [[NSToolbarItem alloc] initWithItemIdentifier:identifier];
    
    [item setLabel:label]; 					// Name for the item in the toolbar
    [item setPaletteLabel:paletteLabel]; 			// Name for the "Customize Toolbar" sheet
    [item setToolTip:toolTip];
    [item setView:view];
    [item setMinSize:NSMakeSize(minWidth ? minWidth : NSWidth([view frame]),
                                minHeight ? minHeight : NSHeight([view frame]))];
    [item setMaxSize:NSMakeSize(maxWidth ? maxWidth : NSWidth([view frame]),
                                maxHeight ? maxHeight : NSHeight([view frame]))];
    
    // If we have a submenu but no item to attach it to, create one:
    if (submenu && !menuItem)
    {
        menuItem = [[[NSMenuItem alloc] init] autorelease];
        [menuItem setTitle:label];
    }
    
    // If we have a submenu, attach it to the menu item:
    if (submenu)
        [menuItem setSubmenu:submenu];
    
    // If we have a menu item, set it as the menu representation of the toolbar item:
    if (menuItem)
        [item setMenuFormRepresentation:menuItem];
    
    [item setEnabled:enable];
    [item setTarget:target];
    [item setAction:action];
    [item setTag:tag];
    
    // Add to toolbar array:
    [self addObject:item];
    
    [item release];
}

/*
 addToolbarItem:label:paletteLabel:toolTip:imageNamed:enabled:target:action:tag:
 
 Sets up an image-based toolbar item.
 
 Written by DJS 2002-08.
 Changed by DJS 2004-02 to convert from NSMutableDictionary edition.
*/

- (void)addToolbarItem:(NSString *)identifier
                 label:(NSString *)label
          paletteLabel:(NSString *)paletteLabel
               toolTip:(NSString *)toolTip
            imageNamed:(NSString *)imageName
               enabled:(BOOL)enable
                target:(id)target
                action:(SEL)action
                   tag:(NSInteger)tag
{
    NSToolbarItem *item = [[NSToolbarItem alloc] initWithItemIdentifier:identifier];
    
    [item setLabel:label]; 					// Name for the item in the toolbar
    [item setPaletteLabel:paletteLabel]; 			// Name for the "Customize Toolbar" sheet
    [item setToolTip:toolTip];
    [item setImage:[NSImage imageNamed:imageName]];
    [item setEnabled:enable];
    [item setTarget:target];
    [item setAction:action];
    [item setTag:tag];
    
    // Add to toolbar array:
    [self addObject:item];
    
    [item release];
}

/*
 addToolbarItem:label:paletteLabel:toolTip:imageNamed:enabled:target:action:
 
 Sets up an image-based toolbar item.
 
 Written by DJS 2002-08.
 Changed by DJS 2004-02 to convert from NSMutableDictionary edition.
*/

- (void)addToolbarItem:(NSString *)identifier
                 label:(NSString *)label
          paletteLabel:(NSString *)paletteLabel
               toolTip:(NSString *)toolTip
            imageNamed:(NSString *)imageName
               enabled:(BOOL)enable
                target:(id)target
                action:(SEL)action
{
    [self addToolbarItem:identifier
                   label:label
            paletteLabel:paletteLabel
                 toolTip:toolTip
              imageNamed:imageName
                 enabled:enable
                  target:target
                  action:action
                     tag:0];
}

@end
