//
//  DSDateCategories.m
//  Shared categories
//
//  Created by David Sinclair on Thu Apr 17 2003.
//  Copyright (c) 2003 - 2007 Dejal Systems, LLC. All rights reserved.
//
//  Redistribution and use in source and binary forms, with or without modification,
//  are permitted provided that the following conditions are met:
//  
//  Redistributions of source code must retain this list of conditions and the following disclaimer.
//  
//  The name of Dejal Systems, LLC may not be used to endorse or promote products derived from this
//  software without specific prior written permission.
//  
//  THIS SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED,
//  INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
//  PURPOSE AND NONINFRINGEMENT OF THIRD PARTY RIGHTS. IN NO EVENT SHALL THE AUTHORS OR
//  COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN
//  ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE
//  SOFTWARE OR THE USE OR OTHER DEALINGS IN THIS SOFTWARE.
//

#import "DSDateCategories.h"


@implementation NSDate (DSDateCategories)

/*
 carbonEpoch
 
 Convenience method creates and returns an autoreleased date representing the Carbon base date (1904-01-01).
 
 Written by DJS 2003-04.
*/

+ (NSDate *)carbonEpoch
{
    static NSCalendarDate *_carbonEpoch;

    if (_carbonEpoch == nil)
        _carbonEpoch = [[NSCalendarDate alloc] initWithYear:1904 month:1 day:1 hour:0 minute:0 second:0
                                                   timeZone:[NSTimeZone localTimeZone]];

    return _carbonEpoch;
}

/*
 dateWithCarbonDate:

 Convenience method creates and returns an autoreleased date representing the specified Carbon date.

 Written by DJS 2003-04.
*/

+ (NSDate *)dateWithCarbonDate:(unsigned long)carbonDate
{
    return [[NSDate carbonEpoch] addTimeInterval:carbonDate];
}

/*
 carbonDate

 Returns the Carbon date representation of the receiver.

 Written by DJS 2003-04.
*/

- (unsigned long)carbonDate
{
    return [self timeIntervalSinceDate:[NSDate carbonEpoch]];
}

/*
 convertToCalendarDate
 
 Creates and returns a NSCalendarDate representation of the receiver.
 
 Written by DJS 2003-07.
*/

- (NSCalendarDate *)convertToCalendarDate
{
    return [NSCalendarDate dateWithTimeIntervalSinceReferenceDate:[self timeIntervalSinceReferenceDate]];
}

/*
 descriptionWithShortDateAndOrTime
 
 Returns the receiver expressed as a string using the user's preferred short date and/or time format.  If the date component is zero, only the time is output; if the time component is zero, only the date is output; if both are non-zero, both are output.  Note: it is more efficient to call one of the following methods, if it is known whether a date and/or time is present.  This method is compatible both with NSDate and NSCalendarDate.
 
 Written by DJS 2004-05.
*/

- (NSString *)descriptionWithShortDateAndOrTime
{
    NSCalendarDate *date;
    
    if ([self isKindOfClass:[NSCalendarDate class]])
        date = (NSCalendarDate *)self;
    else
        date = [self convertToCalendarDate];
    
    if ([date isEqualToDate:[date dateComponent]])
        return [self descriptionWithShortDate];
    else if ([date isEqualToDate:[date timeComponent]])
        return [self descriptionWithTime];
    else
        return [self descriptionWithShortDateTime];
}

/*
 descriptionWithShortDateTime
 
 Returns the receiver expressed as a string using the user's preferred short date and time formats.  This method is compatible both with NSDate and NSCalendarDate.
 
 Written by DJS 2003-11.
 Changed by DJS 2007-10 to avoid deprecated NSShortDateFormatString etc.
*/

- (NSString *)descriptionWithShortDateTime;
{
    NSDateFormatter *dateFormatter = [[NSDateFormatter alloc] init];
    
    [dateFormatter setDateStyle:NSDateFormatterShortStyle];
    [dateFormatter setTimeStyle:NSDateFormatterShortStyle];
    
    return [dateFormatter stringFromDate:self];
}

/*
 descriptionWithShortDate
 
 Returns the receiver expressed as a string using the user's preferred short date format.  This method is compatible both with NSDate and NSCalendarDate.
 
 Written by DJS 2003-11.
 Changed by DJS 2007-10 to avoid deprecated NSShortDateFormatString etc.
*/

- (NSString *)descriptionWithShortDate
{
    NSDateFormatter *dateFormatter = [[NSDateFormatter alloc] init];
    
    [dateFormatter setDateStyle:NSDateFormatterShortStyle];
    [dateFormatter setTimeStyle:NSDateFormatterNoStyle];
    
    return [dateFormatter stringFromDate:self];
}

/*
 descriptionWithTime
 
 Returns the receiver expressed as a string using the user's preferred short time format.  This method is compatible both with NSDate and NSCalendarDate.
 
 Written by DJS 2003-11.
 Changed by DJS 2007-10 to avoid deprecated NSShortDateFormatString etc.
*/

- (NSString *)descriptionWithTime
{
    NSDateFormatter *dateFormatter = [[NSDateFormatter alloc] init];
    
    [dateFormatter setDateStyle:NSDateFormatterNoStyle];
    [dateFormatter setTimeStyle:NSDateFormatterShortStyle];
    
    return [dateFormatter stringFromDate:self];
}

@end


// ----------------------------------------------------------------------------------------
#pragma mark -
// ----------------------------------------------------------------------------------------


@implementation NSCalendarDate (DSDateCategories)

/*
 dateWithYear:month:day:
 
 Returns a new calendar date with the specified date, and the current time.
 
 Written by DJS 2007-04.
*/

+ (id)dateWithYear:(NSInteger)year month:(NSUInteger)month day:(NSUInteger)day;
{
    id calendarDate = [self calendarDate];
    NSInteger yearDiff = year - [calendarDate yearOfCommonEra];
    NSInteger monthDiff = month - [calendarDate monthOfYear];
    NSInteger dayDiff = day - [calendarDate dayOfMonth];
    
    return [calendarDate dateByAddingYears:yearDiff months:monthDiff days:dayDiff hours:0 minutes:0 seconds:0];
}

/*
 dateWithHour:minute:second:
 
 Returns a calendar date with the current date, and the specified time.
 
 Written by DJS 2007-04.
*/

+ (id)dateWithHour:(NSUInteger)hour minute:(NSUInteger)minute second:(NSUInteger)second;
{
    id calendarDate = [self calendarDate];
    NSInteger hourDiff = hour - [calendarDate hourOfDay];
    NSInteger minuteDiff = minute - [calendarDate minuteOfHour];
    NSInteger secondDiff = second - [calendarDate secondOfMinute];
    
    return [calendarDate dateByAddingYears:0 months:0 days:0 hours:hourDiff minutes:minuteDiff seconds:secondDiff];
}

/*
 dateComponent
 
 Returns a new calendar date with just the date portion of the receiver, i.e. stripping out the time.
 
 Written by DJS 2004-05.
*/

- (id)dateComponent
{
    return [self dateByAddingYears:0 months:0 days:0 hours:-[self hourOfDay] minutes:-[self minuteOfHour] seconds:-[self secondOfMinute]];
}

/*
 timeComponent
 
 Returns a new calendar date with just the time portion of the receiver, i.e. stripping out the date.
 
 Written by DJS 2004-05.
*/

- (id)timeComponent
{
    return [self dateByAddingYears:-[self yearOfCommonEra] months:-[self monthOfYear] days:-[self dayOfMonth] hours:0 minutes:0 seconds:0];
}

/*
 dateBySettingDateComponent:
 
 Returns a calendar date with the date component of the value, and the time component of the receiver.
 
 Written by DJS 2003-07.
*/

- (id)dateBySettingDateComponent:(NSCalendarDate *)value
{
    NSInteger yearDiff = [value yearOfCommonEra] - [self yearOfCommonEra];
    NSInteger monthDiff = [value monthOfYear] - [self monthOfYear];
    NSInteger dayDiff = [value dayOfMonth] - [self dayOfMonth];
    
    return [self dateByAddingYears:yearDiff months:monthDiff days:dayDiff hours:0 minutes:0 seconds:0];
}

/*
 dateBySettingTimeComponent:
 
 Returns a calendar date with the date component of the receiver, and the time component of the value.
 
 Written by DJS 2003-07.
*/

- (id)dateBySettingTimeComponent:(NSCalendarDate *)value
{
    NSInteger hourDiff = [value hourOfDay] - [self hourOfDay];
    NSInteger minuteDiff = [value minuteOfHour] - [self minuteOfHour];
    NSInteger secondDiff = [value secondOfMinute] - [self secondOfMinute];

    return [self dateByAddingYears:0 months:0 days:0 hours:hourDiff minutes:minuteDiff seconds:secondDiff];
}

@end

