//
//  DSFileManagerCategories.m
//  Shared categories
//
//  Created by David Sinclair on Tue Mar 23 2004.
//  Copyright (c) 2004 - 2007 Dejal Systems, LLC. All rights reserved.
//
//  Redistribution and use in source and binary forms, with or without modification,
//  are permitted provided that the following conditions are met:
//  
//  Redistributions of source code must retain this list of conditions and the following disclaimer.
//  
//  The name of Dejal Systems, LLC may not be used to endorse or promote products derived from this
//  software without specific prior written permission.
//  
//  THIS SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED,
//  INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
//  PURPOSE AND NONINFRINGEMENT OF THIRD PARTY RIGHTS. IN NO EVENT SHALL THE AUTHORS OR
//  COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN
//  ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE
//  SOFTWARE OR THE USE OR OTHER DEALINGS IN THIS SOFTWARE.
//

#import "DSFileManagerCategories.h"
#import "DSObjectCategories.h"
#import "DSArrayCategories.h"
#import "DSStringCategories.h"


@implementation NSFileManager (DSFileManagerCategories)

/*
 fileSizeAtPath:
 
 Convenience method to return the size of the file specified in the path.
 
 Written by DJS 2004-05.
*/

- (NSInteger)fileSizeAtPath:(NSString *)path
{
    NSDictionary *attributes = [self fileAttributesAtPath:path traverseLink:YES];
    
    return [[attributes objectForKey:NSFileSize] intValue];
}

/*
 fileCreationDateAtPath:
 
 Convenience method to return the creation date of the file specified in the path.
 
 Written by DJS 2006-11.
*/

- (NSDate *)fileCreationDateAtPath:(NSString *)path;
{
    NSDictionary *attributes = [self fileAttributesAtPath:path traverseLink:YES];
    
    return [attributes objectForKey:NSFileCreationDate];
}

/*
 fileModificationDateAtPath:
 
 Convenience method to return the modification date of the file specified in the path.
 
 Written by DJS 2005-11.
*/

- (NSDate *)fileModificationDateAtPath:(NSString *)path;
{
    NSDictionary *attributes = [self fileAttributesAtPath:path traverseLink:YES];
    
    return [attributes objectForKey:NSFileModificationDate];
}

/*
 isDirectoryAtPath:
 
 Convenience method to return YES if the specified path is a directory, otherwise NO.
 
 Written by DJS 2005-10.
*/

- (BOOL)isDirectoryAtPath:(NSString *)path;
{
    return [self isDirectoryAtPath:path butNotPackage:NO];
}

/*
 isDirectoryAtPath:butNotPackage:
 
 Convenience method to return YES if the specified path is a directory and optionally also not a file package (bundle), otherwise NO.
 
 Written by DJS 2007-02.
*/

- (BOOL)isDirectoryAtPath:(NSString *)path butNotPackage:(BOOL)notPackage;
{
    BOOL isDirectory = NO;
    
    [self fileExistsAtPath:path isDirectory:&isDirectory];
    
    if (notPackage)
        return (isDirectory && ![[NSWorkspace sharedWorkspace] isFilePackageAtPath:path]);
    else
        return isDirectory;
}

/*
 renameFileAtPath:toFilename:deleteExisting:handler:
 
 Renames the file or directory specified in the path to have the specified filename, in the same location.  If deleteExisting is YES and the destination already exists, it is first deleted; otherwise an error will occur.  See the standard -movePath:toPath:handler: method description for details of the handler parameter and other relevant information.
 
 Written by DJS 2004-03.
 Changed by DJS 2005-12 to handle a special case when just changing the case of the name.
*/

- (BOOL)renameFileAtPath:(NSString *)path toFilename:(NSString *)filename deleteExisting:(BOOL)deleteExisting handler:(id)handler
{
    NSString *destination = [[path stringByDeletingLastPathComponent] stringByAppendingPathComponent:filename];
    
    // Special case: if the current name and the filename are equivalent, i.e. only differing in case, removing would be wrong and moving would fail, so we need to change the name:
    if ([[path lastPathComponent] isEquivalentTo:filename])
    {
        NSString *oldPath = path;
        
        path = [path stringByAppendingFormat:@"%.2f", [NSDate timeIntervalSinceReferenceDate]];
        
        [self movePath:oldPath toPath:path handler:handler];
    }
    
    if (deleteExisting && [self fileExistsAtPath:destination])
        [self removeFileAtPath:destination handler:handler];
    
    return [self movePath:path toPath:destination handler:handler];
}

/*
 renameWorkingFile:forSuccess:handler:
 
 Given a working file, i.e. one named with a tilde before the extension (see -[NSString backupFilePath]), swaps the working file and the previous final file (if any).
 
 Written by DJS 2004-03.
*/

- (BOOL)renameWorkingFile:(NSString *)workingPath forSuccess:(BOOL)success handler:(id)handler
{
    BOOL okay = YES;
    
    if (success)
    {
        NSString *basePath = [workingPath stringByDeletingLastPathComponent];
        NSString *extension = [workingPath pathExtension];
        NSString *workingFilename = [workingPath lastPathComponent];
        NSString *workingBase = [workingFilename stringByDeletingPathExtension];
        NSString *finalFilename = [workingBase substringToIndex:[workingBase length] - 1];
        NSString *tempFilename = [workingBase stringByAppendingString:@"~"];
        
        if ([extension length])
        {
            finalFilename = [finalFilename stringByAppendingPathExtension:extension];
            tempFilename = [tempFilename stringByAppendingPathExtension:extension];
        }
        
        NSString *finalPath = [basePath stringByAppendingPathComponent:finalFilename];
        NSString *tempPath = [basePath stringByAppendingPathComponent:tempFilename];
        
        // Rename the working file with an extra tilde on the end:
        okay = [self renameFileAtPath:workingPath toFilename:tempFilename deleteExisting:YES handler:handler];
        
        // Rename the previous file the same as the working file was:
        if ([self fileExistsAtPath:finalPath])
            okay = okay && [self renameFileAtPath:finalPath toFilename:workingFilename deleteExisting:NO handler:handler];
        
        // Rename the working file the same as the previous file was:
        okay = okay && [self renameFileAtPath:tempPath toFilename:finalFilename deleteExisting:NO handler:handler];
    }
    else
    {
        // Failed, so just delete the working file:
        okay = [self removeFileAtPath:workingPath handler:handler];
    }
    
    return okay;
}


/*
 removeFileIfExistsAtPath:
 
 If a file or folder exists at the path, it is blindly removed.  As with -removeFileAtPath:handler:, use caution with this method.  Returns YES if the file didn't exist, or was successsfully removed, or NO if it couldn't be removed.
 
 Written by DJS 2004-06.
*/

- (BOOL)removeFileIfExistsAtPath:(NSString *)path
{
    if ([self fileExistsAtPath:path])
        return [self removeFileAtPath:path handler:nil];
    else
        return YES;
}

/*
 trashFileIfExistsAtPath:
 
 If a file or folder exists at the path, it is moved to the Trash.  As with -removeFileAtPath:handler:, use caution with this method.  Returns YES if the file didn't exist, or was successsfully trashed, or NO if it couldn't be trashed.
 
 Written by DJS 2007-07.
*/

- (BOOL)trashFileIfExistsAtPath:(NSString *)path;
{
    if ([self fileExistsAtPath:path])
        return [[NSWorkspace sharedWorkspace] performFileOperation:NSWorkspaceRecycleOperation source:[path stringByDeletingLastPathComponent] destination:@"" files:[NSArray arrayWithObject:[path lastPathComponent]] tag:nil];
    else
        return YES;
}

/*
 backupPath:
 
 Convenience class method to duplicate the path with a " backup" suffix, replacing any old backup.
 
 Written by DJS 2007-04.
*/

+ (BOOL)backupPath:(NSString *)path;
{
    return [[self defaultManager] copyPath:path withSuffix:@" backup" replaceExisting:YES handler:nil];
}

/*
 copyPath:withSuffix:replaceExisting:handler:
 
 Like -copyFile:toPath:handler:, but simply uses the same path as the destination with the suffix appended.  Does nothing if no file exists at the specified path, or the suffix is empty.  Optionally deletes any old file at the new path first.  This is particularly useful for doing duplicate operations or backups of files or folders.
 
 Written by DJS 2007-04.
*/

- (BOOL)copyPath:(NSString *)path withSuffix:(NSString *)suffix replaceExisting:(BOOL)replace handler:(id)handler;
{
    if ([self fileExistsAtPath:path] && [suffix length])
    {
        NSString *destPath = [[path stringByDeletingPathExtension] stringByAppendingString:suffix];
        destPath = [destPath stringByAppendingPathExtension:[path pathExtension]];
        
        if (replace)
            [self removeFileIfExistsAtPath:destPath];
        
        return [self copyPath:path toPath:destPath handler:handler];
    }
    else
        return NO;
}

/*
 pathsWithFragments:isExtension:atPath:deepScan:
 
 Given an array of filenames or extensions (which may be nil or empty) and a directory path, returns an array of full paths that have those filenames or extensions (or all files if no fragments provided) within that directory.  If deep is YES, all subdirectories are scanned, otherwise only the base one is scanned.  If the base path isn't present, or isn't a directory, nil is returned.  The base path may include a tilde.  See also the following more specific methods.
 
 Written by DJS 2005-05.
 Changed by DJS 2005-10 to add support for filenames.
 Changed by DJS 2008-01 to sort in Finder order.
*/

- (NSArray *)pathsWithFragments:(NSArray *)fragments isExtension:(BOOL)isExtension
                         atPath:(NSString *)basePath deepScan:(BOOL)deep;
{
    basePath = [basePath expandedPath];
    BOOL isDirectory = NO;
    
    if (![self fileExistsAtPath:basePath isDirectory:&isDirectory] || !isDirectory)
        return nil;
    
    NSMutableArray *paths = [NSMutableArray array];
    NSArray *subpaths;
    
    if (deep)
        subpaths = [self subpathsAtPath:basePath];
    else
        subpaths = [self directoryContentsAtPath:basePath];
    
    subpaths = [subpaths sortedArrayUsingFinderOrder];
    
    for (NSString *subpath in subpaths)
    {
        NSString *want = isExtension ? [subpath pathExtension] : [subpath lastPathComponentWithoutExtension];
        
        if (!fragments || [fragments containsObjectEquivalentTo:want])
            [paths addObject:[basePath stringByAppendingPathComponent:subpath]];
    }
    
    return paths;
}

/*
 pathsAtPath:deepScan:
 
 Given a directory path, returns an array of full paths within that directory.  If deep is YES, all subdirectories are scanned, otherwise only the base one is scanned.  If the base path isn't present, or isn't a directory, nil is returned.  The base path may include a tilde.
 
 Written by DJS 2007-02.
*/

- (NSArray *)pathsAtPath:(NSString *)basePath deepScan:(BOOL)deep;
{
    return [self pathsWithFragments:nil isExtension:NO atPath:basePath deepScan:deep];
}

/*
 pathsWithExtensions:atPath:deepScan:
 
 Given an array of extensions (which may be nil or empty) and a directory path, returns an array of full paths that have those extensions (or all files if no extensions provided) within that directory.  If deep is YES, all subdirectories are scanned, otherwise only the base one is scanned.  If the base path isn't present, or isn't a directory, nil is returned.  The base path may include a tilde.  See also -pathsWithExtension:atPath:deepScan:.
 
 Written by DJS 2005-05.
*/

- (NSArray *)pathsWithExtensions:(NSArray *)extensions atPath:(NSString *)basePath deepScan:(BOOL)deep
{
    return [self pathsWithFragments:extensions isExtension:YES atPath:basePath deepScan:deep];
}

/*
 pathsWithExtension:atPath:deepScan:
 
 Given an extension (which may be nil or blank) and a directory path, returns an array of full paths that have that extension (or all files if no extension provided) within that directory.  If deep is YES, all subdirectories are scanned, otherwise only the base one is scanned.  If the base path isn't present, or isn't a directory, nil is returned.  The base path may include a tilde.  See also -pathsWithExtensions:atPath:deepScan:.
 
 Written by DJS 2005-05.
*/

- (NSArray *)pathsWithExtension:(NSString *)extension atPath:(NSString *)basePath deepScan:(BOOL)deep
{
    NSArray *paths;
    
    if (![extension length])
        paths = [self pathsWithExtensions:nil atPath:basePath deepScan:deep];
    else
        paths = [self pathsWithExtensions:[NSArray arrayWithObject:extension] atPath:basePath deepScan:deep];
    
    return paths;
}

/*
 pathsWithFilenames:atPath:deepScan:
 
 Given an array of filenames without extensions (which may be nil or empty) and a directory path, returns an array of full paths that have those filenames (or all files if no filenames provided) within that directory.  If deep is YES, all subdirectories are scanned, otherwise only the base one is scanned.  If the base path isn't present, or isn't a directory, nil is returned.  The base path may include a tilde.  See also -pathsWithFilename:atPath:deepScan:.
 
 Written by DJS 2005-10.
 */

- (NSArray *)pathsWithFilenames:(NSArray *)filenames atPath:(NSString *)basePath deepScan:(BOOL)deep
{
    return [self pathsWithFragments:filenames isExtension:NO atPath:basePath deepScan:deep];
}

/*
 pathsWithFilename:atPath:deepScan:
 
 Given a filename without an extension (which may be nil or blank) and a directory path, returns an array of full paths that have that filename (or all files if no filename provided) within that directory.  If deep is YES, all subdirectories are scanned, otherwise only the base one is scanned.  If the base path isn't present, or isn't a directory, nil is returned.  The base path may include a tilde.  See also -pathsWithFilenames:atPath:deepScan:.
 
 Written by DJS 2005-10.
 */

- (NSArray *)pathsWithFilename:(NSString *)filename atPath:(NSString *)basePath deepScan:(BOOL)deep
{
    NSArray *paths;
    
    if (![filename length])
        paths = [self pathsWithFilenames:nil atPath:basePath deepScan:deep];
    else
        paths = [self pathsWithFilenames:[NSArray arrayWithObject:filename] atPath:basePath deepScan:deep];
    
    return paths;
}

@end

