//
//  DSObjectCategories.m
//  Shared categories
//
//  Created by David Sinclair on Tue Sep 24 2002.
//  Copyright � 2002 - 2007 Dejal Systems, LLC. All rights reserved.
//
//  Redistribution and use in source and binary forms, with or without modification,
//  are permitted provided that the following conditions are met:
//  
//  Redistributions of source code must retain this list of conditions and the following disclaimer.
//  
//  The name of Dejal Systems, LLC may not be used to endorse or promote products derived from this
//  software without specific prior written permission.
//  
//  THIS SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED,
//  INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
//  PURPOSE AND NONINFRINGEMENT OF THIRD PARTY RIGHTS. IN NO EVENT SHALL THE AUTHORS OR
//  COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN
//  ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE
//  SOFTWARE OR THE USE OR OTHER DEALINGS IN THIS SOFTWARE.
//

#import "DSObjectCategories.h"


@implementation NSObject (DSObjectFoundationCategories)

/*
 set:to:
 
 Sets the specifie iVar to the specified value, using the standard retain/release approach.  Use &iVar.
 
 Written by DJS 2002-09.
*/

- (void)set:(id *)iVar to:(id)value
{
    [value retain];
    [*iVar release];
    *iVar = value;
}

/*
 set:to:nilAllowed:
 
 Same as -set:to:, above, but if flag is NO and the new value is nil, the iVar is not changed.
 
 Written by DJS 2002-09.
 */

- (void)set:(id *)iVar to:(id)value nilAllowed:(BOOL)flag
{
    if (value || flag)
    {
        [value retain];
        [*iVar release];
        *iVar = value;
    }
}


/*
 isReallySubclassOfClass:
 
 Returns YES if the receiver is an instance of a subclass of aClass, but not an instance of aClass itself.
 
 Written by DJS 2004-01.
*/

- (BOOL)isReallySubclassOfClass:(Class)aClass
{
    return [self isKindOfClass:aClass] && ![self isMemberOfClass:aClass];
}


/*
 isEquivalent:
 
 Like -isEqual, but returns YES if the receiver and the other object are equal when both interpreted as case-insensitive strings.  So, for example, @"THIS" and @"This" are equivalent, and @"123" and [NSNumber numberWithInt:123] are equivalent.  If the other object is nil, returns NO; i.e. a non-nil receiver is not equivalent to a nil value (but, of course, sending this to a nil object will result in 0, which would be incorrect if the other object were also nil).
 
 Written by DJS 2005-03.
*/

- (BOOL)isEquivalent:(id)anObject
{
    if (anObject)
        return ([[self description] caseInsensitiveCompare:[anObject description]] == NSOrderedSame);
    else
        return NO;
}

/*
 isEquivalentTo:
 
 An alias of -isEquivalent:, for when the "To" makes more sense.  For efficiency, the logic is duplicated.
 
 Written by DJS 2005-05.
*/

- (BOOL)isEquivalentTo:(id)anObject
{
    if (anObject)
        return ([[self description] caseInsensitiveCompare:[anObject description]] == NSOrderedSame);
    else
        return NO;
}


/*
 performBoolSelector:
 
 Similar to -performSelector:, but calls a method that returns a boolean value.  Returns NO if the receiever doesn't respond to that selector.
 
 Written by DJS 2006-06.
 */

- (BOOL)performBoolSelector:(SEL)selector;
{
    return [self performIntegerSelector:selector withObject:[NSNull null] withObject:[NSNull null]];
}

/*
 performBoolSelector:withObject:
 
 Similar to -performSelector:withObject:, but calls a method that returns a boolean value.  Returns NO if the receiever doesn't respond to that selector.
 
 Written by DJS 2006-06.
 */

- (BOOL)performBoolSelector:(SEL)selector withObject:(id)object;
{
    return [self performIntegerSelector:selector withObject:object withObject:[NSNull null]];
}

/*
 performBoolSelector:withObject:withObject:
 
 Similar to -performSelector:withObject:withObject:, but calls a method that returns a boolean value.  Returns NO if the receiever doesn't respond to that selector.
 
 Written by DJS 2006-06.
 */

- (BOOL)performBoolSelector:(SEL)selector withObject:(id)object1 withObject:(id)object2;
{
    return [self performIntegerSelector:selector withObject:object1 withObject:object2];
}

/*
 performIntegerSelector:
 
 Similar to -performSelector:, but calls a method that returns an integer value.  Returns zero if the receiever doesn't respond to that selector.
 
 Written by DJS 2004-04.
*/

- (NSInteger)performIntegerSelector:(SEL)selector
{
    return [self performIntegerSelector:selector withObject:[NSNull null] withObject:[NSNull null]];
}

/*
 performIntegerSelector:withObject:
 
 Similar to -performSelector:withObject:, but calls a method that returns an integer value.  Returns zero if the receiever doesn't respond to that selector.
 
 Written by DJS 2004-04.
*/

- (NSInteger)performIntegerSelector:(SEL)selector withObject:(id)object
{
    return [self performIntegerSelector:selector withObject:object withObject:[NSNull null]];
}

/*
 performIntegerSelector:withObject:withObject:
 
 Similar to -performSelector:withObject:withObject:, but calls a method that returns an integer value.  Does nothing (and returns zero) if selector is nil or the receiver doesn't respond to that selector.
 
 Written by DJS 2004-04.
*/

- (NSInteger)performIntegerSelector:(SEL)selector withObject:(id)object1 withObject:(id)object2
{
    if (!selector || ![self respondsToSelector:selector])
        return 0;
    
    NSMethodSignature *signature = [self methodSignatureForSelector:selector];
    NSInvocation *invocation = [NSInvocation invocationWithMethodSignature:signature];
    int result = 0;
    
    [invocation setSelector:selector];
    
    if (![object1 isKindOfClass:[NSNull class]])
        [invocation setArgument:&object1 atIndex:2];
    
    if (![object2 isKindOfClass:[NSNull class]])
        [invocation setArgument:&object2 atIndex:3];
    
    [invocation invokeWithTarget:self];
    [invocation getReturnValue:&result];
    
    return result;
}

/*
 performSelector:withWindow:returnCode:contextInfo:
 
 Similar to -performSelector:withObject:withObject:, but calls a method in the standard pattern of a modal did-end selector.  Does nothing if selector is nil.
 
 Written by DJS 2004-06.
*/

- (void)performSelector:(SEL)selector withWindow:(NSWindow *)window returnCode:(NSInteger)returnCode contextInfo:(void *)contextInfo
{
    if (!selector)
        return;
    
    NSMethodSignature *signature = [self methodSignatureForSelector:selector];
    NSInvocation *invocation = [NSInvocation invocationWithMethodSignature:signature];
    
    [invocation setSelector:selector];
    [invocation setArgument:&window atIndex:2];
    [invocation setArgument:&returnCode atIndex:3];
    [invocation setArgument:&contextInfo atIndex:4];
    [invocation invokeWithTarget:self];
}

/*
 performSelector:withArguments:
 
 Similar to -performSelector:withObject:, but allows any number of arguments.  The arguments are taken from the array, in order.  The selector should expect the same number of arguments.  It's okay for arguments to be nil, if the selector doesn't take any.  Does nothing if selector is nil, or the number of arguments of the selector and array are different.
 
 Written by DJS 2007-03.
*/

- (void)performSelector:(SEL)selector withArguments:(NSArray *)arguments;
{
    if (!selector)
        return;
    
    NSMethodSignature *signature = [self methodSignatureForSelector:selector];
    NSInvocation *invocation = [NSInvocation invocationWithMethodSignature:signature];
    
    // Exclude the hidden self and _cmd arguments:
    if (([signature numberOfArguments] - 2) != [arguments count])
    {
        NSLog(@"performSelector:%@ withObjects:%@ has different number of arguments", NSStringFromSelector(selector), arguments);         // log
        
        return;
    }
    
    [invocation setSelector:selector];
    
    for (NSUInteger i = 0; i < [arguments count]; i++)
    {
        id object = [arguments objectAtIndex:i];
        
        [invocation setArgument:&object atIndex:i + 2];
    }
    
    [invocation invokeWithTarget:self];
}

/*
 performSelector:withEachObjectInArray:
 
 Invokes the selector, passing each object in the array as a parameter.  Not the same as -[NSArray makeObjectsPerformSelector:], as that makes the objects in the array perform the selector, instead of making the receiver object perform the selector.  The selector must take a single argument.
 
 Written by DJS 2004-10.
*/

- (void)performSelector:(SEL)selector withEachObjectInArray:(NSArray *)array
{
    id object;
    
    for (object in array)
        [self performSelector:selector withObject:object];
}

/*
 performSelector:withEachObjectInDictionary:
 
 Invokes the selector, passing each object in the dictionary as a parameter.  The order invoked is not defined.  See -performSelector:withEachObjectInArray: for more information.
 
 Written by DJS 2004-10.
*/

- (void)performSelector:(SEL)selector withEachObjectInDictionary:(NSDictionary *)dict
{
    [self performSelector:selector withEachObjectInArray:[dict allValues]];
}

/*
 performSelector:withEachObjectInSet:
 
 Invokes the selector, passing each object in the set as a parameter.  The order invoked is not defined.  See -performSelector:withEachObjectInArray: for more information.
 
 Written by DJS 2004-10.
*/

- (void)performSelector:(SEL)selector withEachObjectInSet:(NSSet *)set
{
    [self performSelector:selector withEachObjectInArray:[set allObjects]];
}

@end


// ----------------------------------------------------------------------------------------
#pragma mark -
// ----------------------------------------------------------------------------------------


@implementation NSObject (DSObjectAppKitCategories)

/*
 usedCommandModifier
 
 Returns YES if the Command modifier key was pressed during the most recent event.
 
 Written by DJS 2004-04.
*/

- (BOOL)usedCommandModifier
{
    return ([[NSApp currentEvent] modifierFlags] & NSCommandKeyMask) == NSCommandKeyMask;
}

/*
 usedOptionModifier
 
 Returns YES if the Option (Alternate) modifier key was pressed during the most recent event.
 
 Written by DJS 2004-04.
*/

- (BOOL)usedOptionModifier
{
    return ([[NSApp currentEvent] modifierFlags] & NSAlternateKeyMask) == NSAlternateKeyMask;
}

/*
 usedShiftModifier
 
 Returns YES if the Shift modifier key was pressed during the most recent event.
 
 Written by DJS 2004-04.
*/

- (BOOL)usedShiftModifier
{
    return ([[NSApp currentEvent] modifierFlags] & NSShiftKeyMask) == NSShiftKeyMask;
}

/*
 usedControlModifier
 
 Returns YES if the Control modifier key was pressed during the most recent event.
 
 Written by DJS 2004-04.
*/

- (BOOL)usedControlModifier
{
    return ([[NSApp currentEvent] modifierFlags] & NSControlKeyMask) == NSControlKeyMask;
}

@end
