//
//  DSStringCategories.m
//  Shared categories
//
//  Created by David Sinclair on Sat Aug 17 2002.
//  Copyright � 2002 - 2008 Dejal Systems, LLC. All rights reserved.
//
//  Redistribution and use in source and binary forms, with or without modification,
//  are permitted provided that the following conditions are met:
//  
//  Redistributions of source code must retain this list of conditions and the following disclaimer.
//  
//  The name of Dejal Systems, LLC may not be used to endorse or promote products derived from this
//  software without specific prior written permission.
//  
//  THIS SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED,
//  INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
//  PURPOSE AND NONINFRINGEMENT OF THIRD PARTY RIGHTS. IN NO EVENT SHALL THE AUTHORS OR
//  COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN
//  ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE
//  SOFTWARE OR THE USE OR OTHER DEALINGS IN THIS SOFTWARE.
//

#import "DSStringCategories.h"
#import "DSArrayCategories.h"
#import "DSDictionaryCategories.h"


@implementation NSString (DSStringCategories)

/*
 stringWithIntValue:
 
 Given an integer value, returns the corresponding string.  This method is deprecated; use +stringWithIntegerValue: instead.
 
 Written by DJS 2005-04.
*/

+ (NSString *)stringWithIntValue:(NSInteger)value
{
    return [self stringWithFormat:@"%d", value];
}

/*
 stringWithIntegerValue:
 
 Given an integer value, returns the corresponding string.
 
 Written by DJS 2008-07.
*/

+ (NSString *)stringWithIntegerValue:(NSInteger)value;
{
    return [self stringWithFormat:@"%d", value];
}

/*
 stringWithFloatValue:places:
 
 Given a CGFloat value and the number of decimal places to represent it as, returns the corresponding string.
 
 Written by DJS 2005-04.
*/

+ (NSString *)stringWithFloatValue:(CGFloat)value places:(NSInteger)places
{
    NSString *format = [self stringWithFormat:@"%%.%df", places];
    
    return [self stringWithFormat:format, value];
}

/*
 stringWithIntValue:zero:singluar:plural:

 Given an int value, if it is zero, the zero string is returned; if one, the singular string is returned; otherwise the plural string is returned.    This method is deprecated; use +stringWithIntegerValue:zero:singular:plural: instead.
*/

+ (NSString *)stringWithIntValue:(NSInteger)value
                            zero:(NSString *)zero
                        singluar:(NSString *)singular
                          plural:(NSString *)plural
{
    return [self stringWithIntegerValue:value zero:zero singluar:singular plural:plural];
}

/*
 stringWithIntValue:zero:singluar:plural:

 Given an int value, if it is zero, the zero string is returned; if one, the singular string is returned; otherwise the plural string is returned.
*/

+ (NSString *)stringWithIntegerValue:(NSInteger)value
                                zero:(NSString *)zero
                            singluar:(NSString *)singular
                              plural:(NSString *)plural
{
    if (!value && zero)
        return zero;
    else if (value == 1 && singular)
        return singular;
    else
        return plural;
}

/*
 stringWithFloatValue:zero:singluar:plural:
 
 Given a float value, if it is zero, the zero string is returned; if exactly one, the singular string is returned; otherwise the plural string is returned.
 
 Written by DJS 2008-04.
*/

+ (NSString *)stringWithFloatValue:(CGFloat)value
                              zero:(NSString *)zero
                          singluar:(NSString *)singular
                            plural:(NSString *)plural;
{
    if (!value && zero)
        return zero;
    else if (value == 1.0 && singular)
        return singular;
    else
        return plural;
}

/*
 stringWithSeconds:minuteSingular:minutesPlural:secondSingular:secondsPlural:

 Given an integer value representing a time in seconds, and the appropriate words for minutes and seconds in singular and plural, this returns a string expressing this time interval, e.g. @"1 minute 37 seconds".  Pass nil for the minutes or seconds parameter pairs to exclude those units, e.g. passing nil for secondSingular and secondsPlural will result in just "1 minute".  If both are wanted, and one is zero, it is omitted [if different behavor is desired in the future, add a parameter to indicate that; don't change that functionality].  Similar methods can be added to include the option of hours, etc, as needed.
 
 Written by DJS 2003-07.
*/

+ (NSString *)stringWithSeconds:(NSInteger)seconds
                 minuteSingular:(NSString *)minuteSingular
                  minutesPlural:(NSString *)minutesPlural
                 secondSingular:(NSString *)secondSingular
                  secondsPlural:(NSString *)secondsPlural
{
    NSString *minutesString = nil;
    NSString *secondsString = nil;
    NSString *timeString = nil;
    int minutes = seconds / 60;

    BOOL wantSeconds = (secondSingular && secondsPlural);
    BOOL wantMinutes = (minuteSingular && minutesPlural);
    
    if (wantMinutes)
    {
        seconds = seconds % 60;
        
        minutesString = [self stringWithFormat:@"%d %@", minutes,
            [self stringWithIntegerValue:minutes zero:nil singluar:minuteSingular plural:minutesPlural]];
    }

    if (wantSeconds)
        secondsString = [self stringWithFormat:NSLocalizedString(@"%d %@", @"Seconds remaining"), seconds,
            [self stringWithIntegerValue:seconds zero:nil singluar:secondSingular plural:secondsPlural]];

    if (minutes && seconds && wantSeconds && wantMinutes)
        timeString = [NSString stringWithFormat:@"%@ %@", minutesString, secondsString];
    else if (minutes || !wantSeconds)
        timeString = minutesString;
    else
        timeString = secondsString;
    
    return timeString;
}

/*
 stringWithIntValue:minLength:maxLength:paddedWith:padLeft:useEllipsis:

 Given an int value, returns it as a string.  If the length of the resulting string is less than minLength, it is padded with the specified padding characters, on the left or right as requested.  If the string is longer than maxLength, it is truncated to that length, optionally with the ellipsis character at the end.  This method is deprecated; use +stringWithIntegerValue:minLength:maxLength:paddedWith:padLeft:useEllipsis: instead.
*/

+ (NSString *)stringWithIntValue:(NSInteger)value minLength:(NSUInteger)minLength maxLength:(NSUInteger)maxLength
                      paddedWith:(char)padding padLeft:(BOOL)padLeft useEllipsis:(BOOL)ellipsis
{
    return [self stringWithIntegerValue:value minLength:minLength maxLength:maxLength paddedWith:padding padLeft:padLeft useEllipsis:ellipsis];
}

/*
 stringWithIntegerValue:minLength:maxLength:paddedWith:padLeft:useEllipsis:

 Given an int value, returns it as a string.  If the length of the resulting string is less than minLength, it is padded with the specified padding characters, on the left or right as requested.  If the string is longer than maxLength, it is truncated to that length, optionally with the ellipsis character at the end.
*/

+ (NSString *)stringWithIntegerValue:(NSInteger)value minLength:(NSUInteger)minLength maxLength:(NSUInteger)maxLength
                      paddedWith:(char)padding padLeft:(BOOL)padLeft useEllipsis:(BOOL)ellipsis
{
    NSString *temp = [self stringWithFormat:@"%d", value];

    if ([temp length] < minLength)
    {
        NSUInteger shortfall = minLength - [temp length];
        NSUInteger i;

        for (i = 0; i < shortfall; i++)
        {
            if (padLeft)
                temp = [self stringWithFormat:@"%c%@", padding, temp];
            else
                temp = [self stringWithFormat:@"%@%c", temp, padding];
        }
    }
    else if ([temp length] > maxLength)
    {
        //  not implemented yet
    }

    return temp;
}

/*
 stringWithLeadingZeroesForIntValue:digits:
 
 Given an int value, returns it as a string with the specified number of digits.  If there are less digits in the number, it is padded with leading zeroes.  If there are more, it is truncated to that number of digits.  Note that this is equivalent to the somewhat shorter, but less intuitive, +[NSString stringWithFormat:@"%0.4d", value], where the 4 is replaced by the number of digits, except that that approach won't truncate a longer number.  This method is deprecated; use +stringWithLeadingZerosForIntegerValue:digits: instead.
 
 Written by DJS 2004-03.
*/

+ (NSString *)stringWithLeadingZeroesForIntValue:(NSInteger)value digits:(NSInteger)digits
{
    return [[self stringWithFormat:@"%0.10d", value] right:digits];
}

/*
 stringWithLeadingZeroesForIntegerValue:digits:
 
 Given an int value, returns it as a string with the specified number of digits.  If there are less digits in the number, it is padded with leading zeroes.  If there are more, it is truncated to that number of digits.  Note that this is equivalent to the somewhat shorter, but less intuitive, +[NSString stringWithFormat:@"%0.4d", value], where the 4 is replaced by the number of digits, except that that approach won't truncate a longer number.
 
 Written by DJS 2004-03.
*/

+ (NSString *)stringWithLeadingZeroesForIntegerValue:(NSInteger)value digits:(NSInteger)digits;
{
    return [[self stringWithFormat:@"%0.10d", value] right:digits];
}

/*
 stringAsBytesWithInt:
 
 Given a value in bytes, returns it as a string with the suffix " bytes", " KB", " MB", or " GB", as appropriate.  This method is deprecated; use +stringAsBytesWithInteger: instead.
*/

+ (NSString *)stringAsBytesWithInt:(NSInteger)bytes;
{
    return [self stringAsBytesWithInteger:bytes];
}

/*
 stringAsBytesWithInteger:
 
 Given a value in bytes, returns it as a string with the suffix " bytes", " KB", " MB", or " GB", as appropriate.
*/

+ (NSString *)stringAsBytesWithInteger:(NSInteger)bytes;
{
    NSString *temp;
    CGFloat kilobyte = 1024.0;
    CGFloat megabyte = kilobyte * kilobyte;
    CGFloat gigabyte = megabyte * kilobyte;
    
    if (bytes == 0)
        temp = NSLocalizedStringFromTable(@"Zero KB", @"DSShared", @"Zero Kilobytes");
    else if (bytes >= gigabyte)
        temp = [self stringWithFormat:NSLocalizedStringFromTable(@"%.1f GB", @"DSShared", @"Gigabytes"), bytes / gigabyte];
    else if (bytes >= megabyte)
        temp = [self stringWithFormat:NSLocalizedStringFromTable(@"%.1f MB", @"DSShared", @"Megabytes"), bytes / megabyte];
    else if (bytes >= kilobyte)
        temp = [self stringWithFormat:NSLocalizedStringFromTable(@"%.1f KB", @"DSShared", @"Kilobytes"), bytes / kilobyte];
    else
        temp = [self stringWithFormat:NSLocalizedStringFromTable(@"%d bytes", @"DSShared", @"Bytes"), bytes];

    return temp;
}

/*
 stringWithPrefix:keyword:suffix:or:
 
 If the keyword is a valid non-empty string, returns a string containing the prefix, keyword, and suffix (the prefix and/or suffix may be nil if desired).  Otherwise, if the alternative is a valid non-empty string, returns that instead.  If both the keyword and alternative are nil, nil is returned, so if you want a valid string, pass @"" for at least the alternative.
 
 Written by DJS 2004-03.
*/

+ (NSString *)stringWithPrefix:(NSString *)prefix keyword:(NSString *)keyword suffix:(NSString *)suffix or:(NSString *)alternative
{
    if (!keyword && !alternative)
        return nil;
    
    NSMutableString *retString = [NSMutableString string];
    
    [retString appendPrefix:prefix keyword:keyword suffix:suffix or:alternative];
    
    return retString;
}

/*
 or:

 Gotta be one of the shortest method names!  If preferred is not nil, that object is returned, otherwise self is returned.  Useful to provide an optional string, e.g. [@"never" or:someDateObject].
*/

- (id)or:(id)preferred
{
    if (preferred)
        return preferred;
    else
        return self;
}

/*
 containsSomething
 
 Returns YES if the string is non-empty, i.e. not nil and not @"".
 
 Written by DJS 2005-03.
*/

- (BOOL)containsSomething
{
    return ([self length] > 0);
}

/*
 containsString:

 Returns YES if the subString is within the receiver, otherwise NO.  [Note: can easily extend this to take options, and to take a range; see -rangeOfString:options:range:]
*/

- (BOOL)containsString:(NSString *)subString
{
    NSRange range = [self rangeOfString:subString];

    return range.location != NSNotFound;
}

/*
 containsStringCaseInsensitive:

 Returns YES if the subString is within the receiver, otherwise NO.  The case of both strings is ignored.
 
 Written by DJS 2004-12.
*/

- (BOOL)containsStringCaseInsensitive:(NSString *)subString
{
    NSRange range = [self rangeOfString:subString options:NSCaseInsensitiveSearch];
    
    return range.location != NSNotFound;
}

- (NSComparisonResult)caseAndSpaceInsensitiveCompare:(NSString *)otherString
{
    NSString *selfString = [self stringByReplacingAllOccurrencesOf:@" " with:@""];
    NSString *other = [otherString stringByReplacingAllOccurrencesOf:@" " with:@""];

    return [selfString caseInsensitiveCompare:other];
}

/*
 lowercasedLettersOnly
 
 Returns the receiver with all letter characters converted to lowercase, and all other characters removed.  Useful to compare two strings without extraneous punctuation etc.
 
 Written by DJS 2005-02.
 Changed by DJS 2005-04 to use -stringByRemovingCharactersInSet:; was incorrectly only trimming.
*/

- (NSString *)lowercasedLettersOnly
{
    NSCharacterSet *charSet = [[NSCharacterSet lowercaseLetterCharacterSet] invertedSet];
    NSString *clean = [[self lowercaseString] stringByRemovingCharactersInSet:charSet];
    
    return clean;
}

/*
 lowercasedLettersOrDigitsOnly
 
 Returns the receiver with all letter characters converted to lowercase, and all other characters other than digits removed.  Useful to compare two strings without extraneous punctuation etc.
 
 Written by DJS 2007-10.
*/

- (NSString *)lowercasedLettersOrDigitsOnly;
{
    NSCharacterSet *charSet = [[NSCharacterSet alphanumericCharacterSet] invertedSet];
    NSString *clean = [[self lowercaseString] stringByRemovingCharactersInSet:charSet];
    
    return clean;
}

/*
 containsStringLetters:
 
 Returns YES if the receiver contains the other string, when only looking at the letter characters, and ignoring case.
 
 Written by DJS 2005-02.
*/

- (BOOL)containsStringLetters:(NSString *)otherString
{
    return ([[self lowercasedLettersOnly] containsString:[otherString lowercasedLettersOnly]]);
}

/*
 isLetterEquivalentToString:
 
 Returns YES if the receiver and the other string are equal when only looking at the letter characters, and ignoring case.
 
 Written by DJS 2005-02.
*/

- (BOOL)isLetterEquivalentToString:(NSString *)otherString
{
    return ([[self lowercasedLettersOnly] isEqualToString:[otherString lowercasedLettersOnly]]);
}

/*
 stringByRemovingCharactersInSet:
 
 Returns a string with all characters matching the set removed from the receiver.
 
 Written by DJS 2005-04.
*/

- (NSString *)stringByRemovingCharactersInSet:(NSCharacterSet *)set
{
    NSMutableString *mutString = [NSMutableString stringWithString:self];
    
    [mutString deleteCharactersInSet:set];
    
    return [NSString stringWithString:mutString];
}

/*
 stringByRemovingDiacriticalMarks
 
 Returns a string with diacritical marks removed from the receiver.  For example, "expose" with an acute accent will be returned without the accent.
 
 Written by DJS 2007-04.
*/

- (NSString *)stringByRemovingDiacriticalMarks;
{
    NSMutableString *mutie = [[[self decomposedStringWithCanonicalMapping] mutableCopy] autorelease];
    NSCharacterSet *nonBaseSet = [NSCharacterSet nonBaseCharacterSet];
    NSRange range = NSMakeRange([mutie length], 0);
    
    while (range.location > 0)
    {
        range = [mutie rangeOfCharacterFromSet:nonBaseSet options:NSBackwardsSearch range:NSMakeRange(0, range.location)];
        
        if (range.length == 0)
            break;
        
        [mutie deleteCharactersInRange:range];
    }
    
    return mutie;
}

- (NSString *)stringByRemovingQuotesAndSpaces
{
    BOOL insideQuote = NO;
    NSString *nextChar;
    NSMutableString *retString = [NSMutableString string];
    NSUInteger i;

    for (i = 0; i < [self length]; i++)
    {
        unichar nChar = [self characterAtIndex:i];
        nextChar = [NSString stringWithCharacters:&nChar length:1];
        
        if ([nextChar isEqual:@"\""])
        {
            if (!insideQuote)
                insideQuote = YES;
            else
                insideQuote = NO;
            
            continue;
        }
        
        if ([nextChar isEqual:@" "] && insideQuote)
            [retString appendString:@"&SP&"];
        else
            [retString appendString:nextChar];
    }

    return [NSString stringWithString:retString];
}

// 2007-06: The following method is deprecated, as 10.5 implements -stringByReplacingOccurrencesOfString:withString:

- (NSString *)stringByReplacingAllOccurrencesOf:(NSString *)string1 with:(NSString *)string2
{
    NSMutableString *mutString = [NSMutableString stringWithString:self];

    [mutString replaceAllOccurrencesOf:string1 with:string2];

    return [NSString stringWithString:mutString];
}

- (NSString *)stringByDeletingLeadingSpaces:(BOOL)leading trailingSpaces:(BOOL)trailing
{
    NSMutableString *mutString = [NSMutableString stringWithString:self];

    [mutString deleteLeadingSpaces:leading trailingSpaces:trailing];

    return [NSString stringWithString:mutString];
}

/*
 truncatedStringWithIndicator:forFont:withWidth:trimLeft:
 
 Returns the specified string either intact, or truncated with the indicator string added if the original string is too long to fit in the desired width.  The indicator string would typically be an ellipsis (...); passing nil will default to that.  The string is trimmed on the left or right as requested.
 
 Written by DJS 2004-01.
 Changed by DJS 2004-06 to add the trimLeft parameter.
*/

- (NSString *)truncatedStringWithIndicator:(NSString *)indicator forFont:(NSFont *)font withWidth:(NSUInteger)desiredWidth trimLeft:(BOOL)trimLeft
{
    if (!indicator)
        indicator = @"...";
    
    NSString *returnedString;
    NSDictionary *attributes = [NSDictionary dictionaryWithObject:font forKey:@"NSFontAttributeName"];
    NSUInteger indicatorWidth = [indicator sizeWithAttributes:attributes].width;
    
    // Assume our string will fit:
    if ([self sizeWithAttributes:attributes].width <= desiredWidth)
    {
        returnedString = [NSString stringWithString:self];
    }
    else
    {
        // Do a binary search until we fit as tightly as possible:
        BOOL exactFit = NO;
        NSUInteger numChars = [self length] / 2;
        NSUInteger i = numChars;
        NSString *choppedString;
        
        // Assign this string first to deal with pathological case:
        if (trimLeft)
            choppedString = [self right:i];
        else
            choppedString = [self left:i];
        
        while ((numChars > 1) && !exactFit)
        { 
            unsigned int thisWidth = [choppedString sizeWithAttributes:attributes].width + indicatorWidth;
            numChars /= 2;
            
            if (thisWidth < desiredWidth)
                i += numChars;
            else if (thisWidth > desiredWidth)
                i -= numChars;
            else
                exactFit = YES;
            
            if (trimLeft)
                choppedString = [self right:i];
            else
                choppedString = [self left:i];
        }
        
        if (trimLeft)
            returnedString = [indicator stringByAppendingString:choppedString];
        else
            returnedString = [choppedString stringByAppendingString:indicator];
    }
    
    return returnedString;
}

/*
 truncatedStringWithIndicator:forFont:withWidth:
 
 Returns the specified string either intact, or truncated with the indicator string added if the original string is too long to fit in the desired width.  The indicator string would typically be an ellipsis (...); passing nil will default to that.
 
 Written by DJS 2004-01.
*/

- (NSString *)truncatedStringWithIndicator:(NSString *)indicator forFont:(NSFont *)font withWidth:(NSUInteger)desiredWidth
{
    return [self truncatedStringWithIndicator:indicator forFont:font withWidth:desiredWidth trimLeft:NO];
}

/*
 componentsSeparatedByLength:
 
 Returns an array of strings from the receiver, broken up on spaces to be no more than length characters each.  Supports multiple paragraphs, though assumes they are separated by LFs.  This limitation can be tweaked in the future if important.
 
 Written by DJS 2006-10.
*/

- (NSArray *)componentsSeparatedByLength:(NSUInteger)length;
{
    NSMutableArray *array = [NSMutableArray array];
    NSArray *paragraphs = [self componentsSeparatedByString:@"\n"];
    NSString *paragraph;
    
    for (paragraph in paragraphs)
    {
        NSScanner *wordScanner = [NSScanner scannerWithString:paragraph];
        NSCharacterSet *whiteSpace = [NSCharacterSet whitespaceCharacterSet];
        NSString *word = nil;
        NSString *separator = nil;
        NSMutableString *line = [NSMutableString stringWithCapacity:length];
        
        [wordScanner setCharactersToBeSkipped:[NSCharacterSet illegalCharacterSet]];
        
        while (![wordScanner isAtEnd])
        {
            [wordScanner scanUpToCharactersFromSet:whiteSpace intoString:&word];
            [wordScanner scanCharactersFromSet:whiteSpace intoString:&separator];
            
            if (([line length] + [word length] + [separator length]) <= length)
            {
                [line appendString:word];
                [line appendString:separator];
            }
            else
            {
                [array addObject:line];
                line = [NSMutableString stringWithString:word];
                [line appendString:separator];
            }
        }
        
        [array addObject:line];
    }
    
    return array;
}

/*
 lengthOfInteger:
 
 Given an integer, returns its length as a string.
 
 Written by DJS 2008-07.
*/

+ (NSUInteger)lengthOfInteger:(NSInteger)integer;
{
    return [[NSString stringWithFormat:@"%d", integer] length];
}

/*
 wordCount
 
 Returns the number of words in the receiver.
 
 Written by DJS 2004-06.
*/

- (NSInteger)wordCount
{
    NSScanner *wordScanner = [NSScanner scannerWithString:self];
    NSCharacterSet *whiteSpace = [NSCharacterSet whitespaceCharacterSet];
    NSMutableCharacterSet *tempSet = (NSMutableCharacterSet *)[NSMutableCharacterSet whitespaceAndNewlineCharacterSet];
    [tempSet formUnionWithCharacterSet:[NSCharacterSet punctuationCharacterSet]];
    NSCharacterSet *skipSet = [[tempSet copy] autorelease];
    int wordCount = 0;
    
    [wordScanner setCharactersToBeSkipped:skipSet];
    
    while ([wordScanner scanUpToCharactersFromSet:whiteSpace intoString:nil])
        wordCount++;
    
    return wordCount;
}


/*
 dictionaryWithVersionComponents
 
 Given a string containing a version number, of the form "1.2.3b4", returns a dictionary with each component separated out, using the keys "Major", "Minor", "Bug", "Kind" and "Stage", respectively.  In addition, key "Version" will contain the original version string, "General" will contain the general release portion, i.e. omitting the "b4" or whatever, and "IsGeneral" is a boolean indicating if it is a general release or not.  Keys are omitted if the corresponding component of the version string is missing.
 
 Written by DJS 2004-05.
*/

- (NSDictionary *)dictionaryWithVersionComponents
{
    NSRange range = [self rangeOfCharacterFromSet:[NSCharacterSet letterCharacterSet]];
    NSString *general = self;
    NSString *kind = nil;
    NSString *stage = nil;
    
    if (range.location != NSNotFound)
    {
        general = [self substringToIndex:range.location];
        kind = [self substringWithRange:range];
        stage = [self substringFromIndex:range.location + range.length];
    }
    
    NSArray *array = [general componentsSeparatedByString:@"."];
    NSUInteger count = [array count];
    NSString *major = (count >= 1) ? [array objectAtIndex:0] : nil;
    NSString *minor = (count >= 2) ? [array objectAtIndex:1] : nil;
    NSString *bug = (count >= 3) ? [array objectAtIndex:2] : nil;
    
    NSMutableDictionary *dict = [NSMutableDictionary dictionary];
    
    [dict setObject:self forKey:@"Version" removeIfNil:NO];
    [dict setObject:general forKey:@"General" removeIfNil:NO];
    [dict setObject:[NSNumber numberWithBool:!kind] forKey:@"IsGeneral"];
    [dict setObject:major forKey:@"Major" removeIfNil:NO];
    [dict setObject:minor forKey:@"Minor" removeIfNil:NO];
    [dict setObject:bug forKey:@"Bug" removeIfNil:NO];
    [dict setObject:kind forKey:@"Kind" removeIfNil:NO];
    [dict setObject:stage forKey:@"Stage" removeIfNil:NO];
    
    return dict;
}

/*
 versionWithDictionary:
 
 Given a dictionary containing the keys "Major", "Minor", "Bug", "Kind" and "Stage" (some of which may be missing), returns the string representation, e.g. "1.2.3b4".  If the dictionary contains a precomposed "Version" key, it just returns that.
 
 Written by DJS 2004-05.
*/

+ (NSString *)versionWithDictionary:(NSDictionary *)dict
{
    NSString *version = [dict objectForKey:@"Version"];
    
    if ([version length])
        return version;
    
    int major = [[dict objectForKey:@"Major"] integerValue];
    int minor = [[dict objectForKey:@"Minor"] integerValue];
    int bug = [[dict objectForKey:@"Bug"] integerValue];
    NSString *kind = [dict objectForKey:@"Kind"];
    int stage = [[dict objectForKey:@"Stage"] integerValue];
    
    return [self versionWithMajor:major minor:minor bug:bug kind:kind stage:stage];
}

/*
 versionWithMajor:minor:bug:kind:stage:
 
 Given values for the components of a version number (any of which may be 0 or nil), returns the string representation, e.g. "1.2.3b4".
 
 Written by DJS 2004-05.
*/

+ (NSString *)versionWithMajor:(NSInteger)major minor:(NSInteger)minor bug:(NSInteger)bug kind:(NSString *)kind stage:(NSInteger)stage
{
    NSString *version = nil;
    
    if (bug)
        version = [NSString stringWithFormat:@"%d.%d.%d", major, minor, bug];
    else
        version = [NSString stringWithFormat:@"%d.%d", major, minor];
    
    if ([kind length])
        version = [NSString stringWithFormat:@"%@%@%d", version, kind, stage];
    
    return version;
}

/*
 left:
 
 Returns the specified number of characters from the beginning of the receiver, or the entire receiver if there aren't that many.  If a negative value is passed, it returns characters from the beginning of the receiver until the absolute value of the passed value before the length of the receiver, (e.g. [@"Hello!" left:-1] returns @"Hello"), or an empty string if there aren't that many.
 
 Written by DJS 2004-03.
 Changed by DJS 2006-07 to support negative lengths.
*/

- (NSString *)left:(NSInteger)length
{
    if (length < 0)
    {
        if (-length < (NSInteger)[self length])
            return [self substringToIndex:[self length] + length];
        else
            return @"";
    }
    else
    {
        if (length < (NSInteger)[self length])
            return [self substringToIndex:length];
        else
            return self;
    }
}

/*
 right:
 
 Returns the specified number of characters from the end of the receiver, or the entire receiver if there aren't that many.  If a negative value is passed, it returns characters from the end of the receiver until the absolute value of the passed value before the start of the receiver, (e.g. [@"An example" left:-3] returns @"example"), or an empty string if there aren't that many.
 
 Written by DJS 2004-03.
 Changed by DJS 2006-07 to support negative lengths.
*/

- (NSString *)right:(NSInteger)length
{
    if (length < 0)
    {
        NSInteger i = [self length] + length;
        
        if (i > 0)
            return [self substringFromIndex:[self length] - i];
        else
            return @"";
    }
    else
    {
        NSInteger i = [self length] - length;
        
        if (i > 0)
            return [self substringFromIndex:i];
        else
            return self;
    }
}

/*
 from:length:
 
 Returns a substring of the receiver, from the specified position and with the specified length.  If there aren't that many characters from that position, as many as there are are returned; if that position is out of range, an empty string is returned.
 
 Written by DJS 2004-05.
*/

- (NSString *)from:(NSUInteger)position length:(NSUInteger)length
{
    if (position >= [self length])
        return @"";
    else
        return [[self substringFromIndex:position] left:length];
}

/*
 from:to:
 
 Returns a substring of the receiver, from the start position to the end position, both inclusive.  If either position is out of range, as much of that range as is available is returned.
 
 Written by DJS 2004-05.
*/

- (NSString *)from:(NSUInteger)startPosition to:(NSUInteger)endPosition
{
    if (startPosition > endPosition)
        return @"";
    else
        return [self from:startPosition length:endPosition - startPosition + 1];
}

/*
 substringFromString:
 
 Similar to -substringFromIndex:, but takes a string to look for instead of an index.  If the string is within the receiver, the characters from the one after that string to the end are returned.  If it isn't within, the receiver is returned.
 
 Written by DJS 2005-03.
*/

- (NSString *)substringFromString:(NSString *)string
{
    NSRange range = [self rangeOfString:string];
    
    if (range.location == NSNotFound)
        return self;
    else
        return [self substringFromIndex:range.location + range.length];
}

/*
 substringToString:
 
 Similar to -substringToIndex:, but takes a string to look for instead of an index.  If the string is within the receiver, the characters from the start up to but excluding the position of that string are returned.  If it isn't within, the receiver is returned.
 
 Written by DJS 2005-03.
*/

- (NSString *)substringToString:(NSString *)string
{
    NSRange range = [self rangeOfString:string];
    
    if (range.location == NSNotFound)
        return self;
    else
        return [self substringToIndex:range.location];
}

/*
 reverse
 
 Returns the receiver with the characters in the reverse order.
 
 Written by DJS 2004-01.
*/

- (NSString *)reverse
{
    NSString *nextChar;
    NSMutableString *retString = [NSMutableString string];
    NSInteger i;
    
    for (i = [self length] - 1; i >= 0; i--)
    {
        unichar nChar = [self characterAtIndex:i];
        nextChar = [NSString stringWithCharacters:&nChar length:1];
        
        [retString appendString:nextChar];
    }
    
    return [NSString stringWithString:retString];
}

/*
 mask
 
 Returns a lightly encrypted rendition of the receiver, for hiding passwords etc.  Decrypt via -unmask, below.
 
 Written by DJS 2004-01.
*/

- (NSString *)mask
{
    if (![self length])
        return self;
    
    NSMutableString *retString = [NSMutableString string];
    NSString *fragment;
    NSCalendarDate *date = [NSCalendarDate calendarDate];
    NSUInteger randomValue = [date dayOfCommonEra] * [date minuteOfHour] / ([date secondOfMinute] + 5);
    unichar character = 0;
    NSUInteger i;
    
    for (i = 0; i < [self length]; i++)
    {
        character = [self characterAtIndex:i] + 57 + (randomValue * (i + 3));
        fragment = [NSString stringWithFormat:@"%xO", character];
        
        [retString appendString:fragment];
    }
    
    return [NSString stringWithFormat:@"1a7q4%dd39%@", randomValue, [retString reverse]];          // Version 1, in case I want to change the algorithm
}

/*
 unmask
 
 Given a string that was masked via -mask, above, returns the decrypted rendition.  Okay to call with an unmasked string; just returns self in this case.
 
 Written by DJS 2004-01.
*/

- (NSString *)unmask
{
    if (![self length])
        return self;
    
    NSScanner *scanner = [NSScanner scannerWithString:self];
    NSMutableString *retString = [NSMutableString string];
    NSString *fragment;
    unsigned hexInt = 0;
    unichar character = 0;
    int version = 0;
    int randomValue = 0;
    int i = 0;
    
    if (![scanner scanInt:&version])
        return self;
    
    if (![scanner scanString:@"a7q4" intoString:nil])
        return self;
    
    if (![scanner scanInt:&randomValue])
        return self;
    
    if (![scanner scanString:@"d39" intoString:nil])
        return self;
    
    int location = [scanner scanLocation];
    NSString *remaining = [[self substringFromIndex:location] reverse];
    
    scanner = [NSScanner scannerWithString:remaining];
    
    while (![scanner isAtEnd])
    {
        if ([scanner scanHexInt:&hexInt])
        {
            character = hexInt - (randomValue * (i + 3)) - 57;
            fragment = [NSString stringWithCharacters:&character length:1];
            
            [retString appendString:fragment];
        }
        
        [scanner scanString:@"O" intoString:nil];
        
        i++;
    }
    
    return [NSString stringWithString:retString];
}

/*
 rot13
 
 Returns the receiver encoded with the standard Rot13 algorithm: letter characters rotated halfway around the alphabet, other characters untouched.
 
 Written by DJS 2005-03.
*/

- (NSString *)rot13
{
    NSMutableString *string = [NSMutableString string];
    NSUInteger i;
    
    for (i = 0; i < [self length]; ++i)
    {
        unichar character = [self characterAtIndex:i];
        unichar capital = character & 32;
        
        character &= ~capital;
        character = ((character >= 'A') && (character <= 'z')) ? ((character - 'A' + 13) % 26 + 'A') : character;
        character |= capital;
        
        [string appendFormat:@"%c", character];
    }
    
    return string;
}

/*
 universallyUniqueIdentifier
 
 Returns a string that is guaranteed to be globally unique, and can be used as a unique identifier.
 
 Written by DJS 2003-11.
*/

+ (NSString *)universallyUniqueIdentifier
{
    CFUUIDRef uniqueID = CFUUIDCreate(NULL);
    NSString *string = [(NSString *)CFUUIDCreateString(NULL, uniqueID) autorelease];
    CFRelease(uniqueID);
    
    return string;
}


/*
 integerMappedFromString:withDefault:
 
 Convenience method for -mappedFromValue:withDefault: to map a string value to an integer.
 
 Written by DJS 2006-09.
*/

- (NSInteger)integerMappedFromString:(NSString *)value withDefault:(NSInteger)defaultValue;
{
    return [[self mappedFromValue:value withDefault:[NSNumber numberWithInteger:defaultValue]] integerValue];
}

/*
 stringMappedFromInteger:withDefault:
 
 Convenience method for -mappedFromValue:withDefault: map an integer value to a string.
 
 Written by DJS 2006-09.
*/

- (NSString *)stringMappedFromInteger:(NSInteger)value withDefault:(NSString *)defaultValue;
{
    return [self mappedFromValue:[NSNumber numberWithInteger:value] withDefault:defaultValue];
}

/*
 mappedFromValue:withDefault:
 
 Given a mapping of the form "foo=bar;blah=beep;x=y" in the receiver and a value (of any type), returns the corresponding value.  For example, with the above mapping and a value of "blah", "beep" is returned.  It works both ways: if "beep" is passed, "blah" is returned.  If a match can't be found an a default value is provided, the default is returned.  There may be an easier or more efficient way to do this, other than big if/else or switch clauses, but this seems to be tidier from a usage point of view: just one line of code to call.  (I considered adding a DSMapperData class for this, but decided against it for now as I wanted to use it in a Simon plugin, which has class-sharing issues if I ever wanted to use it elsewhere in the app.)
 
 Written by DJS 2006-09.
*/

- (NSString *)mappedFromValue:(id)value withDefault:(id)defaultValue;
{
    NSArray *array = [self componentsSeparatedByString:@";"];
    NSString *desired = [value description];
    NSString *result = nil;
    NSString *mapping = nil;
    
    // Scan through the mapping till a match is found:
    for (mapping in array)
    {
    	NSArray *parts = [mapping componentsSeparatedByString:@"="];
        
        if ([desired isEqualToString:[parts firstObject]])
            result = [parts lastObject];
        else if ([desired isEqualToString:[parts lastObject]])
            result = [parts firstObject];
    }
    
    // If no result and a default was provided, return a copy of the default converted to a string (makes a copy as -[NSString description] returns self):
    if (!result && defaultValue)
        result = [[[defaultValue copy] autorelease] description];
    
    return result;
}

@end


// ----------------------------------------------------------------------------------------
#pragma mark -
// ----------------------------------------------------------------------------------------


@implementation NSString (DSInternetStringCategories)

/*
 stringByReplacingPercentEscapes
 
 An Objective-C wrapper around the Core Foundation function to convert a URL-safe string into a raw format.
 
 Written by DJS 2003-10.
*/

- (NSString *)stringByReplacingPercentEscapes
{
    return [(NSString *)CFURLCreateStringByReplacingPercentEscapes(NULL, (CFStringRef)self, CFSTR("")) autorelease];
}

/*
 stringByAddingPercentEscapes
 
 An Objective-C wrapper around the Core Foundation function to convert a raw string into a URL-safe format.
 
 Written by DJS 2003-07.
*/

- (NSString *)stringByAddingPercentEscapes
{
    return [(NSString *)CFURLCreateStringByAddingPercentEscapes(NULL, (CFStringRef)self, NULL, NULL, kCFStringEncodingUTF8) autorelease];
}

/*
 stringByMakingURLSafe
 
 Returns a string based on the receiver with diacritical marks removed and percent escapes added as needed, safe for passing as a URL parameter.
 
 Written by DJS 2007-04.
*/

- (NSString *)stringByMakingURLSafe;
{
    return [[self stringByRemovingDiacriticalMarks] stringByAddingPercentEscapes];
}

/*
 stringByStrippingHTML
 
 Given a string containing HTML tags like "<p>" and "<a href=...>", strips out all such tags, including any incomplete tags at the start of the string (if it is a substring of the source).
 
 Written by DJS 2006-10.
*/

- (NSString *)stringByStrippingHTML;
{
    NSMutableString *output = [NSMutableString stringWithCapacity:[self length]];
    
    // Scan through the string:
    NSScanner *scanner = [NSScanner scannerWithString:self];
    
    // NSScanner skips spaces and returns by default, which we don't want, so get it to skip illegal characters instead:
    [scanner setCharactersToBeSkipped:[NSCharacterSet illegalCharacterSet]];
    
    while (![scanner isAtEnd])
    {
        NSString *prefix = @"";
        
        // Scan up till a HTML tag and remember for later:
        [scanner scanUpToString:@"<" intoString:&prefix];
        
        // If we found a HTML tag, scan past it:
        if ([scanner scanString:@"<" intoString:nil] && [scanner scanUpToString:@">" intoString:nil])
            [scanner scanString:@">" intoString:nil];
        
        // Append the stuff before the HTML tag to the output:
        [output appendString:prefix];
    }
    
    NSRange range = [output rangeOfString:@">"];
    
    if (range.location != NSNotFound)
        [output deleteCharactersInRange:NSMakeRange(0, range.location + range.length)];
    
    return [NSString stringWithString:output];
}

/*
 stringByReplacingHTMLEntities
 
 Given a string containing HTML entities like "&amp;" and "&copy;", returns the decoded equivalent, i.e. with "&amp;" converted to "&", etc.  If there are no HTML entities in the string, the receiver is returned.
 
 Written by DJS 2006-07.
*/

- (NSString *)stringByReplacingHTMLEntities;
{
    if ([self containsString:@"&"])
    {
        NSData *data = [self dataUsingEncoding:NSUTF8StringEncoding];
        NSAttributedString *attrib = [[[NSAttributedString alloc] initWithHTML:data documentAttributes:nil] autorelease];
        
        return [attrib string];
    }
    else
        return self;
}

/*
 stringByCleaningURL
 
 Given a potential URL, cleans it up to ensure it is valid.  If the receiver is blank, it is returned intact.  It defaults the TLD to ".com" without interfering with the path, and adds the "http://" scheme if needed.  So, for example, "dejal/products" will result in "http://www.dejal.com/products".  To specify a different default scheme, use -stringByCleaningURLWithDefaultScheme: instead.
 
 Written by DJS 2003-11.
 */

- (NSString *)stringByCleaningURL
{
    return [self stringByCleaningURLWithDefaultScheme:nil];
}

/*
 stringByCleaningURLWithDefaultScheme:
 
 Given a potential URL, cleans it up to ensure it is valid.  If the receiver is blank, it is returned intact.  It defaults the TLD to ".com" without interfering with the path, and adds the specified default scheme if needed.  If the scheme is nil, it defaults to "http://".  So, for example, "dejal/products" will result in "http://www.dejal.com/products".  To use "http://" as the default scheme, you could use -stringByCleaningURL instead.
 
 Written by DJS 2003-11.
 Changed by DJS 2004-01 to return blank intact.
 Changed by DJS 2004-06 to support the file:// scheme, which would contain a blank url component.
 Changed by DJS 2007-07 to support the localhost domain.
*/

- (NSString *)stringByCleaningURLWithDefaultScheme:(NSString *)scheme
{
    if (![self length])
        return self;
    
    NSString *url = self;
    NSString *path = @"/";
    NSRange range = [url rangeOfString:@"://"];
    
    if (range.location != NSNotFound)
    {
        scheme = [url substringToIndex:range.location + range.length];
        url = [url substringFromIndex:range.location + range.length];
    }
    
    range = [url rangeOfString:@"/"];
    
    if (range.location != NSNotFound)
    {
        path = [url substringFromIndex:range.location];
        url = [url substringToIndex:range.location];
    }
    
    if (![url containsString:@"."] && ![scheme isEqualToString:@"file://"] && ![url containsString:@"localhost"])
        url = [NSString stringWithFormat:@"www.%@.com", url];
    
    if (!scheme)
        scheme = @"http://";
    
    url = [NSString stringWithFormat:@"%@%@%@", scheme, url, path];
    
    return url;
}

@end


// ----------------------------------------------------------------------------------------
#pragma mark -
// ----------------------------------------------------------------------------------------


@implementation NSString (DSFilePathStringCategories)

/*
 stringByCleaningFilenameWithDefault:
 
 Given a filename (not path), returns it with illegal filename characters removed or replaced.  If the resulting filename is blank, the specified default filename is returned instead.
 
 Written by DJS 2003-11.
*/

- (NSString *)stringByCleaningFilenameWithDefault:(NSString *)defaultFilename
{
    NSMutableString *mutString = [NSMutableString stringWithString:self];
    
    [mutString deleteLeadingSpaces:YES trailingSpaces:YES];
    [mutString replaceAllOccurrencesOf:@"/" with:@"-"];
    [mutString replaceAllOccurrencesOf:@":" with:@"-"];
    
    if ([mutString length])
        return [NSString stringWithString:mutString];
    else
        return defaultFilename;
}

/*
 stringByAppendingPathComponent:cleanWithDefault:
 
 Same as -stringByAppendingPathComponent:, but the filename is first cleaned via -stringByCleaningFilenameWithDefault:.
 
 Written by DJS 2003-11.
*/

- (NSString *)stringByAppendingPathComponent:(NSString *)dirtyFilename cleanWithDefault:(NSString *)defaultFilename
{
    return [self stringByAppendingPathComponent:[dirtyFilename stringByCleaningFilenameWithDefault:defaultFilename]];
}

/*
 lastPathComponentWithoutExtension
 
 Returns the filename portion of a path, without any extension.
 
 Written by DJS 2003-11.
*/

- (NSString *)lastPathComponentWithoutExtension
{
    return [[self lastPathComponent] stringByDeletingPathExtension];
}

/*
 backupFilePath
 
 Given a file or directory path, returns the same path with the tilde character inserted before the extension (if any), as is conventional for backup files.
 
 Written by DJS 2004-03.
*/

- (NSString *)backupFilePath
{
    NSString *extension = [self pathExtension];
    NSString *previous = [self stringByDeletingPathExtension];
    NSString *path = [previous stringByAppendingString:@"~"];
    
    if ([extension length])
        path = [path stringByAppendingPathExtension:extension];
    
    return path;
}

/*
 uniquePath
 
 Given a file or directory path, returns the same path if it is already unique (i.e. no file exists there), or returns that path with a number inserted before the extension to make it unique, if necessary.  See also -uniquePathWithPrefix:, below.
 
 Written by DJS 2005-01.
*/

- (NSString *)uniquePath
{
    return [self uniquePathWithPrefix:nil];
}

/*
 uniquePathWithPrefix:
 
 Given a file or directory path, returns the same path if it is already unique (i.e. no file exists there), or returns that path with the prefix and/or a number inserted before the extension to make it unique (e.g. "filename copy.txt", "filename copy 2.txt", "filename copy 3.txt", etc), if necessary.  See also -uniquePath, above.
 
 Written by DJS 2005-01.
*/

- (NSString *)uniquePathWithPrefix:(NSString *)prefix
{
    if (![[NSFileManager defaultManager] fileExistsAtPath:self])
        return self;
    
    NSString *path = self;
    NSString *extension = [self pathExtension];
    NSString *previous = [self stringByDeletingPathExtension];
    BOOL usePrefix = [prefix length];
    NSUInteger i = !usePrefix;
    
    do
    {
        i++;
        path = previous;
        
        if (usePrefix)
            path = [path stringByAppendingFormat:@" %@", prefix];
        
        if (i > 1)
            path = [path stringByAppendingFormat:@" %d", i];
        
        if ([extension length])
            path = [path stringByAppendingPathExtension:extension];
    }
    while ([[NSFileManager defaultManager] fileExistsAtPath:path]);
    
    return path;
}

/*
 validatedDirectoryPath
 
 Expands a tilde and/or symbolic link in the path, then checks that all directories in the path already exist.  If any don't, they are created.  The expanded path is returned.
 
 Written by DJS 2004-03.
*/

- (NSString *)validatedDirectoryPath
{
    NSString *standardized = [self expandedPath];
    NSArray *components = [standardized pathComponents];
    NSString *component;
    NSString *path = @"";
    
    for (component in components)
    {
        path = [path stringByAppendingPathComponent:component];
        
        if (![[NSFileManager defaultManager] fileExistsAtPath:path])
            [[NSFileManager defaultManager] createDirectoryAtPath:path attributes:nil];
    }
    
    return path;
}

/*
 validatedFilePath
 
 Similar to -validatedDirectoryPath, above, except the last path component is assumed to be a filename, and so is ignored.
 
 Written by DJS 2004-03.
*/

- (NSString *)validatedFilePath
{
    NSString *filename = [self lastPathComponent];
    NSString *directory = [self stringByDeletingLastPathComponent];
    
    directory = [directory validatedDirectoryPath];
    
    NSString *path = [directory stringByAppendingPathComponent:filename];
    
    return path;
}

/*
 expandedPath
 
 Similar to the Cocoa -stringByStandardizingPath method, but also recognizes and expands the special application bundle indicator, "<APP>".  It can be abbreviated with -abbreviatedPath, below.
 
 Written by DJS 2005-05.
*/

- (NSString *)expandedPath
{
    if ([self hasPrefix:@"<APP>"])
        return [self stringByReplacingAllOccurrencesOf:@"<APP>" with:[[NSBundle mainBundle] bundlePath]];
    else
        return [self stringByStandardizingPath];
}

/*
 abbreviatedPath
 
 Similar to the Cocoa -stringByAbbreviatingWithTildeInPath method, but also abbreviates a path within the application bundle with the indicator "<APP>".  It can be expanded with -expandedPath, above.
 
 Written by DJS 2005-05.
*/

- (NSString *)abbreviatedPath
{
    NSString *bundlePath = [[NSBundle mainBundle] bundlePath];
    
    if ([self hasPrefix:bundlePath])
        return [self stringByReplacingAllOccurrencesOf:bundlePath with:@"<APP>"];
    else
        return [self stringByAbbreviatingWithTildeInPath];
}

@end


// ----------------------------------------------------------------------------------------
#pragma mark -
// ----------------------------------------------------------------------------------------


@implementation NSString (DSPropertyListStringCategories)

+ (NSString *)stringWithPoint:(NSPoint)point
{
    return [self stringWithFormat:@"%f, %f", point.x, point.y];
}

- (NSPoint)pointValue
{
    NSArray *components = [self componentsSeparatedByString:@", "];
    NSPoint point = NSZeroPoint;
    
    point.x = [[components objectAtIndex:0] floatValue];
    point.y = [[components objectAtIndex:1] floatValue];
    
    return point;
}

+ (NSString *)stringWithBoolValue:(BOOL)value;
{
    return [self stringWithIntegerValue:value];
}

// 2007-06: The following method is deprecated, as 10.5 implements -boolValue:

- (BOOL)booleanValue;
{
    return [self integerValue];
}

@end


// ----------------------------------------------------------------------------------------
#pragma mark -
// ----------------------------------------------------------------------------------------


@implementation NSString (DSAppKitStringCategories)

/*
 stringWithColor:

 Given a color, returns it encoded as a string, so it can be added to the defaults, etc.

 Written by DJS 2003-07.
*/

+ (NSString *)stringWithColor:(NSColor *)color
{
    CGFloat r, g, b, a;

    [[color colorUsingColorSpaceName:NSCalibratedRGBColorSpace] getRed:&r green:&g blue:&b alpha:&a];

    if (a == 1.0)
        return [self stringWithFormat:@"%g %g %g", r, g, b];
    else
        return [self stringWithFormat:@"%g %g %g %g", r, g, b, a];
}

/*
 color

 Returns a NSColor from the encoded value of the receiver, which must have been encoded by -stringWithColor: above.
 
 Written by DJS 2003-07.
*/

- (NSColor *)color
{
    CGFloat r = 0.0, g = 0.0, b = 0.0, a = 1.0;

    sscanf([self UTF8String], "%f%f%f%f", &r, &g, &b, &a);
    
    return [NSColor colorWithCalibratedRed:r green:g blue:b alpha:a];
}

@end


// ----------------------------------------------------------------------------------------
#pragma mark -
// ----------------------------------------------------------------------------------------


@implementation NSMutableString (DSStringCategories)

/*
 deleteCharactersInSet:
 
 Removes from the receiver all characters matching the set.
 
 Written by DJS 2005-04.
*/

- (void)deleteCharactersInSet:(NSCharacterSet *)set
{
    NSInteger i;
    
    for (i = [self length] - 1; i >= 0; i--)
    {
        unichar nChar = [self characterAtIndex:i];
        
        if ([set characterIsMember:nChar])
            [self deleteCharactersInRange:NSMakeRange(i, 1)];
    }
}

- (void)caseInsensitiveReplaceAllOccurrencesOf:(NSString *)string1 with:(NSString *)string2;
{
    [self replaceOccurrencesOfString:string1 withString:string2 options:NSCaseInsensitiveSearch range:NSMakeRange(0, [self length])];
}

- (void)replaceAllOccurrencesOf:(NSString *)string1 with:(NSString *)string2
{
    [self replaceOccurrencesOfString:string1 withString:string2 options:0 range:NSMakeRange(0, [self length])];
}

/*      my old implementation; now replaced by Apple's (introduced in 10.2):
- (void)replaceAllOccurrencesOf:(NSString *)string1 with:(NSString *)string2
{
    BOOL done = NO;

    while (!done)
    {
        NSRange nextPosition = [self rangeOfString:string1];

        if (nextPosition.length > 0)
            [self replaceCharactersInRange:nextPosition withString:string2];
        else
            done = YES;
    }
}
*/

- (void)deleteLeadingSpaces:(BOOL)leading trailingSpaces:(BOOL)trailing
{
    if (leading)
    {
        while ([self hasPrefix:@" "])
            [self deleteCharactersInRange:NSMakeRange(0, 1)];
    }

    if (trailing)
    {
        while ([self hasSuffix:@" "])
            [self deleteCharactersInRange:NSMakeRange([self length] - 1, 1)];
    }
}

/*
 appendString:or:
 
 If the string is a valid non-empty string, it is appended to the receiver.  Otherwise, if the alternative is a valid non-empty string, it is appended.  If neither applies, the receiver is not changed.
 
 Written by DJS 2004-03.
*/

- (void)appendString:(NSString *)string or:(NSString *)alternative
{
    if ([string length])
        [self appendString:string];
    else if ([alternative length])
        [self appendString:alternative];
}

/*
 appendPrefix:keyword:suffix:or:
 
 If the keyword is a valid non-empty string, the prefix, keyword, and suffix are appended to the receiver (the prefix and/or suffix may be nil if desired).  Otherwise, if the alternative is a valid non-empty string, it is appended.  If neither applies, the receiver is not changed.
 
 Written by DJS 2004-03.
*/

- (void)appendPrefix:(NSString *)prefix keyword:(NSString *)keyword suffix:(NSString *)suffix or:(NSString *)alternative
{
    if ([keyword length])
    {
        [self appendString:prefix or:nil];
        [self appendString:keyword];
        [self appendString:suffix or:nil];
    }
    else
        [self appendString:alternative or:nil];
}

@end


// ----------------------------------------------------------------------------------------
#pragma mark -
// ----------------------------------------------------------------------------------------


@implementation NSString (DSPascalStringCategories)

/*
 stringWithPascalString:
 
 Given a pointer to a Pascal-format string (with an initial length byte), returns the NSString equivalent.
 
 Written by DJS 2003-08.
*/

+ (NSString *)stringWithPascalString:(void *)strPtr
{
    return [(NSString *)CFStringCreateWithPascalString(kCFAllocatorDefault, strPtr, kCFStringEncodingMacRoman) autorelease];
}

/*
 getPascalString:maxLength:
 
 Given a pointer to a Pascal-format string (with an initial length byte) having a capacity of maxLength, sets it to the receiver.  Returns YES if successful.
 
 Written by DJS 2003-08.
*/

- (BOOL)getPascalString:(void *)strPtr maxLength:(NSUInteger)maxLength
{
    NSString *value = self;
    
    if (maxLength < 1 || maxLength > 255)
        maxLength = 255;

    if ([value length] > maxLength)
        value = [value substringToIndex:maxLength];

    return CFStringGetPascalString((CFStringRef)value, (StringPtr)strPtr, maxLength + 1, kCFStringEncodingMacRoman);
}

@end

