//
//  DSUserDefaultsCategories.m
//  Shared categories
//
//  Created by David Sinclair on Thu Jul 03 2003.
//  Copyright (c) 2003 - 2008 Dejal Systems, LLC. All rights reserved.
//
//  Redistribution and use in source and binary forms, with or without modification,
//  are permitted provided that the following conditions are met:
//  
//  Redistributions of source code must retain this list of conditions and the following disclaimer.
//  
//  The name of Dejal Systems, LLC may not be used to endorse or promote products derived from this
//  software without specific prior written permission.
//  
//  THIS SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED,
//  INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
//  PURPOSE AND NONINFRINGEMENT OF THIRD PARTY RIGHTS. IN NO EVENT SHALL THE AUTHORS OR
//  COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN
//  ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE
//  SOFTWARE OR THE USE OR OTHER DEALINGS IN THIS SOFTWARE.
//

#import "DSUserDefaultsCategories.h"
#import "DSStringCategories.h"
#import "DSDictionaryCategories.h"


@implementation NSUserDefaults (DSUserDefaultsCategories)

/*
 setObject:forKey:defaultValue:
 
 Adds an entry to the receiver, consisting of aKey and its corresponding value object anObject.  If anObject is nil, sets the default value instead; does nothing if both objects are nil.
 
 Written by DJS 2007-01.
*/

- (void)setObject:(id)anObject forKey:(id)aKey defaultValue:(id)aDefault;
{
    if (anObject)
        [self setObject:anObject forKey:aKey];
    else if (aDefault)
        [self setObject:aDefault forKey:aKey];
}

/*
 setObject:forKey:removeIfNil:
 
 Adds an entry to the receiver, consisting of aKey and its corresponding value object anObject.  If anObject is nil and removeIfNil is YES, the key is instead removed from the receiver, if already present, otherwise nothing happens; useful to avoid an exception when an object may be nil.
 
 Written by DJS 2007-01.
*/

- (void)setObject:(id)anObject forKey:(id)aKey removeIfNil:(BOOL)removeIfNil;
{
    if (anObject)
        [self setObject:anObject forKey:aKey];
    else if (removeIfNil)
        [self removeObjectForKey:aKey];
}

/*
 incrementIntegerForKey:
 
 Convenience method to increment an integer user default value.
 
 Written by DJS 2008-04.
*/

- (NSInteger)incrementIntegerForKey:(NSString *)defaultName;
{
    NSInteger value = [self integerForKey:defaultName] + 1;
    
    [self setInteger:value forKey:defaultName];
    
    return value;
}

/*
 pointForKey:
 
 Returns a point from the defaults with the specified key.
 
 Written by DJS 2005-04.
*/

- (NSPoint)pointForKey:(NSString *)defaultName;
{
    NSDictionary *dict = [self objectForKey:defaultName];
    NSPoint point = NSZeroPoint;
    
    if (dict)
    {
        point.x = [dict floatForKey:@"X"];
        point.y = [dict floatForKey:@"Y"];
    }
    
    return point;
}

/*
 setPoint:forKey:
 
 Stores the specified point in the defaults with the specified key.
 
 Written by DJS 2005-04.
*/

- (void)setPoint:(NSPoint)point forKey:(NSString *)defaultName;
{
    NSDictionary *dict = [NSDictionary dictionaryWithObjectsAndKeys:
        [NSNumber numberWithFloat:point.x], @"X",
        [NSNumber numberWithFloat:point.y], @"Y",
        nil];
    
    [self setObject:dict forKey:defaultName];
}

/*
 sizeForKey:orDefaultSize:
 
 Returns a size from the defaults with the specified key.  If there is no preference with that key, the specified size is returned instead (e.g. NSZeroSize).
 
 Written by DJS 2006-01.
*/

- (NSSize)sizeForKey:(NSString *)defaultName orDefaultSize:(NSSize)defaultSize;
{
    NSDictionary *dict = [self objectForKey:defaultName];
    NSSize size = defaultSize;
    
    if (dict)
    {
        size.width = [dict floatForKey:@"Width"];
        size.height = [dict floatForKey:@"Height"];
    }
    
    return size;
}

/*
 setSize:forKey:
 
 Stores the specified size in the defaults with the specified key.
 
 Written by DJS 2006-01.
*/

- (void)setSize:(NSSize)size forKey:(NSString *)defaultName;
{
    NSDictionary *dict = [NSDictionary dictionaryWithObjectsAndKeys:
        [NSNumber numberWithFloat:size.width], @"Width",
        [NSNumber numberWithFloat:size.height], @"Height",
        nil];
    
    [self setObject:dict forKey:defaultName];
}

/*
 colorForKey:
 
 Returns a color from the defaults with the specified key.
 
 Written by DJS 2003-07.
*/

- (NSColor *)colorForKey:(NSString *)defaultName;
{
    NSString *value = [self stringForKey:defaultName];
    
    if (value)
        return [value color];
    else
        return nil;
}

/*
 setColor:forKey:
 
 Stores the specified color in the defaults with the specified key.
 
 Written by DJS 2003-07.
*/

- (void)setColor:(NSColor *)color forKey:(NSString *)defaultName;
{
    [self setObject:[NSString stringWithColor:color] forKey:defaultName];
}

/*
 grayForKey:
 
 Returns a shade of gray from the defaults with the specified key.
 
 Written by DJS 2003-07.
*/

- (NSColor *)grayForKey:(NSString *)defaultName;
{
    return [NSColor colorWithCalibratedWhite:[self floatForKey:defaultName] alpha:1.0];
}

/*
 setGray:forKey:
 
 Stores the specified shade of gray in the defaults with the specified key.
 
 Written by DJS 2003-07.
*/

- (void)setGray:(NSColor *)gray forKey:(NSString *)defaultName;
{
    CGFloat grayFloat;

    [[gray colorUsingColorSpaceName:NSCalibratedWhiteColorSpace] getWhite:&grayFloat alpha:NULL];
    [self setFloat:grayFloat forKey:defaultName];
}


/*
 factorySettingsForKeys:
 
 Given an array of user default keys, returns the factory settings, i.e. as set by -registerDefaults:.
 
 Written by DJS 2007-11.
*/

- (NSDictionary *)factorySettingsForKeys:(NSArray *)keysArray;
{
//    return [[self volatileDomainForName:NSRegistrationDomain] subdictionaryWithKeys:keysArray defaultValue:nil];
    return [[self volatileDomainForName:NSRegistrationDomain] dictionaryWithValuesForKeys:keysArray];
}

/*
 changedFromFactorySettingsForKey:

 Given a user default key, returns YES if that default has been changed from its factory setting, i.e. as set by -registerDefaults:.
 
 Written by DJS 2003-07.
*/

- (BOOL)changedFromFactorySettingsForKey:(NSString *)key
{
    NSDictionary *factorySettings = [self volatileDomainForName:NSRegistrationDomain];

    return ![[factorySettings objectForKey:key] isEqualTo:[self objectForKey:key]];
}

/*
 changedFromFactorySettingsForKeys:
 
 Given an array of user default keys, returns YES if that default has been changed from its factory setting, i.e. as set by -registerDefaults:.
 
 Written by DJS 2003-07.
*/

- (BOOL)changedFromFactorySettingsForKeys:(NSArray *)keysArray
{
    NSDictionary *factorySettings = [self volatileDomainForName:NSRegistrationDomain];
    NSEnumerator *enumerator = [keysArray objectEnumerator];
    NSString *key;
    BOOL changed = NO;

    while (!changed && (key = [enumerator nextObject]))
        changed = ![[factorySettings objectForKey:key] isEqualTo:[self objectForKey:key]];

    return changed;
}

/*
 restoreFactorySettingsForKey:

 Given a user default key, restores its value to the factory settings, i.e. as set by -registerDefaults:.
 
 Written by DJS 2004-01.
*/

- (void)restoreFactorySettingsForKey:(NSString *)key
{
    [self removeObjectForKey:key];
}

/*
 restoreFactorySettingsForKeys:
 
 Given an array of user default keys, restores all of their values to the factory settings, i.e. as set by -registerDefaults:.
 
 Written by DJS 2003-07.
 */

- (void)restoreFactorySettingsForKeys:(NSArray *)keysArray
{
    NSString *key;
    
    for (key in keysArray)
        [self restoreFactorySettingsForKey:key];
}

/*
 copyObjectForKey:toKey:replace:
 
 Given a user default key of an old preference, and the new key for that preference, copies the corresponding object to the new key.  If there was no object with the old key, does nothing.  If replace is NO, the object is not copied if a preference already exists for the new key.  Returns YES if an object was copied.
 
 Written by DJS 2006-02.
*/

- (BOOL)copyObjectForKey:(NSString *)oldKey toKey:(NSString *)newKey replace:(BOOL)replace;
{
    id oldObject = [self objectForKey:oldKey];
    BOOL hasNewValue = [self changedFromFactorySettingsForKey:newKey];
    BOOL okayToCopy = (oldObject && (!hasNewValue || replace));
    
    if (okayToCopy)
        [self setObject:[[oldObject copy] autorelease] forKey:newKey];
    
    return okayToCopy;
}

/*
 copyObjectsForKeys:replace:
 
 Given an array of pairs of old and new user default keys, copies the corresponding objects from the old to new keys.  Old keys with no objects are skipped.  If replace is NO, new keys with existing objects are also skipped.  Returns YES if any object were copied.
 
 Written by DJS 2006-02.
*/

- (BOOL)copyObjectsForKeys:(NSArray *)keyPairsArray replace:(BOOL)replace;
{
    NSEnumerator *enumerator = [keyPairsArray objectEnumerator];
    NSString *oldKey;
    NSString *newKey;
    BOOL any = NO;
    
    while ((oldKey = [enumerator nextObject]) && (newKey = [enumerator nextObject]))
    {
    	any = any && [self copyObjectForKey:oldKey toKey:newKey replace:replace];
    }
    
    return any;
}

/*
 copyPreferenceWithKey:fromBundleIdentifier:removeIfMissing:
 
 Copies one preference from the preference file with the specified bundle identifier to the receiver's preferences.  If a preference isn't set in the other file, it can either be skipped or removed from the receiver's preferences.
 
 Written by DJS 2007-03.
*/

- (void)copyPreferenceWithKey:(NSString *)preferenceKey fromBundleIdentifier:(NSString *)bundleIdentifier removeIfMissing:(BOOL)removeIfMissing;
{
    [self copyPreferencesWithKeys:[NSArray arrayWithObject:preferenceKey] fromBundleIdentifier:bundleIdentifier removeIfMissing:removeIfMissing];
}

/*
 copyPreferencesWithKeys:fromBundleIdentifier:removeIfMissing:
 
 Copies any number of preferences from the preference file with the specified bundle identifier to the receiver's preferences.  If a preference isn't set in the other file, it can either be skipped or removed from the receiver's preferences.
 
 Written by DJS 2007-03.
*/

- (void)copyPreferencesWithKeys:(NSArray *)preferenceKeys fromBundleIdentifier:(NSString *)bundleIdentifier removeIfMissing:(BOOL)removeIfMissing;
{
    NSUserDefaults *defaults = [NSUserDefaults standardUserDefaults];
    NSDictionary *domain = [defaults persistentDomainForName:bundleIdentifier];
    NSString *key;
    
    for (key in preferenceKeys)
    {
    	id object = [domain objectForKey:key];
        
        if (object)
            [defaults setObject:object forKey:key];
        else if (removeIfMissing)
            [defaults removeObjectForKey:key];
    }
    
    [defaults synchronize];
}


// ----------------------------------------------------------------------------------------
#pragma mark -
#pragma mark LOGIN SYSTEM PREFERENCES METHODS
// ----------------------------------------------------------------------------------------

/*
 isInLoginItems:inAnyLocation:
 
 Returns whether or not the specified file is in the Login Items for the current user.  If inAnyLocation is YES, returns YES if any application with the same name is there, otherwise only if the exact file specified in the path is there.
 
 Code based on that from <http://cocoa.mamasam.com/MACOSXDEV/2001/06/1/7986.php>.
 Changed by DJS 2006-12 to add the inAnyLocation parameter.
*/

- (BOOL)isInLoginItems:(NSString *)path inAnyLocation:(BOOL)inAnyLocation;
{
    NSString *loginwindow = @"loginwindow";
    NSUserDefaults *u;
    NSArray *a;
    NSDictionary *dict;
    BOOL found = NO;
    
    if (inAnyLocation)
        path = [path lastPathComponentWithoutExtension];
    
    // Get data from user defaults (~/Library/Preferences/loginwindow.plist):
    u = [[[NSUserDefaults alloc] init] autorelease];
    
    if ((a = [[u persistentDomainForName:loginwindow] objectForKey:@"AutoLaunchedApplicationDictionary"]))
    {
        NSEnumerator *enumerator = [a objectEnumerator];
        
        while (!found && (dict = [enumerator nextObject]))
        {
            NSString *other = [dict objectForKey:@"Path"];
            
            if (inAnyLocation)
                other = [other lastPathComponentWithoutExtension];
            
            if ([other isEqualToString:path])
                found = YES;
        }
    }
    
    return found;
}

/*
 loginItem:add:hide:
 
 Adds or removes the specified file (or app) to the Login Items, hiding if requested.  If already present when adding, the old entry is replaced; if not present when removing, it does nothing.
 
 Code based on that from <http://cocoa.mamasam.com/MACOSXDEV/2001/06/1/7986.php>.
 Changed by DJS 2006-01 to remove files with the same filename, to avoid having the same app launch from two different locations.
*/

- (void)loginItem:(NSString *)path add:(BOOL)add hide:(BOOL)hide;
{
    NSString *loginwindow = @"loginwindow";
    NSUserDefaults *u;
    NSMutableDictionary *d;
    NSDictionary *e;
    NSMutableArray *a;
    NSString *filename = [path lastPathComponentWithoutExtension];
    NSInteger i;
    
    // Get data from user defaults (~/Library/Preferences/loginwindow.plist):
    u = [[NSUserDefaults alloc] init];
    
    if (!(d = [[u persistentDomainForName:loginwindow] mutableCopy]))
        d = [[NSMutableDictionary alloc] initWithCapacity:1];
    if (!(a = [[d objectForKey:@"AutoLaunchedApplicationDictionary"] mutableCopy]))
        a = [[NSMutableArray alloc] initWithCapacity:1];
    
    // Remove any existing Login Items for this path; not using an enumerator as objects are removed:
    i = [a count] - 1;
    
    while (i >= 0)
    {
        e = [a objectAtIndex:i];
        
        if ([[[e objectForKey:@"Path"] lastPathComponentWithoutExtension] isEqualToString:filename])
            [a removeObjectAtIndex:i];
        
        i--;
    }
    
    if (add)
    {
        // Build entry:
        e = [NSDictionary dictionaryWithObjectsAndKeys:
            [NSNumber numberWithBool:hide], @"Hide",
            path, @"Path",
            nil];
        
        // Add entry:
        if (e)
        {
            [a insertObject:e atIndex:0];
        }
    }
    
    [d setObject:a forKey:@"AutoLaunchedApplicationDictionary"];
    
    // Update user defaults:
    [u removePersistentDomainForName:loginwindow];
    [u setPersistentDomain:d forName:loginwindow];
    [u synchronize];
    
    // Clean up:
    [a release];
    [d release];
    [u release];
}

@end
