//
//  DSIntervalPicker.m
//  DSInterval
//
//  Created by David Sinclair on 2008-07-11.
//  Copyright 2008 Dejal Systems, LLC. All rights reserved.
//
//  Redistribution and use in source and binary forms, with or without modification,
//  are permitted provided that the following conditions are met:
//  
//  Redistributions of source code must retain this list of conditions and the following disclaimer.
//  
//  The name of Dejal Systems, LLC may not be used to endorse or promote products derived from this
//  software without specific prior written permission.
//  
//  THIS SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED,
//  INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
//  PURPOSE AND NONINFRINGEMENT OF THIRD PARTY RIGHTS. IN NO EVENT SHALL THE AUTHORS OR
//  COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN
//  ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE
//  SOFTWARE OR THE USE OR OTHER DEALINGS IN THIS SOFTWARE.
//

#import "DSIntervalPicker.h"


@implementation DSIntervalPicker

+ (void)initialize;
{
    if (self == [DSIntervalPicker class])
    {
        [self setCellClass:[DSIntervalPickerCell class]];
        
        [self exposeBinding:@"firstAmount"];
        [self exposeBinding:@"secondAmount"];
        [self exposeBinding:@"amount"];
        [self exposeBinding:@"units"];
        [self exposeBinding:@"timeInterval"];
    }
}

+ (Class)cellClass;
{
    return [DSIntervalPickerCell class];
}

- (void)cleanup;
{
    [[NSNotificationCenter defaultCenter] removeObserver:self];
    
    [self unbind:@"firstAmount"];
    [self unbind:@"secondAmount"];
    [self unbind:@"amount"];
    [self unbind:@"units"];
    [self unbind:@"timeInterval"];
}

- (void)dealloc;
{
    [self cleanup];
    
    [super dealloc];
}

- (void)finalize;
{
    [self cleanup];
    
    [super finalize];
}

- (void)setFrameSize:(NSSize)newSize;
{
    [super setFrameSize:newSize];
    
    [[self cell] calcDrawInfo:[self frame]];
}

- (void)sizeToFit;
{
    [self setFrameSize:[[self cell] idealSize]];
}


// ----------------------------------------------------------------------------------------
#pragma mark -
#pragma mark FOCUS RING METHODS
// ----------------------------------------------------------------------------------------

- (void)delayedBecomeFirstResponder:(NSTimer *)timer;
{
    [[self cell] cellBecomeFirstResponderInDirection:[[timer userInfo] integerValue]];
    [super setKeyboardFocusRingNeedsDisplayInRect:[self bounds]];
}

- (BOOL)becomeFirstResponder;
{
    BOOL okay = [super becomeFirstResponder];
    
    if (okay)
        [NSTimer scheduledTimerWithTimeInterval:0.01 target:self selector:@selector(delayedBecomeFirstResponder:) userInfo:[NSNumber numberWithInteger:[[self window] keyViewSelectionDirection]] repeats:NO];
    
    return okay;
}

- (BOOL)resignFirstResponder;
{
    BOOL okay = [super resignFirstResponder];
    
    if (okay)
        [super setKeyboardFocusRingNeedsDisplayInRect:[self bounds]];
    
    return okay;
}

- (void)windowKeyStateDidChange:(NSNotification *)note;
{
    if ([[self window] firstResponder] == self)
        [super setKeyboardFocusRingNeedsDisplayInRect:[self bounds]];
}

- (void)viewDidMoveToWindow;
{
    // If we've been installed in a new window, unregister for notifications in the old window:
    [[NSNotificationCenter defaultCenter] removeObserver:self];
    
    // Then register for notifications in the new window:
    [[NSNotificationCenter defaultCenter] addObserver:self
                                             selector:@selector(windowKeyStateDidChange:)
                                                 name:NSWindowDidBecomeKeyNotification
                                               object:[self window]];
    
    [[NSNotificationCenter defaultCenter] addObserver:self
                                             selector:@selector(windowKeyStateDidChange:)
                                                 name:NSWindowDidResignKeyNotification
                                               object:[self window]];
}


// ----------------------------------------------------------------------------------------
#pragma mark -
#pragma mark ACCESSOR METHODS
// ----------------------------------------------------------------------------------------

- (DSIntervalPickerCell *)pickerCell;
{
    return [self cell];
}

- (DSInterval *)interval;
{
    return self.pickerCell.interval;
}

- (void)setInterval:(DSInterval *)interval;
{
    self.pickerCell.interval = interval;
}

- (CGFloat)firstAmount;
{
    return self.pickerCell.firstAmount;
}

- (void)setFirstAmount:(CGFloat)firstAmount;
{
    self.pickerCell.firstAmount = firstAmount;
}

- (CGFloat)secondAmount;
{
    return self.pickerCell.secondAmount;
}

- (void)setSecondAmount:(CGFloat)secondAmount;
{
    self.pickerCell.secondAmount = secondAmount;
}

- (CGFloat)amount;
{
    return self.pickerCell.amount;
}

- (void)setAmount:(CGFloat)amount;
{
    self.pickerCell.amount = amount;
}

- (DSIntervalUnits)units;
{
    return self.pickerCell.units;
}

- (void)setUnits:(DSIntervalUnits)units;
{
    self.pickerCell.units = units;
}

- (NSTimeInterval)firstTimeInterval;
{
    return self.pickerCell.firstTimeInterval;
}

- (NSTimeInterval)secondTimeInterval;
{
    return self.pickerCell.secondTimeInterval;
}

- (NSTimeInterval)amountTimeInterval;
{
    return self.pickerCell.amountTimeInterval;
}

/*
 objectValue
 
 Returns the DSInterval object using the standard accessor.  Provided as a convenience.
 
 Written by DJS 2008-07.
*/

- (id)objectValue;
{
    return [self.pickerCell objectValue];
}

/*
 setObjectValue:
 
 Sets the DSInterval object using the standard accessor.  Provided as a convenience.  Only sets the value if it is a DSInterval (or subclass).
 
 Written by DJS 2008-07.
*/

- (void)setObjectValue:(id <NSCopying>)object;
{
    [self.pickerCell setObjectValue:object];
}

/*
 stringValue
 
 Returns a string representation of the DSInterval value.  Provided as a convenience.  Note that it isn't possible to set the value via a string.
 
 Written by DJS 2008-07.
*/

- (NSString *)stringValue;
{
    return [self.pickerCell stringValue];
}

/*
 setStringValue:
 
 This is not currently supported for DSIntervalPicker.  Invoking this just logs a warning in the Console.
 
 Written by DJS 2008-09.
*/

- (void)setStringValue:(NSString *)string;
{
    NSLog(@"-setStringValue: is not supported by DSIntervalPicker (tried to set to %@)", string);
}

- (BOOL)usingRange;
{
    return self.pickerCell.usingRange;
}

- (void)setUsingRange:(BOOL)usingRange;
{
    self.pickerCell.usingRange = usingRange;
}

- (DSIntervalPickerRangeFiltering)rangeFiltering;
{
    return self.pickerCell.rangeFiltering;
}

- (void)setRangeFiltering:(DSIntervalPickerRangeFiltering)rangeFiltering;
{
    self.pickerCell.rangeFiltering = rangeFiltering;
}

- (CGFloat)minimumAmount;
{
    return self.pickerCell.minimumAmount;
}

- (void)setMinimumAmount:(CGFloat)minimumAmount;
{
    self.pickerCell.minimumAmount = minimumAmount;
}

- (CGFloat)maximumAmount;
{
    return self.pickerCell.maximumAmount;
}

- (void)setMaximumAmount:(CGFloat)maximumAmount;
{
    self.pickerCell.maximumAmount = maximumAmount;
}

- (BOOL)includeSeconds;
{
    return self.pickerCell.includeSeconds;
}

- (void)setIncludeSeconds:(BOOL)includeSeconds;
{
    self.pickerCell.includeSeconds = includeSeconds;
}

- (BOOL)includeMinutes;
{
    return self.pickerCell.includeMinutes;
}

- (void)setIncludeMinutes:(BOOL)includeMinutes;
{
    self.pickerCell.includeMinutes = includeMinutes;
}

- (BOOL)includeHours;
{
    return self.pickerCell.includeHours;
}

- (void)setIncludeHours:(BOOL)includeHours;
{
    self.pickerCell.includeHours = includeHours;
}

- (BOOL)includeDays;
{
    return self.pickerCell.includeDays;
}

- (void)setIncludeDays:(BOOL)includeDays;
{
    self.pickerCell.includeDays = includeDays;
}

- (BOOL)includeWeeks;
{
    return self.pickerCell.includeWeeks;
}

- (void)setIncludeWeeks:(BOOL)includeWeeks;
{
    self.pickerCell.includeWeeks = includeWeeks;
}

- (BOOL)includeMonths;
{
    return self.pickerCell.includeMonths;
}

- (void)setIncludeMonths:(BOOL)includeMonths;
{
    self.pickerCell.includeMonths = includeMonths;
}

- (BOOL)includeQuarters;
{
    return self.pickerCell.includeQuarters;
}

- (void)setIncludeQuarters:(BOOL)includeQuarters;
{
    self.pickerCell.includeQuarters = includeQuarters;
}

- (BOOL)includeYears;
{
    return self.pickerCell.includeYears;
}

- (void)setIncludeYears:(BOOL)includeYears;
{
    self.pickerCell.includeYears = includeYears;
}

- (BOOL)includeForever;
{
    return self.pickerCell.includeForever;
}

- (void)setIncludeForever:(BOOL)includeForever;
{
    self.pickerCell.includeForever = includeForever;
}

- (BOOL)includeNever;
{
    return self.pickerCell.includeNever;
}

- (void)setIncludeNever:(BOOL)includeNever;
{
    self.pickerCell.includeNever = includeNever;
}

- (void)setupForShortTerm;
{
    [self.pickerCell setupForShortTerm];
}

- (void)setupForLongTerm;
{
    [self.pickerCell setupForLongTerm];
}

- (void)setupForAllUnits;
{
    [self.pickerCell setupForAllUnits];
}

@end

