//
//  DSIntervalPickerCell.m
//  DSInterval
//
//  Created by David Sinclair on 2008-07-11.
//  Copyright 2008 Dejal Systems, LLC. All rights reserved.
//
//  Redistribution and use in source and binary forms, with or without modification,
//  are permitted provided that the following conditions are met:
//  
//  Redistributions of source code must retain this list of conditions and the following disclaimer.
//  
//  The name of Dejal Systems, LLC may not be used to endorse or promote products derived from this
//  software without specific prior written permission.
//  
//  THIS SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED,
//  INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
//  PURPOSE AND NONINFRINGEMENT OF THIRD PARTY RIGHTS. IN NO EVENT SHALL THE AUTHORS OR
//  COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN
//  ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE
//  SOFTWARE OR THE USE OR OTHER DEALINGS IN THIS SOFTWARE.
//

#import "DSIntervalPickerCell.h"
#import "DSIntervalPicker.h"


@interface DSIntervalPickerCell()

@property(retain) NSTextFieldCell *firstAmountCell;
@property(retain) NSTextFieldCell *rangeSeparatorCell;
@property(retain) NSTextFieldCell *secondAmountCell;
@property(retain) NSTextFieldCell *unitsCell;
@property(retain) NSStepperCell *stepperCell;
@property(readonly) NSTextFieldCell *leftmostCell;
@property(assign) NSTextFieldCell *selectedCell;
@property(retain) NSMutableArray *includedUnitsMapping;
@property CGFloat currentValue;
@property NSRect pickerFrame;
@property NSRect firstAmountFrame;
@property NSRect rangeSeparatorFrame;
@property NSRect secondAmountFrame;
@property NSRect unitsFrame;
@property NSRect stepperFrame;
@property BOOL shouldAppendEdit;
@property BOOL autoPopMenu;

- (void)setupCells;
- (void)setupCellFrames;
- (void)updateCells;
- (void)updateStepper;
- (void)updateIncludedUnitsMapping;
- (NSText *)currentEditor;
- (CGFloat)currentMinimum;
- (CGFloat)currentMaximum;
- (CGFloat)currentValue;
- (BOOL)isForeverOrNever;
- (DSIntervalUnits)unitsForIncludedMappingIndex:(NSUInteger)mappingIndex;
- (NSUInteger)includedMappingIndexForUnits:(DSIntervalUnits)units;
- (NSArray *)observedKeyPaths;
- (void)setupObservers;
- (NSWindow *)window;
- (void)showMenu;
- (void)makeFirstResponderIfNecessary;
- (void)editCell:(NSTextFieldCell *)cell;
- (NSMenu *)makeMenuForSelectedCell;

@end


// ----------------------------------------------------------------------------------------
#pragma mark -
// ----------------------------------------------------------------------------------------


@implementation DSIntervalPickerCell

@synthesize interval = _interval;
@synthesize rangeFiltering = _rangeFiltering;
@synthesize minimumAmount = _minimumAmount;
@synthesize maximumAmount = _maximumAmount;
@synthesize includeSeconds = _includeSeconds;
@synthesize includeMinutes = _includeMinutes;
@synthesize includeHours = _includeHours;
@synthesize includeDays = _includeDays;
@synthesize includeWeeks = _includeWeeks;
@synthesize includeMonths = _includeMonths;
@synthesize includeQuarters = _includeQuarters;
@synthesize includeYears = _includeYears;
@synthesize includeForever = _includeForever;
@synthesize includeNever = _includeNever;

@synthesize firstAmountCell = _firstAmountCell;
@synthesize rangeSeparatorCell = _rangeSeparatorCell;
@synthesize secondAmountCell = _secondAmountCell;
@synthesize unitsCell = _unitsCell;
@synthesize stepperCell = _stepperCell;
@synthesize selectedCell = _selectedCell;
@synthesize includedUnitsMapping = _includedUnitsMapping;
@synthesize pickerFrame = _pickerFrame;
@synthesize firstAmountFrame = _firstAmountFrame;
@synthesize rangeSeparatorFrame = _rangeSeparatorFrame;
@synthesize secondAmountFrame = _secondAmountFrame;
@synthesize unitsFrame = _unitsFrame;
@synthesize stepperFrame = _stepperFrame;
@synthesize shouldAppendEdit = _shouldAppendEdit;
@synthesize autoPopMenu = _autoPopMenu;

// ----------------------------------------------------------------------------------------

+ (void)initialize;
{
    [self exposeBinding:@"firstAmount"];
    [self exposeBinding:@"secondAmount"];
    [self exposeBinding:@"amount"];
    [self exposeBinding:@"units"];
    [self exposeBinding:@"timeInterval"];
/*
    // These bindings are also available.  Probably don't need to expose them, though:
    [self exposeBinding:@"usingRange"];
    [self exposeBinding:@"rangeFiltering"];
    [self exposeBinding:@"minimumAmount"];
    [self exposeBinding:@"maximumAmount"];
    [self exposeBinding:@"includeSeconds"];
    [self exposeBinding:@"includeMinutes"];
    [self exposeBinding:@"includeHours"];
    [self exposeBinding:@"includeDays"];
    [self exposeBinding:@"includeWeeks"];
    [self exposeBinding:@"includeMonths"];
    [self exposeBinding:@"includeQuarters"];
    [self exposeBinding:@"includeYears"];
    [self exposeBinding:@"includeForever"];
    [self exposeBinding:@"includeNever"];
*/
}

- (id)init;
{
    self = [super init];
    
    if (self)
    {
        self.interval = [DSInterval intervalWithAmount:1.0 units:DSIntervalMinutes];
        self.rangeFiltering = DSIntervalFirstLessThanSecond;
        self.minimumAmount = 0;
        self.maximumAmount = 999;
        self.includeSeconds = YES;
        self.includeMinutes = YES;
        self.includeHours = YES;
        self.includeDays = YES;
        self.includeWeeks = YES;
        self.includeMonths = YES;
        self.includeQuarters = NO;
        self.includeYears = YES;
        self.includeForever = NO;
        self.includeNever = NO;
        
        [self setBordered:YES];
    }
    
    return self;
}

- (id)initWithCoder:(NSCoder *)coder;
{
    self = [super initWithCoder:coder];
    
    if ([coder allowsKeyedCoding])
    {
        if ([coder decodeBoolForKey:@"usingRange"])
            self.interval = [DSInterval intervalWithRangeFirstAmount:[coder decodeFloatForKey:@"firstAmount"] secondAmount:[coder decodeFloatForKey:@"secondAmount"] units:[coder decodeIntegerForKey:@"units"]];
        else
            self.interval = [DSInterval intervalWithAmount:[coder decodeFloatForKey:@"secondAmount"] units:[coder decodeIntegerForKey:@"units"]];
        
        self.rangeFiltering = [coder decodeIntegerForKey:@"rangeFiltering"];
        self.minimumAmount = [coder decodeFloatForKey:@"minimumAmount"];
        self.maximumAmount = [coder decodeFloatForKey:@"maximumAmount"];
        self.includeSeconds = [coder decodeBoolForKey:@"includeSeconds"];
        self.includeMinutes = [coder decodeBoolForKey:@"includeMinutes"];
        self.includeHours = [coder decodeBoolForKey:@"includeHours"];
        self.includeDays = [coder decodeBoolForKey:@"includeDays"];
        self.includeWeeks = [coder decodeBoolForKey:@"includeWeeks"];
        self.includeMonths = [coder decodeBoolForKey:@"includeMonths"];
        self.includeQuarters = [coder decodeBoolForKey:@"includeQuarters"];
        self.includeYears = [coder decodeBoolForKey:@"includeYears"];
        self.includeForever = [coder decodeBoolForKey:@"includeForever"];
        self.includeNever = [coder decodeBoolForKey:@"includeNever"];
    }
    
    return self;
}

- (void)encodeWithCoder:(NSCoder *)coder;
{
    [super encodeWithCoder:coder];
    
    if ([coder allowsKeyedCoding])
    {
        [coder encodeBool:self.usingRange forKey:@"usingRange"];
        [coder encodeFloat:self.firstAmount forKey:@"firstAmount"];
        [coder encodeFloat:self.secondAmount forKey:@"secondAmount"];
        [coder encodeInteger:self.units forKey:@"units"];
        [coder encodeInteger:self.rangeFiltering forKey:@"rangeFiltering"];
        [coder encodeFloat:self.minimumAmount forKey:@"minimumAmount"];
        [coder encodeFloat:self.maximumAmount forKey:@"maximumAmount"];
        [coder encodeBool:self.includeSeconds forKey:@"includeSeconds"];
        [coder encodeBool:self.includeMinutes forKey:@"includeMinutes"];
        [coder encodeBool:self.includeHours forKey:@"includeHours"];
        [coder encodeBool:self.includeDays forKey:@"includeDays"];
        [coder encodeBool:self.includeWeeks forKey:@"includeWeeks"];
        [coder encodeBool:self.includeMonths forKey:@"includeMonths"];
        [coder encodeBool:self.includeQuarters forKey:@"includeQuarters"];
        [coder encodeBool:self.includeYears forKey:@"includeYears"];
        [coder encodeBool:self.includeForever forKey:@"includeForever"];
        [coder encodeBool:self.includeNever forKey:@"includeNever"];
    }
}

- (id)copyWithZone:(NSZone *)zone;
{
    DSIntervalPickerCell *copy = [super copyWithZone:zone];
    
    // Apparently NSCell implements -copyWithZone: by calling NSCopyObject(), which horribly breaks object ivars (the pointer is copied, but the object isn't retained): <http://gemma.apple.com/documentation/Cocoa/Conceptual/MemoryMgmt/Tasks/ImplementCopy.html#//apple_ref/doc/uid/20000049-997407>.  So we have to correct that by clearing the object ivars:
    copy->_interval = nil;
    copy->_firstAmountCell = nil;
    copy->_rangeSeparatorCell = nil;
    copy->_secondAmountCell = nil;
    copy->_unitsCell = nil;
    copy->_stepperCell = nil;
    copy->_selectedCell = nil;
    copy->_includedUnitsMapping = nil;
    copy->_pickerFrame = NSZeroRect;
    copy->_firstAmountFrame = NSZeroRect;
    copy->_rangeSeparatorFrame = NSZeroRect;
    copy->_secondAmountFrame = NSZeroRect;
    copy->_unitsFrame = NSZeroRect;
    copy->_stepperFrame = NSZeroRect;
    copy->_shouldAppendEdit = NO;
    copy->_autoPopMenu = NO;
    
    // Then set the value we want to copy (the others are automatically allocated when needed):
    copy.interval = self.interval;
    
    return copy;
}

- (void)cleanup;
{
    [self unbind:@"firstAmount"];
    [self unbind:@"secondAmount"];
    [self unbind:@"amount"];
    [self unbind:@"units"];
    [self unbind:@"timeInterval"];
/*
    [self unbind:@"usingRange"];
    [self unbind:@"rangeFiltering"];
    [self unbind:@"minimumAmount"];
    [self unbind:@"maximumAmount"];
    [self unbind:@"includeSeconds"];
    [self unbind:@"includeMinutes"];
    [self unbind:@"includeHours"];
    [self unbind:@"includeDays"];
    [self unbind:@"includeWeeks"];
    [self unbind:@"includeMonths"];
    [self unbind:@"includeQuarters"];
    [self unbind:@"includeYears"];
    [self unbind:@"includeForever"];
    [self unbind:@"includeNever"];
*/
}

- (void)finalize;
{
    [self cleanup];
    
    [super finalize];
}

- (void)dealloc;
{
    [self cleanup];
    
    self.interval = nil;
    self.includedUnitsMapping = nil;
    self.selectedCell = nil;
    self.firstAmountCell = nil;
    self.rangeSeparatorCell = nil;
    self.secondAmountCell = nil;
    self.unitsCell = nil;
    self.stepperCell = nil;
    
    [super dealloc];
}

- (NSTextFieldCell *)setupCellWithString:(NSString *)string isEditable:(BOOL)isEditable alignment:(NSTextAlignment)alignment;
{
    NSTextFieldCell *cell = [[[NSTextFieldCell alloc] initTextCell:string] autorelease];
    
    [cell setDrawsBackground:isEditable];
    [cell setEditable:isEditable];
    [cell setBordered:NO];
    [cell setControlSize:[self controlSize]];
    [cell setFont:[NSFont labelFontOfSize:[NSFont systemFontSizeForControlSize:[self controlSize]]]];
    [cell setAlignment:alignment];
    
    return cell;
}

- (void)setupStepper;
{
    self.stepperCell = [[[NSStepperCell alloc] init] autorelease];
    
    [self.stepperCell bind:@"value" toObject:self withKeyPath:@"currentValue" options:[NSDictionary dictionaryWithObject:[NSNumber numberWithBool:YES] forKey:NSContinuouslyUpdatesValueBindingOption]];
    
    [self.stepperCell setControlSize:[self controlSize]];
    
    [self updateIncludedUnitsMapping];
}

- (NSString *)stringWithIntegerValue:(NSInteger)value;
{
    return [NSString stringWithFormat:@"%d", value];
}

- (void)setControlSize:(NSControlSize)size;
{
    [super setControlSize:size];
    
    [self setupCells];
    
    if ([[self controlView] isKindOfClass:[DSIntervalPicker class]])
        [(DSIntervalPicker *)[self controlView] sizeToFit];
}

- (NSSize)idealSize;
{
    NSSize size = [[self setupCellWithString:[self stringWithIntegerValue:self.maximumAmount] isEditable:YES alignment:NSRightTextAlignment] cellSize];
    
    if (self.usingRange)
        size.width += [[self setupCellWithString:@"-" isEditable:YES alignment:NSCenterTextAlignment] cellSize].width + size.width;
    
    size.width += [[self setupCellWithString:[[DSInterval intervalWithAmount:9.0 units:DSIntervalSeconds] fullUnitsName] isEditable:YES alignment:NSLeftTextAlignment] cellSize].width;
    
    size.width += [self.stepperCell cellSize].width + 5.0;
    size.height += 10.0;
    
    return size;
}

- (void)setupCellFrames;
{
    NSRect baseFrame = self.pickerFrame;
    
    if (NSIsEmptyRect(baseFrame))
        baseFrame = [[self controlView] bounds];
    
    NSRect frame = baseFrame;
    frame.size = [self.stepperCell cellSize];
    frame.origin.x = NSMaxX(baseFrame) - frame.size.width;
    
    if ([self controlSize] == NSMiniControlSize)
        frame.origin.y += 3.0;
    else if ([self controlSize] == NSSmallControlSize)
        frame.origin.y += 1.0;
    
    self.stepperFrame = frame;
    
    frame = NSInsetRect(baseFrame, 3.0, 5.0);
    frame.size = [[self setupCellWithString:[self stringWithIntegerValue:self.maximumAmount] isEditable:YES alignment:NSRightTextAlignment] cellSize];
    
    if (self.usingRange)
    {
        self.firstAmountFrame = frame;
        
        frame.origin.x += floorf(frame.size.width);
        frame.size = [[self setupCellWithString:@"-" isEditable:YES alignment:NSCenterTextAlignment] cellSize];
        
        self.rangeSeparatorFrame = frame;
        
        frame.origin.x += floorf(frame.size.width);
        frame.size = self.firstAmountFrame.size;
    }
    
    self.secondAmountFrame = frame;
    
    frame.origin.x += floorf(frame.size.width);
    frame.size.width = self.stepperFrame.origin.x - frame.origin.x;
    
    if ([self controlSize] == NSMiniControlSize)
        frame.size.width -= 1.0;
    else if ([self controlSize] == NSSmallControlSize)
        frame.size.width -= 2.0;
    
    self.unitsFrame = frame;
}

- (void)setupCells;
{
    // If we're not in a DSIntervalPicker or NSMatrix control, use the unbordered small size by default (using the super implementation to avoid infinite recursion):
    if ([self controlView] && ![[self controlView] isKindOfClass:[DSIntervalPicker class]] && ![[self controlView] isKindOfClass:[NSMatrix class]])
    {
        [self setBordered:NO];
        [super setControlSize:NSSmallControlSize];
    }
    
    self.firstAmountCell = [self setupCellWithString:@"" isEditable:YES alignment:NSRightTextAlignment];
    self.rangeSeparatorCell = [self setupCellWithString:@"-" isEditable:YES alignment:NSCenterTextAlignment];
    self.secondAmountCell = [self setupCellWithString:@"" isEditable:YES alignment:NSRightTextAlignment];
    self.unitsCell = [self setupCellWithString:@"" isEditable:YES alignment:NSLeftTextAlignment];
    
    [self setupStepper];
    [self setupCellFrames];
    [self updateCells];
    [self setupObservers];
}

- (void)setupForShortTerm;
{
    self.includeSeconds = YES;
    self.includeMinutes = YES;
    self.includeHours = YES;
    self.includeDays = NO;
    self.includeWeeks = NO;
    self.includeMonths = NO;
    self.includeQuarters = NO;
    self.includeYears = NO;
}

- (void)setupForLongTerm;
{
    self.includeSeconds = NO;
    self.includeMinutes = NO;
    self.includeHours = NO;
    self.includeDays = YES;
    self.includeWeeks = YES;
    self.includeMonths = YES;
    self.includeQuarters = NO;
    self.includeYears = YES;
}

- (void)setupForAllUnits;
{
    self.includeSeconds = YES;
    self.includeMinutes = YES;
    self.includeHours = YES;
    self.includeDays = YES;
    self.includeWeeks = YES;
    self.includeMonths = YES;
    self.includeQuarters = NO;
    self.includeYears = YES;
}

- (BOOL)sendActionToTarget;
{
    if ([self target] && [self action])
        return [(NSControl *)[self controlView] sendAction:[self action] to:[self target]];
    else
        return NO;
}

- (void)setEnabled:(BOOL)enabled;
{
    if (!enabled && [self currentEditor])
        [[self selectedCell] endEditing:[self currentEditor]];
    
    BOOL canEdit = ![self isForeverOrNever];
    
    [self.firstAmountCell setEditable:canEdit];
    [self.rangeSeparatorCell setEditable:canEdit];
    [self.secondAmountCell setEditable:canEdit];
    
    [self.firstAmountCell setEnabled:enabled && canEdit];
    [self.rangeSeparatorCell setEnabled:enabled && canEdit];
    [self.secondAmountCell setEnabled:enabled && canEdit];
    [self.unitsCell setEnabled:enabled];
    [self.stepperCell setEnabled:enabled];
    
    [super setEnabled:enabled];
}

- (void)updateCells;
{
    if ([self currentEditor])
    {
        if ([self selectedCell] == self.firstAmountCell)
            [[self currentEditor] setString:[self stringWithIntegerValue:self.firstAmount]];
        else if ([self selectedCell] == self.secondAmountCell)
            [[self currentEditor] setString:[self stringWithIntegerValue:self.secondAmount]];
        else
            [[self currentEditor] setString:[self.interval fullUnitsName]];
        
        [[self currentEditor] selectAll:nil];
    }
    
    if ([self isForeverOrNever])
    {
        [self.firstAmountCell setStringValue:@""];
        [self.secondAmountCell setStringValue:@""];
    }
    else
    {
        [self.firstAmountCell setIntegerValue:self.firstAmount];
        [self.secondAmountCell setIntegerValue:self.secondAmount];
    }
    
    [self.unitsCell setStringValue:[self.interval fullUnitsName]];
    
    // Update the enabled state of the sub-cells:
    [self setEnabled:[self isEnabled]];
    
    [self updateStepper];
}

- (void)updateStepper;
{
    [self.stepperCell setMinValue:[self currentMinimum]];
    [self.stepperCell setMaxValue:[self currentMaximum]];
    [self.stepperCell setIntegerValue:[self currentValue]];
    [self.stepperCell setEnabled:[self isEnabled]];
}

- (void)updateIncludedUnitsMapping;
{
    self.includedUnitsMapping = [NSMutableArray array];
    
    if (self.includeSeconds)
        [self.includedUnitsMapping addObject:[NSNumber numberWithInteger:DSIntervalSeconds]];
    
    if (self.includeMinutes)
        [self.includedUnitsMapping addObject:[NSNumber numberWithInteger:DSIntervalMinutes]];
    
    if (self.includeHours)
        [self.includedUnitsMapping addObject:[NSNumber numberWithInteger:DSIntervalHours]];
    
    if (self.includeDays)
        [self.includedUnitsMapping addObject:[NSNumber numberWithInteger:DSIntervalDays]];
    
    if (self.includeWeeks)
        [self.includedUnitsMapping addObject:[NSNumber numberWithInteger:DSIntervalWeeks]];
    
    if (self.includeMonths)
        [self.includedUnitsMapping addObject:[NSNumber numberWithInteger:DSIntervalMonths]];
    
    if (self.includeQuarters)
        [self.includedUnitsMapping addObject:[NSNumber numberWithInteger:DSIntervalQuarters]];
    
    if (self.includeYears)
        [self.includedUnitsMapping addObject:[NSNumber numberWithInteger:DSIntervalYears]];
    
    if (self.includeForever)
        [self.includedUnitsMapping addObject:[NSNumber numberWithInteger:DSIntervalForever]];
    
    if (self.includeNever)
        [self.includedUnitsMapping addObject:[NSNumber numberWithInteger:DSIntervalNever]];
    
    // Convert the units to the mapping index and back, to ensure it is a valid option:
    self.units = [self unitsForIncludedMappingIndex:[self includedMappingIndexForUnits:self.units]];
}

- (NSSize)cellSize;
{
    return NSMakeSize(self.secondAmountFrame.size.width + self.unitsFrame.size.width, self.unitsFrame.size.width + 4);
}

- (void)calcDrawInfo:(NSRect)rect;
{
    [self setupCellFrames];
    [self updateCells];
}

- (void)awakeFromNib;
{
    self.selectedCell = self.leftmostCell;
	
    [self updateStepper];
}

- (NSWindow *)window;
{
    return [[self controlView] window];
}


// ----------------------------------------------------------------------------------------
#pragma mark -
#pragma mark ACCESSOR METHODS
// ----------------------------------------------------------------------------------------

/*
 objectValue
 
 Returns the DSInterval object using the standard accessor.  Provided as a convenience.
 
 Written by DJS 2008-07.
*/

- (id)objectValue;
{
    return self.interval;
}

/*
 setObjectValue:
 
 Sets the DSInterval object using the standard accessor.  Provided as a convenience.  Only sets the value if it is a DSInterval (or subclass).
 
 Written by DJS 2008-07.
*/

- (void)setObjectValue:(id <NSCopying>)object;
{
    if ([[(NSObject *)object class] isKindOfClass:[DSInterval class]])
        self.interval = (DSInterval *)object;
}

/*
 stringValue
 
 Returns a string representation of the DSInterval value.  Provided as a convenience.  Note that it isn't possible to set the value via a string.
 
 Written by DJS 2008-07.
*/

- (NSString *)stringValue;
{
    return [self.interval amountWithFullUnitsName];
}

/*
 setStringValue:
 
 This is not currently supported for DSIntervalPickerCell.  Invoking this just logs a warning in the Console.
 
 Written by DJS 2008-09.
*/

- (void)setStringValue:(NSString *)string;
{
    NSLog(@"-setStringValue: is not supported by DSIntervalPickerCell (tried to set to %@)", string);
}

- (BOOL)usingRange;
{
    return self.interval.usingRange;
}

- (void)setUsingRange:(BOOL)usingRange;
{
    self.interval.usingRange = usingRange;
}

- (CGFloat)firstAmount;
{
    return self.interval.firstAmount;
}

- (void)setFirstAmount:(CGFloat)firstAmount;
{
    self.interval.firstAmount = firstAmount;
}

- (CGFloat)secondAmount;
{
    return self.interval.secondAmount;
}

- (void)setSecondAmount:(CGFloat)secondAmount;
{
    self.interval.secondAmount = secondAmount;
}

- (CGFloat)amount;
{
    return self.interval.amount;
}

- (void)setAmount:(CGFloat)amount;
{
    self.interval.amount = amount;
}

- (DSIntervalUnits)units;
{
    return self.interval.units;
}

- (void)setUnits:(DSIntervalUnits)units;
{
    self.interval.units = units;
}

/*
 firstTimeInterval
 
 Returns the first amount of the receiver represented as a time interval, i.e. in seconds or fractions thereof.
 
 Written by DJS 2008-07.
*/

- (NSTimeInterval)firstTimeInterval;
{
    return self.interval.firstTimeInterval;
}

/*
 secondTimeInterval
 
 Returns the second amount of the receiver represented as a time interval, i.e. in seconds or fractions thereof.
 
 Written by DJS 2008-07.
*/

- (NSTimeInterval)secondTimeInterval;
{
    return self.interval.secondTimeInterval;
}

/*
 amountTimeInterval
 
 Returns the receiver represented as a time interval, i.e. in seconds or fractions thereof.
 
 Written by DJS 2008-07.
*/

- (NSTimeInterval)amountTimeInterval;
{
    return self.interval.amountTimeInterval;
}

- (DSIntervalUnits)unitsForIncludedMappingIndex:(NSUInteger)mappingIndex;
{
    if (mappingIndex >= 0 && mappingIndex < [self.includedUnitsMapping count])
        return [[self.includedUnitsMapping objectAtIndex:mappingIndex] integerValue];
    else
        return [[self.includedUnitsMapping objectAtIndex:0] integerValue];
}

- (NSUInteger)includedMappingIndexForUnits:(DSIntervalUnits)units;
{
    NSUInteger mappingIndex = 0;
    DSIntervalUnits mappedUnits = DSIntervalNever;
    BOOL found = NO;
    
    while (!found && mappingIndex < [self.includedUnitsMapping count])
    {
        mappedUnits = [[self.includedUnitsMapping objectAtIndex:mappingIndex] integerValue];
        found = (mappedUnits == units);
        
        if (!found)
            mappingIndex++;
    }
    
    if (!found)
        return 0;
    else
        return mappingIndex;
}

- (NSText *)currentEditor;
{
    return [(NSControl *)[self controlView] currentEditor];
}

- (CGFloat)currentMinimum;
{
    if (self.selectedCell == self.unitsCell)
        return 0;
    
    if (!self.usingRange || self.rangeFiltering == DSIntervalAnyRange)
        return self.minimumAmount;
    
    CGFloat minimum = self.minimumAmount;
    BOOL isFirstAmountSelected = (self.selectedCell == self.firstAmountCell);
    
    if (isFirstAmountSelected && self.rangeFiltering == DSIntervalFirstGreaterThanSecond)
        minimum = self.secondAmount + 1;
    else if (isFirstAmountSelected && self.rangeFiltering == DSIntervalFirstGreaterOrEqualToSecond)
        minimum = self.secondAmount;
    else if (!isFirstAmountSelected && self.rangeFiltering == DSIntervalFirstLessThanSecond)
        minimum = self.firstAmount + 1;
    else if (!isFirstAmountSelected && self.rangeFiltering == DSIntervalFirstLessOrEqualToSecond)
        minimum = self.firstAmount;
    
    return minimum;
}

- (CGFloat)currentMaximum;
{
    if (self.selectedCell == self.unitsCell)
        return [self.includedUnitsMapping count] - 1;
    
    if (!self.usingRange || self.rangeFiltering == DSIntervalAnyRange)
        return self.maximumAmount;
    
    CGFloat maximum = self.maximumAmount;
    BOOL isFirstAmountSelected = (self.selectedCell == self.firstAmountCell);
    
    if (isFirstAmountSelected && self.rangeFiltering == DSIntervalFirstLessThanSecond)
        maximum = self.secondAmount - 1;
    else if (isFirstAmountSelected && self.rangeFiltering == DSIntervalFirstLessOrEqualToSecond)
        maximum = self.secondAmount;
    else if (!isFirstAmountSelected && self.rangeFiltering == DSIntervalFirstGreaterThanSecond)
        maximum = self.firstAmount - 1;
    else if (!isFirstAmountSelected && self.rangeFiltering == DSIntervalFirstGreaterOrEqualToSecond)
        maximum = self.firstAmount;
    
    return maximum;
}

- (CGFloat)currentValue;
{
    if (self.selectedCell == self.firstAmountCell)
        return self.firstAmount;
    else if (self.selectedCell == self.secondAmountCell)
        return self.secondAmount;
    else
        return [self includedMappingIndexForUnits:self.units];
}

- (void)setCurrentValue:(CGFloat)value;
{
    if (self.selectedCell == self.firstAmountCell)
        self.firstAmount = value;
    else if (self.selectedCell == self.secondAmountCell)
        self.secondAmount = value;
    else
        self.units = [self unitsForIncludedMappingIndex:value];
    
    [self updateCells];
}

- (NSTextFieldCell *)leftmostCell;
{
    if ([self isForeverOrNever])
        return self.unitsCell;
    else if (self.usingRange)
        return self.firstAmountCell;
    else
        return self.secondAmountCell;
}

- (BOOL)isForeverOrNever;
{
    return self.units == DSIntervalForever || self.units == DSIntervalNever;
}


// ----------------------------------------------------------------------------------------
#pragma mark -
#pragma mark BINDINGS METHODS
// ----------------------------------------------------------------------------------------

- (NSArray *)observedKeyPaths;
{
    return [NSArray arrayWithObjects:@"selectedCell", @"usingRange", @"firstAmount", @"secondAmount", @"amount", @"units", @"timeInterval", @"includeNever", @"includeSeconds", @"includeMinutes", @"includeHours", @"includeDays", @"includeWeeks", @"includeMonths", @"includeQuarters", @"includeYears", @"includeForever", @"minimumAmount", @"maximumAmount", nil];
}

- (void)setupObservers;
{
    for (NSString *keyPath in [self observedKeyPaths])
    {
        [self addObserver:self
               forKeyPath:keyPath
                  options:(NSKeyValueObservingOptionNew | NSKeyValueObservingOptionOld)
                  context:NULL];
    }
}

- (Class)valueClassForBinding:(NSString *)bindingName;
{
    return [NSNumber class];
}

- (void)observeValueForKeyPath:(NSString *)keyPath
                      ofObject:(id)object
                        change:(NSDictionary *)change
                       context:(void *)context;
{
    if ([keyPath isEqualToString:@"selectedCell"])
    {
        [self updateStepper];
        [[self controlView] setNeedsDisplay:YES];
    }
    else if ([keyPath isEqualToString:@"usingRange"] || [keyPath isEqualToString:@"minimumAmount"] || [keyPath isEqualToString:@"maximumAmount"])
    {
        [self setupCellFrames];
        [self updateCells];
        [[self controlView] setNeedsDisplay:YES];
    }
    else
    {
        // Avoid infinite recursion, since -updateIncludedUnitsMapping sets the units property:
        if (![keyPath isEqualToString:@"units"])
            [self updateIncludedUnitsMapping];
        
        [self updateCells];
        [[self controlView] setNeedsDisplay:YES];
    }
}


// ----------------------------------------------------------------------------------------
#pragma mark -
#pragma mark DRAWING METHODS
// ----------------------------------------------------------------------------------------

- (BOOL)isOpaque;
{
    return NO;
}

- (NSRect)drawBorder:(NSRect)frame;
{
    frame = NSInsetRect(frame, 2.0, 2.0);
    frame.size.width -= [self.stepperCell cellSize].width - 3.0;
    
    if ([self controlSize] == NSMiniControlSize)
        frame.size.width -= 1.0;
    else if ([self controlSize] == NSSmallControlSize)
        frame.size.width -= 2.0;
    
    if (self.isBordered)
    {
        NSRectEdge sides[] = {NSMaxYEdge, NSMaxXEdge, NSMinXEdge, NSMinYEdge, NSMaxYEdge};
        CGFloat enabledGrays[] = {0.5, 0.8, 0.8, 0.9, 0.9};
        CGFloat disabledGrays[] = {0.8, 0.9, 0.9, 0.95, 0.9};
        
        if ([self isEnabled])
            frame = NSDrawTiledRects(frame, frame, sides, enabledGrays, 5);
        else
            frame = NSDrawTiledRects(frame, frame, sides, disabledGrays, 5);
        
        [[NSColor whiteColor] set];
        NSRectFill(frame);
    }
    
    return frame;
}

- (void)drawAmountAndUnitsCellsInView:(NSView *)controlView;
{
    if (![self isForeverOrNever])
    {
        if (self.usingRange)
        {
            if (NSMaxX(self.firstAmountFrame) < NSMinX(self.stepperFrame))
                [self.firstAmountCell drawWithFrame:self.firstAmountFrame inView:controlView];
            
            if (NSMaxX(self.rangeSeparatorFrame) < NSMinX(self.stepperFrame))
                [self.rangeSeparatorCell drawWithFrame:self.rangeSeparatorFrame inView:controlView];
        }
        
        if (NSMaxX(self.secondAmountFrame) < NSMinX(self.stepperFrame))
            [self.secondAmountCell drawWithFrame:self.secondAmountFrame inView:controlView];
    }
    
    if (NSMinX(self.unitsFrame) < NSMinX(self.stepperFrame))
        [self.unitsCell drawWithFrame:self.unitsFrame inView:controlView];
}

- (void)drawStepperCellInView:(NSView *)controlView;
{
    [self.stepperCell drawWithFrame:self.stepperFrame inView:controlView];
}

- (void)drawFocusRing:(NSRect)cellFrame;
{
    if ([self showsFirstResponder])
    {
        NSRect focusRingFrame = NSInsetRect(cellFrame, -1, -2);
        
        [NSGraphicsContext saveGraphicsState];
        NSSetFocusRingStyle(NSFocusRingOnly);
        [[NSBezierPath bezierPathWithRect:focusRingFrame] fill];
        [NSGraphicsContext restoreGraphicsState];
    }
}

- (void)drawWithFrame:(NSRect)cellFrame inView:(NSView *)controlView;
{
    if (!NSEqualRects(self.pickerFrame, cellFrame))
    {
        self.pickerFrame = cellFrame;
        [self setupCells];
    }
    
    NSRect contentFrame = [self drawBorder:cellFrame];
    
    [self drawAmountAndUnitsCellsInView:controlView];
    [self drawStepperCellInView:controlView];
    [self drawFocusRing:contentFrame];
}


// ----------------------------------------------------------------------------------------
#pragma mark -
#pragma mark EVENT METHODS
// ----------------------------------------------------------------------------------------

- (BOOL)trackMouse:(NSEvent *)theEvent withLocation:(NSPoint)locationInCellFrame forRightEdge:(CGFloat)rightEdge ofTextFieldCell:(NSTextFieldCell *)cell;
{
    if (locationInCellFrame.x > rightEdge)
        return NO;
    
    [self editCell:cell];
    
    return YES;
}

- (BOOL)trackMouse:(NSEvent *)theEvent withLocation:(NSPoint)locationInCellFrame inStepperWithParentFrame:(NSRect)parentFrame untilMouseUp:(BOOL)flag;
{
    if (!NSMouseInRect(locationInCellFrame, self.stepperFrame, [[self controlView] isFlipped]))
        return NO;
    
    self.autoPopMenu = NO;
    
    [self.stepperCell trackMouse:theEvent inRect:parentFrame ofView:[self controlView] untilMouseUp:flag];
    
    self.autoPopMenu = YES;
    self.shouldAppendEdit = NO;
    
    return YES;
}

- (BOOL)trackMouse:(NSEvent *)theEvent inRect:(NSRect)cellFrame ofView:(NSView *)controlView untilMouseUp:(BOOL)flag;
{
    if (![self isEnabled])
        return NO;
    
    [self makeFirstResponderIfNecessary];
    
    NSPoint locationInCellFrame = [controlView convertPoint:[theEvent locationInWindow] fromView:nil];
    
    if (self.usingRange && [self trackMouse:theEvent withLocation:locationInCellFrame forRightEdge:NSMaxX(self.firstAmountFrame) ofTextFieldCell:self.firstAmountCell])
        return YES;
    
    if ([self trackMouse:theEvent withLocation:locationInCellFrame forRightEdge:NSMaxX(self.secondAmountFrame) ofTextFieldCell:self.secondAmountCell])
        return YES;
    
    if ([self trackMouse:theEvent withLocation:locationInCellFrame forRightEdge:self.stepperFrame.origin.x ofTextFieldCell:self.unitsCell])
        return YES;
    
    if ([self trackMouse:theEvent withLocation:locationInCellFrame inStepperWithParentFrame:cellFrame untilMouseUp:flag])
        return YES;
    
    return NO;
}

- (void)editCell:(NSTextFieldCell *)cell;
{
    self.autoPopMenu = NO;
    
    if (self.selectedCell)
        [self.selectedCell endEditing:[self currentEditor]];
    
    if (!cell)
        cell = self.leftmostCell;
    
    NSText *fieldEditor = [[self window] fieldEditor:YES forObject:cell];
    NSInteger length = [[cell stringValue] length];
    NSRect frame;
    
    if (cell == self.firstAmountCell)
        frame = self.firstAmountFrame;
    else if (cell == self.secondAmountCell)
        frame = self.secondAmountFrame;
    else
        frame = self.unitsFrame;
    
    [cell selectWithFrame:frame
                   inView:[self controlView]
                   editor:fieldEditor
                 delegate:self
                    start:0
                   length:length];
    
    self.selectedCell = cell;
    self.shouldAppendEdit = NO;
    self.autoPopMenu = YES;
}

- (void)textDidEndEditing:(NSNotification *)note;
{
    self.autoPopMenu = NO;
    
    [self.selectedCell endEditing:[note object]];
    
    self.selectedCell = nil;
    
    NSMutableDictionary *dict = [[[note userInfo] mutableCopy] autorelease];
    
    [dict setObject:[note object] forKey:@"NSFieldEditor"];
    
    [[NSNotificationCenter defaultCenter] postNotificationName:NSControlTextDidEndEditingNotification
                                                        object:[self controlView]
                                                      userInfo:dict];
}

- (void)selectPreviousKeyView;
{
    [[self window] selectKeyViewPrecedingView:[self controlView]];
    
    if ([[self window] firstResponder] == [self window])
        [self editCell:self.unitsCell];
}

- (void)selectNextKeyView;
{
    [[self window] selectKeyViewFollowingView:[self controlView]];
    
    if ([[self window] firstResponder] == [self window])
        [self editCell:self.leftmostCell];
}

- (void)selectPreviousCellOrKeyView:(BOOL)canSelectKeyView;
{
    if (self.selectedCell == self.unitsCell && ![self isForeverOrNever])
        [self editCell:self.secondAmountCell];
    else if (self.selectedCell == self.secondAmountCell && self.usingRange && ![self isForeverOrNever])
        [self editCell:self.firstAmountCell];
    else if (canSelectKeyView)
        [self selectPreviousKeyView];
    else
        [self editCell:self.unitsCell];
}

- (void)selectNextCellOrKeyView:(BOOL)canSelectKeyView;
{
    if (self.selectedCell == self.firstAmountCell && ![self isForeverOrNever])
        [self editCell:self.secondAmountCell];
    else if (self.selectedCell == self.secondAmountCell)
        [self editCell:self.unitsCell];
    else if (canSelectKeyView)
        [self selectNextKeyView];
    else if (![self isForeverOrNever])
        [self editCell:self.leftmostCell];
    else
        [self editCell:self.unitsCell];
}

- (NSInteger)incrementAmount:(NSInteger)amount withStep:(NSInteger)step minimum:(NSInteger)minimum maximum:(NSInteger)maximum;
{
    amount += step;
    
    if (amount > maximum)
        amount = minimum + ((amount - maximum) - 1);
    
    return amount;
}

- (NSInteger)decrementAmount:(NSInteger)amount withStep:(NSInteger)step minimum:(NSInteger)minimum maximum:(NSInteger)maximum;
{
    amount -= step;
    
    if (amount < minimum)
        amount = maximum - ((minimum - amount) - 1);
    
    return amount;
}

- (void)incrementSelectedValueWithStep:(NSInteger)step;
{
    if (self.selectedCell == self.firstAmountCell)
        self.firstAmount = [self incrementAmount:self.firstAmount withStep:step minimum:[self currentMinimum] maximum:[self currentMaximum]];
    else if (self.selectedCell == self.secondAmountCell)
        self.secondAmount = [self incrementAmount:self.secondAmount withStep:step minimum:[self currentMinimum] maximum:[self currentMaximum]];
    else
    {
        NSUInteger mappingIndex = [self includedMappingIndexForUnits:self.units];
        
        mappingIndex = [self incrementAmount:mappingIndex withStep:step minimum:0 maximum:[self currentMaximum]];
        
        self.units = [self unitsForIncludedMappingIndex:mappingIndex];
    }
    
    self.shouldAppendEdit = NO;
    
    [self updateCells];
}

- (void)decrementSelectedValueWithStep:(NSInteger)step;
{
    if (self.selectedCell == self.firstAmountCell)
        self.firstAmount = [self decrementAmount:self.firstAmount withStep:step minimum:[self currentMinimum] maximum:[self currentMaximum]];
    else if (self.selectedCell == self.secondAmountCell)
        self.secondAmount = [self decrementAmount:self.secondAmount withStep:step minimum:[self currentMinimum] maximum:[self currentMaximum]];
    else
    {
        NSUInteger mappingIndex = [self includedMappingIndexForUnits:self.units];
        
        mappingIndex = [self decrementAmount:mappingIndex withStep:step minimum:0 maximum:[self currentMaximum]];
        
        self.units = [self unitsForIncludedMappingIndex:mappingIndex];
    }
    
    self.shouldAppendEdit = NO;
    
    [self updateCells];
}

- (void)chooseMinimumSelectedValue;
{
    if (self.selectedCell == self.firstAmountCell)
        self.firstAmount = [self currentMinimum];
    else if (self.selectedCell == self.secondAmountCell)
        self.secondAmount = [self currentMinimum];
    else
        self.units = [self unitsForIncludedMappingIndex:[self currentMinimum]];
    
    self.shouldAppendEdit = NO;
    
    [self updateCells];
}

- (void)chooseMaximumSelectedValue;
{
    if (self.selectedCell == self.firstAmountCell)
        self.firstAmount = [self currentMaximum];
    else if (self.selectedCell == self.secondAmountCell)
        self.secondAmount = [self currentMaximum];
    else
        self.units = [self unitsForIncludedMappingIndex:[self currentMaximum]];
    
    self.shouldAppendEdit = NO;
    
    [self updateCells];
}

- (void)showMenu;
{
    self.autoPopMenu = NO;
    
    NSMenu *menu = [self makeMenuForSelectedCell];
    
    // Convert the control frame origin to the window's coordinates (this change made without testing!):
    NSPoint location = [[self controlView] convertPoint:[[self controlView] frame].origin toView:[[self window] contentView]];
    NSPoint cellOrigin = self.secondAmountFrame.origin;
    
    if (self.usingRange)
        cellOrigin = self.firstAmountFrame.origin;
    
    location.x += cellOrigin.x;
    location.y += cellOrigin.y - 5;
    
    NSEvent *event = [NSEvent mouseEventWithType:NSLeftMouseDown location:location modifierFlags:0 timestamp:0.0 windowNumber:[[self window] windowNumber] context:nil eventNumber:0 clickCount:1 pressure:1.0];
    
    [NSMenu popUpContextMenu:menu withEvent:event forView:[self controlView] withFont:[NSFont labelFontOfSize:13]];
    
    self.autoPopMenu = YES;
}

- (BOOL)textView:(NSTextView *)textView doCommandBySelector:(SEL)selector;
{
    self.autoPopMenu = NO;
    
    if (selector == @selector(insertBacktab:))
        [self selectPreviousCellOrKeyView:YES];
    else if (selector == @selector(insertTab:))
        [self selectNextCellOrKeyView:YES];
    else if (selector == @selector(moveLeft:) || selector == @selector(moveLeftAndModifySelection:) || selector == @selector(moveWordLeft:) || selector == @selector(moveBackwardAndModifySelection:))
        [self selectPreviousCellOrKeyView:NO];
    else if (selector == @selector(moveRight:) || selector == @selector(moveRightAndModifySelection:) || selector == @selector(moveWordRight:) || selector == @selector(moveForwardAndModifySelection:) || selector == @selector(insertTabIgnoringFieldEditor:))
        [self selectNextCellOrKeyView:NO];
    else if (selector == @selector(selectPreviousKeyView:))
        [self selectPreviousKeyView];
    else if (selector == @selector(selectNextKeyView:))
        [self selectNextKeyView];
    else if (selector == @selector(moveUp:))
        [self incrementSelectedValueWithStep:1];
    else if (selector == @selector(moveDown:))
        [self decrementSelectedValueWithStep:1];
    else if (selector == @selector(moveUpAndModifySelection:) || selector == @selector(pageUp:) || selector == @selector(scrollPageUp:) || selector == @selector(moveBackward:))
        [self incrementSelectedValueWithStep:5];
    else if (selector == @selector(moveDownAndModifySelection:) || selector == @selector(pageDown:) || selector == @selector(scrollPageDown:) || selector == @selector(moveForward:))
        [self decrementSelectedValueWithStep:5];
    else if (selector == @selector(moveToBeginningOfDocument:) || selector == @selector(scrollToBeginningOfDocument:))
        [self chooseMinimumSelectedValue];
    else if (selector == @selector(moveToEndOfDocument:) || selector == @selector(scrollToEndOfDocument:))
        [self chooseMaximumSelectedValue];
    else if (selector == @selector(complete:))
        [self showMenu];
    else if (selector == @selector(deleteBackward:) || selector == @selector(insertNewline:) || selector == @selector(cancelOperation:))
        return NO;
    // Ignore anything else
    
    return YES;
}

- (void)textView:(NSTextView *)textView changeAmountWithReplacementString:(NSString *)replacementString;
{
    // If the replacement string contains any non-digit characters, reject it outright:
    if ([[replacementString stringByTrimmingCharactersInSet:[NSCharacterSet decimalDigitCharacterSet]] length])
        return;
    
    // We'll adjust the field editor string as desired, then change the text ourselves:
    NSString *editorString = [textView string];
    NSUInteger editorLength = [editorString length];
    
    if (![replacementString length])
    {
        // If deleting (i.e. no replacement string), and there's more than one character, delete the last character and append next time, otherwise overwrite next time:
        if (editorLength > 1)
        {
            editorString = [editorString substringToIndex:editorLength - 1];
            self.shouldAppendEdit = YES;
        }
        else
            self.shouldAppendEdit = NO;
    }
    else
    {
        // If the minimum is more than one, trim off the first digit and append to the end; otherwise append if we can, or overwrite:
        
        if ([self currentMinimum] > 1 && editorLength > 1 && editorLength == [[self stringWithIntegerValue:[self currentMaximum]] length])
        {
            editorString = [[editorString substringFromIndex:1] stringByAppendingString:replacementString];
            
            // But if the resulting value would be less than the minimum, replace the first digit with the minimum's first digit, so the result is valid:
            if ([editorString floatValue] < [self currentMinimum])
                editorString = [[[self stringWithIntegerValue:[self currentMinimum]] substringToIndex:1] stringByAppendingString:[editorString substringFromIndex:1]];
        }
        else if (self.shouldAppendEdit && editorLength < [[self stringWithIntegerValue:[self currentMaximum]] length])
            editorString = [editorString stringByAppendingString:replacementString];
        else
            editorString = [[replacementString copy] autorelease];
        
        // Always append if we can:
        self.shouldAppendEdit = YES;
    }
    
    // Validate the value range:
    CGFloat value = [editorString floatValue];
    
    if (value < [self currentMinimum])
        value = [self currentMinimum];
    
    if (value > [self currentMaximum])
        value = [self currentMaximum];
    
    // Keep our property up-to-date:
    if (self.selectedCell == self.firstAmountCell)
        self.firstAmount = value;
    else
        self.secondAmount = value;
    
    // Update all of the cells to synchronize the amount to the field editor, cell, and other cells:
    [self updateCells];
}

- (void)textView:(NSTextView *)textView changeUnitsWithReplacementString:(NSString *)replacementString;
{
    // Ignore deleting (i.e. no replacement string):
    if (![replacementString length])
        return;
    
    // We'll adjust the field editor string as desired, then change the text ourselves:
    NSString *editorString = [textView string];
    NSUInteger editorMaximum = [editorString length] - 1;
    DSInterval *interval = [DSInterval intervalWithAmount:0 units:DSIntervalSeconds];
    NSInteger mappingMaximum = [self currentMaximum];
    NSInteger characterIndex = 0;
    BOOL found = NO;
    
    // Scan through all of the available units strings, and see if the replacement string matches the beginning of any, at any character position:
    while (!found && characterIndex <= (editorMaximum))
    {
        NSString *tempString = [[editorString substringToIndex:characterIndex] stringByAppendingString:replacementString];
        NSInteger mappingIndex = 0;
        
        while (!found && mappingIndex <= mappingMaximum)
        {
            interval.units = [self unitsForIncludedMappingIndex:mappingIndex];
            found = [[[interval fullUnitsName] lowercaseString] hasPrefix:[tempString lowercaseString]];
            
            mappingIndex++;
        }
        
        characterIndex++;
    }
    
    if (!found)
        return;
    
    // If we found a match, use it:
    self.units = interval.units;
    
    // Update all of the cells to synchronize the units to the field editor and cell:
    [self updateCells];
}

- (BOOL)textView:(NSTextView *)textView shouldChangeTextInRange:(NSRange)affectedCharRange replacementString:(NSString *)replacementString;
{
    // Pop the menu if the spacebar is pressed:
    if ([replacementString isEqualToString:@" "])
    {
        [self showMenu];
        return NO;
    }
    
    self.autoPopMenu = NO;
    
    // Special case: allow decrementing and incrementing via - and + keys:
    if ([replacementString isEqualToString:@"-"] || [replacementString isEqualToString:@"_"])
        [self decrementSelectedValueWithStep:1];
    else if ([replacementString isEqualToString:@"+"] || [replacementString isEqualToString:@"="])
        [self incrementSelectedValueWithStep:1];
    
    if (self.selectedCell == self.unitsCell)
        [self textView:textView changeUnitsWithReplacementString:replacementString];
    else
        [self textView:textView changeAmountWithReplacementString:replacementString];
	
    self.autoPopMenu = YES;
    
    return NO;
}

- (NSRange)textView:(NSTextView *)textView willChangeSelectionFromCharacterRange:(NSRange)oldSelectedCharRange toCharacterRange:(NSRange)newSelectedCharRange;
{
    if (self.autoPopMenu)
        [NSTimer scheduledTimerWithTimeInterval:0.1 target:self selector:@selector(showMenu) userInfo:nil repeats:NO];
    
    self.autoPopMenu = NO;
    
    return NSMakeRange(0, [[textView string] length]);
}

- (BOOL)acceptsFirstResponder;
{
    return [self isEnabled];
}

- (BOOL)needsPanelToBecomeKey;
{
    return YES;
}

- (void)makeFirstResponderIfNecessary;
{
    if ([[self window] firstResponder] != [self controlView] && [[self window] firstResponder] != [self currentEditor])
    {
        [[self window] makeFirstResponder:[self controlView]];
        [self cellBecomeFirstResponderInDirection:[[self window] keyViewSelectionDirection]];
    }
}

- (void)cellBecomeFirstResponderInDirection:(NSSelectionDirection)selectionDirection;
{
    // Lazily set up the first time this is drawn:
    if (!self.unitsCell)
        [self setupCells];
    
    if (selectionDirection == NSSelectingNext)
        [self editCell:self.leftmostCell];
    else if (selectionDirection == NSSelectingPrevious)
        [self editCell:self.unitsCell];
    else
        [self editCell:self.selectedCell];
}

- (void)addItemToMenu:(NSMenu *)menu withTitle:(NSString *)title tag:(NSInteger)tag minimum:(NSInteger)minimum maximum:(NSInteger)maximum wantSeparator:(BOOL)wantSeparator;
{
    if (tag < minimum || tag > maximum)
        return;
    
    if (!title)
        title = [self stringWithIntegerValue:tag];
    
    if (wantSeparator && [menu numberOfItems])
        [menu addItem:[NSMenuItem separatorItem]];
    
    NSMenuItem *item = [[[NSMenuItem alloc] initWithTitle:title action:@selector(menuItemChosen:) keyEquivalent:@""] autorelease];
    NSDictionary *attributes = [NSDictionary dictionaryWithObject:[self.unitsCell font] forKey:NSFontAttributeName];
    
    [item setAttributedTitle:[[[NSAttributedString alloc] initWithString:title attributes:attributes] autorelease]];
    [item setTarget:self];
    [item setTag:tag];
    
    [menu addItem:item];
}

- (void)menuItemChosen:(id)sender;
{
    self.autoPopMenu = NO;
    self.currentValue = [sender tag];
    self.autoPopMenu = YES;
}

- (NSMenu *)makeMenuForSelectedCell;
{
    NSMenu *menu = [[[NSMenu alloc] init] autorelease];
    
    if (self.selectedCell == self.unitsCell)
    {
        DSInterval *interval = [DSInterval intervalWithAmount:0 units:DSIntervalSeconds];
        
        for (NSInteger mappingIndex = 0; mappingIndex <= [self currentMaximum]; mappingIndex++)
        {
            interval.units = [self unitsForIncludedMappingIndex:mappingIndex];
            
            BOOL wantSeparator = (interval.units == DSIntervalDays || interval.units == DSIntervalForever || interval.units == DSIntervalNever);
            
            [self addItemToMenu:menu withTitle:[interval fullUnitsName] tag:mappingIndex minimum:0 maximum:[self currentMaximum] wantSeparator:wantSeparator];
        }
    }
    else
    {
        NSInteger amounts[] = {0, 1, 2, 3, 4, 5, 10, 15, 20, 25, 30, 40, 50, 60, 90, 120};
        NSInteger amountsCount = sizeof(amounts) / sizeof(NSInteger);
        
        for (NSInteger amountsIndex = 0; amountsIndex < amountsCount; amountsIndex++)
            [self addItemToMenu:menu withTitle:nil tag:amounts[amountsIndex] minimum:[self currentMinimum] maximum:[self currentMaximum] wantSeparator:NO];
    }
    
    return menu;
}

@end

