//
//  DSAgentController.m
//  Shared classes
//
//  Created by David Sinclair on 2007-03-14.
//  Copyright 2007 Dejal Systems, LLC. All rights reserved.
//

#import "DSAgentController.h"
#import "DSDictionaryCategories.h"
#import "DSExceptionController.h"
#import "DSSharedDefines.h"


@implementation DSAgentController

+ (id)mainController;
{
    static id controller = nil;
    
    if (!controller)
    {
        controller = [NSApp delegate];
    }
    
    return controller;
}

- (id)init;
{
    self = [super init];
    
    [self setFullyStarted:NO];
    [self setQuitting:NO];
    [self setOwnerAgent:nil];
    [self setTimer:nil];
    
    [[NSUserDefaults standardUserDefaults] registerDefaults:[self defaultPreferences]];
    
    [self upgradePreferences];
    
    return self;
}

- (void)dealloc;
{
    [self setTimer:nil];
    [self setOwnerAgent:nil];
    
    [self debugWithFormat:@"dealloc"];
    
	[super dealloc];
}

- (void)awakeFromNib;
{
}

- (void)someAppTerminated:(NSNotification *)notification;
{
	NSDictionary *info = [notification userInfo];
	NSString *quitBundleIdentifier = [info objectForKey:@"NSApplicationBundleIdentifier"];
    
    [self debugWithFormat:@"%@ has quit", quitBundleIdentifier];
    
    // If our owner has quit, we may want to quit as well:
    if ([quitBundleIdentifier isEqualToString:[[self ownerAgent] bundleIdentifier]])
    {
        [self debugWithFormat:@"... it's our owner!"];
        
        if ([self shouldTerminateWithOwner])
            [NSApp terminate:self];
    }
    else
        [self someApplicationDidTerminateWithBundleIdentifier:quitBundleIdentifier];
}

- (void)debugWithFormat:(NSString *)format, ...;
{
    if ([self debugMode])
    {
        va_list args;
        
        va_start(args, format);
        NSLogv(format, args);
        va_end(args);
    }
}

/*
 fatalError:
 
 Logs a message and quits.  Useful for significant errors that can't be recovered from, like possible license hacking.
 
 Written by DJS 2007-03.
*/

+ (void)fatalError:(NSString *)message;
{
    NSLog(message);
    
    [NSApp terminate:self];
}

/*
 upgradePreferences
 
 If any of the shared preferences are out of date or need resetting (e.g. when adding a column to a table), do so here and increment the version number.  Subclasses should implement this, calling super, and use the key "PreferencesVersion" and do something similar for their own keys.  Subclasses should not override this; override -upgradePreferencesFromVersion: instead.
 
 Written by DJS 2007-03.
*/

- (void)upgradePreferences;
{
    /*      commented out until needed
    // Increment the new number when adding new lines below:
    int oldSharedPreferencesVersion = [PREFERENCES integerForKey:@"DSSharedPreferencesVersion"];
    int newSharedPreferencesVersion = 1;
    
    if (oldSharedPreferencesVersion < 1)
    [PREFERENCES restoreFactorySettingsForKey:xxx];
    
    if (newSharedPreferencesVersion > oldSharedPreferencesVersion)
    [PREFERENCES setInteger:newSharedPreferencesVersion forKey:@"DSSharedPreferencesVersion"];
    */
    
    // Upgrade the preferences specific to this app too:
    int oldSpecificPreferencesVersion = [[NSUserDefaults standardUserDefaults] integerForKey:@"DSSpecificPreferencesVersion"];
    int newSpecificPreferencesVersion = [self upgradePreferencesFromVersion:oldSpecificPreferencesVersion];
    
    if (newSpecificPreferencesVersion > oldSpecificPreferencesVersion)
        [[NSUserDefaults standardUserDefaults] setInteger:newSpecificPreferencesVersion forKey:@"DSSpecificPreferencesVersion"];
}


// ----------------------------------------------------------------------------------------
#pragma mark -
#pragma mark AGENT DELEGATE METHODS
// ----------------------------------------------------------------------------------------

/*
 agentDidLaunch:withRelease:
 
 Delegate method received when a remote agent sends this signature to us, typically when it is launched (which has the side effect of launching the receiver if it isn't already running).  The sender is the DSAgentManager of the agent that sent the message (e.g. the owner), and the release is the release number of the sender.  By default it sends back an -agent:wasWrongRelease: signature if the release doesn't match, and quits itself.  Subclasses may override (optionally calling super) if desired.
 
 Written by DJS 2007-03.
*/

- (void)agentDidLaunch:(DSAgentManager *)sender withRelease:(NSString *)release;
{
    [self debugWithFormat:@"agentDidLaunch received from %@ with release %@", sender, release];
    
    // If it's a different release, and we were already running prior to receiving this message, we probably want to get the sender to relaunch us (but don't want to do that if we were launched via this message, as then it must be the best available version):
    if (![release isEqualToString:PRODUCT_RELEASE] && ![sender hasLaunchArguments])
    {
        [self debugWithFormat:@"... but is a different release (we're %@)", PRODUCT_RELEASE];
        
        // Tell the owner it was the wrong release and terminate outselves if the owner is still running (if it isn't, it won't get the message and thus won't relaunch us, so might as well stick with what we've got):
        if ([[self ownerAgent] isRunning])
        {
            [[self ownerAgent] remotelyPerformAgentWasWrongRelease];
            [NSApp terminate:self];
        }
    }
}

/*
 agentRequestsTermination:
 
 Delegate method received when a remote agent sends this signature to us.  The sender is the DSAgentManager of the agent that sent the message (e.g. the owner).  Terminates the agent by default (via -agentCanTerminate and -agentCleanup), but subclasses may override this (optionally calling super) if desired (e.g. to make it dependent on the agent requesting it).
 
 Written by DJS 2007-03.
*/

- (void)agentRequestsTermination:(DSAgentManager *)sender;
{
    [self debugWithFormat:@"agentRequestsTermination received from %@", sender];
    
    [NSApp terminate:self];
}

/*
 agentJustCheckingIn:
 
 Delegate method received when a remote agent sends this signature to us, typically to do nothing but make sure we're still running.  The sender is the DSAgentManager of the agent that sent the message (e.g. the owner).  Does nothing by default, but subclasses may override (optionally calling super) if desired.
 
 Written by DJS 2007-03.
*/

- (void)agentJustCheckingIn:(DSAgentManager *)sender;
{
    [self debugWithFormat:@"agentJustCheckingIn received from %@", sender];
}


// ----------------------------------------------------------------------------------------
#pragma mark -
#pragma mark OVERRIDE METHODS
// ----------------------------------------------------------------------------------------

/*
 defaultPreferences
 
 Returns a mutable dictionary containing the default preference settings.  Subclasses should override this, call super, and add their own preference keys.
 
 Written by DJS 2007-03.
*/

- (NSMutableDictionary *)defaultPreferences;
{
    NSMutableDictionary *dict = [NSMutableDictionary dictionary];
    
//    [dict setObject:[NSNumber numberWithBool:CONFIRM_QUIT_DEFAULT] forKey:SHARED_CONFIRM_QUIT];
//    [dict setObject:[NSNumber numberWithBool:CONFIRM_PREFS_DEFAULT] forKey:SHARED_CONFIRM_RESTORE_PREFS];
    
    return dict;
}

/*
 upgradePreferencesFromVersion:
 
 If any of the preferences are out of date or need resetting, do so here.  The parameter is the version number from the preference file; if this is less than the current version number, do the upgrade.  Return the current version number.  Note: for things like tables, it is often better to just change the autosave name (e.g. by appending the app version number), so the old pref isn't used, but is retained for backward compatibility.
 
 Written by DJS 2007-03.
*/

- (int)upgradePreferencesFromVersion:(int)preferencesVersion;
{
    // Subclasses should override this, without calling super, to upgrade their preferences
    
    return 0;
}

/*
 agentWillSetupOwner
 
 Called just before setting up the owner agent.  Subclasses may override this, without calling super, to do any initial setup.
 
 Written by DJS 2007-03.
*/

- (void)agentWillSetupOwner;
{
    // Does nothing by default
}

/*
 agentDidSetupOwner
 
 Called just after setting up the owner agent.  Subclasses may override this, without calling super, to set up any additional agents, or other setup.
 
 Written by DJS 2007-03.
*/

- (void)agentDidSetupOwner;
{
    // Does nothing by default
}

/*
 agentDidFinishLaunching
 
 Called when the receiver is fully started.  Subclasses may override this, without calling super, to start the agent operations.
 
 Written by DJS 2007-03.
*/

- (void)agentDidFinishLaunching;
{
    // Does nothing by default
}

/*
 agentCanTerminate
 
 Returns YES if it's okay to stop the agent's operation.  Simply returns YES, but subclasses may override this, without calling super, to add more logic.  Note: returning NO will stop termination, so be careful to only do that when appropriate.
 
 Written by DJS 2007-03.
*/

- (BOOL)agentCanTerminate;
{
    return YES;
}

/*
 agentCleanup
 
 Called when the agent's operation is being stopped.  It should discontinue what it's doing.  Subclasses may override this, without calling super.
 
 Written by DJS 2007-03.
*/

- (void)agentCleanup;
{
    // Does nothing by default
}

/*
 ownerName
 
 Returns the application name of the the owner.  Subclasses should override this, without calling super, to set up the owner agent.  If they don't, or return nil, no owner is set.
 
 Written by DJS 2007-03.
*/

- (NSString *)ownerName;
{
    return nil;
}

/*
 ownerBundleIdentifier
 
 Returns the bundle identifier of the the owner.  Subclasses should override this, without calling super, to set up the owner agent.  If they don't, or return nil, no owner is set.
 
 Written by DJS 2007-03.
*/

- (NSString *)ownerBundleIdentifier;
{
    return nil;
}

/*
 ownerLocation
 
 Returns where the owner is relative to the receiver, in the bundle hierarchy: it's parent (at the enclosing level), sibling (at the same level), or child (within the receiver's bundle).  Assumes the owner is closely related like that.  By default this returns the parent level, as is normal for an agent.  Subclasses may override this, without calling super, if a different location is appropriate.
 
 Written by DJS 2007-03.
*/

- (DSAgentLocation)ownerLocation;
{
    return DSAgentIsParent;
}

/*
 shouldTerminateWithOwner
 
 Called when the owner has quit.  Returns YES if the agent should terminate when the owner is quit, or NO if it should remain running.  By default it will terminate.  The subclass may override this if it wants to remain running, and/or do special processing when the owner quits.
 
 Written by DJS 2007-03.
*/

- (BOOL)shouldTerminateWithOwner;
{
    return YES;
}

/*
 someApplicationDidTerminateWithBundleIdentifier:
 
 Called when an application other than the owner has quit, passing its bundle identifier.  The subclass may overide this to do any appropriate processing.  No need to call super.
 
 Written by DJS 2007-03.
*/

- (void)someApplicationDidTerminateWithBundleIdentifier:(NSString *)bundleIdentifier;
{
    // Does nothing by default
}


// ----------------------------------------------------------------------------------------
#pragma mark -
#pragma mark ACCESSOR METHODS
// ----------------------------------------------------------------------------------------

- (void)setMustQuitWithoutConfirmation;
{
    [self setFullyStarted:NO];
}

- (BOOL)fullyStarted;
{
    return _fullyStarted;
}

- (void)setFullyStarted:(BOOL)value;
{
    _fullyStarted = value;
}

- (BOOL)isQuitting;
{
    return _quitting;
}

- (void)setQuitting:(BOOL)value;
{
    _quitting = value;
}

- (BOOL)debugMode;
{
    return _debugMode;
}

- (void)setDebugMode:(BOOL)value;
{
    _debugMode = value;
}

- (DSAgentManager *)ownerAgent;
{
    return _ownerAgent;
}

- (void)setOwnerAgent:(DSAgentManager *)value;
{
    [value retain];
    [_ownerAgent autorelease];
    _ownerAgent = value;
}

/*
 dataFolderPath
 
 Returns the data folder path, which is guaranteed to exist.  It returns the path as it was when first accessed, even if since changed in the Advanced preferences.
 
 Written by DJS 2007-04.
*/

- (NSString *)dataFolderPath;
{
    static NSString *_dataFolderPath = nil;
    
    // Retain a copy of the data folder path when first accessed, in case it is changed in the prefs later:
    if (!_dataFolderPath)
        _dataFolderPath = [[[PREFERENCES objectForKey:SHARED_DATA_FOLDER_PATH] validatedDirectoryPath] copy];
    
    return _dataFolderPath;
}


// ----------------------------------------------------------------------------------------
#pragma mark -
#pragma mark CONVENIENCE ACCESSOR METHODS
// Provided for subclasses to use freely
// ----------------------------------------------------------------------------------------

- (NSTimer *)timer;
{
    return _timer;
}

- (void)setTimer:(NSTimer *)value;
{
    [value retain];
    [_timer invalidate];
    [_timer autorelease];
    _timer = value;
}

@end


// ----------------------------------------------------------------------------------------
#pragma mark -
// ----------------------------------------------------------------------------------------


@implementation DSAgentController (NSApplicationDelegate)

- (void)applicationWillFinishLaunching:(NSNotification *)aNotification;
{
    // Hook up my custom exception handler; done in will-finish rather than did-finish to better catch any startup exceptions.  Note that NSApp hasn't been (fully?) initialized yet:
    [DSExceptionController sharedExceptionController];
}

- (void)applicationDidFinishLaunching:(NSNotification *)notification;
{
    // DSDEBUG is defined in the OTHER_CFLAGS of the project for the Debug build configuration (set to -DDSDEBUG=1):
#if DSDEBUG
    [self setDebugMode:YES];
#else
    [self setDebugMode:NO];
#endif
	
    [self debugWithFormat:@"agent launched"];
    
    // Watch app quits, so we can quit ourselves if our owner quits, or do other processing:
    [[[NSWorkspace sharedWorkspace] notificationCenter] addObserver:self selector:@selector(someAppTerminated:) name:NSWorkspaceDidTerminateApplicationNotification object:nil];
	
    [self agentWillSetupOwner];
    
    NSString *name = [self ownerName];
    NSString *bundleIdentifier = [self ownerBundleIdentifier];
    
    if ([name length] && [bundleIdentifier length])
        [self setOwnerAgent:[DSAgentManager agentWithName:name bundleIdentifier:bundleIdentifier
                                         relativeLocation:[self ownerLocation] delegate:self]];
    
    [self agentDidSetupOwner];
    
    // Quit immediately if not started with appropriate parameters:
    if (![[[self ownerAgent] bundleIdentifier] length])
        [NSApp terminate:self];
    
    [self setFullyStarted:YES];
    
    [self agentDidFinishLaunching];
}

- (NSApplicationTerminateReply)applicationShouldTerminate:(NSApplication *)sender;
{
    [self setQuitting:YES];
    
    if ([self agentCanTerminate])
    {
        [self agentCleanup];
        [self debugWithFormat:@"terminating the agent"];
        return NSTerminateNow;
    }
    else
    {
        [self setQuitting:NO];
        return NSTerminateCancel;
    }
}

@end

