//
//  DSAgentManager.m
//  Shared classes
//
//  Created by David Sinclair on 2007-03-20.
//  Copyright 2007 Dejal Systems, LLC. All rights reserved.
//

#import "DSAgentManager.h"
#import "DSSharedDefines.h"


@interface DSAgentManager (Private)

- (BOOL)remotelyPerformSignature:(NSString *)signature withDictionary:(NSDictionary *)dict;
- (BOOL)launchWithSignature:(NSString *)signature dictionary:(NSDictionary *)dict;

- (void)informDelegateUsingSignature:(NSString *)signature withArgumentsFromDictionary:(id)dict;

- (void)processLaunchArguments;
- (void)processNotification:(NSNotification *)note;

- (NSString *)notificationNameForSending:(BOOL)sending;

@end


// ----------------------------------------------------------------------------------------
#pragma mark -
// ----------------------------------------------------------------------------------------


@implementation DSAgentManager

/*
 agentWithName:bundleIdentifier:relativeLocation:delegate:
 
 Convenience method to return a new DSAgentManager instance.
 
 Written by DJS 2007-03.
*/

+ (id)agentWithName:(NSString *)name bundleIdentifier:(NSString *)bundleIdentifier relativeLocation:(DSAgentLocation)relativeLocation delegate:(id)delegate;
{
    return [[[self alloc] initWithName:name bundleIdentifier:bundleIdentifier relativeLocation:relativeLocation delegate:delegate] autorelease];
}

/*
 initWithName:bundleIdentifier:relativeLocation:delegate:
 
 Designated initializer.
 
 Written by DJS 2007-03.
*/

- (id)initWithName:(NSString *)name bundleIdentifier:(NSString *)bundleIdentifier relativeLocation:(DSAgentLocation)relativeLocation delegate:(id)delegate;
{
    if ((self = [super init]))
    {
        [self setName:name];
        [self setBundleIdentifier:bundleIdentifier];
        [self setDelegate:delegate];
        [self setSingleInstance:YES];
        [self setShouldStopAgentWithParent:YES];
        [self setRelativeLocation:relativeLocation];
        
        // Register for notifications from another agent:
        [[NSDistributedNotificationCenter defaultCenter] addObserver:self
                                                            selector:@selector(processNotification:)
                                                                name:[self notificationNameForSending:NO]
                                                              object:nil];
        
        // If there are launch arguments for this agent, process them after the agent has fully started:
        if ([self hasLaunchArguments])
            [NSTimer scheduledTimerWithTimeInterval:0.1 target:self selector:@selector(processLaunchArguments) userInfo:nil repeats:NO];
    }
    
    return self;
}

- (void)dealloc;
{
    [[NSDistributedNotificationCenter defaultCenter] removeObserver:self];
    
    if ([self shouldStopAgentWithParent])
        [self remotelyPerformAgentRequestsTermination];
    
    [self setName:nil];
    [self setBundleIdentifier:nil];
    
	[super dealloc];
}

/*
 remotelyPerformSignature:
 
 Performs the selector on the remote agent's delegate, using a selector that accepts the agent and no other arguments (e.g. -agent:).  Note that the first parameter is always the agent.  If the agent isn't already running, it is launched.
 
 Written by DJS 2007-03.
*/

- (BOOL)remotelyPerformSignature:(NSString *)signature;
{
    return [self remotelyPerformSignature:signature withDictionary:nil];
}

/*
 remotelyPerformSignature:withObject:
 
 Performs the selector on the remote agent's delegate, using a selector that accepts the agent and one other object as arguments (e.g. -agent:withObject:).  Note that the first parameter is always the agent.  If the agent isn't already running, it is launched.  The object value must only be a string or number: no dictionaries, arrays, or custom objects are allowed (unless their -description method results in something meaningful).
 
 Written by DJS 2007-03.
*/

- (BOOL)remotelyPerformSignature:(NSString *)signature withObject:(id)object;
{
    return [self remotelyPerformSignature:signature withArguments:[NSArray arrayWithObject:object]];
}

/*
 remotelyPerformSignature:withObject:object:
 
 Performs the selector on the remote agent's delegate, using a selector that accepts the agent and two other objects as arguments (e.g. -agent:withObject:object:).  Note that the first parameter is always the agent.  If the agent isn't already running, it is launched.  The object values must only be strings or numbers: no dictionaries, arrays, or custom objects are allowed (unless their -description method results in something meaningful).
 
 Written by DJS 2007-03.
*/

- (BOOL)remotelyPerformSignature:(NSString *)signature withObject:(id)object1 object:(id)object2;
{
    return [self remotelyPerformSignature:signature withArguments:[NSArray arrayWithObjects:object1, object2, nil]];
}

/*
 remotelyPerformSignature:withArguments:
 
 Performs the selector on the remote agent's delegate, using a selector that accepts the agent and any number of arguments (e.g. -agent:withObject:object:object:object:).  Note that the first parameter is always the agent.  If the agent isn't already running, it is launched.  The array values must only be strings or numbers: no dictionaries, arrays, or custom objects are allowed (unless their -description method results in something meaningful).
 
 Written by DJS 2007-03.
*/

- (BOOL)remotelyPerformSignature:(NSString *)signature withArguments:(NSArray *)arguments;
{
    NSMutableDictionary *dict = [NSMutableDictionary dictionaryWithCapacity:[arguments count]];
    int index;
    
    for (index = 0; index < [arguments count]; index++)
    {
        [dict setObject:[[arguments objectAtIndex:index] description]
                 forKey:[NSString stringWithFormat:@"DSAgentManagerObject%d", index]];
    }
    
    return [self remotelyPerformSignature:signature withDictionary:dict];
}

/*
 remotelyPerformSignature:withDictionary:
 
 Performs the selector on the remote agent's delegate, using a selector that accepts the agent and a dictionary as arguments (e.g. -agent:withDictionary:).  Note that the first parameter is always the agent.  If the agent isn't already running, it is launched.  The dictionary keys must not contain spaces, and the values must only be strings or numbers: no sub-dictionaries, arrays, or custom objects are allowed (unless their -description method results in something meaningful).
 
 Written by DJS 2007-03.
*/

- (BOOL)remotelyPerformSignature:(NSString *)signature withDictionary:(NSDictionary *)dict;
{
    if ([self isRunning])
    {
        [[NSDistributedNotificationCenter defaultCenter] postNotificationName:[self notificationNameForSending:YES]
                                                                       object:signature
                                                                     userInfo:dict];
        return YES;
    }
    else
    {
        return [self launchWithSignature:signature dictionary:dict];
    }
}

- (void)_addKey:(NSString *)key value:(NSString *)value toArray:(NSMutableArray *)array;
{
    [array addObject:[@"-" stringByAppendingString:key]];
    [array addObject:value];
}

/*
 launchWithSignature:dictionary:
 
 Launch the agent with the specified dictionary values, passing them on the command line.  The dictionary keys must not contain spaces, and the values must only be strings or numbers: no sub-dictionaries or arrays are allowed.  Use -remotelyPerformSignature:::, above, instead to avoid launching another instance if it is already running (or call -stop then this).
 
 Written by DJS 2007-03.
*/

- (BOOL)launchWithSignature:(NSString *)signature dictionary:(NSDictionary *)dict;
{
    NSMutableArray *arguments = nil;
    BOOL okay = NO;
    
NS_DURING
    arguments = [NSMutableArray arrayWithCapacity:[dict count] + 2];
    
    [self _addKey:@"DSAgentManagerBundleIdentifier" value:[[NSBundle mainBundle] bundleIdentifier] toArray:arguments];
    [self _addKey:@"DSAgentManagerSignature" value:signature toArray:arguments];
    
    NSEnumerator *enumerator = [dict keyEnumerator];
    NSString *key;
    
    while ((key = [enumerator nextObject]))
    {
    	NSString *value = [[dict objectForKey:key] description];
        
        [self _addKey:key value:value toArray:arguments];
    }
    
    NSLog(@"launchWithSignature:dictionary: arguments: %@", arguments);         // log
    
    okay = ([NSTask launchedTaskWithLaunchPath:[self executablePath] arguments:arguments] != nil);
NS_HANDLER
    NSLog(@"Unable to launch the %@ agent (from %@), with arguments %@.", [self name], [self executablePath], arguments);         // log
NS_ENDHANDLER
    
    return okay;
}

/*
 _pathForProcessSerialNumber:
 
 Returns a path for the specified process.  Used by -isRunning, below.
 
 Written by DJS 2007-04, based on NDProcess by Nathan Day <http://homepage.mac.com/nathan_day/pages/source.xml>.
*/

- (NSString *)_pathForProcessSerialNumber:(ProcessSerialNumber)processSerialNumber;
{
    ProcessInfoRec infoRec;
    FSSpec theSpec;
    NSURL *url = nil;
    
    infoRec.processInfoLength = sizeof(ProcessInfoRec);
    infoRec.processName = NULL;
    infoRec.processAppSpec = &theSpec;
    
    if (GetProcessInformation(&processSerialNumber, &infoRec) == noErr && infoRec.processAppSpec != NULL)
    {
        FSRef theRef;
        
        if (FSpMakeFSRef(&theSpec, &theRef) == noErr)
            url = [(NSURL *)CFURLCreateFromFSRef(kCFAllocatorDefault, &theRef) autorelease];
        
        if (url)
            return [url path];
    }
    
    return nil;
}

/*
 processSerialNumber
 
 Returns the process serial number of the agent if is currently executing, or one with kNoProcess in the high long if it isn't.
 
 Written by DJS 2007-04.
 Changed by DJS 2007-12 to return the PSN.
*/

- (ProcessSerialNumber)processSerialNumber;
{
    ProcessSerialNumber currentProcessSerialNumber;
    currentProcessSerialNumber.highLongOfPSN = kNoProcess;
    currentProcessSerialNumber.lowLongOfPSN = 0;
    ProcessSerialNumber noProcessSerialNumber = currentProcessSerialNumber;
	
	while (GetNextProcess(&currentProcessSerialNumber) == noErr && !(currentProcessSerialNumber.highLongOfPSN == kNoProcess && currentProcessSerialNumber.lowLongOfPSN == 0))
	{
		NSString *path = [self _pathForProcessSerialNumber:currentProcessSerialNumber];
        path = [[[path stringByDeletingLastPathComponent] stringByDeletingLastPathComponent] stringByDeletingLastPathComponent];
        
        if ([[NSWorkspace sharedWorkspace] isFilePackageAtPath:path])
        {        
            NSBundle *bundle = [NSBundle bundleWithPath:path];
            NSString *identifier = [bundle bundleIdentifier];
            
            if ([identifier isEqualToString:[self bundleIdentifier]])
                return currentProcessSerialNumber;
        }
	}
    
    return noProcessSerialNumber;
}

/*
 isRunning
 
 Returns YES if the agent is currently executing, or NO if it isn't.
 
 Written by DJS 2007-04.
 Changed by DJS 2007-12 to use -processSerialNumber.
*/

- (BOOL)isRunning;
{
    return ([self processSerialNumber].highLongOfPSN != kNoProcess);
}

/*
 isRunning
 
 Returns YES if the agent is currently executing, or NO if it isn't.
 
 Written by DJS 2007-03.
*/

/*
- (BOOL)isRunning;
{
    NSArray *runningApps = [[NSWorkspace sharedWorkspace] launchedApplications];
    NSDictionary *app = [runningApps objectMatching:[self bundleIdentifier] usingKey:@"NSApplicationBundleIdentifier"];
    
    return (app != nil);
}
*/

/*
 stop
 
 Quits the agent, if it's running.  Does nothing if it isn't.  Normally you'd use the "agent requests termination" mechanism instead.
 
 Written by DJS 2007-12.
*/

- (void)stop;
{
    ProcessSerialNumber processSerialNumber = [self processSerialNumber];
    
    if (processSerialNumber.highLongOfPSN == kNoProcess)
        return;
    
    AppleEvent quitEvent = {typeNull, NULL};
    AEBuildError buildError;
    
    OSErr err = AEBuildAppleEvent(kCoreEventClass, kAEQuitApplication,
                                  typeProcessSerialNumber, processSerialNumber, sizeof(ProcessSerialNumber),
                                  kAutoGenerateReturnID, kAnyTransactionID, &quitEvent, &buildError, "");
    
    if (err == noErr)
    {
        AppleEvent reply = {};
        err = AESend(&quitEvent, &reply, kAENoReply | kAENeverInteract, kAENormalPriority, kNoTimeOut, NULL, NULL);
        (void)AEDisposeDesc(&quitEvent);
    }
}


// ----------------------------------------------------------------------------------------
#pragma mark -
#pragma mark REMOTELY PERFORM CONVENIENCE METHODS
// ----------------------------------------------------------------------------------------

/*
 remotelyPerformAgentDidLaunchWithRelease
 
 Convenience method to perform -agentDidLaunch:withRelease: on the remote agent's delegate, to tell it that we've launched.  If the agent isn't already running, it is launched.
 
 Written by DJS 2007-03.
*/

- (BOOL)remotelyPerformAgentDidLaunchWithRelease;
{
    return [self remotelyPerformSignature:@"agentDidLaunch:withRelease:" withObject:PRODUCT_RELEASE];
}

/*
 remotelyPerformAgentWasWrongRelease
 
 Convenience method to perform -agent:wasWrongRelease: on the remote agent's delegate, to tell it they're a different version than us.  Does nothing if it isn't already running.
 
 Written by DJS 2007-03.
*/

- (BOOL)remotelyPerformAgentWasWrongRelease;
{
    if (![self isRunning])
        return NO;
    
    return [self remotelyPerformSignature:@"agent:wasWrongRelease:" withObject:PRODUCT_RELEASE];
}

/*
 remotelyPerformAgentRequestsTermination
 
 Convenience method to perform -agentRequestsTermination: on the remote agent's delegate, to ask it to stop what it's doing and quit.  Does nothing if it isn't already running.
 
 Written by DJS 2007-03.
*/

- (BOOL)remotelyPerformAgentRequestsTermination;
{
    if (![self isRunning])
        return NO;
    
    return [self remotelyPerformSignature:@"agentRequestsTermination:"];
}

/*
 remotelyPerformAgentJustCheckingIn
 
 Convenience method to perform -agentJustCheckingIn: on the remote agent's delegate, to just say hi (and ensure it's running).  If the agent isn't already running, it is launched.
 
 Written by DJS 2007-03.
*/

- (BOOL)remotelyPerformAgentJustCheckingIn;
{
    return [self remotelyPerformSignature:@"agentJustCheckingIn:"];
}

/*
 remotelyPerformAgentDidChangeUpdatesPreferences
 
 Convenience method to perform -agentDidChangeUpdatesPreferences: on the remote agent's delegate, to let it know to reload the user-settable Updates preferences from the sender.  Typically sent from Main to Status.
 
 Written by DJS 2007-03.
*/

- (BOOL)remotelyPerformAgentDidChangeUpdatesPreferences;
{
    return [self remotelyPerformSignature:@"agentDidChangeUpdatesPreferences:"];
}

/*
 remotelyPerformAgentCheckForUpdates:runModal:
 
 Convenience method to perform -agent:didCheckForUpdates:runModal: on the remote agent's delegate, to perform a check for updates with the specified mode and modality.  Should normally call -[MainController checkForUpdates:runModally:] instead; it'll either do it locally or remotely as appropriate.
 
 Written by DJS 2007-03.
*/

- (BOOL)remotelyPerformAgentCheckForUpdates:(int)checkingMode runModal:(BOOL)runModal;
{
    return [self remotelyPerformSignature:@"agent:checkForUpdates:runModal:" withArguments:[NSArray arrayWithObjects:[NSNumber numberWithInt:checkingMode], [NSNumber numberWithBool:runModal], nil]];
}


// ----------------------------------------------------------------------------------------
#pragma mark -
#pragma mark PRIVATE INCOMING METHODS
// ----------------------------------------------------------------------------------------

/*
 informDelegateUsingSignature:withArgumentsFromDictionary:
 
 Converts the arguments from within the dictionary into an array, and then informs the delegate with them.  The dict is an id as it may be user defaults instead of a dictionary.  Only the keys with the DSAgentManagerObject prefix are used.
 
 Written by DJS 2007-03.
*/

- (void)informDelegateUsingSignature:(NSString *)signature withArgumentsFromDictionary:(id)dict;
{
    NSMutableArray *arguments = [NSMutableArray array];
    id object = nil;
    int index = 0;
    
    while ((object = [dict objectForKey:[NSString stringWithFormat:@"DSAgentManagerObject%d", index++]]))
    {
        [arguments addObject:object];
    }
    
    [self informDelegateUsingSignature:signature withArguments:arguments];
}

/*
 processLaunchArguments
 
 A timer handler, invoked after the agent has started, if the agent was launched with DSAgentManager arguments for the receiver.  It performs the selector on the delegate.
 
 Written by DJS 2007-03.
*/

- (void)processLaunchArguments;
{
    NSUserDefaults *defaults = [NSUserDefaults standardUserDefaults];
    NSString *signature = [defaults stringForKey:@"DSAgentManagerSignature"];
    
    [self informDelegateUsingSignature:signature withArgumentsFromDictionary:defaults];
}

/*
 processNotification:
 
 Handles a notification from an agent by performing the selector on the delegate.
 
 Written by DJS 2007-03.
*/

- (void)processNotification:(NSNotification *)note;
{
    NSString *signature = [note object];
    NSDictionary *dict = [note userInfo];
    
    [self informDelegateUsingSignature:signature withArgumentsFromDictionary:dict];
}


// ----------------------------------------------------------------------------------------
#pragma mark -
#pragma mark ACCESSOR METHODS
// ----------------------------------------------------------------------------------------

- (NSString *)description;
{
    return [self notificationNameForSending:NO];
}

- (NSString *)name;
{
    return _name;
}

- (void)setName:(NSString *)value;
{
    [value retain];
    [_name autorelease];
    _name = value;
}

- (NSString *)bundleIdentifier;
{
    return _bundleIdentifier;
}

- (void)setBundleIdentifier:(NSString *)value;
{
    [value retain];
    [_bundleIdentifier autorelease];
    _bundleIdentifier = value;
}

- (NSString *)notificationNameForSending:(BOOL)sending;
{
    NSString *localBundleIdentifier = [[NSBundle mainBundle] bundleIdentifier];
    NSString *remoteBundleIdentifier = [self bundleIdentifier];
    
    if (sending)
        return [NSString stringWithFormat:@"DSAgentManager from %@ to %@", localBundleIdentifier, remoteBundleIdentifier];
    else
        return [NSString stringWithFormat:@"DSAgentManager from %@ to %@", remoteBundleIdentifier, localBundleIdentifier];
}

/*
 hasLaunchArguments
 
 Returns YES if the receiver was launched with a delegate message, that will be received once fully started.  A client may call this to determine whether to do some default action on startup, or wait for the delegate message.  (Note that if there are multiple agents, any one may return YES for this, and others won't.)
 
 Written by DJS 2007-03.
*/

- (BOOL)hasLaunchArguments;
{
    NSString *bundleIdentifier = [[NSUserDefaults standardUserDefaults] stringForKey:@"DSAgentManagerBundleIdentifier"];
    BOOL isMatch = [bundleIdentifier isEqualToString:[self bundleIdentifier]];
    
    NSLog(@"hasLaunchArguments: %@ = %@: %d", bundleIdentifier, [self bundleIdentifier], isMatch);         // log
    
    return isMatch;
}

- (BOOL)isSingleInstance;
{
    return _isSingleInstance;
}

- (void)setSingleInstance:(BOOL)value;
{
    _isSingleInstance = value;
}

- (BOOL)shouldStopAgentWithParent;
{
    return _shouldStopAgentWithParent;
}

- (void)setShouldStopAgentWithParent:(BOOL)value;
{
    _shouldStopAgentWithParent = value;
}

- (DSAgentLocation)relativeLocation;
{
    return _relativeLocation;
}

- (void)setRelativeLocation:(DSAgentLocation)value;
{
    _relativeLocation = value;
}

- (NSString *)executablePath;
{
    NSString *nameWithExtension = [[self name] stringByAppendingPathExtension:@"app"];
    NSBundle *mainBundle = [NSBundle mainBundle];
    NSString *agentBundlePath = nil;
    
    if ([self relativeLocation] == DSAgentIsParent)
    {
        // Get our path, remove our app name, then go up two levels (i.e. remove Contents/MacOS):
        agentBundlePath = [[mainBundle bundlePath] stringByDeletingLastPathComponent];
        agentBundlePath = [[agentBundlePath stringByDeletingLastPathComponent] stringByDeletingLastPathComponent];
        
        NSLog(@"executablePath of parent: %@", agentBundlePath);         // log
    }
    else if ([self relativeLocation] == DSAgentIsSibling)
    {
        // Get our path, remove our app name, then add the agent's name:
        agentBundlePath = [[mainBundle bundlePath] stringByDeletingLastPathComponent];
        agentBundlePath = [agentBundlePath stringByAppendingPathComponent:nameWithExtension];
        
        NSLog(@"executablePath of sibling: %@", agentBundlePath);         // log
    }
    else
        agentBundlePath = [mainBundle pathForAuxiliaryExecutable:nameWithExtension];
    
    agentBundlePath = [agentBundlePath stringByResolvingSymlinksInPath];
    
    NSBundle *agentBundle = [NSBundle bundleWithPath:agentBundlePath];
    
    return [agentBundle executablePath];
}

@end

