//
//  DSStatusAgentController.m
//  Shared classes
//
//  Created by David Sinclair on 2007-03-14.
//  Copyright 2007 Dejal Systems, LLC. All rights reserved.
//


#import "DSStatusAgentController.h"
#import "MainDefines.h"
#import "DSSharedPrefDefines.h"
#import "DSLicenseController.h"
#import "DSUpdateCheckController.h"
#import "DSStringCategories.h"


@implementation DSStatusAgentController

/*
 upgradePreferences
 
 If any of the shared preferences are out of date or need resetting (e.g. when adding a column to a table), do so here and increment the version number.  Subclasses should implement this, calling super, and use the key "PreferencesVersion" and do something similar for their own keys.  Subclasses should not override this; override -upgradePreferencesFromVersion: instead.
 
 Written by DJS 2007-03.
*/

- (void)upgradePreferences;
{
    [super upgradePreferences];
    
    [[self updateController] setupDefaultPrefs];
}

/*
 licenseAssistantDidFinish
 
 This is called from DSLicenseController after the license assistant is finished, or if it isn't needed.  Until this is called, we're still starting up.  Note that this can also be called after starting, e.g. via a random license reminder or manually-initiated setup assistant.
 
 Written by DJS 2007-03.
*/

- (void)licenseAssistantDidFinish;
{
    [[NSNotificationCenter defaultCenter] postNotificationName:@"DSLicenseAssistantDidClose" object:self];
    
    if ([self fullyStarted])
        return;
    
    // Finish starting up:
    [super applicationDidFinishLaunching:nil];
}

/*
 licenseAssistantWasPrematurelyClosed
 
 This is called from DSLicenseController if the license assistant is closed before completing the essential interns (i.e. the license agreement).  If this happens, the app quits if it wasn't yet fully started.
 
 Written by DJS 2007-03.
*/

- (void)licenseAssistantWasPrematurelyClosed;
{
    [[NSNotificationCenter defaultCenter] postNotificationName:@"DSLicenseAssistantDidClose" object:self];
    
    if ([self fullyStarted])
        return;
    
    [NSApp terminate:nil];
}

- (NSString *)productNumberString;
{
    return PRODUCT_NUMBER_STRING;
}

- (NSString *)productSpecificStats;
{
    // Note: don't use the "&", "@", "/", ":", "=" symbols, space characters, or anything else invalid in URLs
    
    // Override this in the MainController subclass to provide app-specific stats
    
    return @"--";
}

/*
 makeKind:price:kagiProduct:kagiPart:
 
 Constructs and returns a dictionary with information for the Kagi KRM licensing feature.  Note that the kagiPart value is optional; it works fine without it, but is available in case that changes in the future.
 
 Written by DJS 2007-03.
*/

- (NSDictionary *)makeKind:(int)kind price:(NSString *)price
               kagiProduct:(NSString *)kagiProduct kagiPart:(NSString *)kagiPart;
{
    if (!kagiPart)
        kagiPart = @"";
    
    return [NSDictionary dictionaryWithObjectsAndKeys:
        [NSNumber numberWithInt:kind], @"Kind",
        price, @"Price",
        kagiProduct, @"KagiProduct",
        kagiPart, @"KagiPart",
        nil];
}

/*
 availableLicensesArray
 
 Returns an array of dictionaries with information on the available licenses for the licensing features.
 
 Written by DJS 2007-03.
*/

- (NSArray *)availableLicensesArray;
{
    NSAssert(NO, @"Forgot to override -availableLicensesArray in the main controller.");
    
    return nil;
}

/*
 readyForLicenseAssistant
 
 Returns YES if it's okay to display the license assistant for a reminder, or no if it should be skipped this time.  Primarily for use in agents to avoid displaying a reminder if the main app is running, since it would do it.  Returns YES if the owner isn't running, or NO if it is; subclasses may override if desired.
 
 Written by DJS 2007-03.
*/

- (BOOL)readyForLicenseAssistant;
{
    return (![[self ownerAgent] isRunning]);
}

/*
 updateController
 
 Returns the controller instance to use for updates.  By default returns the DSUpdateCheckController, but subclasses may override.
 
 Written by DJS 2007-03.
*/

- (id)updateController;
{
    return [DSUpdateCheckController sharedUpdateController];
}


// ----------------------------------------------------------------------------------------
#pragma mark -
#pragma mark AGENT DELEGATE METHODS
// ----------------------------------------------------------------------------------------

/*
 agentDidChangeUpdatesPreferences:
 
 Delegate method received when a remote agent sends this signature to us.  The sender is the DSAgentManager of the agent that sent the message.  Received when the Main app changes Updates preferences, to syncronize the user-settable prefs to the Status agent's prefs.
 
 Written by DJS 2007-03.
*/

- (void)agentDidChangeUpdatesPreferences:(DSAgentManager *)sender;
{
    [self debugWithFormat:@"agentDidChangeUpdatesPreferences received from %@", sender];
    
    [[NSUserDefaults standardUserDefaults] copyPreferencesWithKeys:SU_USER_PREFERENCE_KEYS fromBundleIdentifier:[[self ownerAgent] bundleIdentifier] removeIfMissing:NO];
}

/*
 agent:checkForUpdates:runModal:
 
 Delegate method received when a remote agent sends this signature to us.  The sender is the DSAgentManager of the agent that sent the message (e.g. the owner).  Starts a check for updates.
 
 Written by DJS 2007-03.
*/

- (void)agent:(DSAgentManager *)sender checkForUpdates:(NSString *)checkingMode runModal:(NSString *)runModal;
{
    [self debugWithFormat:@"agentCheckForUpdates:%@ runModal:%@ received from %@", checkingMode, runModal, sender];
    
    [[self updateController] checkForUpdates:[checkingMode intValue] runModal:[runModal boolValue]];
}

@end


// ----------------------------------------------------------------------------------------
#pragma mark -
// ----------------------------------------------------------------------------------------


@implementation DSStatusAgentController (NSApplicationDelegate)

- (void)applicationDidFinishLaunching:(NSNotification *)notification;
{
    [[NSUserDefaults standardUserDefaults] copyPreferenceWithKey:LICENSE_AGREEMENT_RELEASE fromBundleIdentifier:[self ownerBundleIdentifier] removeIfMissing:NO];
    
    [[DSLicenseController alloc] init];
    
    // Uses a timer so it doesn't interfere with the license panel:
    [NSTimer scheduledTimerWithTimeInterval:1.0 * INTERVAL_MINUTE
                                     target:[self updateController]
                                   selector:@selector(automaticUpdateCheck)
                                   userInfo:nil repeats:NO];
    
    // Note: do not call super in this method; it is called in -licenseAssistantDidFinish.
}

@end

