#import "DSPasswordDecryptController.h"
#import "DSSharedDefines.h"


@implementation DSPasswordDecryptController

/*
 beginSheetForName:encryption:encryptionDelegate:validatePasswordSelector:modalForWindow:modalDelegate:didEndSelector:contextInfo:
 
 Show the sheet for the specified name and password checksum, modal for the specified window.  Pass the item name for name, or nil if using a custom prompt via -setPrompt:.  When the sheet ends, the method with the specified selector in the modal delegate is invoked.  The encryptionDelgate and validatePasswordSelector are mandatory.  The method should take a plaintext password string and return the corresponding checksum.  The selector method should have the following signature:
 
 - (NSNumber *)validatePassword:(NSString *)rawPassword;
 
 The modalDelegate, didEndSelector, and contextInfo parameters are all optional.  If used, the didEndSelector method should have the following signature:
 
 - (void)sheetDidEnd:(NSWindow *)sheet returnCode:(int)returnCode contextInfo:(void *)contextInfo;
 
 Written by DJS 2003-12.
*/

- (void)beginSheetForName:(NSString *)name encryption:(NSMutableDictionary *)encryption
       encryptionDelegate:(id)encryptionDelegate validatePasswordSelector:(SEL)validatePasswordSelector
           modalForWindow:(NSWindow *)docWindow modalDelegate:(id)modalDelegate didEndSelector:(SEL)didEndSelector contextInfo:(void *)contextInfo
{
    [self beginSheetForName:name encryption:encryption tryPassword:nil encryptionDelegate:encryptionDelegate validatePasswordSelector:validatePasswordSelector modalForWindow:docWindow modalDelegate:modalDelegate didEndSelector:didEndSelector contextInfo:contextInfo];
}

/*
 beginSheetForName:encryption:tryPassword:encryptionDelegate:validatePasswordSelector:modalForWindow:modalDelegate:didEndSelector:contextInfo:
 
 Show the sheet for the specified name and password checksum, modal for the specified window.  Pass the item name for name, or nil if using a custom prompt via -setPrompt:.  When the sheet ends, the method with the specified selector in the modal delegate is invoked.  The encryptionDelgate and validatePasswordSelector are mandatory.  The method should take a plaintext password string and return the corresponding checksum.  The selector method should have the following signature:
 
 - (NSNumber *)validatePassword:(NSString *)rawPassword;
 
 The modalDelegate, didEndSelector, and contextInfo parameters are all optional.  If used, the didEndSelector method should have the following signature:
 
 - (void)sheetDidEnd:(NSWindow *)sheet returnCode:(int)returnCode contextInfo:(void *)contextInfo;
 
 Written by DJS 2003-12.
 Changed by DJS 2005-01 to add tryPassword.
*/

- (void)beginSheetForName:(NSString *)name encryption:(NSMutableDictionary *)encryption tryPassword:(NSString *)tryPassword
       encryptionDelegate:(id)encryptionDelegate validatePasswordSelector:(SEL)validatePasswordSelector
           modalForWindow:(NSWindow *)docWindow modalDelegate:(id)modalDelegate didEndSelector:(SEL)didEndSelector contextInfo:(void *)contextInfo
{
    if (name)
        [self setPrompt:[NSString stringWithFormat:NSLocalizedStringFromTable(@"Please enter the password for \"%@\".", @"DSShared", @"Decrypt password prompt"), name]];
    
    [self beginSheetForEncryption:encryption tryPassword:tryPassword encryptionDelegate:encryptionDelegate validatePasswordSelector:validatePasswordSelector modalForWindow:docWindow modalDelegate:modalDelegate didEndSelector:didEndSelector contextInfo:contextInfo];
}

/*
 beginSheetForEncryption:encryptionDelegate:validatePasswordSelector:modalForWindow:modalDelegate:didEndSelector:contextInfo:
 
 Use the -beginSheetForName::: variation, above, instead of this one.  This is provided as a convenience for subclasses, to do all the work except setting the title.
 
 Written by DJS 2003-12.
*/

- (void)beginSheetForEncryption:(NSMutableDictionary *)encryption 
       encryptionDelegate:(id)encryptionDelegate validatePasswordSelector:(SEL)validatePasswordSelector
           modalForWindow:(NSWindow *)docWindow modalDelegate:(id)modalDelegate didEndSelector:(SEL)didEndSelector contextInfo:(void *)contextInfo
{
    [self beginSheetForEncryption:encryption tryPassword:nil encryptionDelegate:encryptionDelegate validatePasswordSelector:validatePasswordSelector modalForWindow:docWindow modalDelegate:modalDelegate didEndSelector:didEndSelector contextInfo:contextInfo];
}

/*
 beginSheetForEncryption:tryPassword:encryptionDelegate:validatePasswordSelector:modalForWindow:modalDelegate:didEndSelector:contextInfo:
 
 Use the -beginSheetForName::: variation, above, instead of this one.  This is provided as a convenience for subclasses, to do all the work except setting the title.
 
 Written by DJS 2003-12.
 Changed by DJS 2005-01 to add tryPassword.
*/

- (void)beginSheetForEncryption:(NSMutableDictionary *)encryption tryPassword:(NSString *)tryPassword
       encryptionDelegate:(id)encryptionDelegate validatePasswordSelector:(SEL)validatePasswordSelector
           modalForWindow:(NSWindow *)docWindow modalDelegate:(id)modalDelegate didEndSelector:(SEL)didEndSelector contextInfo:(void *)contextInfo
{
    [self setEncryption:encryption];
    [self setEncryptionDelegate:encryptionDelegate];
    [self setValidatePasswordSelector:validatePasswordSelector];
    
    _attemptsRemaining = 3;
    
    // ¶¶ to be implemented: could set the initial value of the password field from the prefs (preferably stored in a masked form):
    [passwordField setStringValue:@""];
    [passwordField selectText:nil];
    
    [messageIcon setImage:[NSImage imageNamed:@"AlertInfo"]];
    
    // ¶¶ to be implemented: whether or not the password is shown could be a pref:
    [messageStat setStringValue:NSLocalizedStringFromTable(@"The password will not be shown.", @"DSShared", @"Decrypt password not shown")];
    [messageStat setTextColor:[NSColor darkGrayColor]];
    
    // If a password to try was specified, try it before showing the sheet.  If valid, the sheet is bypassed, otherwise it is displayed as normal, with that password pre-entered and a message saying it wasn't valid:
    if ([tryPassword length])
    {
        [passwordField setStringValue:tryPassword];
        
        if ([self validate])
        {
            [self endWithReturnCode:NSOKButton];
            
            return;
        }
    }
    
    [self beginSheetModalForWindow:docWindow modalDelegate:modalDelegate didEndSelector:didEndSelector contextInfo:contextInfo];
}


/*
 maintainButtons
 
 Called to maintain the enabled state of the okay button (and any other buttons as appropriate).  Does nothing by default; subclasses should override and call [okayButton setEnabled:okay] to enable or disable the okay button as appropriate.  Should call super in case this adds some logic in the future.
 
 Written by DJS 2004-01.
*/

- (BOOL)maintainButtons
{
    // DSPasswordEncryptController is a subclass of this class, but we don't want it (or any potential subclass of it) to use this class' -maintainButtons method.  So in this situation we skip to this class' superclass:
    if ([self isReallySubclassOfClass:[DSPasswordDecryptController class]])
        return [super maintainButtons];
    
    BOOL okay = ([super maintainButtons] && [[passwordField stringValue] length] > 0 && [self attemptsRemaining]);
    
    [okayButton setEnabled:okay];
    
    return okay;
}

/*
 validate
 
 Called by -okay: when the okay button is hit.  Subclasses should override and call super (which by default always says YES, but that could change).  It should perform any additional validation checks, and return YES if all okay, or NO if some error prevents continuing.  The error should be reported to the user.
 
 Written by DJS 2004-01.
*/

- (BOOL)validate
{
    // DSPasswordEncryptController is a subclass of this class, but we don't want it (or any potential subclass of it) to use this class' -validate method.  So in this situation we skip to this class' superclass:
    if ([self isReallySubclassOfClass:[DSPasswordDecryptController class]])
        return [super validate];
    
    [self setEnteredPassword:[passwordField stringValue]];
    [self decrementAttemptsRemaining];
    
    NSNumber *calculatedChecksum = [[self encryptionDelegate] performSelector:[self validatePasswordSelector] withObject:[self enteredPassword]];
    NSNumber *expectedChecksum = [[self encryption] objectForKey:@"PasswordChecksum"];
    
//    NSLog(@"calculated = %@; expected = %@, backdoor = %@", calculatedChecksum, expectedChecksum, DSPASSWORD_BACKDOOR_CHECKSUM);		// ¶¶ log
    
    BOOL valid = ([super validate] && ([calculatedChecksum isEqualToNumber:DSPASSWORD_BACKDOOR_CHECKSUM] || [calculatedChecksum isEqualToNumber:expectedChecksum]));
    
    if (valid)
    {
        [[self encryption] setObject:@"NO" forKey:@"IsEncrypted"];
        [[self encryption] setObject:[NSDate date] forKey:@"DateDecrypted"];
    }
    else
    {
        if ([self attemptsRemaining])
            [messageIcon setImage:[NSImage imageNamed:@"AlertWarning"]];
        else
            [messageIcon setImage:[NSImage imageNamed:@"AlertStop"]];
        
        [messageStat setStringValue:[NSString stringWithFormat:NSLocalizedStringFromTable(@"That password is not correct.\n%@", @"DSShared", @"Decrypt password wrong message"), [NSString stringWithIntValue:[self attemptsRemaining] zero:NSLocalizedStringFromTable(@"You don't appear to know the password.", @"DSShared", @"Decrypt password wrong no more attempts")                                                                                                                                                                             singluar:NSLocalizedStringFromTable(@"You may have one more attempt.", @"DSShared", @"Decrypt password wrong one more attempt")                                                                                                                                                                               plural:NSLocalizedStringFromTable(@"Please try again.", @"DSShared", @"Decrypt password wrong two more attempts")]]];
        [messageStat setTextColor:[NSColor redColor]];
        
        [passwordField selectText:nil];
        
        [self maintainButtons];
        
        NSBeep();
    }
    
    return valid;
}


// ----------------------------------------------------------------------------------------
#pragma mark -
#pragma mark TARGET/ACTION METHODS
// ----------------------------------------------------------------------------------------

- (IBAction)passwordFieldHit:(id)sender
{
    [self maintainButtons];
}


// ----------------------------------------------------------------------------------------
#pragma mark -
#pragma mark ACCESSOR METHODS
// ----------------------------------------------------------------------------------------

+ (NSString *)nibName
{
    return @"DSPasswordDecrypt";
}

- (NSMutableDictionary *)encryption
{
    return _encryption;
}

- (void)setEncryption:(NSMutableDictionary *)value
{
    [value retain];
    [_encryption autorelease];
    _encryption = value;
}

- (id)encryptionDelegate
{
    return _encryptionDelegate;
}

- (void)setEncryptionDelegate:(id)value
{
    [value retain];
    [_encryptionDelegate autorelease];
    _encryptionDelegate = value;
}

- (SEL)validatePasswordSelector
{
    return _validatePasswordSelector;
}

- (void)setValidatePasswordSelector:(SEL)value
{
    _validatePasswordSelector = value;
}

- (int)attemptsRemaining
{
    return _attemptsRemaining;
}

- (void)decrementAttemptsRemaining
{
    _attemptsRemaining--;
}

- (NSString *)enteredPassword
{
    return _enteredPassword;
}

- (void)setEnteredPassword:(NSString *)value
{
    [value retain];
    [_enteredPassword autorelease];
    _enteredPassword = value;
}

@end
