#import "DSPasswordEncryptController.h"
#import "DSSharedDefines.h"


@implementation DSPasswordEncryptController

/*
 beginSheetForName:encryptionDelegate:validatePasswordSelector:modalForWindow:modalDelegate:didEndSelector:contextInfo:
 
 Show the sheet for the specified name, modal for the specified window.  Pass the item name for name, or nil if using a custom prompt via -setPrompt:.  When the sheet ends, the method with the specified selector in the modal delegate is invoked.  The encryptionDelgate and validatePasswordSelector are mandatory.  The method should take a plaintext password string and return the corresponding checksum.  The selector method should have the following signature:
 
 - (NSNumber *)validatePassword:(NSString *)rawPassword;
 
 The modalDelegate, didEndSelector, and contextInfo parameters are all optional.  If used, the didEndSelector method should have the following signature:
 
 - (void)sheetDidEnd:(NSWindow *)sheet returnCode:(int)returnCode contextInfo:(void *)contextInfo;
 
 Written by DJS 2003-12.
*/

- (void)beginSheetForName:(NSString *)name encryption:(NSMutableDictionary *)encryption
       encryptionDelegate:(id)encryptionDelegate validatePasswordSelector:(SEL)validatePasswordSelector
           modalForWindow:(NSWindow *)docWindow modalDelegate:(id)modalDelegate didEndSelector:(SEL)didEndSelector contextInfo:(void *)contextInfo
{
    if (name)
        [self setPrompt:[NSString stringWithFormat:NSLocalizedStringFromTable(@"Please enter a password to use for \"%@\".", @"DSShared", @"Encrypt password prompt"), name]];
    
    // ¶¶ to be implemented: could set the initial value of the confirm field from the prefs (preferably stored in a masked form):
    [confirmField setStringValue:@""];
    
    [self beginSheetForEncryption:encryption encryptionDelegate:encryptionDelegate validatePasswordSelector:validatePasswordSelector modalForWindow:docWindow modalDelegate:modalDelegate didEndSelector:didEndSelector contextInfo:contextInfo];
}


/*
 maintainButtons
 
 Called to maintain the enabled state of the okay button (and any other buttons as appropriate).  Does nothing by default; subclasses should override and call [okayButton setEnabled:okay] to enable or disable the okay button as appropriate.  Should call super in case this adds some logic in the future.
 
 Written by DJS 2004-01.
*/

- (BOOL)maintainButtons
{
    BOOL okay = ([super maintainButtons] && [[passwordField stringValue] length] > 0 && [[confirmField stringValue] length] > 0);
    
    [okayButton setEnabled:okay];
    
    return okay;
}

/*
 validate
 
 Called by -okay: when the okay button is hit.  Subclasses should override and call super (which by default always says YES, but that could change).  It should perform any additional validation checks, and return YES if all okay, or NO if some error prevents continuing.  The error should be reported to the user.
 
 Written by DJS 2004-01.
*/

- (BOOL)validate
{
    [self setEnteredPassword:[passwordField stringValue]];
    
    NSNumber *calculatedChecksum = [[self encryptionDelegate] performSelector:[self validatePasswordSelector] withObject:[self enteredPassword]];
    
    if (![self encryption])
        [self setEncryption:[NSMutableDictionary dictionary]];
    
    // ¶¶ to be implemented: may want to set other values in the dictionary, e.g. algorithm, etc:
    [[self encryption] setObject:@"YES" forKey:@"IsEncrypted"];
    [[self encryption] setObject:[NSDate date] forKey:@"DateEncrypted"];
    [[self encryption] setObject:calculatedChecksum forKey:@"PasswordChecksum"];
    
    BOOL passwordsMatch = [[self enteredPassword] isEqualToString:[confirmField stringValue]];
    BOOL okay = ([super validate] && [[self enteredPassword] length] && passwordsMatch);
    
    if (!okay)
    {
        [messageIcon setImage:[NSImage imageNamed:@"AlertWarning"]];
        
        // ¶¶ to be implemented: could add and enforce prefs of password length and content
        
        if (!passwordsMatch)
        {
            [messageStat setStringValue:NSLocalizedStringFromTable(@"The passwords do not match.", @"DSShared", @"Encrypt password mismatch")];
            
            [confirmField selectText:nil];
        }
        else
        {
            [messageStat setStringValue:NSLocalizedStringFromTable(@"Please enter a password.", @"DSShared", @"Encrypt password missing")];
            
            [passwordField selectText:nil];
        }
        
        [messageStat setTextColor:[NSColor redColor]];
        
        NSBeep();
    }
    
    return okay;
}


// ----------------------------------------------------------------------------------------
#pragma mark -
#pragma mark TARGET/ACTION METHODS
// ----------------------------------------------------------------------------------------

- (IBAction)confirmFieldHit:(id)sender
{
    [self maintainButtons];
}


// ----------------------------------------------------------------------------------------
#pragma mark -
#pragma mark ACCESSOR METHODS
// ----------------------------------------------------------------------------------------

+ (NSString *)nibName
{
    return @"DSPasswordEncrypt";
}

@end
