//
//  DSScriptController.m
//  Shared classes
//
//  Created by David Sinclair on Thu Apr 15 2004.
//  Copyright (c) 2004 Dejal Systems, LLC. All rights reserved.
//

#import "DSScriptController.h"
#import "DSSharedDefines.h"


@interface DSScriptController (Private)

- (NSString *)keyEquivalentForString:(NSString *)string returningModifierMask:(unsigned int *)modifierMask;

- (void)scanDirectory:(NSString *)directory intoMenu:(NSMenu *)menu;

@end


// ----------------------------------------------------------------------------------------
#pragma mark -
// ----------------------------------------------------------------------------------------


@implementation DSScriptController

id sharedScriptController = nil;

+ (id)sharedScriptController
{
    if (!sharedScriptController)
        sharedScriptController = [[self alloc] init];
    
    return sharedScriptController;
}

- (id)init
{
    self = [super init];
    
    [self setMenu:nil];
    [self setPath:nil];
    
    return self;
}

- (void)dealloc
{
    [self setMenu:nil];
    [self setPath:nil];
    
    [super dealloc];
}


/*
 setupWithMenu:parentPath:
 
 Sets up the Scripts menu, using the icon for the title, and sets the parent directory of the Scripts folder.  Automatically adds the Scripts directory component to the path, and expands and validates the directory.  The directory will be created, if possible.
 
 Written by DJS 2004-04.
*/

- (void)setupWithMenu:(NSMenu *)menu parentPath:(NSString *)path
{
    NSMenu *mainMenu = [menu supermenu];
    NSMenuItem *item = [mainMenu itemWithTitle:NSLocalizedStringFromTable(@"Scripts", @"DSShared", @"Scripts menu title")];
    
    [item setImage:[NSImage imageNamed:@"MenuScripts"]];
    
    // Validate the path before appending the Scripts folder, so that folder isn't created automatically:
    path = [[path validatedDirectoryPath] stringByAppendingPathComponent:@"Scripts"];
    
    BOOL doesExist = [[NSFileManager defaultManager] fileExistsAtPath:path];
    
    if (!doesExist)
    {
        // If there isn't anything in the expected location, look in the bundle for a default file (looks first in the non-localized resources, then in the localized ones):
        NSString *defaultPath = [[NSBundle mainBundle] pathForResource:[path lastPathComponent] ofType:nil];
        
        [[NSFileManager defaultManager] copyPath:defaultPath toPath:path handler:nil];
    }
    
    [self setMenu:menu];
    [self setPath:path];
    
    [self updateMenu];
}


/*
 openFolder
 
 Opens the Scripts folder in the Finder.  Call this from the action method for the Open Scripts Folder command.
 
 Written by DJS 2004-04.
*/

- (void)openFolder
{
    [[NSWorkspace sharedWorkspace] openFile:[[self path] validatedDirectoryPath]];
}

/*
 updateMenu
 
 Builds or rebuilds the Scripts menu.  Assumes that the menu always contains Open Scripts Folder and Update Scripts Menu commands at the top, set in the Nib.  Call this from the action method for the Update Scripts Menu command.
 
 Written by DJS 2004-04.
*/

- (void)updateMenu
{
    NSMenu *menu = [self menu];
    NSString *path = [self path];
    
    if (!menu || ![path length])
        return;
    
    // Remove scripts:
    while ([menu numberOfItems] > 2)
        [menu removeItemAtIndex:2];
    
    // Add a separator item:
    [menu addItem:[NSMenuItem separatorItem]];
    
    // Recursively add scripts:
    [self scanDirectory:path intoMenu:menu];
    
    // If no scripts, remove the separator:
    if ([menu numberOfItems] == 3)
        [menu removeItemAtIndex:2];
}


// ----------------------------------------------------------------------------------------
#pragma mark -
#pragma mark TARGET/ACTION METHODS
// ----------------------------------------------------------------------------------------

/*
 performScript:
 
 Action method for all scripts in the Scripts menu.
 
 Written by DJS 2004-04.
*/

- (IBAction)performScript:(id)sender
{
    NSString *path = [sender representedObject];
    
    // Option key down:
    if ([self usedOptionModifier])
    {
        // Open the script file:
        [[NSWorkspace sharedWorkspace] openFile:path];
    }
    else
    {
        // Execute the script:
        NSURL *url = [NSURL fileURLWithPath:path];
        NSDictionary *error;
        
        if (!url)
            return;
        
        NSAppleScript *script = [[[NSAppleScript alloc] initWithContentsOfURL:url error:&error] autorelease];
        
        [script executeAndReturnError:&error];
    }
}


// ----------------------------------------------------------------------------------------
#pragma mark -
#pragma mark ACCESSOR METHODS
// ----------------------------------------------------------------------------------------

- (NSMenu *)menu
{
    return _menu;
}

- (void)setMenu:(NSMenu *)value
{
    [value retain];
    [_menu autorelease];
    _menu = value;
}

- (NSString *)path
{
    return _path;
}

- (void)setPath:(NSString *)value
{
    [value retain];
    [_path autorelease];
    _path = value;
}


@end


// ----------------------------------------------------------------------------------------
#pragma mark -
// ----------------------------------------------------------------------------------------


@implementation DSScriptController (Private)

/*
 keyEquivalentForString:
 
 Private method to figure out a keyboard equivalent based on text at the end of the filename.
 
 Written by DJS 2004-04.
*/

- (NSString *)keyEquivalentForString:(NSString *)string returningModifierMask:(unsigned int *)modifierMask
{
    NSArray *components = [[string lowercaseString] componentsSeparatedByString:@"-"];
    NSString *keyEquiv = [components lastObject];
    
    *modifierMask = NSCommandKeyMask & NSControlKeyMask;
    
    // ¶¶ to be implemented: doesn't yet support custom modifiers, or function keys, etc.
    
    return keyEquiv;
}

/*
 scanDirectory:intoMenu:
 
 Private method to recursively scan the contents of the specified path into the specified menu.
 
 Written by DJS 2004-04.
*/

- (void)scanDirectory:(NSString *)directory intoMenu:(NSMenu *)menu
{
    NSFileManager *manager = [NSFileManager defaultManager];
    NSArray *files = [manager directoryContentsAtPath:directory];
    NSEnumerator *enumerator = [files objectEnumerator];
    NSString *filename;
    NSString *path;
    NSString *title;
    NSString *keyEquiv;
    unsigned int modifierMask;
    BOOL isDirectory;
    
    while ((filename = [enumerator nextObject]))
    {
        path = [directory stringByAppendingPathComponent:filename];
        title = [filename stringByDeletingPathExtension];
        keyEquiv = @"";
        
        NSRange position = [title rangeOfString:@"___"];
        
        if (position.location != NSNotFound)
        {
            keyEquiv = [title substringFromIndex:position.location + position.length];
            keyEquiv = [self keyEquivalentForString:keyEquiv returningModifierMask:&modifierMask];
            title = [title substringToIndex:position.location];
        }
        
        if ([manager fileExistsAtPath:path isDirectory:&isDirectory] && isDirectory)
        {
            NSMenuItem *item = [menu addItemWithTitle:title action:nil keyEquivalent:@""];
            NSMenu *submenu = [[NSMenu alloc] initWithTitle:title];
            
            [menu setSubmenu:submenu forItem:item];
            
            [self scanDirectory:path intoMenu:submenu];
        }
        else if ([[path pathExtension] isEqualToString:@"scpt"])
        {
            [menu addItemWithTitle:title target:self action:@selector(performScript:) keyEquivalent:keyEquiv modifierMask:modifierMask icon:nil representedObject:path];
        }
    }
}

@end
