#import "DSSharedController.h"
#import "MainDefines.h"
#import <SystemConfiguration/SCNetwork.h>
#import "DSExceptionController.h"

@interface NSObject (DSUpdateCategories)

+ (id)sharedUpdateController;
- (void)setupDefaultPrefs;
- (void)automaticUpdateCheck;
- (void)checkForUpdates:(NSInteger)checkingMode runModal:(BOOL)runModal;

@end


// ----------------------------------------------------------------------------------------
#pragma mark -
// ----------------------------------------------------------------------------------------


@implementation DSSharedController

+ (id)mainController
{
    static id controller = nil;

    if (!controller)
    {
        controller = [NSApp delegate];
    }

    return controller;
}

- (id)init
{
    self = [super init];
    
    [self setFullyStarted:NO];
    [self setQuitting:NO];
    
    // Start the crash reporter; the email address strips off the "mailto:" tag:
//    [[ILCrashReporter defaultReporter] launchReporterForCompany:COMPANY_NAME_BRIEF
//                                                     reportAddr:[URL_COMPANY_EMAIL substringFromIndex:7]];
    
    // Test crashing!
//    [[[NSDate date] release] release];
    
//    NSLog(@"DSSharedController init before registerDefaults");		//  log
    
    DSRandomSeed();
    
    [PREFERENCES registerDefaults:[self defaultPreferences]];
    
    [self upgradePreferences];
    
    dockIconAnimation = 0;
    dockIconDate = nil;
    dockIconAnimationArray = nil;	// Allocate this in the subclass if Dock icon animation is desired
        
    [[NSNotificationCenter defaultCenter] addObserver:self selector:@selector(checkForUpdates:) name:@"CheckForUpdates" object:nil];
    
    return self;
}

- (void)dealloc
{
    [[NSNotificationCenter defaultCenter] removeObserver:self];

    [dockIconDate release];
    [dockIconAnimationArray release];

    [super dealloc];

    return;
}

/*
 preferenceClasses
 
 Returns an array of class names (as strings) for the preference pages.  Subclasses should override this, without calling super.
 
 Written by DJS 2005-04.
*/

- (NSArray *)preferenceClasses
{
    NSAssert(NO, @"Forgot to override -preferenceClasses in the main controller.");
    
    return [NSArray arrayWithObjects:@"DSPrefsGeneral", @"DSPrefsUpdates", @"DSPrefsAdvanced", nil];
}

/*
 defaultPreferences
 
 Returns a mutable dictionary containing the default preference settings.  Subclasses should override this, call super, and add their own preference keys.
 
 Written by DJS 2004-06.
 Changed by DJS 2006-01 to add the prefs confirmation.
 Changed by DJS 2007-03 to add the data folder path.
*/

- (NSMutableDictionary *)defaultPreferences
{
    NSMutableDictionary *dict = [NSMutableDictionary dictionary];
    
    [dict setObject:[NSNumber numberWithBool:CONFIRM_QUIT_DEFAULT] forKey:SHARED_CONFIRM_QUIT];
    [dict setObject:[NSNumber numberWithBool:CONFIRM_PREFS_DEFAULT] forKey:SHARED_CONFIRM_RESTORE_PREFS];
    
    [dict setObject:[[@"~/Library/Application Support" stringByAppendingPathComponent:COMPANY_NAME_BRIEF] stringByAppendingPathComponent:PRODUCT_NAME] forKey:SHARED_DATA_FOLDER_PATH];
    
    return dict;
}

/*
 upgradePreferences
 
 If any of the shared preferences are out of date or need resetting (e.g. when adding a column to a table), do so here and increment the version number.  Subclasses should implement this, calling super, and use the key "PreferencesVersion" and do something similar for their own keys.  Subclasses should not override this; override -upgradePreferencesFromVersion: instead.
 
 Written by DJS 2004-06.
*/

- (void)upgradePreferences
{
/*       commented out until needed
    // Increment the new number when adding new lines below:
    NSInteger oldSharedPreferencesVersion = [PREFERENCES integerForKey:@"DSSharedPreferencesVersion"];
    NSInteger newSharedPreferencesVersion = 1;
    
    if (oldSharedPreferencesVersion < 1)
        [PREFERENCES restoreFactorySettingsForKey:xxx];
    
    if (newSharedPreferencesVersion > oldSharedPreferencesVersion)
        [PREFERENCES setInteger:newSharedPreferencesVersion forKey:@"DSSharedPreferencesVersion"];
*/
    
    // Upgrade the preferences specific to this app too:
    NSInteger oldSpecificPreferencesVersion = [PREFERENCES integerForKey:@"DSSpecificPreferencesVersion"];
    NSInteger newSpecificPreferencesVersion = [self upgradePreferencesFromVersion:oldSpecificPreferencesVersion];
    
    if (newSpecificPreferencesVersion > oldSpecificPreferencesVersion)
        [PREFERENCES setInteger:newSpecificPreferencesVersion forKey:@"DSSpecificPreferencesVersion"];
}

/*
 upgradePreferencesFromVersion:
 
 If any of the preferences are out of date or need resetting, do so here.  The parameter is the version number from the preference file; if this is less than the current version number, do the upgrade.  Return the current version number.  Note: for things like tables, it is often better to just change the autosave name (e.g. by appending the app version number), so the old pref isn't used, but is retained for backward compatibility.
 
 Written by DJS 2004-06.
*/

- (NSInteger)upgradePreferencesFromVersion:(NSInteger)preferencesVersion
{
    // Subclasses should override this, without calling super, to upgrade their preferences
    
    return 0;
}


- (void)applicationWillFinishLaunching:(NSNotification *)aNotification;
{
    // Hook up my custom exception handler; done in will-finish rather than did-finish to better catch any startup exceptions.  Note that NSApp hasn't been (fully?) initialized yet:
    [DSExceptionController sharedExceptionController];
}

- (void)applicationDidFinishLaunching:(NSNotification *)aNotification
{
    // Load the application data:
    [self loadInitialApplicationData];
    [self setFullyStarted:YES];
    [self displayInitialApplicationViews];
}

/*
 setupAssistantSettings
 
 Returns the settings for the Setup Assistant intern, including the DSInternClass object (a Class type) with the class of the intern.  Return nil if no Setup Assistant is desired.
 
 Written by DJS 2006-05.
*/

- (NSDictionary *)setupAssistantSettings;
{
    return nil;
}

/*
 loadInitialApplicationData
 
 Loads the application's data while the splash is still visible (if applicable).  Don't display any UI yet (other than temporary loading UI).  Returns YES if the data was loaded, or NO if there is no data yet, i.e. the Setup Assistant should be displayed.
 
 Written by DJS 2006-05.
*/

- (BOOL)loadInitialApplicationData;
{
    // Override this in the MainController subclass without calling super
    
    NSAssert(NO, @"Forgot to override -loadInitialApplicationData in the main controller.");
    
    return NO;
}

/*
 displayInitialApplicationViews
 
 Displays any standard application windows or other UI.  Returns YES if the windows were successfully displayed, or NO if they couldn't be displayed for some reason, in which case the application terminates.
 
 Written by DJS 2006-05.
*/

- (BOOL)displayInitialApplicationViews;
{
    // Override this in the MainController subclass without calling super
    
    NSAssert(NO, @"Forgot to override -displayInitialApplicationViews in the main controller.");
    
    return NO;
}

- (BOOL)validateMenuItem:(NSMenuItem *)menuItem;
{
    // Override this in the MainController subclass to validate the Dock menu items
    
    return YES;
}    


// ----------------------------------------------------------------------------------------
#pragma mark -
#pragma mark ACCESSOR METHODS
// ----------------------------------------------------------------------------------------

- (void)setMustQuitWithoutConfirmation
{
    [self setFullyStarted:NO];
}

- (BOOL)fullyStarted
{
    return _fullyStarted;
}

- (void)setFullyStarted:(BOOL)value
{
    _fullyStarted = value;
}

- (BOOL)isQuitting
{
    return _quitting;
}

- (void)setQuitting:(BOOL)value
{
    _quitting = value;
}

/*
 dataFolderPath
 
 Returns the data folder path, which is guaranteed to exist.  It returns the path as it was when first accessed, even if since changed in the Advanced preferences.
 
 Written by DJS 2007-04.
*/

- (NSString *)dataFolderPath;
{
    static NSString *_dataFolderPath = nil;
    
    // Retain a copy of the data folder path when first accessed, in case it is changed in the prefs later:
    if (!_dataFolderPath)
        _dataFolderPath = [[[PREFERENCES objectForKey:SHARED_DATA_FOLDER_PATH] validatedDirectoryPath] copy];
    
    return _dataFolderPath;
}

/*
 uniqueTemporaryDataPath
 
 Returns the path of a unique subdirectory of the temporary data directory for the application, which is guaranteed to exist.  This directory is universally unique, and is automatically deleted when the app quits.
 
 Written by DJS 2008-01.
*/

- (NSString *)uniqueTemporaryDataPath;
{
    return [[[self temporaryDataPathWithoutCreating] stringByAppendingPathComponent:[NSString universallyUniqueIdentifier]] validatedDirectoryPath];
}

/*
 temporaryDataPath
 
 Returns the temporary data path for the application, which is guaranteed to exist.  This directory is unique to the application, and is automatically deleted when it quits.
 
 Written by DJS 2008-01.
*/

- (NSString *)temporaryDataPath;
{
    return [[self temporaryDataPathWithoutCreating] validatedDirectoryPath];
}

/*
 temporaryDataPathWithoutCreating
 
 Returns the temporary data path for the application, which may or may not exist.  This directory is unique to the application, and is automatically deleted when it quits.
 
 Written by DJS 2008-01.
*/

- (NSString *)temporaryDataPathWithoutCreating;
{
    return [NSTemporaryDirectory() stringByAppendingPathComponent:[[[NSBundle mainBundle] infoDictionary] objectForKey:@"CFBundleIdentifier"]];
}

@synthesize dockIconAnimationTimer;
@synthesize dockIconDate;
@synthesize dockIconAnimation;
@synthesize dockIconAnimationArray;
@end

