//
//  DSSheetController.m
//  Shared classes
//
//  Created by David Sinclair on Fri Jan 09 2004.
//  Copyright (c) 2004 - 2008 Dejal Systems, LLC. All rights reserved.
//

#import "DSSheetController.h"
#import "DSSharedDefines.h"
#import <Carbon/Carbon.h>


@implementation DSSheetController

NSMutableArray *sharedSheetControllers = nil;

/*
 controller
 
 Returns either a new or recycled sheet controller of the receiver class.  If there are none available, it instantiates a new one, but if there is an old one that isn't in use anymore (i.e. isn't currently visible), it is returned instead.  Subclasses shouldn't need to override this.
 
 Written by DJS 2004-01.
*/

+ (id)controller
{
    if (!sharedSheetControllers)
        sharedSheetControllers = [[NSMutableArray array] retain];
    
    NSEnumerator *enumerator = [sharedSheetControllers objectEnumerator];
    id controller;
    BOOL found = NO;
    
    while (!found && (controller = [enumerator nextObject]))
    {
    	found = [controller isMemberOfClass:self] && ![[controller window] isVisible];
    }
    
    if (found)
    {
        [controller recycle];
    }
    else
    {
        controller = [[[self alloc] initWithWindowNibName:[self nibName]] autorelease];
        
        // Force the nib to make the connections, if it hasn't already:
        [controller window];
        
        [sharedSheetControllers addObject:controller];
    }
    
    return controller;
}

/*
 controllerForObject:
 
 If the specified object is already being edited, brings that window to the front, otherwise recycles a controller instance if any invisible ones are available, or creates a new controller and window for it.  Subclasses shouldn't need to override this, though they may want to provide a replacement that specifies the object type, and calls this.  See also +controllerForObject:modelessDelegate:didEndSelector:contextInfo:, below, for a variation that allows a calling did-end method when the OK or Cancel button is hit.
 
 Written by DJS 2004-06.
*/

+ (id)controllerForObject:(id)object
{
    return [self controllerForObject:object modelessDelegate:nil didEndSelector:nil contextInfo:nil];
}

/*
 controllerForObject:modelessDelegate:didEndSelector:contextInfo:
 
 If the specified object is already being edited, brings that window to the front, otherwise recycles a controller instance if any invisible ones are available, or creates a new controller and window for it.  Subclasses shouldn't need to override this, though they may want to provide a replacement that specifies the object type and/or omits unneeded parameters, and calls this.  See also +controllerForObject:, above, for a variation that doesn't set the delegate etc (they can still be set later if desired).
 
 Written by DJS 2004-06.
*/

+ (id)controllerForObject:(id)object modelessDelegate:(id)modelessDelegate didEndSelector:(SEL)didEndSelector contextInfo:(void *)contextInfo
{
    id controller = nil;
    
    // If an object was specified, find the controller for that object, if any:
    if (object)
        controller = [self findControllerForObject:object];
    
    // If one is found, the window for that object is already visible, so just bring it to the front:
    if (controller)
        [controller showWindow];
    // Otherwise, get a new or recycled instance and set it up:
    else
        controller = [[self controller] beginWindowForObject:object modelessDelegate:modelessDelegate didEndSelector:didEndSelector contextInfo:contextInfo];
    
    return controller;
}

/*
 initWithWindowNibName:
 
 An override of the NSWindowController method, to additionally set the window frame autosave name and other configuration.  Subclasses shouldn't need to override this.
 
 Written by DJS 2004-01.
 Changed by DJS 2006-01 to add -setCanRecycle:.
 Changed by DJS 2007-08 to set can recycle to NO by default.
 Changed by DJS 2007-09 to set can recycle back to YES, as NO broke windows that assumed it was recycled, e.g. the Assistant window.
*/

- (id)initWithWindowNibName:(NSString *)nibName
{
    self = [super initWithWindowNibName:nibName];
    
    [self setCanRecycle:YES];
    [self setWindowFrameAutosaveName:nibName];
    [[self window] setFrameUsingName:nibName];
    
    [self setup];
    [self recycle];
    
    return self;
}

/*
 setup
 
 Does any initialization that should only be done when the Nib file is first loaded for this controller instance, not when recycling.  Subclasses should override this, calling super, if they need to do anything here, e.g. adding notification observers.
 
 Written by DJS 2004-06.
*/

- (void)setup
{
    // Nothing to do here currently
}

/*
 recycle
 
 Clears the ivars, so a new or recycled instance is clean.  Subclasses should override this, calling super, if they define any ivars.  UI adjustment should be done in -prepare rather than here.
 
 Written by DJS 2004-01.
*/

- (void)recycle
{
    _isModal = YES;
    
    [self setIdentifier:nil];
    [self setObject:nil];
    [self setUserInfo:nil];
    [self setAltering:NO];
    [self setDelegate:nil];
    [self setDidEndSelector:nil];
    [self setContextInfo:nil];
}

/*
 dealloc
 
 Deallocator method.  Not used for recycled windows, but is for non-recycled, or when -prepare returns NO.
 
 Written by DJS 2004-01.
*/

- (void)dealloc
{
    [[NSNotificationCenter defaultCenter] removeObserver:self];
    
    [self setIdentifier:nil];
    [self setObject:nil];
    [self setUserInfo:nil];
    [self setDelegate:nil];
    [self setDidEndSelector:nil];
    [self setContextInfo:nil];
    
    [super dealloc];
}

/*
 topmostController
 
 Returns the topmost controller for the receiver class, or nil if there are none visible.  Subclasses shouldn't need to override this.
 
 Written by DJS 2004-06.
*/

+ (id)topmostController
{
    return [self findControllerForObject:nil];
}

/*
 findControllerForObject:
 
 Looks through the visible windows for a controller of the receiver's class using the specified object, and returns it if found, or nil if not.  If object is nil, returns the first controller found (see +topmostController, above).  Subclasses shouldn't need to override this.
 
 Written by DJS 2004-06.
*/

+ (id)findControllerForObject:(id)object
{
    // If none, bail out:
    if (!sharedSheetControllers)
        return nil;
    
    // Scan in window order:
    NSEnumerator *enumerator = [[NSApp windows] objectEnumerator];
    NSWindow *window;
    id controller = nil;
    BOOL found = NO;
    
    // Enumerate through the windows:
    while (!found && (window = [enumerator nextObject]))
    {
        controller = [window windowController];
        
        // Found if the window is visible, the controller is the same class as the receiver, and the controller is in the shared controllers array:
        found = [window isVisible] && [controller isMemberOfClass:[self class]] && [sharedSheetControllers containsObject:controller];
        
        // If an object is specified, also see if the object matches:
        if (found && object)
            found = [[controller object] isEqual:object];
    }
    
    if (!found)
        controller = nil;
    
    return controller;
}


/*
 beginSheetModalForWindow:modalDelegate:didEndSelector:contextInfo:
 
 Shows the sheet, modal for the specified window.  When the sheet ends, the method with the specified selector in the modal delegate is invoked.  All parameters are optional.  Returns the receiver.  If used, the didEndSelector method should have the following signature; returnCode will be one of NSOKButton or NSCancelButton:
 
 - (void)sheetDidEnd:(NSWindow *)sheet returnCode:(NSInteger)returnCode contextInfo:(void *)contextInfo;
 
 Written by DJS 2004-01.
 Changed by DJS 2006-01 to not show the window if -prepare returns NO.
*/

- (id)beginSheetModalForWindow:(NSWindow *)docWindow modalDelegate:(id)modalDelegate didEndSelector:(SEL)didEndSelector contextInfo:(void *)contextInfo
{
    _isModal = YES;
    
    [self setDelegate:modalDelegate];
    [self setDidEndSelector:didEndSelector];
    [self setContextInfo:contextInfo];
    
    // If we can't prepare for any reason, assume this controller is invalid, and forget about it:
    if (![self prepare])
    {
        [sharedSheetControllers removeObject:self];
        return nil;
    }
    
    [self maintainButtons];
    
    [NSApp beginSheet:[self window] modalForWindow:docWindow modalDelegate:modalDelegate didEndSelector:didEndSelector contextInfo:contextInfo];
    
    return self;
}

/*
 beginWindowForObject:modelessDelegate:didEndSelector:contextInfo:
 
 Shows the window modelessly to edit the specified object.  When the window ends (i.e. the OK or Cancel button is clicked), the method with the specified selector in the modeless delegate is invoked.  All parameters are optional.  Returns the receiver.  If used, the didEndSelector method should have the following signature; returnCode will be one of NSOKButton or NSCancelButton:
 
 - (void)windowDidEnd:(NSWindow *)window returnCode:(NSInteger)returnCode contextInfo:(void *)contextInfo;
 
 Written by DJS 2004-06.
 Changed by DJS 2006-01 to not show the window if -prepare returns NO.
*/

- (id)beginWindowForObject:(id)object modelessDelegate:(id)modelessDelegate didEndSelector:(SEL)didEndSelector contextInfo:(void *)contextInfo
{
    _isModal = NO;
    
    [self setAltering:object != nil];
    [self setObject:object];
    [self setDelegate:modelessDelegate];
    [self setDidEndSelector:didEndSelector];
    [self setContextInfo:contextInfo];
    
    // If we can't prepare for any reason, assume this controller is invalid, and forget about it:
    if (![self prepare])
    {
        [sharedSheetControllers removeObject:self];
        return nil;
    }
    
    [self maintainButtons];
    [self showWindow];
    
    return self;
}


/*
 prepare
 
 Called from -beginSheetModalForWindow:modalDelegate:didEndSelector:contextInfo: and -beginWindowForObject:modelessDelegate:didEndSelector:contextInfo: to do any additional setup of the sheet or window before showing it.  Does nothing by default; subclasses may override if desired.  Often you'd want to provide a replacement for that method instead, though, to add additional parameters.  Should call super in case this adds some logic in the future.
 
 Written by DJS 2004-02.
*/

- (BOOL)prepare
{
    return YES;
}

/*
 showWindow
 
 Shows the receiver's window, if not already visible, and brings it to the front.  Subclasses shouldn't need to override this.
 
 Written by DJS 2004-06.
*/

- (void)showWindow
{
    [self showWindow:nil];
}

/*
 maintainButtons
 
 Called to maintain the enabled state of the okay button (and any other buttons as appropriate).  Does nothing by default; subclasses should override and call [okayButton setEnabled:okay] to enable or disable the okay button as appropriate.  Should call super in case this adds some logic in the future.
 
 Written by DJS 2004-01.
*/

- (BOOL)maintainButtons
{
    return YES;
}

/*
 validate
 
 Called by -okay: when the okay button is hit.  Subclasses should override and call super (which by default always says YES, but that could change).  It should perform any additional validation checks, and return YES if all okay, or NO if some error prevents continuing.  The error should be reported to the user.
 
 Written by DJS 2004-01.
*/

- (BOOL)validate
{
    return YES;
}

/*
 shouldEndWithReturnCode:
 
 The sheet, panel, or window is about to be removed.  Last chance to do any cleanups or operations that should occur when either the okay or cancel buttons are pressed.  See also -validate, above.  Returns YES by default.  Subclassses may override this if desired; return NO to stop the window from closing.
 
 Written by DJS 2006-01.
*/

- (BOOL)shouldEndWithReturnCode:(NSInteger)returnCode;
{
    return YES;
}

/*
 endWithReturnCode:
 
 Removes the sheet, panel, or window, and invokes the did-end method, if any.  Subclasses shouldn't need to override this.
 
 Written by DJS 2004-06.
 Changed by DJS 2006-01 to add -shouldEndWithReturnCode:.
*/

- (void)endWithReturnCode:(NSInteger)returnCode
{
    if (![self shouldEndWithReturnCode:returnCode])
        return;
    
    [[self window] forceEndEditingForView:nil];
    [[self window] orderOut:nil];
    
    if ([self isModal])
        [NSApp endSheet:[self window] returnCode:returnCode];
    else
        [[self delegate] performSelector:[self didEndSelector] withWindow:[self window] returnCode:returnCode contextInfo:[self contextInfo]];
    
    // If we're not allowed to recycle, clean up:
    if (![self canRecycle])
    {
        // Make autoreleased without changing the current retain count:
        [[self retain] autorelease];
        
        // Remove from the shared shared controllers, which will release our interest in it:
        [sharedSheetControllers removeObject:self];
    }
    
    [PREFERENCES synchronize];
}


// ----------------------------------------------------------------------------------------
#pragma mark -
#pragma mark TARGET/ACTION METHODS
// ----------------------------------------------------------------------------------------

/*
 cancel:
 
 Action method for the cancel button.  Removes the sheet, panel, or window and calls the didEndSelector (if any) with the NSCancelButton return code.  Subclasses shouldn't need to override this.
 
 Written by DJS 2004-01.
*/

- (IBAction)cancel:(id)sender
{
    [self endWithReturnCode:NSCancelButton];
}

/*
 okay:
 
 Action method for the okay button.  First calls -validate to ensure all is well.  If that returns NO, does nothing.  Otherwise removes the sheet, panel, or window and calls the didEndSelector (if any) with the NSOKButton return code.  Subclasses shouldn't need to override this.
 
 Written by DJS 2004-01.
*/

- (IBAction)okay:(id)sender
{
    if ([self validate])
        [self endWithReturnCode:NSOKButton];
}

/*
 help:
 
 Action method for the help (?) button.  Subclasses shouldn't need to override this; instead override -helpBookPath to return the path for the help book page on the sheet.
 
 Written by DJS 2004-01.
*/

- (IBAction)help:(id)sender
{
    AHGotoPage((CFStringRef)HELP_BOOK_NAME, (CFStringRef)[self helpBookPath], NULL);
}

/*
 hit:
 
 Action method available for fields or other controls that are hit.  Simply calls -maintainButtons.  Subclasses can override this to perform any other operations required, and should call super.
 
 Written by DJS 2004-01.
*/

- (IBAction)hit:(id)sender
{
    [self maintainButtons];
}


// ----------------------------------------------------------------------------------------
#pragma mark -
#pragma mark TEXTFIELD DELEGATE METHODS
// ----------------------------------------------------------------------------------------

/*
 controlTextDidChange:
 
 Any fields with the controller set as a delegate will cause this to be called, which will in turn call -maintainButtons, e.g. to enable the okay button as soon as they enter some value.  Subclasses shouldn't need to override this.
 
 Written by DJS 2004-01.
*/

- (void)controlTextDidChange:(NSNotification *)notification
{
    [self maintainButtons];
}


// ----------------------------------------------------------------------------------------
#pragma mark -
#pragma mark USER INTERFACE ACCESSOR METHODS
// ----------------------------------------------------------------------------------------

/*
 nibName
 
 Returns the nib name to use for this sheet.  Subclasses must override this and return the appropriate nib name.
 
 Written by DJS 2004-01.
*/

+ (NSString *)nibName
{
    NSAssert(NO, @"Must override +nibName");
    
    return nil;
}

/*
 helpBookPath
 
 Returns the path of the help book page for this sheet, for use by the help (?) button.  By default returns the path of the main help page.  Subclasses should override this to provide the specific page.
 
 Written by DJS 2008-02.
*/

- (NSString *)helpBookPath;
{
    return HELP_BOOK_ROOT;
}

/*
 icon
 
 Returns the icon that is typically in the top-left corner of the sheet.
 
 Written by DJS 2004-01.
*/

- (NSImage *)icon
{
    return [iconImageView image];
}

/*
 setIcon:
 
 Changes the icon that is typically in the top-left corner of the sheet.
 
 Written by DJS 2004-01.
*/

- (void)setIcon:(NSImage *)value
{
    if (value)
        [iconImageView setImage:value];
}

/*
 prompt
 
 Returns the current value of the prompt typically at the top of the sheet.
 
 Written by DJS 2004-01.
*/

- (NSString *)prompt
{
    return [promptStat stringValue];
}

/*
 setPrompt:
 
 Sets the prompt text to the specified value.
 
 Written by DJS 2004-01.
*/

- (void)setPrompt:(NSString *)value
{
    if (value)
        [promptStat setStringValue:value];
}

/*
 cancelTitle
 
 Returns the current cancel button title.
 
 Written by DJS 2004-01.
*/

- (NSString *)cancelTitle
{
    return [cancelButton title];
}

/*
 setCancelTitle:
 
 Sets the cancel button title to the specified value.  Use this to use a custom button title.
 
 Written by DJS 2004-01.
*/

- (void)setCancelTitle:(NSString *)value
{
    if (value)
        [cancelButton setTitle:value];
}

/*
 okayTitle
 
 Returns the current okay button title.
 
 Written by DJS 2004-01.
*/

- (NSString *)okayTitle
{
    return [okayButton title];
}

/*
 setOkayTitle:
 
 Sets the okay button title to the specified value.  Use this to use a custom button title.
 
 Written by DJS 2004-01.
*/

- (void)setOkayTitle:(NSString *)value
{
    if (value)
        [okayButton setTitle:value];
}

/*
 windowTitle
 
 Returns the current window title.
 
 Written by DJS 2004-01.
*/

- (NSString *)windowTitle
{
    return [[self window] title];
}

/*
 setWindowTitle:
 
 Sets the window title to the specified value.  Use this to use a custom window title, if not using as a sheet.
 
 Written by DJS 2004-01.
*/

- (void)setWindowTitle:(NSString *)value
{
    if (value)
        [[self window] setTitle:value];
}

/*
 setAddWindowTitle:addPrompt:addOkayTitle:addIconNamed:alterWindowTitle:alterPrompt:alterOkayTitle:alterIconNamed:
 
 Convenience method to set the window title, prompt, okay button title, and icon, depending on whether altering or adding.  All parameters are optional; if nil, that value will not be changed.
 
 Written by DJS 2004-06.
*/

- (void)setAddWindowTitle:(NSString *)addWindowTitle addPrompt:(NSString *)addPrompt
             addOkayTitle:(NSString *)addOkayTitle addIconNamed:(NSString *)addIconName
         alterWindowTitle:(NSString *)alterWindowTitle alterPrompt:(NSString *)alterPrompt
           alterOkayTitle:(NSString *)alterOkayTitle alterIconNamed:(NSString *)alterIconName
{
    if ([self isAltering])
    {
        [self setWindowTitle:alterWindowTitle];
        [self setPrompt:alterPrompt];
        [self setOkayTitle:alterOkayTitle];
        
        if (alterIconName)
            [self setIcon:[NSImage imageNamed:alterIconName]];
    }
    else
    {
        [self setWindowTitle:addWindowTitle];
        [self setPrompt:addPrompt];
        [self setOkayTitle:addOkayTitle];
        
        if (addIconName)
            [self setIcon:[NSImage imageNamed:addIconName]];
    }
}


// ----------------------------------------------------------------------------------------
#pragma mark -
#pragma mark DATA ACCESSOR METHODS
// ----------------------------------------------------------------------------------------

/*
 identifier
 
 Convenience accessor available to subclasses to return an identifier string, if desired.  Initialized to nil, but not used by this abstract class.
 
 Written by DJS 2004-01.
*/

- (NSString *)identifier
{
    return _identifier;
}

/*
 setIdentifier:
 
 Convenience accessor available to subclasses to store an identifier string, if desired.  Initialized to nil, but not used by this abstract class.
 
 Written by DJS 2004-01.
*/

- (void)setIdentifier:(NSString *)value
{
    [value retain];
    [_identifier autorelease];
    _identifier = value;
}

/*
 object
 
 Convenience accessor available to subclasses to return an arbitrary object, if desired.  Initialized to nil; referred to by this abstract class for modeless operations only.
 
 Written by DJS 2004-01.
*/

- (id)object
{
    return _object;
}

/*
 setObject:
 
 Convenience accessor available to subclasses to store an arbitrary object, if desired.  Initialized to nil; referred to by this abstract class for modeless operations only.
 
 Written by DJS 2004-01.
*/

- (void)setObject:(id)value
{
    [value retain];
    [_object autorelease];
    _object = value;
}

/*
 userInfo
 
 Convenience accessor available to subclasses to return arbitrary data, if desired.  Initialized to nil, but not used by this abstract class.  Note that this is not the same as the contextInfo.
 
 Written by DJS 2004-01.
*/

- (NSMutableDictionary *)userInfo
{
    return _userInfo;
}

/*
 setUserInfo:
 
 Convenience accessor available to subclasses to store arbitrary data, if desired.  Initialized to nil, but not used by this abstract class.  Note that this is not the same as the contextInfo.
 
 Written by DJS 2004-01.
*/

- (void)setUserInfo:(NSMutableDictionary *)value
{
    [value retain];
    [_userInfo autorelease];
    _userInfo = value;
}

/*
 isAltering
 
 Returns YES if an object was passed to -beginWindowForObject:modelessDelegate:didEndSelector:contextInfo: for a modeless window.  Currently not used by this abstract class for modal sheets and panels; defaults to NO.
 
 Written by DJS 2004-06.
*/

- (BOOL)isAltering
{
    return _isAltering;
}

/*
 setAltering:
 
 Sets to YES if an object was passed to -beginWindowForObject:modelessDelegate:didEndSelector:contextInfo: for a modeless window.  Currently not used by this abstract class for modal sheets and panels; defaults to NO.
 
 Written by DJS 2004-06.
*/

- (void)setAltering:(BOOL)value
{
    _isAltering = value;
}

/*
 isModal
 
 Returns YES if the controller is for a modal sheet or panel, or NO if it is for a modeless window.  This value is read-only, and shouldn't be subclassed.
 
 Written by DJS 2004-06.
*/

- (BOOL)isModal
{
    return _isModal;
}

/*
 delegate
 
 Modal or modeless delegate.
 
 Written by DJS 2004-06.
*/

- (id)delegate
{
    return _delegate;
}

/*
 setDelegate:
 
 Modal or modeless delegate.  Automatically set by -beginSheetModalForWindow::: and -beginWindowForObject:::.
 
 Written by DJS 2004-06.
*/

- (void)setDelegate:(id)value
{
    [value retain];
    [_delegate autorelease];
    _delegate = value;
}

/*
 didEndSelector
 
 Selector of the method to invoke when the OK or Cancel button is hit.
 
 Written by DJS 2004-06.
*/

- (SEL)didEndSelector
{
    return _didEndSelector;
}

/*
 setDidEndSelector:
 
 Selector of the method to invoke when the OK or Cancel button is hit.  Automatically set by -beginSheetModalForWindow::: and -beginWindowForObject:::.
 
 Written by DJS 2004-06.
*/

- (void)setDidEndSelector:(SEL)value
{
    _didEndSelector = value;
}

/*
 contextInfo
 
 Arbitrary data passed to -beginSheetModalForWindow::: and -beginWindowForObject:::, and passed along to the did-end method.  Note that this is not the same as the userInfo.
 
 Written by DJS 2004-06.
*/

- (id)contextInfo
{
    return _contextInfo;
}

/*
 setContextInfo:
 
 Arbitrary data passed to -beginSheetModalForWindow::: and -beginWindowForObject:::, and passed along to the did-end method.  Automatically set by those methods.  Note that this is not the same as the userInfo.
 
 Written by DJS 2004-06.
*/

- (void)setContextInfo:(id)value
{
    [value retain];
    [_contextInfo autorelease];
    _contextInfo = value;
}

/*
 canRecycle
 
 Returns YES if this instance can be recycled for future sheets, or NO if it should be released after each use.  Defaults to YES.
 
 Written by DJS 2006-01.
*/

- (BOOL)canRecycle;
{
    return _canRecycle;
}

/*
 setCanRecycle:
 
 Enables or disables recycling of this instance for future sheets.
 
 Written by DJS 2006-01.
*/

- (void)setCanRecycle:(BOOL)value;
{
    _canRecycle = value;
}

@synthesize _object;
@synthesize _isModal;
@synthesize _contextInfo;
@synthesize _identifier;
@synthesize _userInfo;
@synthesize _delegate;
@end
