//
//  DSSoundController.m
//  Shared classes
//
//  Created by David Sinclair on Fri May 06 2005.
//  Copyright (c) 2005 Dejal Systems, LLC. All rights reserved.
//

#import "DSSoundController.h"
#import "DSSharedDefines.h"


@interface DSSoundController (Private)

- (void)openPanelDidEnd:(NSOpenPanel *)sheet returnCode:(int)returnCode contextInfo:(void *)contextInfo;

- (void)addFilesAtPath:(NSString *)basePath;

- (void)addBuiltInSoundNames;

- (void)playStopNotification:(NSNotification *)note;

- (void)notifyOfChanges;

@end


// ----------------------------------------------------------------------------------------
#pragma mark -
// ----------------------------------------------------------------------------------------


@implementation DSSoundController

/*
 controllerWithDictionary:popup:pathField:playButton:
 
 Convienience method to create a sound controller.  The settings dictionary may be empty or nil, as may the popup, path field, and play button.
 
 Written by DJS 2005-05.
*/

+ (id)controllerWithDictionary:(NSDictionary *)settings
                         popup:(NSPopUpButton *)popup 
                     pathField:(NSTextField *)pathField
                    playButton:(NSButton *)playButton
{
    return [[[self alloc] initWithDictionary:settings popup:popup pathField:pathField playButton:playButton] autorelease];
}

/*
 initWithDictionary:popup:pathField:playButton:
 
 Designated initializer.  The settings dictionary may be empty or nil, as may the popup, path field, and play button.
 
 Written by DJS 2005-05.
*/

- (id)initWithDictionary:(NSDictionary *)settings
                   popup:(NSPopUpButton *)popup
               pathField:(NSTextField *)pathField
              playButton:(NSButton *)playButton
{
    self = [super init];
    
    [self setSoundData:[DSSoundData dataWithDictionary:settings]];
    [self setPopup:popup];
    [self setPathField:pathField];
    [self setPlayButton:playButton];
    [self setBaseItemCount:[popup numberOfItems]];
    
    [self reload];
    
    if (playButton)
        [[NSNotificationCenter defaultCenter] addObserver:self
                                                 selector:@selector(playStopNotification:)
                                                     name:@"DSSoundPlayStopNotification"
                                                   object:[self soundData]];
    return self;
}

- (id)init
{
    return [self initWithDictionary:nil popup:nil pathField:nil playButton:nil];
}

- (void)dealloc
{
    [[self soundData] stop];
    
    [[NSNotificationCenter defaultCenter] removeObserver:self];
    
    [self setSoundData:nil];
    [self setPopup:nil];
    [self setPathField:nil];
    [self setPlayButton:nil];
    
    [super dealloc];
}

/*
 reloadWithDictionary:
 
 Sets up the various controls based on the sound data, updating them from the dictionary.
 
 Written by DJS 2005-05.
*/

- (void)reloadWithDictionary:(NSDictionary *)settings
{
    [[self soundData] reloadWithDictionary:settings];
    
    [self reload];
}

/*
 reload
 
 Sets up the various controls based on the sound data, using the current values.
 
 Written by DJS 2005-05.
*/

- (void)reload
{
    if ([self popup])
        [self addBuiltInSoundNames];
    
    int mode = [[self soundData] mode];
    
    if (mode == 5)
        [[self popup] selectItemWithRepresentedObject:[[self soundData] currentPath]];
    else
        [[self popup] selectItemAtIndex:[[self popup] indexOfItemWithTag:mode]];
    
    [[self pathField] setFieldValue:[[self soundData] otherPath]];
}


// ----------------------------------------------------------------------------------------
#pragma mark -
#pragma mark ACTION METHODS
// ----------------------------------------------------------------------------------------

/*
 changeMode
 
 Handles changing sound modes.  Call from the mode popup action method.
 
 Written by DJS 2005-05.
*/

- (void)changeModeForWindow:(NSWindow *)parentWindow
{
    int mode = [[[self popup] selectedItem] tag];
    
    if (mode == 1)
        [self openSoundForWindow:parentWindow];
    else
    {
        [[self soundData] setMode:mode];
        
        if (mode == 5)
        {
            NSMenuItem *item = [[self popup] selectedItem];
            
            [[self soundData] setCurrentPath:[item representedObject]];
        }
        
        [self notifyOfChanges];
    }
}

/*
 playStop
 
 Starts or stops playing the current sound.  Call from the play button action method.
 
 Written by DJS 2005-05.
*/

- (void)playStop
{
    [[self soundData] playStop];
}

/*
 pathChanged
 
 Updates the sound data when the path changes directly.  Call from the path field action method, if it is based on a FilePathField.
 
 Written by DJS 2005-05.
*/

- (void)pathChanged
{
    NSString *path = [[[self pathField] stringValue] abbreviatedPath];
    
    [[self soundData] setOtherPath:path];
    [[self soundData] setCurrentPath:path];
    [[self soundData] setMode:1];
    
    [[self popup] selectItemAtIndex:[[self popup] indexOfItemWithTag:1]];
    
    [self notifyOfChanges];
}

/*
 openSoundForWindow:
 
 Displays an open sheet for the specified window (which may be nil to use a panel instead).  Call from the Set button action method.
 
 Written by DJS 2005-05.
*/

- (void)openSoundForWindow:(NSWindow *)parentWindow
{
    NSOpenPanel *oPanel = [NSOpenPanel openPanel];
    NSArray *fileTypes = [NSSound soundUnfilteredFileTypes];
    NSString *path = [[self soundData] otherPath];
    NSString *directory = [path stringByDeletingLastPathComponent];
    NSString *filename = [path lastPathComponent];
    
    [oPanel setAllowsMultipleSelection:NO];
    [oPanel beginSheetForDirectory:directory
                              file:filename
                             types:fileTypes
                    modalForWindow:parentWindow
                     modalDelegate:self 
                    didEndSelector:@selector(openPanelDidEnd:returnCode:contextInfo:)
                       contextInfo:nil];
}


// ----------------------------------------------------------------------------------------
#pragma mark -
#pragma mark ACCESSOR METHODS
// ----------------------------------------------------------------------------------------

/*
 dictionary
 
 Returns a dictionary representation of the data, suitable for saving in preferences etc.
 
 Written by DJS 2005-05.
*/

- (NSDictionary *)dictionary
{
    return [[self soundData] dictionary];
}

- (DSSoundData *)soundData
{
    return _soundData;
}

- (void)setSoundData:(DSSoundData *)value
{
    [value retain];
    [_soundData autorelease];
    _soundData = value;
}

- (NSPopUpButton *)popup
{
    return _popup;
}

- (void)setPopup:(NSPopUpButton *)value
{
    [value retain];
    [_popup autorelease];
    _popup = value;
}

- (NSTextField *)pathField
{
    return _pathField;
}

- (void)setPathField:(NSTextField *)value
{
    [value retain];
    [_pathField autorelease];
    _pathField = value;
}

- (NSButton *)playButton
{
    return _playButton;
}

- (void)setPlayButton:(NSButton *)value
{
    [value retain];
    [_playButton autorelease];
    _playButton = value;
}

- (int)baseItemCount
{
    return _baseItemCount;
}

- (void)setBaseItemCount:(int)value
{
    _baseItemCount = value;
}

@end


// ----------------------------------------------------------------------------------------
#pragma mark -
// ----------------------------------------------------------------------------------------


@implementation DSSoundController (Private)

- (void)openPanelDidEnd:(NSOpenPanel *)sheet returnCode:(int)returnCode contextInfo:(void *)contextInfo
{
    if (returnCode == NSOKButton)
    {
        NSString *filename = [[sheet filename] abbreviatedPath];
        
        [[self soundData] setOtherPath:filename];
        [[self soundData] setCurrentPath:filename];
        [[self soundData] setMode:1];
        
        [[self pathField] setStringValue:filename];
        [[self popup] selectItemAtIndex:[[self popup] indexOfItemWithTag:1]];
        
        [self notifyOfChanges];
    }
    else
    {
        [[self popup] selectItemAtIndex:[[self popup] indexOfItemWithTag:[[self soundData] mode]]];
    }
}

- (void)addFilesAtPath:(NSString *)basePath
{
    NSArray *paths = [[NSFileManager defaultManager] pathsWithExtensions:[NSSound soundUnfilteredFileTypes] atPath:basePath deepScan:YES];
    
    [[self popup] addItemsWithPaths:paths prefixDivider:YES tag:5 abbreviatedObject:YES];
}

- (void)addBuiltInSoundNames
{
    while ([[self popup] numberOfItems] > [self baseItemCount])
        [[self popup] removeItemAtIndex:[self baseItemCount]];
    
    [self addFilesAtPath:[[[NSBundle mainBundle] resourcePath] stringByAppendingPathComponent:@"Sounds"]];
    [self addFilesAtPath:@"/System/Library/Sounds"];
    [self addFilesAtPath:@"/Library/Sounds"];
    [self addFilesAtPath:@"~/Library/Sounds"];
}

- (void)playStopNotification:(NSNotification *)note
{
    NSDictionary *info = [note userInfo];
    BOOL isPlaying = [info boolForKey:@"DSSoundIsPlaying"];
    
    if (isPlaying)
        [[self playButton] setImage:[NSImage imageNamed:@"SoundStop"]];
    else
        [[self playButton] setImage:[NSImage imageNamed:@"SoundPlay"]];
}

/*
 notifyOfChanges
 
 Posts a DSSoundSettingsChangedNotification notification, with self as the object and the dictionary representation of the sound data in the user info.  The latter can be saved to the preferences or whatever if desired.
 
 Written by DJS 2005-05.
*/

- (void)notifyOfChanges
{
    [[NSNotificationCenter defaultCenter] postNotificationName:@"DSSoundSettingsChangedNotification" object:self userInfo:[self dictionary]];
}

@end
