//
//  DSToolbarController.m
//  Shared classes
//
//  Created by David Sinclair on Sat Feb 21 2004.
//  Copyright (c) 2004 Dejal Systems, LLC. All rights reserved.
//

#import "DSToolbarController.h"
#import "DSSharedDefines.h"


@implementation DSToolbarController

/*
 awakeFromNib
 
 Sets up the toolbar automatically.
 
 Written by DJS 2004-02.
 Changed by DJS 2004-05 to add check if toolbar is already set up.
*/

- (void)awakeFromNib
{
    // Do nothing if the toolbar is already set up:
    if ([window toolbar])
        return;
    
    [self setToolbarItems:[NSMutableArray array]];
    
    NSToolbar *toolbar = [[NSToolbar alloc] initWithIdentifier:[self identifier]];
    
    [self setToolbar:toolbar];
    
    [toolbar setDelegate:self];
    [toolbar setAllowsUserCustomization:YES];
    [toolbar setAutosavesConfiguration:YES];
    
    [self addToolbarItems];
    
    [window setToolbar:toolbar];
}

- (void)dealloc
{
    [self setToolbar:nil];
    [self setToolbarItems:nil];
    
    [super dealloc];
}


// ----------------------------------------------------------------------------------------
#pragma mark -
#pragma mark OVERRIDE METHODS
// ----------------------------------------------------------------------------------------

/*
 addToolbarItems
 
 Adds the toolbar items.  Subclasses should override this, and call super if these standard items are acceptable, or not if not.  They can also tweak the configuration of the toolbar here, if desired (e.g. to change to only icon mode by default).
 
 Written 2004-02.
*/

- (void)addToolbarItems
{
    NSMutableArray *toolbarItems = [self toolbarItems];
    
    [toolbarItems addToolbarItem:CMD_SOFTWARE_UPDATE
                           label:NSLocalizedStringFromTable(@"Check for Updates", @"DSShared", @"short label for Check for Updates")
                    paletteLabel:NSLocalizedStringFromTable(@"Check for Updates", @"DSShared", @"long label for Check for Updates")
                         toolTip:NSLocalizedStringFromTable(@"Check for software updates", @"DSShared", @"tooltip for Check for Updates")
                      imageNamed:@"ToolbarPrefsUpdates"
                         enabled:YES
                          target:nil
                          action:@selector(checkForUpdates:)];
    
    [toolbarItems addToolbarItem:CMD_PREFERENCES
                           label:NSLocalizedStringFromTable(@"Preferences", @"DSShared", @"short label for Preferences")
                    paletteLabel:NSLocalizedStringFromTable(@"Preferences", @"DSShared", @"long label for Preferences")
                         toolTip:NSLocalizedStringFromTable(@"Show the Preferences window", @"DSShared", @"tooltip for Preferences")
                      imageNamed:@"ToolbarPrefsGeneral"
                         enabled:YES
                          target:nil
                          action:@selector(showPreferences:)];
    
    [toolbarItems addToolbarItem:CMD_FEEDBACK
                           label:NSLocalizedStringFromTable(@"Forums", @"DSShared", @"short label for Forums")
                    paletteLabel:NSLocalizedStringFromTable(@"Visit Forums", @"DSShared", @"long label for Forums")
                         toolTip:NSLocalizedStringFromTable(@"Visit the Forums", @"DSShared", @"tooltip for Forums")
                      imageNamed:@"ToolbarFeedback"
                         enabled:YES
                          target:nil
                          action:@selector(helpFeedback:)];
    
    [toolbarItems addToolbarItem:CMD_GUIDE
                           label:NSLocalizedStringFromTable(@"User Guide", @"DSShared", @"short label for User Guide")
                    paletteLabel:NSLocalizedStringFromTable(@"View User Guide", @"DSShared", @"long label for User Guide")
                         toolTip:NSLocalizedStringFromTable(@"View the User Guide in your browser", @"DSShared", @"tooltip for User Guide")
                      imageNamed:@"ToolbarGuide"
                         enabled:YES
                          target:nil
                          action:@selector(guideButtonHit:)];
}

/*
 identifier
 
 Returns the identifier to use for the toolbar.  Subclasses must override and return an appropriate string.
 
 Written 2004-02.
*/

- (NSString *)identifier
{
    NSAssert(NO, @"DSToolbarController -identifier -- Forgot to override to specify the toolbar identifier.");
    
    return @"DSToolbar";
}


// ----------------------------------------------------------------------------------------
#pragma mark -
#pragma mark UTILITY METHODS
// ----------------------------------------------------------------------------------------

/*
 toolbarIsVisible
 
 Returns YES if the toolbar is currently visible, or NO if not.
 
 Written 2004-02.
*/

- (BOOL)toolbarIsVisible
{
    return [[self toolbar] isVisible];
}


// ----------------------------------------------------------------------------------------
#pragma mark -
#pragma mark NSTOOLBAR DELEGATE METHODS
// ----------------------------------------------------------------------------------------

- (NSToolbarItem *)toolbar:(NSToolbar *)toolbar
     itemForItemIdentifier:(NSString *)itemIdentifier
 willBeInsertedIntoToolbar:(BOOL)flag
{
    return [[self toolbarItems] toolbarItemForIdentifier:itemIdentifier];
}

- (NSArray *)toolbarAllowedItemIdentifiers:(NSToolbar *)toolbar
{
    return [[self toolbarItems] toolbarItemIdentifiersIncludingStandardItems:YES];
}

/*
 toolbarDefaultItemIdentifiers:
 
 Defines the default toolbar items and order.  Subclasses should override this (probably without calling super) to have additional buttons in the default setup.
 
 Written 2004-02.
*/

- (NSArray *)toolbarDefaultItemIdentifiers:(NSToolbar *)toolbar
{
    return [NSArray arrayWithObjects:NSToolbarPrintItemIdentifier, NSToolbarSeparatorItemIdentifier, NSToolbarShowFontsItemIdentifier, NSToolbarShowColorsItemIdentifier, NSToolbarFlexibleSpaceItemIdentifier, CMD_FEEDBACK, CMD_GUIDE, nil];
}

- (void)toolbarWillAddItem:(NSNotification *)notification
{
    //    NSToolbarItem *addedItem = [[notification userInfo] objectForKey:@"item"];
    
    // set up the item here
}

- (void)toolbarDidRemoveItem:(NSNotification *)notification
{
    //    NSToolbarItem *addedItem = [[notification userInfo] objectForKey:@"item"];
    
    // clear associated info here
}

- (IBAction)showHideToolbar:(id)sender
{
    [[self toolbar] setVisible:![[self toolbar] isVisible]];
}

- (IBAction)customizeToolbar:(id)sender
{
    [[self toolbar] runCustomizationPalette:sender];
}


// ----------------------------------------------------------------------------------------
#pragma mark -
#pragma mark ACCESSOR METHODS
// ----------------------------------------------------------------------------------------

- (NSToolbar *)toolbar
{
    return _toolbar;
}

- (void)setToolbar:(NSToolbar *)value
{
    [value retain];
    [_toolbar autorelease];
    _toolbar = value;
}

- (NSMutableArray *)toolbarItems
{
    return _toolbarItems;
}

- (void)setToolbarItems:(NSMutableArray *)value
{
    [value retain];
    [_toolbarItems autorelease];
    _toolbarItems = value;
}

@end
