//
//  DSDelegateData.m
//  Shared classes
//
//  Created by David Sinclair on 2006-02-12.
//  Copyright 2006 - 2007 Dejal Systems, LLC. All rights reserved.
//

#import "DSDelegateData.h"
#import "DSSharedDefines.h"


@implementation DSDelegateData

- (id)init;
{
    self = [super init];
    
    [self setDelegate:nil];
    
    return self;
}

- (void)dealloc;
{
	[self setDelegate:nil];
    
	[super dealloc];
}

/*
 _mainThreadInformDelegate:
 
 Worker method to inform the delegate on the main thread.
 
 Written by DJS 2005-05.
 Changed by DJS 2006-02 to move from DSStreamData to DSDelgateData.
 Changed by DJS 2007-03 to support multiple objects as parameters.
*/

- (void)_mainThreadInformDelegate:(NSDictionary *)parameters
{
    NSString *signature = [parameters objectForKey:@"Signature"];
    SEL selector = NSSelectorFromString(signature);
    SEL debugSel = @selector(delegateData:sentDelegateMessage:);
    id object = [parameters objectForKey:@"Object"];
    int objectCount = [parameters integerForKey:@"ObjectCount"];
    
    if ([[self delegate] respondsToSelector:selector])
    {
        if (objectCount < 0)
        {
            NSMutableArray *arguments = [NSMutableArray arrayWithObject:self];
            [arguments addObjectsFromArray:object];
            [[self delegate] performSelector:selector withArguments:arguments];
        }
        else if (objectCount == 1)
            [[self delegate] performSelector:selector withObject:self withObject:object];
        else
            [[self delegate] performSelector:selector withObject:self];
    }
    
    if ([[self delegate] respondsToSelector:debugSel])
        [[self delegate] performSelector:debugSel withObject:self withObject:parameters];
}

/*
 informDelegateUsingSignature:
 
 If a delegate has been registered, and it responds to a method with the specified signature, it is invoked, passing self.  It is invoked on the main thread.
 
 Written by DJS 2004-07.
 Changed by DJS 2006-02 to move from DSStreamData to DSDelgateData.
 Changed by DJS 2007-03 to use ObjectCount instead of WantObject.
*/

- (void)informDelegateUsingSignature:(NSString *)signature
{
    NSDictionary *parameters = [NSDictionary dictionaryWithObjectsAndKeys:
        signature, @"Signature",
        [NSNumber numberWithInt:0], @"ObjectCount",
        nil];
    
    [self performSelectorOnMainThread:@selector(_mainThreadInformDelegate:) withObject:parameters waitUntilDone:YES];
}

/*
 informDelegateUsingSignature:withObject:
 
 If a delegate has been registered, and it responds to a method with the specified signature, it is invoked, passing self and the object.  It is invoked on the main thread.
 
 Written by DJS 2004-07.
 Changed by DJS 2006-02 to move from DSStreamData to DSDelgateData.
 Changed by DJS 2007-03 to use ObjectCount instead of WantObject.
*/

- (void)informDelegateUsingSignature:(NSString *)signature withObject:(id)object
{
    NSDictionary *parameters = [NSDictionary dictionaryWithObjectsAndKeys:
        signature, @"Signature",
        object, @"Object",
        [NSNumber numberWithInt:1], @"ObjectCount",
        nil];
    
    [self performSelectorOnMainThread:@selector(_mainThreadInformDelegate:) withObject:parameters waitUntilDone:YES];
}

/*
 informDelegateUsingSignature:withArguments:
 
 If a delegate has been registered, and it responds to a method with the specified signature, it is invoked, passing self and the objects (as individual arguments, not an array).  It's okay to pass nil for the arguments.  It is invoked on the main thread.
 
 Written by DJS 2007-03.
*/

- (void)informDelegateUsingSignature:(NSString *)signature withArguments:(NSArray *)arguments;
{
    if (![arguments count])
    {
        [self informDelegateUsingSignature:signature];
        return;
    }
    
    NSDictionary *parameters = [NSDictionary dictionaryWithObjectsAndKeys:
        signature, @"Signature",
        arguments, @"Object",
        [NSNumber numberWithInt:-1], @"ObjectCount",
        nil];
    
    [self performSelectorOnMainThread:@selector(_mainThreadInformDelegate:) withObject:parameters waitUntilDone:YES];
}


// ----------------------------------------------------------------------------------------
#pragma mark -
#pragma mark ACCESSOR METHODS
// ----------------------------------------------------------------------------------------

- (id)delegate
{
    return _delegate;
}

- (void)setDelegate:(id)value
{
    [value retain];
    [_delegate autorelease];
    _delegate = value;
}

@end
