//
//  DSImageData.m
//  Shared classes
//
//  Created by David Sinclair on Tue Jan 20 2004.
//  Copyright (c) 2004 Dejal Systems, LLC. All rights reserved.
//

#import "DSImageData.h"


@implementation DSImageData

id sharedImageData;

+ (id)sharedImageData
{
    if (!sharedImageData)
        sharedImageData = [[self alloc] init];
    
    return sharedImageData;
}

- (id)init
{
    self = [super init];
    
    [self setCache:[NSMutableDictionary dictionary]];
    [self setUserPath:nil];
    
    return self;
}

- (void)dealloc
{
    [self setCache:nil];
    [self setUserPath:nil];
    
    [super dealloc];
}


// ----------------------------------------------------------------------------------------
#pragma mark -
#pragma mark IMAGE HANDLING METHODS
// ----------------------------------------------------------------------------------------

/*
 imageNamed:forDomain:delegate:
 
 Returns an image with the specified name from the icon cache for the domain, the user data's domain folder, or the app bundle's domain folder, or the app bundle itself, searching in that order.  If not already in the cache, it is added so next time will be much faster.
 
 Written by DJS 2004-01.
 Changed by DJS 2004-02 to add the delegate.
*/

- (NSImage *)imageNamed:(NSString *)name forDomain:(NSString *)domain delegate:(id)delegate
{
    if (![name length] || ![domain length])
        return nil;
    
    NSMutableDictionary *dict = [[self cache] objectForKey:domain];
    NSImage *image = [dict objectForKey:name];
    
    // If no image with that name in the cache for that domain, try reading from disk:
    if (!image)
    {
        NSString *path = [[self userPath] stringByExpandingTildeInPath];
        
        if ([path length])
            image = [self imageNamed:name forDomain:domain fromPath:path delegate:delegate];
    }
    
    // If still no image, try reading from the app's resources:
    if (!image)
    {
        if ((image = [NSImage imageNamed:name]))
            [self setImage:image forDomain:domain withName:name persistent:NO];
    }
    
    return image;
}

/*
 imageNamed:forDomain:
 
 Returns an image with the specified name from the icon cache for the domain, the user data's domain folder, or the app bundle's domain folder, or the app bundle itself, searching in that order.  If not already in the cache, it is added so next time will be much faster.
 
 Written by DJS 2004-01.
*/

- (NSImage *)imageNamed:(NSString *)name forDomain:(NSString *)domain
{
    return [self imageNamed:name forDomain:domain delegate:nil];
}

/*
 imageNamed:forDomain:ofSize:
 
 Similar to -imageNamed:forDomain:, above, but allows specifying a sized image.  This is useful for icons.  The requested size of image will be returned, or if that size isn't available, an unsized image will be fetched and resized to the requested size, then returned.
 
 Written by DJS 2004-01.
*/

- (NSImage *)imageNamed:(NSString *)name forDomain:(NSString *)domain ofSize:(int)size
{
    if (![name length] || ![domain length] || !size)
        return nil;
    
    NSString *sizedName = [NSString stringWithFormat:@"%@-%d", name, size];
    NSImage *image = [self imageNamed:sizedName forDomain:domain];
    
    if (!image)
        image = [self imageNamed:name forDomain:domain];
    
    // Always re-set the image, to ensure that it really is of the requested size:
    if (image)
        [self setImage:image forDomain:domain withName:name size:size persistent:NO];
    
    return image;
}

/*
 imageNamed:forDomain:fromPath:delegate:
 
 Similar to -imageNamed:forDomain:, above, but always fetches the image from disk, from the domain folder within the path folder.  Normally you wouldn't call this.
 
 Written by DJS 2004-01.
 Changed by DJS 2004-02 to add the delegate.
*/

- (NSImage *)imageNamed:(NSString *)name forDomain:(NSString *)domain fromPath:(NSString *)path delegate:(id)delegate
{
    NSString *pathWithDomain = [path stringByAppendingPathComponent:domain];
    NSString *pathWithName = [pathWithDomain stringByAppendingPathComponent:name];
    NSString *extension = [self extensionForDomain:domain delegate:delegate];
    NSString *pathWithExtension = [pathWithName stringByAppendingPathExtension:extension];
    BOOL dataCameFromFileWithDelegateExtension = YES;
    
    BOOL doesExist = [[NSFileManager defaultManager] fileExistsAtPath:pathWithExtension];
    
    if (!doesExist && delegate)
    {
        // If there isn't a file with that extension (and there is a delegate to provide a different extension), try again with the standard extension:
        dataCameFromFileWithDelegateExtension = NO;
        extension = @"tiff";
        pathWithExtension = [pathWithName stringByAppendingPathExtension:extension];
        
        doesExist = [[NSFileManager defaultManager] fileExistsAtPath:pathWithExtension];
    }
    
    if (!doesExist)
    {
        // If there isn't anything in the expected location, look in the bundle for a default file (looks first in the non-localized resources, then in the localized ones):
        dataCameFromFileWithDelegateExtension = NO;
        NSString *pathInBundle = [[NSBundle mainBundle] pathForResource:domain ofType:nil];
        pathWithName = [pathInBundle stringByAppendingPathComponent:name];
        extension = @"tiff";
        pathWithExtension = [pathWithName stringByAppendingPathExtension:extension];
        
        doesExist = [[NSFileManager defaultManager] fileExistsAtPath:pathWithExtension];
    }
    
    if (!doesExist)
        return nil;
    
    NSData *data = [NSData dataWithContentsOfFile:pathWithExtension];
    
    if (!data)
        return nil;
    
    // If there's a reading delegate, and the data is relevant to the delegate, get it to decode it:
    if ([delegate respondsToSelector:@selector(imageDomain:name:dataForReadingData:)] && dataCameFromFileWithDelegateExtension)
        data = [delegate imageDomain:domain name:name dataForReadingData:data];
    
    NSImage *image = [[[NSImage alloc] initWithData:data] autorelease];
    
    [self setImage:image forDomain:domain withName:name persistent:NO];
    
    return image;
}

/*
 imageNamed:forDomain:fromPath:
 
 Similar to -imageNamed:forDomain:, above, but always fetches the image from disk, from the domain folder within the path folder.  Normally you wouldn't call this.
 
 Written by DJS 2004-01.
*/

- (NSImage *)imageNamed:(NSString *)name forDomain:(NSString *)domain fromPath:(NSString *)path
{
    return [self imageNamed:name forDomain:domain fromPath:path delegate:nil];
}

/*
 setImage:forDomain:withName:persistent:delegate:
 
 Adds or replaces a cached image.  If persistent is YES, the image is saved in the user's domain folder, otherwise it is only saved in the cache.  If a delegate is provided, it is given a chance to override some aspects.
 
 Written by DJS 2004-01.
 Changed by DJS 2004-02 to support deleting the image if nil, and add the delegate.
*/

- (void)setImage:(NSImage *)image forDomain:(NSString *)domain withName:(NSString *)name persistent:(BOOL)persistent delegate:(id)delegate
{
    if (![domain length] || ![name length])
        return;
    
    if (image)
        [image setName:name];
    
    NSMutableDictionary *dict = [[self cache] objectForKey:domain];
    
    if (!dict)
        dict = [NSMutableDictionary dictionary];
    
    if (image)
    {
        [dict setObject:image forKey:name];
        [[self cache] setObject:dict forKey:domain];
    }
    else
        [[self cache] removeObjectForKey:domain];
    
    NSString *path = [[self userPath] stringByExpandingTildeInPath];
    
    NSAssert(!persistent || [path length], @"DSImageData -setImage:forDomain:withName:persistent: no userPath set; either don't ask for persistence, or call -setUserPath prior to calling this.");
    
    if (!persistent || ![path length])
        return;
    
    if (![[NSFileManager defaultManager] fileExistsAtPath:path])
        [[NSFileManager defaultManager] createDirectoryAtPath:path attributes:nil];
    
    path = [path stringByAppendingPathComponent:domain];
    
    if (![[NSFileManager defaultManager] fileExistsAtPath:path])
        [[NSFileManager defaultManager] createDirectoryAtPath:path attributes:nil];
    
    NSString *extension = [self extensionForDomain:domain delegate:delegate];
    
    path = [[path stringByAppendingPathComponent:name] stringByAppendingPathExtension:extension];
    
    if (image)
    {
        NSData *data = [image TIFFRepresentation];
        
        if (data && [delegate respondsToSelector:@selector(imageDomain:name:dataForWritingData:)])
            data = [delegate imageDomain:domain name:name dataForWritingData:data];
        
        if (data)
            [data writeToFile:path atomically:YES];
    }
    else
    {
        if ([[NSFileManager defaultManager] fileExistsAtPath:path])
            [[NSFileManager defaultManager] removeFileAtPath:path handler:nil];
    }
}

/*
 setImage:forDomain:withName:persistent:
 
 Adds or replaces a cached image.  If persistent is YES, the image is saved in the user's domain folder, otherwise it is only saved in the cache.
 
 Written by DJS 2004-01.
*/

- (void)setImage:(NSImage *)image forDomain:(NSString *)domain withName:(NSString *)name persistent:(BOOL)persistent
{
    [self setImage:image forDomain:domain withName:name persistent:persistent delegate:nil];
}

/*
 setImage:forDomain:withName:size:persistent:
 
 As for -setImage:forDomain:withName:persistent:, above, but allows specifying a size as well.  The image is resized (with scaling) to that size, if necessary.
 
 Written by DJS 2004-01.
*/

- (void)setImage:(NSImage *)image forDomain:(NSString *)domain withName:(NSString *)name size:(int)size persistent:(BOOL)persistent
{
    NSString *sizedName = [NSString stringWithFormat:@"%@-%d", name, size];
    NSSize oldSize = [image size];
    
    if (oldSize.width != size || oldSize.height != size)
    {
        image = [[image copy] autorelease];
        
        [image setScalesWhenResized:YES];
        [image setSize:NSMakeSize(size, size)];
    }
    
    [self setImage:image forDomain:domain withName:sizedName persistent:persistent];
}

/*
 removeImageForDomain:withName:persistent:
 
 Removes a cached image.  If persistent is YES, the image is removed from the user's domain folder, otherwise it is only removed from the cache.  This is equivalent to calling -setImage:forDomain:withName:persistent: with nil for the image.
 
 Written by DJS 2004-02.
*/

- (void)removeImageForDomain:(NSString *)domain withName:(NSString *)name persistent:(BOOL)persistent
{
    [self setImage:nil forDomain:domain withName:name persistent:persistent];
}

/*
 extensionForDomain:delegate:
 
 Returns the file extension to use for the persistent storage for the specified domain, using the delegate if specified.  Normally you wouldn't need to call this.
 
 Written by DJS 2004-02.
*/

- (NSString *)extensionForDomain:(NSString *)domain delegate:(id)delegate
{
    NSString *extension;
    
    if ([delegate respondsToSelector:@selector(extensionForImageDomain:)])
        extension = [delegate extensionForImageDomain:domain];
    else
        extension = [self extensionForImageDomain:domain];
    
    return extension;
}


// ----------------------------------------------------------------------------------------
#pragma mark -
#pragma mark DELEGATE METHODS
// ----------------------------------------------------------------------------------------

/*
 extensionForImageDomain:
 
 If the delegate implements this method, it can change the file extension used for persistent image storage.
 
 Written by DJS 2004-02.
*/

- (NSString *)extensionForImageDomain:(NSString *)domain
{
    return @"tiff";
}

/*
 imageDomain:name:dataForReadingData:
 
 If the delegate implements this method, it can alter the data used when reading from the persistent image storage (e.g. to decrypt it).
 
 Written by DJS 2004-02.
*/

- (NSData *)imageDomain:(NSString *)domain name:(NSString *)name dataForReadingData:(NSData *)data
{
    return data;
}

/*
 imageDomain:name:dataForWritingData:
 
 If the delegate implements this method, it can alter the data used when writing to the persistent image storage (e.g. to encrypt it).
 
 Written by DJS 2004-02.
*/

- (NSData *)imageDomain:(NSString *)domain name:(NSString *)name dataForWritingData:(NSData *)data
{
    return data;
}


// ----------------------------------------------------------------------------------------
#pragma mark -
#pragma mark ACCESSOR METHODS
// ----------------------------------------------------------------------------------------

- (NSMutableDictionary *)cache
{
    return _cache;
}

- (void)setCache:(NSMutableDictionary *)value
{
    [value retain];
    [_cache autorelease];
    _cache = value;
}

- (NSString *)userPath
{
    return _userPath;
}

- (void)setUserPath:(NSString *)value
{
    [value retain];
    [_userPath autorelease];
    _userPath = value;
}

@end
