//
//  DSIntervalData.m
//  Shared classes
//
//  Created by David Sinclair on 2007-04-05.
//  Copyright 2007-2008 Dejal Systems, LLC. All rights reserved.
//
//  See also DSDateData.
//

#import "DSIntervalData.h"
#import "DSSharedBaseDefines.h"
#import "DSStringCategories.h"
#import "DSDictionaryCategories.h"


@implementation DSIntervalData

@synthesize usingRange = _usingRange;
@synthesize firstAmount = _firstAmount;
@synthesize secondAmount = _secondAmount;
@synthesize units = _units;
@synthesize mode = _mode;

/*
 intervalWithAmount:units:
 
 Convenience class method to return a new DSIntervalData with the specified amount and units.
 
 Written by DJS 2007-04.
 Changed by DJS 2008-07 to support ranges.
*/

+ (id)intervalWithAmount:(NSInteger)amount units:(NSInteger)units;
{
    return [[[self alloc] initWithRange:NO firstAmount:amount secondAmount:amount units:units mode:0] autorelease];
}

/*
 intervalWithAmount:units:mode:
 
 Convenience class method to return a new DSIntervalData with the specified amount, units and mode.
 
 Written by DJS 2007-04.
 Changed by DJS 2008-07 to support ranges.
*/

+ (id)intervalWithAmount:(NSInteger)amount units:(NSInteger)units mode:(NSInteger)mode;
{
    return [[[self alloc] initWithRange:NO firstAmount:amount secondAmount:amount units:units mode:mode] autorelease];
}

/*
 intervalWithRangeFirstAmount:secondAmount:units:
 
 Convenience class method to return a new DSIntervalData with the specified range of amounts and units.
 
 Written by DJS 2008-07.
*/

+ (id)intervalWithRangeFirstAmount:(NSInteger)firstAmount secondAmount:(NSInteger)secondAmount units:(NSInteger)units;
{
    return [[[self alloc] initWithRange:YES firstAmount:firstAmount secondAmount:secondAmount units:units mode:0] autorelease];
}

/*
 intervalWithDictionary:
 
 Convenience class method to return a new DSIntervalData with the amount, units and mode specified in the dictionary representation (as returned by -dictionary).
 
 Written by DJS 2007-04.
 Changed by DJS 2008-07 to support ranges.
*/

+ (id)intervalWithDictionary:(NSDictionary *)dict;
{
    if ([dict boolForKey:@"UsingRange"])
        return [self intervalWithRangeFirstAmount:[dict integerForKey:@"FirstAmount"]
                                     secondAmount:[dict integerForKey:@"SecondAmount"]
                                            units:[dict integerForKey:@"Units"]];
    else
        return [self intervalWithAmount:[dict integerForKey:@"SecondAmount"]
                                  units:[dict integerForKey:@"Units"]
                                   mode:[dict integerForKey:@"Mode"]];
}

/*
 initWithRange:firstAmount:secondAmount:units:mode:
 
 Designated initializer.
 
 Written by DJS 2007-04.
 Changed by DJS 2008-07 to use properties and support ranges.
*/

- (id)initWithRange:(BOOL)usingRange firstAmount:(NSInteger)firstAmount secondAmount:(NSInteger)secondAmount units:(NSInteger)units mode:(NSInteger)mode;
{
    if ((self = [super init]))
    {
        self.usingRange = usingRange;
        self.firstAmount = firstAmount;
        self.secondAmount = secondAmount;
        self.units = units;
        self.mode = mode;
    }
    
    return self;
}

/*
 copyWithZone:
 
 NSCopying protocol method; returns a new copy of the receiver.
 
 Written by DJS 2007-07.
 Changed by DJS 2008-07 to use properties and support ranges.
*/

- (id)copyWithZone:(NSZone *)zone;
{
    return [[[self class] allocWithZone:zone] initWithRange:self.usingRange firstAmount:self.firstAmount secondAmount:self.secondAmount units:self.units mode:self.mode];
}

/*
 description
 
 Returns a string that describes the contents of the receiver.
 
 Written by DJS 2007-04.
 Changed by DJS 2008-07 to use properties and support ranges.
*/

- (NSString *)description;
{
    if (self.usingRange)
        return [NSString stringWithFormat:@"%@ = %.2f - %.2f seconds", [self amountWithFullUnitsName], self.firstTimeInterval, self.secondTimeInterval];
    else if (self.mode)
        return [NSString stringWithFormat:@"%@ = %.2f seconds (mode %d)", [self amountWithFullUnitsName], self.amountTimeInterval, self.mode];
    else
        return [NSString stringWithFormat:@"%@ = %.2f seconds", [self amountWithFullUnitsName], self.amountTimeInterval];
}

/*
 dictionary
 
 Returns a dictionary representation of the receiver, suitable for saving in a plist etc.
 
 Written by DJS 2007-04.
 Changed by DJS 2008-07 to use properties and support ranges.
*/

- (NSDictionary *)dictionary;
{
    return [NSDictionary dictionaryWithObjectsAndKeys:
            [NSNumber numberWithBool:self.usingRange], @"UsingRange",
            [NSNumber numberWithInt:self.firstAmount], @"FirstAmount",
            [NSNumber numberWithInt:self.secondAmount], @"SecondAmount",
            [NSNumber numberWithInt:self.units], @"Units",
            [NSNumber numberWithInt:self.mode], @"Mode",
            nil];
}

/*
 amountTimeInterval
 
 Returns the receiver represented as a time interval, i.e. in seconds or fractions thereof.
 
 Written by DJS 2008-07.
*/

- (NSTimeInterval)amountTimeInterval;
{
    return self.secondTimeInterval;
}

/*
 firstTimeInterval
 
 Returns the first amount of the receiver represented as a time interval, i.e. in seconds or fractions thereof.
 
 Written by DJS 2008-07.
*/

- (NSTimeInterval)firstTimeInterval;
{
    if (self.units == UNITS_NEVER)
        return INTERVAL_NEVER;
    else if (self.units == UNITS_FOREVER)
        return INTERVAL_FOREVER;
    else
        return self.firstAmount * self.unitsTimeInterval;
}

/*
 secondTimeInterval
 
 Returns the second amount of the receiver represented as a time interval, i.e. in seconds or fractions thereof.
 
 Written by DJS 2008-07.
*/

- (NSTimeInterval)secondTimeInterval;
{
    if (self.units == UNITS_NEVER)
        return INTERVAL_NEVER;
    else if (self.units == UNITS_FOREVER)
        return INTERVAL_FOREVER;
    else
        return self.secondAmount * self.unitsTimeInterval;
}

/*
 rangeDifferenceInterval
 
 Returns the time difference between the first and second amounts, represented as a time interval, i.e. in seconds or fractions thereof.  The result is negative if the first amount is greater than the second one.
 
 Written by DJS 2008-07.
*/

- (NSTimeInterval)rangeDifferenceInterval;
{
    return self.secondTimeInterval - self.firstTimeInterval;
}

/*
 unitsTimeInterval
 
 Returns the units of the receiver represented as a time interval, i.e. in seconds or fractions thereof.  This value can be multiplied by the amounts.
 
 Written by DJS 2008-07.
*/

- (NSTimeInterval)unitsTimeInterval;
{
    switch (self.units)
    {
        case UNITS_MINUTE:
            return INTERVAL_MINUTE;
        case UNITS_HOUR:
            return INTERVAL_HOUR;
        case UNITS_DAY:
            return INTERVAL_DAY;
        case UNITS_WEEK:
            return INTERVAL_WEEK;
        case UNITS_MONTH:
            return INTERVAL_MONTH;
        case UNITS_YEAR:
            return INTERVAL_YEAR;
        case UNITS_NEVER:
            return INTERVAL_NEVER;
        case UNITS_FOREVER:
            return INTERVAL_FOREVER;
        default:
            return INTERVAL_SECOND;
    }
}

/*
 amount
 
 Accessor for the amount property.  Returns the second amount; provided as a more intuitive property name when not using a range.
 
 Written by DJS 2008-07.
*/

- (NSInteger)amount;
{
    return self.secondAmount;
}

/*
 setAmount:
 
 Accessor for the amount property.  Sets both amounts; provided as a more intuitive property name when not using a range.
 
 Written by DJS 2008-07.
*/

- (void)setAmount:(NSInteger)amount;
{
    self.firstAmount = amount;
    self.secondAmount = amount;
}

/*
 amountWithBriefUnitsName
 
 Returns a string with the amount and brief units name, e.g. "5 mins" or "1 wk".
 
 Written by DJS 2008-07.
*/

- (NSString *)amountWithBriefUnitsName;
{
    if (self.units == UNITS_NEVER || self.units == UNITS_FOREVER)
        return [self briefUnitsName];
    
    if (self.usingRange)
        return [NSString stringWithFormat:@"%d - %d %@", self.firstAmount, self.secondAmount, [self briefUnitsName]];
    else
        return [NSString stringWithFormat:@"%d %@", self.secondAmount, [self briefUnitsName]];
}

/*
 amountWithFullUnitsName
 
 Returns a string with the amount and full units name, e.g. "5 minutes" or "1 week".
 
 Written by DJS 2008-07.
*/

- (NSString *)amountWithFullUnitsName;
{
    if (self.units == UNITS_NEVER || self.units == UNITS_FOREVER)
        return [self fullUnitsName];
    
    if (self.usingRange)
        return [NSString stringWithFormat:@"%d - %d %@", self.firstAmount, self.secondAmount, [self fullUnitsName]];
    else
        return [NSString stringWithFormat:@"%d %@", self.secondAmount, [self fullUnitsName]];
}

/*
 briefUnitsName
 
 Returns a brief localized string for the units of the receiver, using singular or plural as appropriate for the amount, e.g. "min" or "wks".
 
 Written by DJS 2007-04.
 Changed by DJS 2007-07 to add forever.
 Changed by DJS 2008-07 to use properties.
*/

- (NSString *)briefUnitsName;
{
    switch (self.units)
    {
        case UNITS_MINUTE:
            return [NSString stringWithIntValue:self.secondAmount zero:nil
                                       singluar:NSLocalizedStringFromTable(@"min", @"DSShared", @"minute singular")
                                         plural:NSLocalizedStringFromTable(@"mins", @"DSShared", @"minute plural")];
        case UNITS_HOUR:
            return [NSString stringWithIntValue:self.secondAmount zero:nil
                                       singluar:NSLocalizedStringFromTable(@"hr", @"DSShared", @"hour singular")
                                         plural:NSLocalizedStringFromTable(@"hrs", @"DSShared", @"hour plural")];
        case UNITS_DAY:
            return [NSString stringWithIntValue:self.secondAmount zero:nil
                                       singluar:NSLocalizedStringFromTable(@"day", @"DSShared", @"day singular")
                                         plural:NSLocalizedStringFromTable(@"days", @"DSShared", @"day plural")];
        case UNITS_WEEK:
            return [NSString stringWithIntValue:self.secondAmount zero:nil
                                       singluar:NSLocalizedStringFromTable(@"wk", @"DSShared", @"week singular")
                                         plural:NSLocalizedStringFromTable(@"wks", @"DSShared", @"week plural")];
        case UNITS_MONTH:
            return [NSString stringWithIntValue:self.secondAmount zero:nil
                                       singluar:NSLocalizedStringFromTable(@"mth", @"DSShared", @"month singular")
                                         plural:NSLocalizedStringFromTable(@"mths", @"DSShared", @"month plural")];
        case UNITS_YEAR:
            return [NSString stringWithIntValue:self.secondAmount zero:nil
                                       singluar:NSLocalizedStringFromTable(@"yr", @"DSShared", @"year singular")
                                         plural:NSLocalizedStringFromTable(@"yrs", @"DSShared", @"year plural")];
        case UNITS_NEVER:
            return NSLocalizedStringFromTable(@"never", @"DSShared", @"never");
        
        case UNITS_FOREVER:
            return NSLocalizedStringFromTable(@"forever", @"DSShared", @"forever");
        
        default:
            return [NSString stringWithIntValue:self.secondAmount zero:nil
                                       singluar:NSLocalizedStringFromTable(@"sec", @"DSShared", @"second singular")
                                         plural:NSLocalizedStringFromTable(@"secs", @"DSShared", @"second plural")];
    }
}

/*
 fullUnitsName
 
 Returns a longer localized string for the units of the receiver, using singular or plural as appropriate for the amount, e.g. "minute" or "weeks".
 
 Written by DJS 2007-04.
 Changed by DJS 2007-07 to add forever.
 Changed by DJS 2008-07 to use properties.
*/

- (NSString *)fullUnitsName;
{
    switch (self.units)
    {
        case UNITS_MINUTE:
            return [NSString stringWithIntValue:self.secondAmount zero:nil
                                       singluar:NSLocalizedStringFromTable(@"minute", @"DSShared", @"minute singular")
                                         plural:NSLocalizedStringFromTable(@"minutes", @"DSShared", @"minute plural")];
        case UNITS_HOUR:
            return [NSString stringWithIntValue:self.secondAmount zero:nil
                                       singluar:NSLocalizedStringFromTable(@"hour", @"DSShared", @"hour singular")
                                         plural:NSLocalizedStringFromTable(@"hours", @"DSShared", @"hour plural")];
        case UNITS_DAY:
            return [NSString stringWithIntValue:self.secondAmount zero:nil
                                       singluar:NSLocalizedStringFromTable(@"day", @"DSShared", @"day singular")
                                         plural:NSLocalizedStringFromTable(@"days", @"DSShared", @"day plural")];
        case UNITS_WEEK:
            return [NSString stringWithIntValue:self.secondAmount zero:nil
                                       singluar:NSLocalizedStringFromTable(@"week", @"DSShared", @"week singular")
                                         plural:NSLocalizedStringFromTable(@"weeks", @"DSShared", @"week plural")];
        case UNITS_MONTH:
            return [NSString stringWithIntValue:self.secondAmount zero:nil
                                       singluar:NSLocalizedStringFromTable(@"month", @"DSShared", @"month singular")
                                         plural:NSLocalizedStringFromTable(@"months", @"DSShared", @"month plural")];
        case UNITS_YEAR:
            return [NSString stringWithIntValue:self.secondAmount zero:nil
                                       singluar:NSLocalizedStringFromTable(@"year", @"DSShared", @"year singular")
                                         plural:NSLocalizedStringFromTable(@"years", @"DSShared", @"year plural")];
        case UNITS_NEVER:
            return NSLocalizedStringFromTable(@"never", @"DSShared", @"never");
        
        case UNITS_FOREVER:
            return NSLocalizedStringFromTable(@"forever", @"DSShared", @"forever");
        
        default:
            return [NSString stringWithIntValue:self.secondAmount zero:nil
                                       singluar:NSLocalizedStringFromTable(@"second", @"DSShared", @"second singular")
                                         plural:NSLocalizedStringFromTable(@"seconds", @"DSShared", @"second plural")];
    }
}

@end

