//
//  DSKeyedAccessorData.m
//  Shared classes
//
//  Created by David Sinclair on Fri Feb 20 2004.
//  Copyright (c) 2004 Dejal Systems, LLC. All rights reserved.
//

#import "DSKeyedAccessorData.h"
#import "DSSharedDefines.h"


@implementation DSKeyedAccessorData

/*
 init
 
 Initializes the receiver with no existing data.  This is the designated initializer.
 
 Written by DJS 2004-02.
*/

- (id)init
{
    self = [super init];
    
    [self setDictionary:[NSDictionary dictionary]];
    
    return self;
}

- (void)dealloc
{
    [self setDictionary:nil];
    
    [super dealloc];
}

/*
 initWithDictionary:
 
 Initializes the receiver with the specified dictionary.
 
 Written by DJS 2004-02.
*/

- (id)initWithDictionary:(NSDictionary *)dict
{
    self = [self init];
    
    if (dict)
        [self setDictionary:dict];
    
    if ([self dataFormat] > [self currentFormat])
    {
        NSLog(@"The data is too new for this version of the application.  (Data format is %d, latest the app knowns about is format %d.)", [self dataFormat], [self currentFormat]);		// ¶¶ log
        
        [self autorelease];
        return nil;
    }
    
    return self;
}

/*
 copyWithZone:
 
 NSCopying protocol method to support copying the receiver.  You'd normally just call -copy.  Note that the returned instance is not autoreleased.  Note also that this class is always mutable, so a -mutableCopy method isn't needed.
 
 Written by DJS 2004-10.
*/

- (id)copyWithZone:(NSZone *)zone
{
    return [[[self class] allocWithZone:zone] initWithDictionary:[self dictionary]];
}

/*
 dictionary
 
 Returns a dictionary representation of the receiver.  Also sets the data version and format to the current values.  This is usually used when saving the data.
 
 Written 2004-02.
*/

- (NSDictionary *)dictionary
{
    [self setDataVersion:[self currentVersion]];
    [self setDataFormat:[self currentFormat]];
    
    return [self mutableDictionary];
}

/*
 mutableDictionary
 
 Returns a mutable dictionary representation of the receiver.  This is usually only used by this class and its subclasses.
 
 Written 2004-02.
*/

- (NSMutableDictionary *)mutableDictionary
{
    return _dictionary;
}

- (void)setDictionary:(NSDictionary *)value
{
    NSMutableDictionary *dict = [value mutableCopy];
    
    [_dictionary autorelease];
    _dictionary = dict;
}


// ----------------------------------------------------------------------------------------
#pragma mark -
#pragma mark VERSIONING METHODS
// ----------------------------------------------------------------------------------------

- (int)dataVersion
{
    return [self integerForKey:@"DataVersion" defaultValue:[self currentVersion]];
}

- (void)setDataVersion:(int)value
{
    [self setInteger:value forKey:@"DataVersion"];
}

- (int)dataFormat
{
    return [self integerForKey:@"DataFormat" defaultValue:[self currentFormat]];
}

- (void)setDataFormat:(int)value
{
    [self setInteger:value forKey:@"DataFormat"];
}

/*
 currentVersion
 
 Returns the current version number.  This indicates minor non-critical changes to the data, to help handle things differently when upgrading.  Subclasses may override this if necessary.
 
 Written 2004-02.
*/

- (int)currentVersion
{
    return 1;
}

/*
 currentFormat
 
 Returns the current format number.  This indicates major critical changes to the data.  Older apps will refuse to read data with format numbers greater than the current format.  Subclasses may override this if necessary.
 
 Written 2004-02.
*/

- (int)currentFormat
{
    return 1;
}


// ----------------------------------------------------------------------------------------
#pragma mark -
#pragma mark ACCESSOR METHODS
// ----------------------------------------------------------------------------------------

- (void)removeAllObjects
{
	return; // [[self mutableDictionary] removeAllObjects];
}

/*
 boolForKey:defaultValue:
 
 Returns the BOOL value from the receiver with the specified key, or the default value if that key hasn't hasn't been set yet.
 
 Written 2004-02.
*/

- (BOOL)boolForKey:(NSString *)key defaultValue:(BOOL)defaultValue
{
    NSNumber *value = [[self mutableDictionary] objectForKey:key];
    
    if (value)
        return [value boolValue];
    else
        return defaultValue;
}

/*
 setBool:forKey:
 
 Sets the key in the receiver to the specified value.
 
 Written 2004-02.
*/

- (void)setBool:(BOOL)value forKey:(NSString *)key
{
    if (key)
        [[self mutableDictionary] setObject:[NSNumber numberWithBool:value] forKey:key];
}

/*
 integerForKey:defaultValue:
 
 Returns the int value from the receiver with the specified key, or the default value if that key hasn't hasn't been set yet.
 
 Written 2004-02.
*/

- (int)integerForKey:(NSString *)key defaultValue:(int)defaultValue
{
    NSNumber *value = [[self mutableDictionary] objectForKey:key];
    
    if (value)
        return [value intValue];
    else
        return defaultValue;
}

/*
 setInteger:forKey:
 
 Sets the key in the receiver to the specified value.
 
 Written 2004-02.
*/

- (void)setInteger:(int)value forKey:(NSString *)key
{
    if (key)
        [[self mutableDictionary] setObject:[NSNumber numberWithInt:value] forKey:key];
}

/*
 floatForKey:defaultValue:
 
 Returns the float value from the receiver with the specified key, or the default value if that key hasn't hasn't been set yet.
 
 Written 2004-02.
*/

- (float)floatForKey:(NSString *)key defaultValue:(float)defaultValue
{
    NSNumber *value = [[self mutableDictionary] objectForKey:key];
    
    if (value)
        return [value floatValue];
    else
        return defaultValue;
}

/*
 setFloat:forKey:
 
 Sets the key in the receiver to the specified value.
 
 Written 2004-02.
*/

- (void)setFloat:(float)value forKey:(NSString *)key
{
    if (key)
        [[self mutableDictionary] setObject:[NSNumber numberWithFloat:value] forKey:key];
}

/*
 numberForKey:defaultValue:
 
 Returns the NSNumber value from the receiver with the specified key, or the default value if that key hasn't hasn't been set yet.
 
 Written 2004-02.
*/

- (NSNumber *)numberForKey:(NSNumber *)key defaultValue:(NSNumber *)defaultValue
{
    NSNumber *value = [[self mutableDictionary] objectForKey:key];
    
    if (value)
        return value;
    else
        return defaultValue;
}

/*
 setNumber:forKey:
 
 Sets the key in the receiver to the specified value.  If the value is nil, any old value for the key is removed.
 
 Written 2004-02.
*/

- (void)setNumber:(NSNumber *)value forKey:(NSString *)key
{
    if (key)
        [[self mutableDictionary] setObject:value forKey:key removeIfNil:YES];
}

/*
 stringForKey:defaultValue:
 
 Returns the NSString value from the receiver with the specified key, or the default value if that key hasn't hasn't been set yet.
 
 Written 2004-02.
*/

- (NSString *)stringForKey:(NSString *)key defaultValue:(NSString *)defaultValue
{
    NSString *value = [[self mutableDictionary] objectForKey:key];
    
    if (value)
        return value;
    else
        return defaultValue;
}

/*
 setString:forKey:
 
 Sets the key in the receiver to the specified value.  If the value is nil, any old value for the key is removed.
 
 Written 2004-02.
*/

- (void)setString:(NSString *)value forKey:(NSString *)key
{
    if (key)
        [[self mutableDictionary] setObject:value forKey:key removeIfNil:YES];
}

/*
 arrayForKey:defaultValue:
 
 Returns the NSMutableArray value from the receiver with the specified key, or the default value if that key hasn't hasn't been set yet.
 
 Written 2004-03.
*/

- (NSMutableArray *)arrayForKey:(NSString *)key defaultValue:(NSArray *)defaultValue
{
    NSMutableArray *value = [[self mutableDictionary] objectForKey:key];
    
    if (!value)
        value = [[defaultValue mutableCopy] autorelease];
    
    return [[value mutableCopy] autorelease];
}

/*
 setArray:forKey:
 
 Sets the key in the receiver to the specified value.  If the value is nil, any old value for the key is removed.
 
 Written 2004-03.
*/

- (void)setArray:(NSArray *)value forKey:(NSString *)key
{
    if (key)
        [[self mutableDictionary] setObject:[[value copy] autorelease] forKey:key removeIfNil:YES];
}

/*
 dictionaryForKey:defaultValue:
 
 Returns the NSMutableDictionary value from the receiver with the specified key, or the default value if that key hasn't hasn't been set yet.
 
 Written 2004-03.
*/

- (NSMutableDictionary *)dictionaryForKey:(NSString *)key defaultValue:(NSDictionary *)defaultValue
{
    NSMutableDictionary *value = [[self mutableDictionary] objectForKey:key];
    
    if (!value)
        value = [[defaultValue mutableCopy] autorelease];
    
    return [[value mutableCopy] autorelease];
}

/*
 setDictionary:forKey:
 
 Sets the key in the receiver to the specified value.  If the value is nil, any old value for the key is removed.
 
 Written 2004-03.
*/

- (void)setDictionary:(NSDictionary *)value forKey:(NSString *)key
{
    if (key)
        [[self mutableDictionary] setObject:[[value copy] autorelease] forKey:key removeIfNil:YES];
}

/*
 objectForKey:defaultValue:
 
 Returns some object value from the receiver with the specified key, or the default value if that key hasn't hasn't been set yet.
 
 Written 2004-02.
*/

- (id)objectForKey:(NSString *)key defaultValue:(id)defaultValue
{
    id value = [[self mutableDictionary] objectForKey:key];
    
    if (value)
        return value;
    else
        return defaultValue;
}

/*
 setObject:forKey:
 
 Sets the key in the receiver to the specified value.  If the value is nil, any old value for the key is removed.
 
 Written 2004-02.
*/

- (void)setObject:(id)value forKey:(NSString *)key
{
    if (key)
        [[self mutableDictionary] setObject:value forKey:key removeIfNil:YES];
}

/*
 keyedAccessorForKey:class:
 
 Returns the DSKeyedAccessorData value of the specified class from the receiver with the specified key, or a default value if that key hasn't hasn't been set yet.
 
 Written 2004-03.
*/

- (id)keyedAccessorForKey:(NSString *)key class:(Class)class
{
    NSDictionary *value = [[self mutableDictionary] objectForKey:key];
    
    if (!value)
        value = [NSDictionary dictionary];
    
    return [[[class alloc] initWithDictionary:value] autorelease];
}

/*
 setKeyedAccessor:forKey:
 
 Sets the key in the receiver to the specified value.  If the value is nil, any old value for the key is removed.
 
 Written 2004-03.
*/

- (void)setKeyedAccessor:(id)value forKey:(NSString *)key
{
    if (key)
        [[self mutableDictionary] setObject:[[[value dictionary] copy] autorelease] forKey:key removeIfNil:YES];
}

@end
