//
//  DSKeyedCollectionData.m
//  Shared classes
//
//  Created by David Sinclair on Mon Feb 09 2004.
//  Copyright (c) 2004 Dejal Systems, LLC. All rights reserved.
//

#import "DSKeyedCollectionData.h"
#import "DSSharedDefines.h"


@implementation DSKeyedCollectionData

/*
 collection
 
 Creates a new empty collection.  This doesn't need to be subclassed.
 
 Written by DJS 2004-02.
*/

+ (id)collection
{
    return [[[self alloc] init] autorelease];
}

/*
 collectionWithDictionary:forClass:
 
 Creates a new collection based on the specified dictionary, creating records based on the specified class, via the -recordWithDictionary: method.  This doesn't need to be subclassed.
 
 Written by DJS 2004-02.
*/

+ (id)collectionWithDictionary:(NSDictionary *)dict forClass:(Class)class
{
    return [[[self alloc] initWithDictionary:dict forClass:class] autorelease];
}

- (id)init
{
    self = [super init];
    
    [self setCollection:[NSMutableDictionary dictionary]];
    
    return self;
}

- (void)dealloc
{
    [_collection release];
    _collection = nil;
    
    [super dealloc];
}

/*
 initWithDictionary:forClass:

 Initializes the receiver with the specified dictionary, creating records based on the specified class, via the -recordWithDictionary: method.  Subclasses should call [super initWithDictionary:forClass:] to set the base infomation, then set their own items.
 
 Written by DJS 2004-02.
*/

- (id)initWithDictionary:(NSDictionary *)dict forClass:(Class)class
{
    if (!dict || ![class instancesRespondToSelector:@selector(initWithDictionary:)])
    {
        if (!dict)
            NSAssert(NO, @"DSKeyedCollectionData -initWithDictionary:forClass: - missing dictionary");
        else
            NSAssert(NO, @"DSKeyedCollectionData -initWithDictionary:forClass: - class instances don't respond to -initWithDictionary:");
        
        return nil;
    }
    
    self = [self init];
    
    NSEnumerator *enumerator = [dict keyEnumerator];
    NSString *identifier;
    id object = nil;
    
    while ((identifier = [enumerator nextObject]))
    {
        object = [dict objectForKey:identifier];
        
        id record = [[[class alloc] initWithDictionary:object] autorelease];
        
        [[self collection] setObject:record forKey:identifier];
    }
    
    return self;
}

/*
 dictionary
 
 Returns a dictionary representation of the receiver and its contents.  This doesn't need to be subclassed.
 
 Written by DJS 2004-02.
*/

- (NSDictionary *)dictionary
{
    return [self mutableDictionary];
}

/*
 mutableDictionary
 
 Returns a mutable dictionary representation of the receiver and its contents.  This probably doesn't need to be subclassed.
 
 Written by DJS 2004-02.
*/

- (NSMutableDictionary *)mutableDictionary
{
    NSMutableDictionary *dict = [NSMutableDictionary dictionaryWithCapacity:[self count]];
    NSEnumerator *enumerator = [self identifierEnumerator];
    NSString *identifier;
    id record = nil;
    NSDictionary *subdict = nil;
    
    while ((identifier = [enumerator nextObject]))
    {
        record = [self recordForIdentifier:identifier];
        subdict = [record dictionary];
        
        if (subdict)
            [dict setObject:subdict forKey:identifier];
    }
    
    return dict;
}


// ----------------------------------------------------------------------------------------
#pragma mark -
#pragma mark ADD/REMOVE METHODS
// ----------------------------------------------------------------------------------------

/*
 setRecord:forIdentifier:
 
 Sets the specified record with the specified identifier in the receiver.
 
 Written by DJS 2004-02.
*/

- (void)setRecord:(id)newRecord forIdentifier:(NSString *)identifier
{
    if (newRecord && [identifier length])
        [[self collection] setObject:newRecord forKey:identifier];
}

/*
 removeRecordForIdentifier:
 
 Removes the record with the specified identifier from the receiver, if present.
 
 Written by DJS 2004-02.
*/

- (void)removeRecordForIdentifier:(NSString *)identifier
{
    if (![identifier length])
        return;
    
    [[self collection] removeObjectForKey:identifier];
}

/*
 removeRecord:
 
 Removes the specified record from the receiver, if present.
 
 Written by DJS 2004-02.
*/

- (void)removeRecord:(id)oldRecord
{
    if (!oldRecord)
        return;
    
    [self removeRecordForIdentifier:[self identifierForRecord:oldRecord]];
}


// ----------------------------------------------------------------------------------------
#pragma mark -
#pragma mark CONTENTS METHODS
// ----------------------------------------------------------------------------------------

/*
 identifierForRecord:
 
 Returns the identifier of the specified record within the receiver, or nil if not found.
 
 Written by DJS 2004-02.
*/

- (NSString *)identifierForRecord:(id)record
{
    NSArray *array = [[self collection] allKeysForObject:record];
    
    if ([array count])
        return [array firstObject];
    else
        return nil;
}

/*
 recordForIdentifier:
 
 Returns the record from within the receiver with the specified identifer, or nil if not found.
 
 Written by DJS 2004-02.
*/

- (id)recordForIdentifier:(NSString *)identifier
{
    if (identifier)
        return [[self collection] objectForKey:identifier];
    else
        return nil;
}

/*
 identifiersForRecords:
 
 Given an array of records, returns an array with the identifiers for those records.
 
 Written by DJS 2004-02.
*/

- (NSArray *)identifiersForRecords:(NSArray *)records
{
    NSMutableArray *identifiers = [NSMutableArray arrayWithCapacity:[records count]];
    NSEnumerator *enumerator = [records objectEnumerator];
    id record;
    
    while ((record = [enumerator nextObject]))
    {
        [identifiers addObject:[record identifier]];
    }
    
    return identifiers;
}

/*
 recordsForIdentifiers:
 
 Given an array of identifiers, returns an array with the corresponding records.
 
 Written by DJS 2004-02.
*/

- (NSArray *)recordsForIdentifiers:(NSArray *)identifiers
{
    NSMutableArray *records = [NSMutableArray arrayWithCapacity:[identifiers count]];
    NSEnumerator *enumerator = [identifiers objectEnumerator];
    NSString *identifier;
    
    while ((identifier = [enumerator nextObject]))
    {
        [records addObject:[self recordForIdentifier:identifier]];
    }
    
    return records;
}

/*
 allIdentifiers
 
 Returns an array containing all identifiers within the receiver, in no particular order.
 
 Written by DJS 2004-02.
*/

- (NSArray *)allIdentifiers
{
    return [[self collection] allKeys];
}

/*
 allRecords
 
 Returns an array containing all records within the receiver, in no particular order.
 
 Written by DJS 2004-02.
*/

- (NSArray *)allRecords
{
    return [[self collection] allValues];
}

/*
 identifierEnumerator
 
 Returns an enumerator of the identifiers within the receiver.
 
 Written by DJS 2004-02.
*/

- (NSEnumerator *)identifierEnumerator
{
    return [[self collection] keyEnumerator];
}

/*
 objectEnumerator
 
 Returns an enumerator of the records within the receiver.
 
 Written by DJS 2004-02.
*/

- (NSEnumerator *)recordEnumerator
{
    return [[self collection] objectEnumerator];
}

/*
 count
 
 Returns the number of records within this collection.
 
 Written by DJS 2004-02.
*/

- (int)count
{
    return [[self collection] count];
}


// ----------------------------------------------------------------------------------------
#pragma mark -
#pragma mark ACCESSOR METHODS
// ----------------------------------------------------------------------------------------

- (NSMutableDictionary *)collection
{
    return _collection;
}

- (void)setCollection:(NSMutableDictionary *)dict
{
    [dict retain];
    [_collection autorelease];
    _collection = dict;
}

/*
 description
 
 Useful for debugging.  Override and include [super description] in subclasses to add more information.
 
 Written by DJS 2004-01.
*/

- (NSString *)description
{
    return [NSString stringWithFormat:@"%@: contains %d records", [self className], [self count]];
}

@end
