//
//  DSKeyedRecordData.m
//  Shared classes
//
//  Created by David Sinclair on Mon Feb 09 2004.
//  Copyright (c) 2004 Dejal Systems, LLC. All rights reserved.
//

#import "DSKeyedRecordData.h"
#import "DSSharedDefines.h"


@implementation DSKeyedRecordData

/*
 record
 
 Creates a new default record.  This doesn't need to be subclassed.
 
 Written by DJS 2004-02.
*/

+ (id)record
{
    return [[[self alloc] init] autorelease];
}

/*
 recordWithDictionary:
 
 Creates a new record based on the specified dictionary.  This doesn't need to be subclassed.
 
 Written by DJS 2004-02.
*/

+ (id)recordWithDictionary:(NSDictionary *)dict
{
    return [[[self alloc] initWithDictionary:dict] autorelease];
}

- (id)init
{
    self = [super init];
    
    [self setIdentifier:[NSString universallyUniqueIdentifier]];
    [self setName:@""];
    [self setCreatedDate:[NSDate date]];
    [self setModifiedDate:[self createdDate]];
    [self setViewedDate:[self createdDate]];
    [self setCreatedBy:NSFullUserName()];
    [self setModifiedBy:[self createdBy]];
    [self setViewedBy:[self createdBy]];
    [self setSavedData:nil];
    
    return self;
}

- (void)dealloc
{
    [self setIdentifier:nil];
    [self setName:nil];
    [self setCreatedDate:nil];
    [self setModifiedDate:nil];
    [self setViewedDate:nil];
    [self setCreatedBy:nil];
    [self setModifiedBy:nil];
    [self setViewedBy:nil];
    [self setSavedData:nil];
    
    [super dealloc];
}

/*
 initWithDictionary:

 Initializes the receiver with the specified dictionary.  Subclasses should call [super initWithDictionary:] to set the base infomation, then set their own items.
 
 Written by DJS 2004-02.
*/

- (id)initWithDictionary:(NSDictionary *)dict
{
    self = [self init];

    [self setSavedData:dict];

    // The dictionary may be missing some keys, so only set those that are valid; defaults for all iVars have already been set via init:
    [self set:&_identifier to:[dict objectForKey:@"RecordIdentifier"] nilAllowed:NO];
    [self set:&_name to:[dict objectForKey:@"RecordName"] nilAllowed:NO];
    [self set:&_createdDate to:[dict objectForKey:@"RecordCreatedDate"] nilAllowed:NO];
    [self set:&_modifiedDate to:[dict objectForKey:@"RecordModifiedDate"] nilAllowed:NO];
    [self set:&_viewedDate to:[dict objectForKey:@"RecordViewedDate"] nilAllowed:NO];
    [self set:&_createdBy to:[dict objectForKey:@"RecordCreatedBy"] nilAllowed:NO];
    [self set:&_modifiedBy to:[dict objectForKey:@"RecordModifiedBy"] nilAllowed:NO];
    [self set:&_viewedBy to:[dict objectForKey:@"RecordViewedBy"] nilAllowed:NO];
    
    return self;
}

/*
 dictionary
 
 Returns a dictionary representation of the receiver.  Subclasses shouldn't need to override this.
 
 Written by DJS 2004-02.
*/

- (NSDictionary *)dictionary
{
    return [self mutableDictionary];
}

/*
 mutableDictionary
 
 Returns a mutable dictionary representation of the receiver.  Subclasses should override this to add additional items (after calling [super mutableDictionary]).
 
 Written by DJS 2004-02.
*/

- (NSMutableDictionary *)mutableDictionary
{
    NSMutableDictionary *dict = [[[self savedData] mutableCopy] autorelease];

    if (!dict)
        dict = [NSMutableDictionary dictionary];

    // Only need to write out the current keys, since the old values will be preserved in the old keys.  Unless a new value directly maps to an old one, and you want to use that old value if they downgrade.  Note that any new records they add in a new version will be missing important data when downgrading, if that data isn't retrosaved to the old keys.

    // Remove these from the saved dict before adding them again, so obsolete entries within them aren't kept:
//    [dict removeObjectsForKeys:[NSArray arrayWithObjects:xxx, nil]];
    
    [dict setObject:[self identifier] forKey:@"RecordIdentifier"];
    [dict setObject:[self name] forKey:@"RecordName"];
    [dict setObject:[self createdDate] forKey:@"RecordCreatedDate"];
    [dict setObject:[self modifiedDate] forKey:@"RecordModifiedDate"];
    [dict setObject:[self viewedDate] forKey:@"RecordViewedDate"];
    [dict setObject:[self createdBy] forKey:@"RecordCreatedBy"];
    [dict setObject:[self modifiedBy] forKey:@"RecordModifiedBy"];
    [dict setObject:[self viewedBy] forKey:@"RecordViewedBy"];
    
    return dict;
}


// ----------------------------------------------------------------------------------------
#pragma mark -
#pragma mark ADD/REMOVE METHODS
// ----------------------------------------------------------------------------------------

/*
 modified
 
 Marks the receiver as being modified; the modified date and user name are updated.
 
 Written 2004-02.
*/

- (void)modified
{
    [self setModifiedDate:[NSDate date]];
    [self setModifiedBy:NSFullUserName()];
}

/*
 viewed
 
 Marks the receiver as being viewed; the viewed date and user name are updated.
 
 Written 2004-02.
*/

- (void)viewed
{
    [self setViewedDate:[NSDate date]];
    [self setViewedBy:NSFullUserName()];
}


// ----------------------------------------------------------------------------------------
#pragma mark -
#pragma mark ACCESSOR METHODS
// ----------------------------------------------------------------------------------------

- (NSString *)identifier
{
    return _identifier;
}

- (void)setIdentifier:(NSString *)value
{
    [value retain];
    [_identifier autorelease];
    _identifier = value;
}

- (NSString *)name
{
    return _name;
}

- (void)setName:(NSString *)value
{
    [value retain];
    [_name autorelease];
    _name = value;
}

- (NSDate *)createdDate
{
    return _createdDate;
}

- (void)setCreatedDate:(NSDate *)value
{
    [value retain];
    [_createdDate autorelease];
    _createdDate = value;
}

- (NSDate *)modifiedDate
{
    return _modifiedDate;
}

- (void)setModifiedDate:(NSDate *)value
{
    [value retain];
    [_modifiedDate autorelease];
    _modifiedDate = value;
}

- (NSDate *)viewedDate
{
    return _viewedDate;
}

- (void)setViewedDate:(NSDate *)value
{
    [value retain];
    [_viewedDate autorelease];
    _viewedDate = value;
}

- (NSString *)createdBy
{
    return _createdBy;
}

- (void)setCreatedBy:(NSString *)value
{
    [value retain];
    [_createdBy autorelease];
    _createdBy = value;
}

- (NSString *)modifiedBy
{
    return _modifiedBy;
}

- (void)setModifiedBy:(NSString *)value
{
    [value retain];
    [_modifiedBy autorelease];
    _modifiedBy = value;
}

- (NSString *)viewedBy
{
    return _viewedBy;
}

- (void)setViewedBy:(NSString *)value
{
    [value retain];
    [_viewedBy autorelease];
    _viewedBy = value;
}

- (NSDictionary *)savedData
{
    return _savedData;
}

- (void)setSavedData:(NSDictionary *)value
{
    [self set:&_savedData to:value];
}


/*
 compare:
 
 Returns a comparison result based on the receiver and the specified record.  Subclasses should override this for more meaningful comparisons.
 
 Written 2004-02.
 */

- (NSComparisonResult)compare:(id)record
{
    return [[self description] compare:[record description]];
}


/*
 searchableString
 
 A string containing all the relevant keywords from the data, for indexing and searching via the -searchData.  Override and include super in subclasses to add more information.
 
 Written 2004-02.
*/

- (NSString *)searchableString
{
    return [self name];
}

/*
 description
 
 Useful for debugging.  Override and include [super description] in subclasses to add more information.
 
 Written 2004-02.
*/

- (NSString *)description
{
    return [NSString stringWithFormat:@"%@ %@: name = %@", [self className], [self identifier], [self name]];
}

@end
