/*
 *  DSResourceForkData.m
 *  Shared
 *
 *  Created by nathan on Wed Dec 05 2001.
 *  Co-opted and modified by David Sinclair on Thu Mar 17 2003.
 *  Copyright (c) 2001 Nathan Day. All rights reserved.
 *
 *	Currently ResourceFork will not add resource forks to files
 *	or create new files with resource forks
 *
 */

#import "DSResourceForkData.h"
#import "DSCarbonCategories.h"


@implementation DSResourceForkData

/*
 +resourceForkContentsAtPath:

 Similar to NSFileManager -contentsAtPath:, this reads the entire contents of the resource fork of the specified file, and returns as a NSData.  If the file doesn't have a resource fork, it returns nil.
*/

+ (NSData *)resourceForkContentsAtPath:(NSString *)path
{
    if (![[NSFileManager defaultManager] fileExistsAtPath:path])
        return nil;

    NSMutableData *data = nil;
    HFSUniStr255 forkName;
    FSRef theFsRef;
    short rsrcRef = 0;
    signed long long count = 0;
    OSErr err, ignored;

    [path getFSRef:&theFsRef];

    err = FSGetResourceForkName(&forkName);

    if (!err)
        err = FSOpenFork(&theFsRef, forkName.length, forkName.unicode, fsRdPerm, &rsrcRef);

    if (!err)
        err = FSGetForkSize(rsrcRef, &count);

    if (!err)
        data = [NSMutableData dataWithLength:count];

    if (!err)
        err = FSReadFork(rsrcRef, fsFromStart, 0, count, [data mutableBytes], NULL);

    if (rsrcRef)
        ignored = FSCloseFork(rsrcRef);

    return data;
}

/*
 +createResourceForkAtPath:contents:attributes:

 Similar to NSFileManager -createFileAtPath:contents:attributes:, this creates the file if it doesn't already exist, then saves the contents data to the resource fork of the file.  Returns YES if successful.
*/

+ (BOOL)createResourceForkAtPath:(NSString *)path contents:(NSData *)contents
                      attributes:(NSDictionary *)attributes
{
    if (![[NSFileManager defaultManager] fileExistsAtPath:path])
        [[NSFileManager defaultManager] createFileAtPath:path contents:nil attributes:attributes];

    HFSUniStr255 forkName;
    FSRef theFsRef;
    short rsrcRef = 0;
    ByteCount count = [contents length];
    OSErr err, ignored;
    
    [path getFSRef:&theFsRef];
    
//    err = FSpOpenRF(&(FSSpec)theFsRef, fsWrPerm, &rsrcRef);

    err = FSGetResourceForkName(&forkName);

    if (!err)
        err = FSOpenFork(&theFsRef, forkName.length, forkName.unicode, fsWrPerm, &rsrcRef);
    
    if (!err)
        err = FSWriteFork(rsrcRef, fsFromStart, 0, count, [contents bytes], &count);
    
//        err = FSWrite(rsrcRef, &count, [contents bytes]);

    if (rsrcRef)
        ignored = FSCloseFork(rsrcRef);

    return err ? NO : YES;
}


/*
 * resourceForkForReadingAtPath:
*/

+ (id)resourceForkForReadingAtPath:(NSString *)aPath
{
    return [[[self alloc] initForReadingAtPath:aPath] autorelease];
}

/*
 * resourceForkForWritingAtPath:
*/

+ (id)resourceForkForWritingAtPath:(NSString *)aPath
{
    return [[[self alloc] initForWritingAtPath:aPath] autorelease];
}

- (id)initForPermission:(char)aPermission atPath:(NSString *)aPath
{
    OSErr theError = !noErr;
    FSRef theFsRef, theParentFsRef;

    if ((self = [self init]))
    {
        fileReference = 0;
        
        /*
         * if write permission then create resource fork
         */
        if( (aPermission & 0x06) != 0 )		// if write permission
        {
            if ( [[aPath stringByDeletingLastPathComponent] getFSRef:&theParentFsRef] )
            {
                unsigned int theNameLength;
                unichar theUnicodeName[ PATH_MAX ];
                NSString *theName;

                theName = [aPath lastPathComponent];
                theNameLength = [theName length];

                if( theNameLength <= PATH_MAX )
                {
                    [theName getCharacters:theUnicodeName range:NSMakeRange(0,theNameLength)];

                    FSCreateResFile( &theParentFsRef, theNameLength, theUnicodeName, 0, NULL, NULL, NULL );		// doesn't replace if already exists

                    theError =  ResError( );

                    if( theError == noErr || theError == dupFNErr )
                    {
                        [aPath getFSRef:&theFsRef];
                        fileReference = FSOpenResFile ( &theFsRef, aPermission );
                        theError = fileReference > 0 ? ResError( ) : !noErr;
                    }
                }
                else
                    theError = !noErr;
            }
        }
        else		// dont have write permission
        {
            [aPath getFSRef:&theFsRef];
            fileReference = FSOpenResFile ( &theFsRef, aPermission );
            theError = fileReference > 0 ? ResError( ) : !noErr;
        }

    }

    if( noErr != theError && theError != dupFNErr )
    {
        [self release];
        self = nil;
    }

    return self;
}

/*
 * initForReadingAtPath:
*/

- (id)initForReadingAtPath:(NSString *)aPath
{
    if( [[NSFileManager defaultManager] fileExistsAtPath:aPath] )
        return [self initForPermission:fsRdPerm atPath:aPath];
    else
        return nil;
}

/*
 * initForWritingAtPath:
*/

- (id)initForWritingAtPath:(NSString *)aPath
{
    return [self initForPermission:fsWrPerm atPath:aPath];
}

/*
 * dealloc
*/

- (void)dealloc
{
    if (fileReference)
        CloseResFile(fileReference);
    
    fileReference = 0;
    
    [super dealloc];
}


/*
 resourceForkSizeAtPath:
 
 Given a path to a file with a resource fork, this returns the size of the resource fork.
 
 Written by DJS 2003-08.
*/

+ (long)resourceForkSizeAtPath:(NSString *)path
{
    if (![[NSFileManager defaultManager] fileExistsAtPath:path])
        return 0;

    HFSUniStr255 forkName;
    FSRef theFsRef;
    short rsrcRef = 0;
    signed long long count = 0;
    OSErr err, ignored;

    [path getFSRef:&theFsRef];

    err = FSGetResourceForkName(&forkName);

    if (!err)
        err = FSOpenFork(&theFsRef, forkName.length, forkName.unicode, fsRdPerm, &rsrcRef);

    if (!err)
        err = FSGetForkSize(rsrcRef, &count);

    if (rsrcRef)
        ignored = FSCloseFork(rsrcRef);

    return count;
}


- (BOOL)addData:(NSData *)aData type:(ResType)aType withID:(short)anID name:(NSString *)aName
{
    Handle theResHandle;

    if( [self removeType:aType withID:anID] )
    {
        short thePreviousRefNum;

        thePreviousRefNum = CurResFile();	// save current resource
        UseResFile( fileReference );    			// set this resource to be current

        // copy NSData's bytes to a handle
        if ( noErr == PtrToHand ( [aData bytes], &theResHandle, [aData length] ) )
        {
            Str255 thePName;

//            CopyCStringToPascal ( [aName lossyCString], thePName );
            
            CFStringGetPascalString((CFStringRef)aName, (StringPtr)&thePName, 255, kCFStringEncodingMacRoman);

            HLock( theResHandle );
            AddResource( theResHandle, aType, anID, thePName );
            HUnlock( theResHandle );

            UseResFile( thePreviousRefNum );     		// reset back to resource previously set

            //			DisposeHandle( theResHandle );
            return ( ResError( ) == noErr );
        }
    }

    return NO;
}

/*
 * dataForType:withID:
*/

- (NSData *)dataForType:(ResType)aType withID:(short)anID
{
    NSData *theData = nil;
    Handle theResHandle;
    short thePreviousRefNum;

    thePreviousRefNum = CurResFile();	// save current resource

    UseResFile( fileReference );    		// set this resource to be current

    if( noErr ==  ResError( ) )
    {
        theResHandle = Get1Resource( aType, anID );

        if( theResHandle && noErr ==  ResError( ) )
        {
            HLock(theResHandle);
            theData = [NSData dataWithBytes:*theResHandle length:GetHandleSize( theResHandle )];
            HUnlock(theResHandle);
        }

        if ( theResHandle )
            ReleaseResource( theResHandle );
    }

    UseResFile( thePreviousRefNum );     		// reset back to resource previously set

    return theData;
}

/*
 * -addString:type:withID:name:
 *		adds a string to the resource fork as a pascal string
*/

- (BOOL)addString:(NSString *)aString type:(ResType)aType withID:(short)anID name:(NSString *)aName
{
    unsigned int theLength;

    theLength = [aString length];

    if( theLength < 256 )
    {
        NSMutableData *theData;

        theData = [NSMutableData dataWithLength:1];
        *((char*)[theData mutableBytes]) = (char)theLength;
        [theData appendData:[aString dataUsingEncoding:NSASCIIStringEncoding allowLossyConversion:YES]];
        return [self addData:theData type:aType withID:anID name:aName];
    }
    else
        return NO;
}

/*
 * -stringForType:withID:
 */
- (NSString *)stringForType:(ResType)aType withID:(short)anID
{
    NSData *theData;

    theData = [self dataForType:aType withID:anID];

    return theData ? [NSString stringWithCString:([theData bytes] + 1) length:[theData length] - 1] : nil;
}

/*
 * removeType:withID:
*/

- (BOOL)removeType:(ResType)aType withID:(short)anID
{
    Handle theResHandle;
    OSErr theErr;

    UseResFile( fileReference );    			// set this resource to be current

    theResHandle = Get1Resource( aType, anID );
    theErr = ResError( );

    if( theResHandle && theErr == noErr )
    {
        RemoveResource( theResHandle );		// Disposed of in current resource file
        theErr = ResError( );
    }

    return theErr == noErr;
}

@end
