//
//  DSSearchData.m
//  Shared classes
//
//  Created by David Sinclair on Sat Dec 06 2003.
//  Copyright (c) 2003 - 2004 Dejal Systems, LLC. All rights reserved.
//

#import "DSSearchData.h"


@implementation DSSearchData

/*
 searchDataWithBuffer:
 
 Creates a new autoreleased root-level search data object, and creates an index if the buffer is nil, or opens the index if the buffer is non-nil.
 
 Written by DJS 2004-06.
*/

+ (id)searchDataWithBuffer:(NSMutableData *)buffer
{
    return [[[self alloc] initWithBuffer:buffer] autorelease];
}

/*
 searchDataWithPath:
 
 Creates a new autoreleased root-level search data object, and creates or opens the index at the specified path.
 
 Written by DJS 2003-12.
*/

+ (id)searchDataWithPath:(NSString *)path
{
    return [[[self alloc] initWithPath:path] autorelease];
}

/*
 searchDataWithIdentifier:string:parentSearchData:
 
 Creates a new autoreleased search node for the specified values.
 
 Written by DJS 2003-12.
*/

+ (id)searchDataWithIdentifier:(NSString *)identifier string:(NSString *)searchableString parentSearchData:(id)parentSearchData
{
    return [[[self alloc] initWithIdentifier:identifier string:searchableString parentSearchData:parentSearchData] autorelease];
}

/*
 initWithBuffer:
 
 Initializes a root-level search data object, and creates an index if the buffer is nil, or opens the index if the buffer is non-nil.  This is a designated initializer.
 
 Written by DJS 2004-06.
*/

- (id)initWithBuffer:(NSMutableData *)buffer
{
    self = [super init];
    
//        SKLoadDefaultExtractorPlugIns();
    
    // This is the root, so set self as such:
    [self setRoot:self];
    
    SKIndexRef index = nil;
    BOOL exists = buffer != nil;
    
    if (!exists)
        buffer = [NSMutableData data];
    
    [self setSearchKitBuffer:buffer];
    
    if (exists)
        index = SKIndexOpenWithMutableData((CFMutableDataRef)[self searchKitBuffer], NULL);
    else
        index = SKIndexCreateWithMutableData((CFMutableDataRef)[self searchKitBuffer], NULL, kSKIndexInverted, NULL);
    
    [self setSearchKitIndex:index];
    
    CFRelease(index);
    
    return self;
}

/*
 initWithPath:
 
 Initializes a root-level search data object, and creates or opens the index at the specified path.  This is a designated initializer.
 
 Written by DJS 2003-12.
*/

- (id)initWithPath:(NSString *)path
{
    self = [self init];
    
//        SKLoadDefaultExtractorPlugIns();
    
    // This is the root, so set self as such:
    [self setRoot:self];
    
    NSURL *url = nil;
    SKIndexRef index;
    
    if (path)
        url = [NSURL fileURLWithPath:path];
    
    if (url && [[NSFileManager defaultManager] fileExistsAtPath:path])
        index = SKIndexOpenWithURL((CFURLRef)url, NULL, true);
    else
        index = SKIndexCreateWithURL((CFURLRef)url, NULL, kSKIndexInverted, NULL);
    
    [self setSearchKitIndex:index];
    
    CFRelease(index);
    
    return self;
}

/*
 initWithIdentifier:string:parentSearchData:
 
 Initializes a search node for the specified values.  This is a designated initializer.
 
 Written by DJS 2003-12.
*/

- (id)initWithIdentifier:(NSString *)identifier string:(NSString *)searchableString parentSearchData:(id)parentSearchData
{
    self = [self init];
    
    [self setRoot:[parentSearchData root]];
    [self setSearchKitIndex:[parentSearchData searchKitIndex]];
    
    SKDocumentRef doc = SKDocumentCreate((CFStringRef)@"DSSearchData", [parentSearchData searchKitDocument], (CFStringRef)identifier);
    
    [self setSearchKitDocument:doc];
    
    CFRelease(doc);
    
    [self updateWithString:searchableString];
    
//    NSLog(@"doc retain count = %d; doc count = %d; root doc count = %d", CFGetRetainCount(doc), SKIndexGetDocumentCount([self searchKitIndex]), SKIndexGetDocumentCount([[self root] searchKitIndex]));		// ¶¶ log
    
//    [[self root] searchForString:@"dejal"];            // ¶¶ hack
    
    return self;
}

- (id)init
{
    self = [super init];
    
    [self setRoot:nil];
    [self setSearchKitBuffer:nil];
    [self setSearchKitIndex:nil];
    [self setSearchKitDocument:nil];
    
    return self;
}

- (void)dealloc
{
    // Auto-remove from the search tree, if needed:
    [self remove];
    
    [self setSearchKitDocument:nil];
    [self setSearchKitIndex:nil];
    [self setSearchKitBuffer:nil];
    [self setRoot:nil];
    
    [super dealloc];
}


/*
 flush
 
 Write out the index data to disk.
 
 Written 2003-12.
*/

- (void)flush
{
    SKIndexFlush([self searchKitIndex]);
}

/*
 remove
 
 Remove the search node from the index.  No need to call this normally, as it is done automatically when the receiver is released.
 
 Written 2003-12.
*/

- (void)remove
{
    if ([self searchKitIndex] && [self searchKitDocument])
    {
        SKIndexRemoveDocument([self searchKitIndex], [self searchKitDocument]);
        
        [self setSearchKitDocument:nil];
    }
}

/*
 updateWithString:
 
 Update the index to use the specified string.  Call this after the relevant data changes.
 
 Written 2003-12.
*/

- (void)updateWithString:(NSString *)searchableString
{
    if (!searchableString)
        searchableString = @"";
    
    if (![self searchKitIndex] || ![self searchKitDocument])
        return;
    
    SKIndexAddDocumentWithText([self searchKitIndex], [self searchKitDocument], (CFStringRef)searchableString, true);
    
    [self flush];
}

/*
 searchForString:
 
 Perform a search operation for the specified string.  Returns an array of identifiers of matches, or nil if no matches.
 
 Written 2003-12.
*/

- (NSArray *)searchForString:(NSString *)searchString
{
    [self flush];
    
//    NSLog(@"search string = %@; doc count = %d", searchString, SKIndexGetDocumentCount([self searchKitIndex]));		// ¶¶ log
    
    CFMutableArrayRef indexArray = CFArrayCreateMutable(kCFAllocatorDefault, 1, NULL);
    CFArrayAppendValue(indexArray, [self searchKitIndex]);
    SKSearchGroupRef searchGroup = SKSearchGroupCreate(indexArray);
    SKSearchResultsRef searchResults = SKSearchResultsCreateWithQuery(searchGroup, (CFStringRef)searchString, kSKSearchPrefixRanked, 100, NULL, NULL);
    CFIndex count = SKSearchResultsGetCount(searchResults);
    SKDocumentRef documentsArray[100];
    
//    NSLog(@"search = %@; count = %d; doc count = %d", searchString, count, SKIndexGetDocumentCount([self searchKitIndex]));		// ¶¶ log
    
    
    count = SKSearchResultsGetInfoInRange(searchResults, CFRangeMake(0, count), documentsArray, NULL, NULL);
    
    // ¶¶ to be implemented
    
//    NSLog(@"..new count = %d", count);		// ¶¶ log
    
    NSMutableArray *matches = [NSMutableArray arrayWithCapacity:count];
    NSString *identifier;
    int i;
    
    for (i = 0; i < count; i++)
    {
        identifier = (NSString *)SKDocumentGetName(documentsArray[i]);
        
        if ([identifier length])
            [matches addObject:identifier];
    }
    
    CFRelease(searchResults);
    
    return matches;
}


// ----------------------------------------------------------------------------------------
#pragma mark -
#pragma mark ACCESSOR METHODS
// ----------------------------------------------------------------------------------------

- (id)root
{
    return _root;
}

- (void)setRoot:(id)value
{
    [value retain];
    [_root autorelease];
    _root = value;
}

- (NSMutableData *)searchKitBuffer
{
    return _searchKitBuffer;
}

- (void)setSearchKitBuffer:(NSMutableData *)value
{
    [value retain];
    [_searchKitBuffer autorelease];
    _searchKitBuffer = value;
}

- (SKIndexRef)searchKitIndex
{
    return _searchKitIndex;
}

- (void)setSearchKitIndex:(SKIndexRef)value
{
    if (value)
        CFRetain(value);
    
    if (_searchKitIndex)
        CFRelease(_searchKitIndex);
    
    _searchKitIndex = value;
}

- (SKDocumentRef)searchKitDocument
{
    return _searchKitDocument;
}

- (void)setSearchKitDocument:(SKDocumentRef)value
{
    if (value)
        CFRetain(value);
    
    if (_searchKitDocument)
        CFRelease(_searchKitDocument);
    
    _searchKitDocument = value;
}

@end
