//
//  DSSocketStreamData.m
//  Shared classes
//
//  Created by David Sinclair on Thu Jul 01 2004.
//  Copyright (c) 2004 - 2006 Dejal Systems, LLC. All rights reserved.
//

#import "DSSocketStreamData.h"
#import "DSSharedDefines.h"


@implementation DSSocketStreamData

/*
 connectToHost:onPort:forDelegate:
 
 Establishes a connection to the specified host (domain or IP address) on the specified port, and registers the delegate to receive information about the session.  The delegate is not retained.  If the delegate implements -streamDataConnecting:, it is invoked.  If it implements -streamDataShouldUseSameThread:, it is invoked to ask whether to connect on the same thread; by default it will use a new thread.
 
 Written by DJS 2005-05.
 Changed by DJS 2005-11 to use a DSStreamData abstract superclass.
 Changed by DJS 2007-08 to ask the delegate if it wants to use the same thread.
*/

- (void)connectToHost:(NSString *)host onPort:(int)port forDelegate:(id)delegate
{
    // Must set the delegate first, since the -informDelegateUsingSignature:withObject: method uses it:
    [self setDelegate:delegate];
    
    if (![host length])
    {
        [self informDelegateOfError:@"Can't connect as no host domain or address was specified" terminate:NO];
        
        return;
    }
    
    BOOL useSameThread = ([delegate respondsToSelector:@selector(streamDataShouldUseSameThread:)] && [delegate streamDataShouldUseSameThread:self]);
    
    NSDictionary *parameters = [NSDictionary dictionaryWithObjectsAndKeys:
        host, @"Host",
        [NSNumber numberWithInt:port], @"Port",
        [NSNumber numberWithBool:useSameThread], @"UseSameThread",
        nil];
    
    // Establish the connection:
    [self connectWithParameters:parameters];
}

/*
 setStreamsWithParameters:
 
 Given the parameters dictionary of -connectToHost:onPort:forDelegate:, connects and sets the input and output streams.  Executes on a sub-thread.
 
 Written by DJS 2005-11.
*/

- (void)setStreamsWithParameters:(NSDictionary *)parameters;
{
    NSString *host = [parameters objectForKey:@"Host"];
    int port = [parameters integerForKey:@"Port"];
    
    unichar lastCharacter = [host characterAtIndex:[host length] - 1];
    NSHost *tempHost = nil;
    NSInputStream *input = nil;
    NSOutputStream *output = nil;
    
    if (lastCharacter >= '0' && lastCharacter <= '9')
        tempHost = [NSHost hostWithAddress:host];
    else
        tempHost = [NSHost hostWithName:host];
    
    // Connect to the specified host:
    [NSStream getStreamsToHost:tempHost
                          port:port
                   inputStream:&input
                  outputStream:&output];
    
    [self setInputStream:input];
    [self setOutputStream:output];
}

/*
 handleHasBytesAvailableForStream:
 
 Invoked by -stream:handleEvent: when there is an bytes available event.
 
 Written by DJS 2005-11.
*/

- (void)handleHasBytesAvailableForStream:(NSStream *)stream;
{
    [self receiveStringForStream:stream];
}

@end

