//
//  DSSortTermsBitData.m
//  Shared classes
//
//  Created by David Sinclair on Sun Mar 21 2004.
//  Copyright (c) 2004 Dejal Systems, LLC. All rights reserved.
//
//  This class is private to DSSortTermsData.
//

#import "DSSortTermsBitData.h"
#import "DSSharedDefines.h"


@implementation DSSortTermsBitData

/*
 sortTermWithTerm:object:
 
 Creates a new sort term instance based on the specified term and object.
 
 Written by DJS 2004-03.
*/

+ (id)sortTermWithTerm:(id)term object:(id)object
{
    return [[[self alloc] initWithTerm:term object:object] autorelease];
}

/*
 initWithTerm:object:
 
 Initializes the sort term with the specified term and object.  This is the designated initializer.
 
 Written by DJS 2004-03.
*/

- (id)initWithTerm:(id)term object:(id)object
{
    self = [super init];
    
    [self setTerm:term];
    [self setObject:object];
    
    return self;
}

/*
 dealloc
 
 Standard deallocation method.
 
 Written by DJS 2004-03.
*/

- (void)dealloc
{
    [self setTerm:nil];
    [self setObject:nil];
    
    [super dealloc];
}

/*
 description
 
 Standard description method; returns a string that represents the contents of the receiver.
 
 Written by DJS 2004-03.
*/

- (NSString *)description
{
    return [NSString stringWithFormat:@"%@: term = \"%@\", object = \"%@\"", [[self class] description], [self term], [self object]];
}


// ----------------------------------------------------------------------------------------
#pragma mark -
#pragma mark SORTING METHODS
// ----------------------------------------------------------------------------------------

/*
 compare:
 
 Returns NSOrderedAscending if the receiver's term preceeds the specified term, or NSOrderedDescending if the reverse is true, or NSOrderedSame if they are equivalent.  Any class of object can be compared, but all objects in the collection must be the same kind of class.
 
 Written by DJS 2004-03.
 Changed by DJS 2004-04 to support comparing any object class.
*/

- (NSComparisonResult)compare:(id)other
{
    id selfTerm = [self term];
    id otherTerm = [other term];
    NSComparisonResult result = NSOrderedSame;
    
    // Note: calling -performIntegerSelector:withObject: repeatedly may be inefficient.  If so, the invocation could be cached:
    if ([selfTerm respondsToSelector:@selector(compare:)])
        result = [selfTerm performIntegerSelector:@selector(compare:) withObject:otherTerm];
    
    return result;
}

/*
 caseInsensitiveCompare:
 
 Returns NSOrderedAscending if the receiver's term preceeds the specified term, or NSOrderedDescending if the reverse is true, or NSOrderedSame if they are equivalent.  This does a case-insensitive comparison of strings.  If the term is not a string, it falls back on the -compare: method, above.
 
 Written by DJS 2004-04.
*/

- (NSComparisonResult)caseInsensitiveCompare:(id)other
{
    if ([[self term] isKindOfClass:[NSString class]])
    {
        NSString *selfTerm = [self term];
        NSString *otherTerm = [other term];
        
        return [selfTerm caseInsensitiveCompare:otherTerm];
    }
    else
    {
        return [self compare:other];
    }
}


// ----------------------------------------------------------------------------------------
#pragma mark -
#pragma mark ACCESSOR METHODS
// ----------------------------------------------------------------------------------------

- (id)term
{
    return _term;
}

- (void)setTerm:(id)value
{
    [value retain];
    [_term autorelease];
    _term = value;
}

- (id)object
{
    return _object;
}

- (void)setObject:(id)value
{
    [value retain];
    [_object autorelease];
    _object = value;
}

@end
