//
//  DSSortTermsData.m
//  Shared classes
//
//  Created by David Sinclair on Sun Mar 21 2004.
//  Copyright (c) 2004 Dejal Systems, LLC. All rights reserved.
//

#import "DSSortTermsData.h"
#import "DSSortTermsBitData.h"
#import "DSSharedDefines.h"


@interface DSSortTermsData (Private)

- (NSMutableArray *)collection;
- (void)setCollection:(NSMutableArray *)value;

@end


// ----------------------------------------------------------------------------------------
#pragma mark -
// ----------------------------------------------------------------------------------------

@implementation DSSortTermsData

/*
 sortObjectsFromArray:usingTermSelector:
 
 Convenience method for a one-line use of this class.  Pass an array of objects and a selector for an accessor method (that takes no parameters and returns an object value) for the sort term.  Returns the array sorted by those terms.  Also available as an instance edition, which is useful if you want to do anything else with the sort term instance afterwards.
 
 Written by DJS 2004-03.
*/

+ (NSArray *)sortObjectsFromArray:(NSArray *)objects usingTermSelector:(SEL)selector
{
    return [[self sortTerms] sortObjectsFromArray:objects usingTermSelector:selector];
}

/*
 sortTerms
 
 Creates a new empty sort terms collection instance.
 
 Written by DJS 2004-03.
*/

+ (id)sortTerms
{
    return [[[self alloc] init] autorelease];
}

/*
 init
 
 Initializes the sort terms collection.  This is the designated initializer.
 
 Written by DJS 2004-03.
*/

- (id)init
{
    self = [super init];
    
    [self setCollection:[NSMutableArray array]];
    
    return self;
}

/*
 dealloc
 
 Standard deallocation method.
 
 Written by DJS 2004-03.
*/

- (void)dealloc
{
    [self setCollection:nil];
    
    [super dealloc];
}

/*
 description
 
 Standard description method; returns a string that represents the contents of the receiver.
 
 Written by DJS 2004-03.
*/

- (NSString *)description
{
    return [NSString stringWithFormat:@"%@: collection contains %d terms:\n%@", [[self class] description], [[self collection] count], [self collection]];
}


// ----------------------------------------------------------------------------------------
#pragma mark -
#pragma mark SORTING METHODS
// ----------------------------------------------------------------------------------------

/*
 sortObjectsFromArray:usingTermSelector:
 
 Convenience method for a one-line use of this class.  Pass an array of objects and a selector for an accessor method (that takes no parameters and returns an object value) for the sort term.  Returns the array sorted by those terms.  The class edition of this method is even more convenient, but this instance edition is handy if you want to do anything else with the sort term instance afterwards.
 
 Written by DJS 2004-03.
*/

- (NSArray *)sortObjectsFromArray:(NSArray *)objects usingTermSelector:(SEL)selector
{
    NSEnumerator *enumerator = [objects objectEnumerator];
    id object = nil;
    id term = nil;
    
    while ((object = [enumerator nextObject]))
    {
        term = [object performSelector:selector];
        
    	[self addTerm:term forObject:object];
    }
    
    [self sort];
    
    NSArray *array = [self objects];
    
    return array;
}

/*
 addTerm:forObject:
 
 Adds a sort term for the specified term and object to the collection.  The term should be an object that responds to -compare:, e.g. NSString, NSDate, NSNumber, or a subclass, or a custom object that implements -compare:.  All terms added to the collection should be the same kind of class.
 
 Written by DJS 2004-03.
*/

- (void)addTerm:(id)term forObject:(id)object
{
    NSAssert(term, @"-addTerm:forObject: -- The term is missing.");
    NSAssert(object, @"-addTerm:forObject: -- The object is missing.");
    
    DSSortTermsBitData *bit = [DSSortTermsBitData sortTermWithTerm:term object:object];
    
    if (bit)
        [[self collection] addObject:bit];
}

/*
 addTermsFromArray:forObjectsFromArray:
 
 Adds sort terms for the specified terms and objects to the collection.  The number and order of the terms and objects must be the same, i.e. the first term is for the first object, the second term for the second object, and so on.
 
 Written by DJS 2004-03.
*/

- (void)addTermsFromArray:(NSArray *)terms forObjectsFromArray:(NSArray *)objects
{
    NSAssert([terms count] == [objects count], @"-addTermsFromArray:forObjectsFromArray: -- The number of terms and objects does not match.");
    
    NSEnumerator *termsEnumerator = [terms objectEnumerator];
    NSEnumerator *objectsEnumerator = [objects objectEnumerator];
    id term;
    id object;
    
    while ((term = [termsEnumerator nextObject]) && (object = [objectsEnumerator nextObject]))
    {
    	[self addTerm:term forObject:object];
    }
}

/*
 sort
 
 Sorts the collection case insensitively.  Returns YES if successful.
 
 Written by DJS 2004-03.
*/

- (BOOL)sort
{
    return [self sortCaseSensitive:NO];
}

/*
 sortCaseSensitive:
 
 Sorts the collection, either case sensitively or not as indicated.  Returns YES if successful.
 
 Written by DJS 2004-03.
*/

- (BOOL)sortCaseSensitive:(BOOL)caseSensitive
{
    if (caseSensitive)
        [[self collection] sortUsingSelector:@selector(compare:)];
    else
        [[self collection] sortUsingSelector:@selector(caseInsensitiveCompare:)];
    
    return YES;
}

/*
 terms
 
 Returns an array of the terms in the collection.  They will be in the sorted order if -sort is called first.
 
 Written by DJS 2004-03.
*/

- (NSArray *)terms
{
    NSArray *array = [[self collection] arrayUsingSelector:@selector(term)];
    
    return array;
}

/*
 objects
 
 Returns an array of the objects in the collection.  They will be in the sorted order if -sort is called first.
 
 Written by DJS 2004-03.
*/

- (NSArray *)objects
{
    NSArray *array = [[self collection] arrayUsingSelector:@selector(object)];
    
    return array;
}

@end


// ----------------------------------------------------------------------------------------
#pragma mark -
// ----------------------------------------------------------------------------------------

@implementation DSSortTermsData (Private)

- (NSMutableArray *)collection
{
    return _collection;
}

- (void)setCollection:(NSMutableArray *)value
{
    [value retain];
    [_collection autorelease];
    _collection = value;
}

@end
