//
//  DSSortedData.m
//  Shared classes
//
//  Created by David Sinclair on Mon Feb 09 2004.
//  Copyright (c) 2004 Dejal Systems, LLC. All rights reserved.
//

#import "DSSortedData.h"
#import "DSSharedDefines.h"
#import "DSSortCriteriaData.h"


@implementation DSSortedData

/*
 sortedData
 
 Creates a new empty sorted data instance.  Subclasses shouldn't need to override this.
 
 Written by DJS 2004-02.
*/

+ (id)sortedData
{
    return [[[self alloc] init] autorelease];
}

/*
 sortedDataWithArray:criteria:
 
 Creates a new sorted data instance based on the specified array and sorting criteria, and sorts it.  Subclasses shouldn't need to override this.
 
 Written by DJS 2004-02.
*/

+ (id)sortedDataWithArray:(NSArray *)array criteria:(id)criteria
{
    id instance = [[[self alloc] initWithArray:array criteria:criteria] autorelease];
    
    [instance sort];
    
    return instance;
}

/*
 init
 
 Initializes an empty collection.  This is an optional initializer.  Subclasses shouldn't need to override this.
 
 Written by DJS 2004-02.
*/

- (id)init
{
    return [self initWithArray:nil criteria:nil];
}

/*
 initWithArray:criteria:
 
 Initializes the collection based on the specified array and sorting criteria.  This is the designated initializer.  Subclasses shouldn't need to override this, unless they add any instance variables.
 
 Written by DJS 2004-02.
*/

- (id)initWithArray:(NSArray *)array criteria:(id)criteria
{
    self = [super init];
    
    if (!array)
        array = [NSArray array];
    
    if (!criteria)
    {
        Class class = [self sortCriteriaClass];
        
        criteria = [class criteria];
    }
    
    [self setSortedArray:array];
    [self setCriteria:criteria];
    
    return self;
}

/*
 dealloc
 
 Standard deallocation method.  Subclasses should override this if they add any instance variables that need releasing.
 
 Written by DJS 2004-02.
*/

- (void)dealloc
{
    [self setSortedArray:nil];
    [self setCriteria:nil];
    
    [super dealloc];
}

/*
 sortCriteriaClass
 
 Returns the class to use for the sort criteria.  Subclasses should override this if a subclass of DSSortCriteria is used.
 
 Written by DJS 2004-03.
*/

- (Class)sortCriteriaClass
{
    return [DSSortCriteriaData class];
}


// ----------------------------------------------------------------------------------------
#pragma mark -
#pragma mark SORTING METHODS
// ----------------------------------------------------------------------------------------

/*
 sort
 
 Sorts the collection, based on the provided criteria.  Returns YES if successful (this implementation always returns YES).  The default implementation sorts the objects via the -compare: method, and reverses the sort if the DSSortDescending criteria is YES.  Subclasses should override this method if this isn't appropriate, e.g. if the contained objects are identifiers to other objects, or more complex sorting behavior is desired.  They probably shouldn't call super, but probably should call -reverseIfNeeded.
 
 Written by DJS 2004-03.
*/

- (BOOL)sort
{
    [[self sortedArray] sortUsingSelector:@selector(compare:)];
    
    [self reverseIfNeeded];
    
    return YES;
}

/*
 reverseIfNeeded
 
 Reverses the order of the collection if the criteria indicates it should be in descending order.  Using this, the -sort method can always sort in ascending order, then just call this method to reverse it if needed.  Returns YES if the collection was reversed, or NO if unchanged.  Subclasses shouldn't need to override this.
 
 Written by DJS 2004-03.
*/

- (BOOL)reverseIfNeeded
{
    BOOL isDescending = [[self criteria] descending];
    
    if (isDescending)
        [self setSortedArray:[[self sortedArray] reverseArray]];
    
    return isDescending;
}


// ----------------------------------------------------------------------------------------
#pragma mark -
#pragma mark CONTENTS METHODS
// ----------------------------------------------------------------------------------------

/*
 isValidIndex:
 
 Returns YES if the specified index is within the appropriate range for the collection, or NO if it is not.  Subclasses shouldn't need to override this.
 
 Written by DJS 2004-02.
*/

- (BOOL)isValidIndex:(int)index
{
    return [[self sortedArray] isValidIndex:index];
}

/*
 objectAtIndex:
 
 Returns the object at the specified index, or nil if the index is out of range.  Subclasses shouldn't need to override this.
 
 Written by DJS 2004-02.
*/

- (id)objectAtIndex:(int)index
{
    if ([self isValidIndex:index])
        return [[self sortedArray] objectAtIndex:index];
    else
        return nil;
}

/*
 indexOfObject:
 
 Returns the index of the first occurence of the specified object, or NSNotFound if it is not present.  Subclasses shouldn't need to override this.
 
 Written by DJS 2004-02.
*/

- (int)indexOfObject:(id)object
{
    if (object)
        return [[self sortedArray] indexOfObject:object];
    else
        return NSNotFound;
}

/*
 objectEnumerator
 
 Returns a NSEnumerator instance for the collection, which can be used to iterate over the contained objects.  Subclasses shouldn't need to override this.
 
 Written by DJS 2004-03.
*/

- (NSEnumerator *)objectEnumerator
{
    return [[self sortedArray] objectEnumerator];
}

/*
 count
 
 Returns the number of objects in the collection.  Subclasses shouldn't need to override this.
 
 Written by DJS 2004-02.
*/

- (int)count
{
    return [[self sortedArray] count];
}


// ----------------------------------------------------------------------------------------
#pragma mark -
#pragma mark ACCESSOR METHODS
// ----------------------------------------------------------------------------------------

- (NSMutableArray *)sortedArray
{
    return _sortedArray;
}

- (void)setSortedArray:(NSArray *)value
{
    // Mutable copy rather than retain:
    NSMutableArray *mutableValue = [value mutableCopy];
    
    [_sortedArray autorelease];
    _sortedArray = mutableValue;
}

- (id)criteria
{
    return _criteria;
}

- (void)setCriteria:(id)value
{
    [value retain];
    [_criteria autorelease];
    _criteria = value;
}

@end
