//
//  DSSoundData.m
//  Shared classes
//
//  Created by David Sinclair on Fri May 06 2005.
//  Copyright (c) 2005 Dejal Systems, LLC. All rights reserved.
//

#import "DSSoundData.h"
#import "DSSharedDefines.h"


@implementation DSSoundData

/*
 dataWithDictionary:
 
 Convenience method to create the data and assign settings from a dictionary (which may be empty or nil).
 
 Written by DJS 2005-05.
*/

+ (id)dataWithDictionary:(NSDictionary *)settings
{
    return [[[self alloc] initWithDictionary:settings] autorelease];
}

/*
 playWithDictionary:
 
 Convenience method to play a sound using existing settings (which may be empty or nil).  The instance retains itself while playing, then releases itself.  So if you don't retain it, it'll be released automatically, which is normally what you'd want for one-off playing.
 
 Written by DJS 2005-05.
*/

+ (id)playWithDictionary:(NSDictionary *)settings
{
    DSSoundData *instance = [self dataWithDictionary:settings];
    
    [instance play];
    
    return instance;
}

/*
 initWithDictionary:
 
 Designated initializer.  Assigns the settings from a dictionary (which may be empty or nil).
 
 Written by DJS 2005-05.
*/

- (id)initWithDictionary:(NSDictionary *)settings
{
    self = [super init];
    
    [self reloadWithDictionary:settings];
    
    return self;
}

- (id)init
{
    return [self initWithDictionary:nil];
}

- (void)dealloc
{
    [self setOtherPath:nil];
    [self setCurrentPath:nil];
    [self setCurrentSound:nil];
    
    [super dealloc];
}

/*
 isPlaying
 
 Returns YES if the current sound is currently playing, otherwise NO.
 
 Written by DJS 2005-05.
*/

- (BOOL)isPlaying
{
    return [[self currentSound] isPlaying];
}

/*
 playStop
 
 If currently playing, stops, otherwise starts playing.
 
 Written by DJS 2005-05.
*/

- (void)playStop
{
    if ([self isPlaying])
        [self stop];
    else
        [self play];
}

/*
 play
 
 Start playing the current sound, if any.
 
 Written by DJS 2005-05.
*/

- (void)play
{
    [self stop];
    
    switch ([self mode])
    {
        case 1:			// Sound file
        case 5:         // Built-in sound
            [self playSoundAtPath:[self currentPath]];
            break;
        case 2:			// System beep
            NSBeep();
            break;
    }
}

/*
 stop
 
 Stop playing the current sound, if any.
 
 Written by DJS 2005-05.
*/

- (void)stop
{
    [[self currentSound] stop];
}

/*
 playSoundAtPath:
 
 Given a path for a sound file, plays that sound.  Normally you'd just call -play.
 
 Written by DJS 2005-05.
*/

- (void)playSoundAtPath:(NSString *)path
{
    if ([path length])
    {
        // Retain ourselves, to make sure we don't go away till the sound is done:
        [self retain];
        
        path = [path expandedPath];
        
        NSSound *sound = [[[NSSound alloc] initWithContentsOfFile:path byReference:YES] autorelease];
        [sound setDelegate:self];
        
        [self setCurrentSound:sound];
        
        if ([sound play])
            [self notifyPlayStop:YES];
    }
    else
        NSBeep();
}

/*
 sound:didFinishPlaying:
 
 Sound delegate method for when the sound finishes playing.
 
 Written by DJS 2005-05.
*/

- (void)sound:(NSSound *)sound didFinishPlaying:(BOOL)aBool
{
    [self notifyPlayStop:NO];
    [self setCurrentSound:nil];
    
    // Balance the retain when playing started:
    [self autorelease];
}

/*
 notifyPlayStop:
 
 Posts a DSSoundPlayStopNotification notification, with self as the object and the playing state in the DSSoundIsPlaying key of the user info.  The dictionary representaton can be retrieved via [object dictionary], if desired.
 
 Written by DJS 2005-05.
*/

- (void)notifyPlayStop:(BOOL)isPlaying
{
    NSDictionary *userInfo = [NSDictionary dictionaryWithObject:[NSNumber numberWithBool:isPlaying] forKey:@"DSSoundIsPlaying"];
    
    [[NSNotificationCenter defaultCenter] postNotificationName:@"DSSoundPlayStopNotification" object:self userInfo:userInfo];
}


// ----------------------------------------------------------------------------------------
#pragma mark -
#pragma mark ACCESSOR METHODS
// ----------------------------------------------------------------------------------------

/*
 dictionary
 
 Returns a dictionary representation of the data, suitable for saving in preferences etc.
 
 Written by DJS 2005-05.
*/

- (NSDictionary *)dictionary
{
    return [NSDictionary dictionaryWithObjectsAndKeys:
        [NSNumber numberWithInt:[self mode]], @"Mode",
        [[self currentPath] abbreviatedPath], @"CurrentPath",
        [[self otherPath] abbreviatedPath], @"OtherPath",
        nil];
}

/*
 reloadWithDictionary:
 
 Given a dictionary representation of the data, as returned by -dictionary, above, sets up the data.
 
 Written by DJS 2005-05.
*/

- (void)reloadWithDictionary:(NSDictionary *)settings
{
    [self setMode:[settings integerForKey:@"Mode"]];
    [self setOtherPath:[settings emptyStringOrObjectForKey:@"OtherPath"]];
    [self setCurrentPath:[settings emptyStringOrObjectForKey:@"CurrentPath"]];
}

- (int)mode
{
    return _mode;
}

- (void)setMode:(int)value
{
    _mode = value;
}

- (NSString *)otherPath
{
    return _otherPath;
}

- (void)setOtherPath:(NSString *)value
{
    [value retain];
    [_otherPath autorelease];
    _otherPath = value;
}

- (NSString *)currentPath
{
    return _currentPath;
}

- (void)setCurrentPath:(NSString *)value
{
    [value retain];
    [_currentPath autorelease];
    _currentPath = value;
}

- (NSSound *)currentSound
{
    return _currentSound;
}

- (void)setCurrentSound:(NSSound *)value
{
    [_currentSound stop];
    
    [value retain];
    [_currentSound autorelease];
    _currentSound = value;
}

@end
