//
//  DSStreamData.m
//  Shared classes
//
//  Created by David Sinclair on Thu Jul 01 2004.
//  Copyright (c) 2004 - 2007 Dejal Systems, LLC. All rights reserved.
//

#import "DSStreamData.h"
#import "DSDictionaryCategories.h"


// Note: this is an abstract class.  See DSSocketStreamData and DSFTPStreamData for concrete subclasses.


@implementation DSStreamData

/*
 streamData
 
 Convenience method allocates, initializes, and returns a new instance of this class.
 
 Written by DJS 2004-07.
*/

+ (id)streamData
{
    return [[[self alloc] init] autorelease];
}

- (id)init
{
    self = [super init];
    
    [self setRunLoop:nil];
    [self setInputStream:nil];
    [self setOutputStream:nil];
    [self resetConnectionCount];
    
    return self;
}

- (void)dealloc
{
    if ([self isConnected])
        [self terminate];
    
    [self setInputStream:nil];
    [self setOutputStream:nil];
    [self setRunLoop:nil];
    [self resetConnectionCount];
    
    [super dealloc];
}


/*
 disconnect
 
 Disconnects from the remote host.  If the delegate implements -streamDataDisconnected, it is invoked.
 
 Written by DJS 2004-07.
 Changed by DJS 2007-07 to use a separate thread.
 Changed by DJS 2007-08 to use an ivar instead.
*/

- (void)disconnect
{
    // Simply set an ivar telling the -run method to stop:
    _needToStop = YES;
    
    [self informDelegateUsingSignature:@"streamDataDisconnected:"];
    
//    [NSThread detachNewThreadSelector:@selector(threadedDisconnect) toTarget:self withObject:nil];
}

/*
 sendString:
 
 Sends the specified string to the remote host.  If the delegate implements -streamData:sentString:, it is invoked on success, or -streamData:errorOccurred: if implemented and not currently connected.
 
 Written by DJS 2004-07.
*/

- (void)sendString:(NSString *)string
{
    if ([self isConnected])
    {
        // Append a \r\n to the end for telnet-like behavior:
        string = [string stringByAppendingString:@"\r\n"];
        
        NSData *data = [string dataUsingEncoding:NSUTF8StringEncoding];
        
        [[self outputStream] write:[data bytes] maxLength:[data length]];
        
        [self informDelegateUsingSignature:@"streamData:sentString:" withObject:string];
    }
    else
        [self informDelegateOfError:@"Can't send as not connected" terminate:NO];
}


// ----------------------------------------------------------------------------------------
#pragma mark -
#pragma mark ACCESSOR METHODS
// ----------------------------------------------------------------------------------------

- (NSRunLoop *)runLoop
{
    return _runLoop;
}

- (void)setRunLoop:(NSRunLoop *)value
{
    [value retain];
    [_runLoop autorelease];
    _runLoop = value;
}

- (NSInputStream *)inputStream 
{
    return _inputStream;
}

- (void)setInputStream:(NSInputStream *)value 
{
    [value retain];
    [_inputStream autorelease];
    _inputStream = value;
}

- (NSOutputStream *)outputStream 
{
    return _outputStream;
}

- (void)setOutputStream:(NSOutputStream *)value
{
    [value retain];
    [_outputStream autorelease];
    _outputStream = value;
}

/*
 isConnected
 
 Returns YES if both streams are currently connected, or NO if not.
 
 Written by DJS 2004-07.
*/

- (BOOL)isConnected
{
    return _connectionCount == 2;
}

/*
 connectionCount
 
 Returns the number of currently connected streams.
 
 Written by DJS 2004-07.
*/

- (int)connectionCount
{
    return _connectionCount;
}


// ----------------------------------------------------------------------------------------
#pragma mark -
#pragma mark OVERRIDE METHODS (for subclasses)
// ----------------------------------------------------------------------------------------

/*
 setStreamsWithParameters:
 
 Given a parameters dictionary from a connection method, connects and sets the input and output streams.  Executes on a sub-thread.  Subclasses must override this, without calling super.
 
 Written by DJS 2005-11.
*/

- (void)setStreamsWithParameters:(NSDictionary *)parameters;
{
    NSAssert(NO, @"Forgot to override -setStreamsWithParameters:");
}

/*
 handleNoEventForStream:
 
 Invoked by -stream:handleEvent: when there is no event.  Subclasses may override this; no need to call super.
 
 Written by DJS 2005-11.
*/

- (void)handleNoEventForStream:(NSStream *)stream;
{
    // Does nothing by default
}

/*
 handleOpenCompletedForStream:
 
 Invoked by -stream:handleEvent: when there is an open completed event.  Subclasses may override this; no need to call super.
 
 Written by DJS 2005-11.
*/

- (void)handleOpenCompletedForStream:(NSStream *)stream;
{
    // Does nothing by default
}

/*
 handleHasBytesAvailableForStream:
 
 Invoked by -stream:handleEvent: when there is an bytes available event.  Subclasses may override this; no need to call super.
 
 Written by DJS 2005-11.
*/

- (void)handleHasBytesAvailableForStream:(NSStream *)stream;
{
    // Does nothing by default; call -receiveStringForStream: to treat the input as a string.
}

/*
 handleHasSpaceAvailableForStream:
 
 Invoked by -stream:handleEvent: when there is a space available event.  Subclasses may override this; no need to call super.
 
 Written by DJS 2005-11.
*/

- (void)handleHasSpaceAvailableForStream:(NSStream *)stream;
{
    // Does nothing by default
}

/*
 handleErrorOccurred:forStream:
 
 Invoked by -stream:handleEvent: when there is an error occurred event.  Subclasses may override this; no need to call super.  Return YES to terminate the connection, or NO if you want to try continuing (not recommended).
 
 Written by DJS 2005-11.
*/

- (BOOL)handleErrorOccurred:(NSError *)error forStream:(NSStream *)stream;
{
    // Does nothing by default
    
    return YES;
}

/*
 handleEndEncounteredForStream:
 
 Invoked by -stream:handleEvent: when there is an end encountered event.  Subclasses may override this; no need to call super.  Return YES to terminate the connection, or NO if you want to try continuing (not recommended).
 
 Written by DJS 2005-11.
*/

- (BOOL)handleEndEncounteredForStream:(NSStream *)stream;
{
    // Does nothing by default
    
    return YES;
}

/*
 handleOtherEvent:forStream:
 
 Invoked by -stream:handleEvent: when there is an unknown event.  Subclasses may override this; no need to call super.
 
 Written by DJS 2005-11.
*/

- (void)handleOtherEvent:(NSStreamEvent)streamEvent forStream:(NSStream *)stream;
{
    // Does nothing by default
}


// ----------------------------------------------------------------------------------------
#pragma mark -
#pragma mark PRIVATE METHODS (only for subclasses)
// ----------------------------------------------------------------------------------------

/*
 connectWithParameters:
 
 Establishes a connection using the parameters.  Invokes a worker method on a new thread, or the same thread, if the UseSameThread parameter key is YES.
 
 Written by DJS 2005-11.
 Changed by DJS 2006-01 to add support for the UseSameThread parameter.
*/

- (void)connectWithParameters:(NSDictionary *)parameters;
{
    if ([parameters boolForKey:@"UseSameThread"])
        [self threadedConnectWithParameters:parameters];
    else
        [NSThread detachNewThreadSelector:@selector(threadedConnectWithParameters:) toTarget:self
                           withObject:parameters];
}

/*
 threadedConnectWithParameters:
 
 Worker routine on a separate thread for -connectWithParameters.
 
 Written by DJS 2004-07.
 Changed by DJS 2005-05 to use a separate thread.
 Changed by DJS 2005-11 to be an abstract method and call subclasses for the actual streams.
*/

- (void)threadedConnectWithParameters:(NSDictionary *)parameters;
{
    NSAutoreleasePool *pool = [[NSAutoreleasePool alloc] init];
    BOOL failed = NO;
    
    NS_DURING
        [self setRunLoop:[NSRunLoop currentRunLoop]];
        
        // Get the subclass to connect the input and output streams:
        [self setStreamsWithParameters:parameters];
        
        // Set ourself to to the delegate, schedule ourselves in the run loop, and open the streams:
        [[self inputStream] setDelegate:self];
        [[self outputStream] setDelegate:self];
        
        [[self inputStream] scheduleInRunLoop:[self runLoop] forMode:NSDefaultRunLoopMode];
        [[self outputStream] scheduleInRunLoop:[self runLoop] forMode:NSDefaultRunLoopMode];
        
        [[self inputStream] open];
        [[self outputStream] open];
        
        [self informDelegateUsingSignature:@"streamDataConnecting:"];
        
        // Loop until we are finished, one way or another:
        [self run];
    NS_HANDLER
        NSLog(@"An exception occurred while connecting to a remote server.");
        failed = YES;
    NS_ENDHANDLER
    
    NS_DURING
        if (failed)
            [self terminate];
    NS_HANDLER
        NSLog(@"An exception occurred while terminating a connection after a connection failure.");
    NS_ENDHANDLER
    
    [self informDelegateUsingSignature:@"streamDataEnded:"];
    
    [pool release];
}

/*
 threadedDisconnect
 
 Disconnects from the remote hoast, called on a separate thread.
 
 Written by DJS 2007-07.
*/

/*
- (void)threadedDisconnect;
{
    NSAutoreleasePool *pool = [[NSAutoreleasePool alloc] init];
    
    [self terminate];
    
    [self informDelegateUsingSignature:@"streamDataDisconnected:"];
    
    [pool release];
}
*/

/*
 stream:handleEvent:
 
 Handles all stream events (stream open, close, data available, etc), calling the delegate as appropriate.
 
 Written by DJS 2004-07.
 Changed by DJS 2005-11 to provide hooks for subclasses to override.
*/

- (void)stream:(NSStream *)stream handleEvent:(NSStreamEvent)streamEvent
{
    switch (streamEvent)
    {
        case NSStreamEventNone:
            [self handleNoEventForStream:stream];
            break;
        case NSStreamEventOpenCompleted:
            [self handleOpenCompletedForStream:stream];
            
            // If the stream has completed opening, tell the delegate:
            [self incrementConnectionCount];
            [self informDelegateUsingSignature:@"streamData:connectedToStream:" withObject:stream];
            
            if ([self isConnected])
                [self informDelegateUsingSignature:@"streamDataConnected:"];
            break;
        case NSStreamEventHasBytesAvailable:
            [self handleHasBytesAvailableForStream:stream];
            break;
        case NSStreamEventHasSpaceAvailable:
            [self handleHasSpaceAvailableForStream:stream];
            break;
        case NSStreamEventErrorOccurred:
            if ([self handleErrorOccurred:[stream streamError] forStream:stream])
            {
                // If an error occurred, tell the delegate and terminate:
                [self informDelegateOfError:[stream streamError] terminate:YES];
            }
            break;
        case NSStreamEventEndEncountered:
            if ([self handleEndEncounteredForStream:stream])
            {
                // If the connection was terminated by the remote side, tell the delegate and terminate:
                [self terminate];
                [self informDelegateUsingSignature:@"streamDataConnectionTerminated:"];
            }
            break;
        default:
            [self handleOtherEvent:streamEvent forStream:stream];
            break;
    }
}

/*
 receiveStringForStream:
 
 Given an input stream, reads it as a string.  Useful to call from -handleHasBytesAvailableForStream: if this behavior is desired.
 
 Written by DJS 2005-11.
*/

- (void)receiveStringForStream:(NSStream *)stream;
{
    uint8_t buffer[1024];
    
    // If the stream has bytes available to read, read it:
    int bytesRead = [(NSInputStream *)stream read:buffer maxLength:1024];
    
    if (bytesRead <= 0)
        return;
    
    NSData *data = [NSData dataWithBytes:buffer length:bytesRead];
    
    NSString *string = [[[NSString alloc] initWithData:data encoding:NSUTF8StringEncoding] autorelease];
    
    [self informDelegateUsingSignature:@"streamData:receivedString:" withObject:string];
}

/*
 run
 
 Runs the run loop; a custom loop, instead of using -[NSRunLoop run], to guarantee we can escape.
 
 Written by DJS 2005-07.
 Changed by DJS 2005-12 to add a local autorelease pool; may not give any benefit, if the run loop does that too, but can't hurt.
 Changed by DJS 2007-07 to add a local exception handler.
 Changed by DJS 2007-08 to add _needToStop logic.
*/

- (void)run
{
    _needToStop = NO;
    
    // The needToStop ivar can get set to YES from the main thread via -disconnect:
    while (!_needToStop && [self runLoop] && [self inputStream] && [self outputStream])
    {
        NSAutoreleasePool *pool = [[NSAutoreleasePool alloc] init];
        
        NS_DURING
            [[self runLoop] runMode:NSDefaultRunLoopMode beforeDate:[NSDate dateWithTimeIntervalSinceNow:0.1]];
        NS_HANDLER
            NSLog(@"Exception while communiating with a remote server: %@ (%@)", [localException name], [localException reason]);
        NS_ENDHANDLER
        
        [pool release];
    }
}

/*
 terminate
 
 Disconnects, closes and releases the streams.  Private method; delegates should call -disconnect instead.
 
 Written by DJS 2004-07.
 Changed by DJS 2005-05 to remove the streams from the run loop.
*/

- (void)terminate
{
    [self resetConnectionCount];
    
    [[self inputStream] close];
    [[self outputStream] close];
    
//    [[self inputStream] removeFromRunLoop:[self runLoop] forMode:NSDefaultRunLoopMode];
//    [[self outputStream] removeFromRunLoop:[self runLoop] forMode:NSDefaultRunLoopMode];
    
    [self setInputStream:nil];
    [self setOutputStream:nil];
}

/*
 informDelegateOfError:terminate:
 
 Informs the delegate of an error, and optionally terminates.  The error parameter may be a NSError or NSString.
 
 Written by DJS 2005-11.
*/

- (void)informDelegateOfError:(id)error terminate:(BOOL)shouldTerminate;
{
    if (shouldTerminate)
        [self terminate];
    
    [self informDelegateUsingSignature:@"streamData:errorOccurred:" withObject:error];
}

/*
 resetConnectionCount
 
 Clears the connection count, i.e. indicating disconnection.
 
 Written by DJS 2004-07.
*/

- (int)resetConnectionCount
{
    _connectionCount = 0;
    
    return 0;
}

/*
 incrementConnectionCount
 
 Increments the count of connected streams.
 
 Written by DJS 2004-07.
*/

- (int)incrementConnectionCount
{
    _connectionCount++;
    
    return _connectionCount;
}

@end

