//
//  DSTaskData.m
//  Shared classes
//
//  Created by David Sinclair on 2006-02-12.
//  Copyright 2006 Dejal Systems, LLC. All rights reserved.
//

#import "DSTaskData.h"


@interface DSTaskData (Private)

- (void)didReceiveString:(NSNotification *)note;
- (void)errorOccurred:(NSNotification *)note;
- (void)didTerminate:(NSNotification *)note;

- (void)cleanup;

- (void)setTask:(NSTask *)value;

- (NSMutableString *)mutableOutput;
- (void)setOutput:(NSMutableString *)value;
- (void)setError:(NSString *)value;

- (NSFileHandle *)outputFileHandle;
- (void)setOutputFileHandle:(NSFileHandle *)value;

- (NSFileHandle *)errorFileHandle;
- (void)setErrorFileHandle:(NSFileHandle *)value;

@end


// ----------------------------------------------------------------------------------------
#pragma mark -
// ----------------------------------------------------------------------------------------


@implementation DSTaskData

/*
 launchedTaskWithLaunchPath:arguments:delegate:
 
 Creates and launches a task with the executable specified in path, passing it the array of arguments.  The same as the NSTask method of the same name, except you can use delegate methods.
 
 Written by DJS 2006-02.
*/

+ (id)launchedTaskWithLaunchPath:(NSString *)path arguments:(NSArray *)arguments delegate:(id)delegate;
{
    id data = [[[self alloc] initWithLaunchPath:path arguments:arguments delegate:delegate] autorelease];
    
    [data launch];
    
    return data;
}

- (void)dealloc;
{
    // We don't stop a running task, as the caller may just want to start it and let it do its thing.
    
    [self cleanup];
    
    [self setDelegate:nil];
    [self setOutput:nil];
    [self setError:nil];
    
    [super dealloc];
}

/*
 initWithLaunchPath:arguments:delegate:
 
 Designated initializer.  The task is not launched; call -launch to start it.
 
 Written by DJS 2006-02.
*/

- (id)initWithLaunchPath:(NSString *)path arguments:(NSArray *)arguments delegate:(id)delegate;
{
    [self setDelegate:delegate];
    
    NSPipe *outputPipe = [NSPipe pipe];
    NSPipe *errorPipe = [NSPipe pipe];
    
    [self setOutput:[NSMutableString string]];
    [self setError:nil];
    
    // Prepare the task:
    [self setTask:[[[NSTask alloc] init] autorelease]];
        
    [[self task] setLaunchPath:path];
    [[self task] setArguments:arguments];
    
    // Capture standard output and error:
    [[self task] setStandardOutput:outputPipe];
    [[self task] setStandardError:errorPipe];
    
    // Create file handles for the output and error pipes:
    [self setOutputFileHandle:[outputPipe fileHandleForReading]];
    [self setErrorFileHandle:[errorPipe fileHandleForReading]];
    
    // Register myself as an observer of the pipes and the task:
    [[NSNotificationCenter defaultCenter] addObserver:self selector:@selector(didReceiveString:) name:NSFileHandleReadCompletionNotification object:[self outputFileHandle]];
    [[NSNotificationCenter defaultCenter] addObserver:self selector:@selector(errorOccurred:) name:NSFileHandleReadCompletionNotification object:[self errorFileHandle]];
    [[NSNotificationCenter defaultCenter] addObserver:self selector:@selector(didTerminate:) name:NSTaskDidTerminateNotification object:[self task]];
    
    // Put it in the background - don't hold UI hostage:
    [[self outputFileHandle] readInBackgroundAndNotify];
    [[self errorFileHandle] readInBackgroundAndNotify];
    
    return self;
}

/*
 launch
 
 Launches the task represented by the receiver.
 
 Written by DJS 2006-02.
*/

- (void)launch;
{
    [[self task] launch];
    
    if ([self isRunning])
        [self informDelegateUsingSignature:@"taskDataLaunched:"];
    else
        [self informDelegateUsingSignature:@"taskDataFailedLaunch:"];
}

/*
 stop
 
 Interrupts the task represented by the receiver.  If it terminates as a result, which is the default behavior, the delegate will receive a -taskDataEnded: message.
 
 Written by DJS 2006-02.
*/

- (void)stop;
{
    [[self task] interrupt];
}

/*
 isRunning
 
 Returns YES if the receiver's task is still running, or NO if either the receiver could not run or it has terminated.
 
 Written by DJS 2006-02.
*/

- (BOOL)isRunning;
{
    return [[self task] isRunning];
}


// ----------------------------------------------------------------------------------------
#pragma mark -
#pragma mark ACCESSOR METHODS
// ----------------------------------------------------------------------------------------

/*
 output
 
 Returns the output string of the task.
 
 Written by DJS 2006-02.
*/

- (NSString *)output;
{
    return _output;
}

/*
 error
 
 Returns a string for any error that occurred, or nil if there was no error.
 
 Written by DJS 2006-02.
*/

- (NSString *)error;
{
    return _error;
}

/*
 task
 
 Returns the underlying NSTask, so additional setup or other operations can be performed, if necessary.
 
 Written by DJS 2006-02.
*/

- (NSTask *)task;
{
    return _task;
}

@end


// ----------------------------------------------------------------------------------------
#pragma mark -
// ----------------------------------------------------------------------------------------


@implementation DSTaskData (Private)

- (void)didReceiveString:(NSNotification *)note;
{
    // Get the data from the notification:
    NSData *theData = [[note userInfo] objectForKey:NSFileHandleNotificationDataItem];
    
    NSString *theString = [[[NSString alloc] initWithData:theData encoding:NSNEXTSTEPStringEncoding] autorelease];
    
    // Reset the notifier:
    [[self outputFileHandle] readInBackgroundAndNotify];
    
    // Put the stuff in the text:
    if (theString)
    {
        [[self mutableOutput] appendString:theString];
        
        [self informDelegateUsingSignature:@"taskData:receivedString:" withObject:theString];
    }
}

- (void)errorOccurred:(NSNotification *)note;
{
    // Get the data from the notification:
    NSData *errData = [[note userInfo] objectForKey:NSFileHandleNotificationDataItem];
    
    NSString *errString = [[[NSString alloc] initWithData:errData encoding:NSNEXTSTEPStringEncoding] autorelease];
    
    if ([errData length])
    {    
        // Tell the user what the problem is:
        [self setError:errString];
        
        [self informDelegateUsingSignature:@"taskData:errorOccurred:" withObject:errString];
        
        // We're done:
        [self cleanup];
    }
}

- (void)didTerminate:(NSNotification *)note;
{
    NSMutableString *output = [self mutableOutput];
    
    // Trim returns from the end:
    while ([output hasSuffix:@"\n"] || [output hasSuffix:@"\r"])
        [output deleteCharactersInRange:NSMakeRange([output length] - 1, 1)];
    
    [self informDelegateUsingSignature:@"taskDataEnded:"];
    
    [self cleanup];
}

- (void)cleanup;
{
    [self setTask:nil];
    
    // Unregister for notifications:
    [[NSNotificationCenter defaultCenter] removeObserver:self];
    
    // Clear the output and error file handles:
    [self setOutputFileHandle:nil];
    [self setErrorFileHandle:nil];
}


// ----------------------------------------------------------------------------------------
#pragma mark -
#pragma mark ACCESSOR METHODS
// ----------------------------------------------------------------------------------------

- (void)setTask:(NSTask *)value
{
    // We don't stop a running task, as the caller may just want to start it and let it do its thing.
    
    [value retain];
    [_task autorelease];
    _task = value;
}

- (NSMutableString *)mutableOutput;
{
    return _output;
}

- (void)setOutput:(NSMutableString *)value
{
    [value retain];
    [_output autorelease];
    _output = value;
}

- (void)setError:(NSString *)value
{
    [value retain];
    [_error autorelease];
    _error = value;
}

- (NSFileHandle *)outputFileHandle
{
    return _outputFileHandle;
}

- (void)setOutputFileHandle:(NSFileHandle *)value
{
    [value retain];
    [_outputFileHandle autorelease];
    _outputFileHandle = value;
}

- (NSFileHandle *)errorFileHandle
{
    return _errorFileHandle;
}

- (void)setErrorFileHandle:(NSFileHandle *)value
{
    [value retain];
    [_errorFileHandle autorelease];
    _errorFileHandle = value;
}

@end
