#import "DSGraphController.h"
#import <SM2DGraphView/SM2DGraphView.h>
#import "DSTableViewCategories.h"


@interface DSGraphController (Private)

- (NSMutableArray *)groupNamesArray;
- (void)setGroupNamesArray:(NSMutableArray *)value;

- (NSMutableArray *)groupColorsArray;
- (void)setGroupColorsArray:(NSMutableArray *)value;

- (NSMutableArray *)valueLabelsArray;
- (void)setValueLabelsArray:(NSMutableArray *)value;

- (NSMutableArray *)groupedValuesArray;
- (void)setGroupedValuesArray:(NSMutableArray *)value;

- (NSMutableArray *)groupedLinesArray;
- (void)setGroupedLinesArray:(NSMutableArray *)value;

@end


// ----------------------------------------------------------------------------------------
#pragma mark -
// ----------------------------------------------------------------------------------------


@implementation DSGraphController

+ (id)graph
{
   return [[[self alloc] init] autorelease];
}

- (id)init
{
    self = [self initWithWindowNibName:@"DSGraph"];

    [self setGroupNamesArray:[NSMutableArray array]];
    [self setValueLabelsArray:[NSMutableArray array]];
    [self setGroupedValuesArray:[NSMutableArray array]];
    [self setGroupedLinesArray:nil];

    _highestCount = 0.0;

    return self;
}

- (void)dealloc
{
    [_groupedLinesArray release];
    [_groupedValuesArray release];
    [_valueLabelsArray release];
    [_groupNamesArray release];

    [super dealloc];
}


- (void)setTitle:(NSString *)title withCountName:(NSString *)countName groupName:(NSString *)groupName
{
    if ([countName isEqualToString:@"Date & Time"])
        countName = @"Date";

    if ([groupName isEqualToString:@"Date & Time"])
        groupName = @"Date";

    if ([groupName length] && ![groupName isEqualToString:@"None"])
        [[self window] setTitle:[NSString stringWithFormat:@"%@: %@ by %@", title, countName, groupName]];
    else
        [[self window] setTitle:[NSString stringWithFormat:@"%@: %@", title, countName]];
}

- (void)displayWindow
{
    // Autoreleased till now, so vanishes if not displayed:
    [self retain];

    [self setupColors];
    [self simplify];

    [legendTableView reloadData];

    // Inset the X Axis a bit to give enough room for the bars to display.
    [graphView setAxisInset:[SM2DGraphView barWidth] * [[self groupNamesArray] count] forAxis:kSM2DGraph_Axis_X];
    [graphView setNumberOfTickMarks:[[self valueLabelsArray] count] forAxis:kSM2DGraph_Axis_X];
    [graphView refreshDisplay:self];

    [self showWindow:nil];

    if ([[self groupNamesArray] count] > 1)
        [self openLegendDrawer:nil];
}

/*
 simplify

 Converts the groupedValuesArray (with arrays of ints) into the format used by the graph framework (with arrays of stringified points), to avoid having to do this when drawing.
*/

- (void)simplify
{
    NSMutableArray *groupedLinesArray = [NSMutableArray array];

    NSArray *legendArray = nil;
    NSEnumerator *groupEnumerator = [[self groupedValuesArray] objectEnumerator];
    NSMutableArray *valuesArray;

    if ([legendTableView numberOfRows])
        legendArray = [[legendTableView selectedOrAllRowsEnumerator] allObjects];
    
    _highestCount = 0.0;
    int legendIndex = 0;
    
    while ((valuesArray = [groupEnumerator nextObject]))
    {
        if (!legendArray || [legendArray containsObject:[NSNumber numberWithInt:legendIndex]])
        {
            NSEnumerator *valuesEnumerator = [valuesArray objectEnumerator];
            NSNumber *countInt;
            NSMutableArray *linesArray = [NSMutableArray array];
            float valuesIndex = 0.0;

            // Build the lines array from the values array:
            while ((countInt = [valuesEnumerator nextObject]))
            {
                float count = [countInt floatValue];

                if (count > _highestCount)
                    _highestCount = count;

                [linesArray addObject:NSStringFromPoint(NSMakePoint(valuesIndex, count))];
                valuesIndex += 1;
            }

            int requiredNumber = [[self valueLabelsArray] count];

            // If not enough, add more:
            while ([linesArray count] < requiredNumber)
            {
                [linesArray addObject:NSStringFromPoint(NSMakePoint(valuesIndex, 0))];
                valuesIndex += 1;
            }

            [groupedLinesArray addObject:linesArray];
        }

        legendIndex++;
    }

    [self setGroupedLinesArray:groupedLinesArray];
}

- (void)setupColors
{
    int numGroups = [[self groupNamesArray] count];
    NSMutableArray *groupColorsArray = [NSMutableArray arrayWithCapacity:numGroups];

    int colorsLoop;

    for (colorsLoop = 0; colorsLoop < numGroups; colorsLoop++)
    {
        switch (colorsLoop % 9)
        {
            case 0:
                [groupColorsArray addObject:[NSColor greenColor]];
                break;
            case 1:
                [groupColorsArray addObject:[NSColor redColor]];
                break;
            case 2:
                [groupColorsArray addObject:[NSColor blueColor]];
                break;
            case 3:
                [groupColorsArray addObject:[NSColor orangeColor]];
                break;
            case 4:
                [groupColorsArray addObject:[NSColor purpleColor]];
                break;
            case 5:
                [groupColorsArray addObject:[NSColor brownColor]];
                break;
            case 6:
                [groupColorsArray addObject:[NSColor magentaColor]];
                break;
            case 7:
                [groupColorsArray addObject:[NSColor cyanColor]];
                break;
            default:
                [groupColorsArray addObject:[NSColor blackColor]];
                break;
        }
    }

    [self setGroupColorsArray:groupColorsArray];
}


- (void)awakeFromNib
{
    [self setShouldCascadeWindows:NO];
    [self setWindowFrameAutosaveName:@"DSGraph"];
    [[self window] setFrameUsingName:@"DSGraph"];
}


// ----------------------------------------------------------------------------------------
#pragma mark -
#pragma mark LEGEND DRAWER METHODS
// ----------------------------------------------------------------------------------------

- (IBAction)openLegendDrawer:(id)sender
{
    [legendDrawer openOnEdge:NSMaxXEdge];
}

- (IBAction)closeLegendDrawer:(id)sender
{
    [legendDrawer close];
}

- (IBAction)toggleLegendDrawer:(id)sender
{
    NSDrawerState state = [legendDrawer state];
    
    if (state == NSDrawerOpeningState || state == NSDrawerOpenState)
        [self closeLegendDrawer:sender];
    else
        [self openLegendDrawer:sender];
}


// ----------------------------------------------------------------------------------------
#pragma mark -
#pragma mark LEGEND TABLEVIEW DATASOURCE METHODS
// ----------------------------------------------------------------------------------------

- (int)numberOfRowsInTableView:(NSTableView *)tableView
{
    return [[self groupNamesArray] count];
}

- (id)tableView:(NSTableView *)tableView objectValueForTableColumn:(NSTableColumn *)tableColumn row:(int)row
{
    return [[self groupNamesArray] objectAtIndex:row];
}

- (void)tableView:(NSTableView *)tableView willDisplayCell:(id)cell
   forTableColumn:(NSTableColumn *)tableColumn row:(int)row
{
    [cell setTextColor:[[self groupColorsArray] objectAtIndex:row]];
}


// ----------------------------------------------------------------------------------------
#pragma mark -
#pragma mark LEGEND TABLEVIEW DELEGATE METHODS
// ----------------------------------------------------------------------------------------

- (void)tableViewSelectionDidChange:(NSNotification *)aNotification
{
    [self simplify];
    
    [graphView refreshDisplay:self];
}


// ----------------------------------------------------------------------------------------
#pragma mark -
#pragma mark SM2DGRAPHVIEW DATASOURCE METHODS
// ----------------------------------------------------------------------------------------

- (unsigned int)numberOfLinesInTwoDGraphView:(SM2DGraphView *)inGraphView
{
    return [[self groupedLinesArray] count];
}

- (NSArray *)twoDGraphView:(SM2DGraphView *)inGraphView dataForLineIndex:(unsigned int)inLineIndex
{
    NSMutableArray *result = [[self groupedLinesArray] objectAtIndex:inLineIndex];
    
    return result;
}

- (double)twoDGraphView:(SM2DGraphView *)inGraphView maximumValueForLineIndex:(unsigned int)inLineIndex
                forAxis:(SM2DGraphAxisEnum)inAxis
{
    double result = 0.0;

    if (inAxis == kSM2DGraph_Axis_X)
        result = [[self valueLabelsArray] count] - 1;
    else
        result = _highestCount + (_highestCount / 10);
    
    return result;
}

- (double)twoDGraphView:(SM2DGraphView *)inGraphView minimumValueForLineIndex:(unsigned int)inLineIndex
                forAxis:(SM2DGraphAxisEnum)inAxis
{
    double result = 0;

    return result;
}

- (NSDictionary *)twoDGraphView:(SM2DGraphView *)inGraphView attributesForLineIndex:(unsigned int)inLineIndex
{
    NSDictionary *result = nil;
    NSArray *legendArray = [[legendTableView selectedOrAllRowsEnumerator] allObjects];
    NSNumber *index = [legendArray objectAtIndex:inLineIndex];
    
    // Make this a bar graph:
    result = [NSDictionary dictionaryWithObjectsAndKeys:
        [NSNumber numberWithBool:YES], SM2DGraphBarStyleAttributeName,
        [[self groupColorsArray] objectAtIndex:[index intValue]], NSForegroundColorAttributeName,
//        [[self groupColorsArray] objectAtIndex:inLineIndex], NSForegroundColorAttributeName,
        nil];

    return result;
}

// ----------------------------------------------------------------------------------------
#pragma mark -
#pragma mark SM2DGRAPHVIEW DELEGATE METHODS
// ----------------------------------------------------------------------------------------

- (NSString *)twoDGraphView:(SM2DGraphView *)inGraphView labelForTickMarkIndex:(unsigned int)inTickMarkIndex
                    forAxis:(SM2DGraphAxisEnum)inAxis defaultLabel:(NSString *)inDefault
{
    NSString *result = inDefault;

    if (inAxis == kSM2DGraph_Axis_X)
    {
        if (inTickMarkIndex < [[self valueLabelsArray] count])
            result = [[self valueLabelsArray] objectAtIndex:inTickMarkIndex];
        else
            result = @"";
    }

    return result;
}

/*
- (void)twoDGraphView:(SM2DGraphView *)inGraphView didClickPoint:(NSPoint)inPoint
{
    NSString *tempString;
    NSPoint testPoint;
    int min, max, index;

    if ( inGraphView == _sm_trigGraph )
    {
        // In this case the line index doesn't matter because both lines have the same scale.
        inPoint = [ _sm_trigGraph convertPoint:inPoint fromView:_sm_trigGraph toLineIndex:0 ];

        // Quick search through the data array for the closest point.
        index = 0;
        max = [ _sm_line_sin count ] - 1;
        min = 0;
        while ( max > min )
        {
            index = ( max + min ) / 2;
            testPoint = NSPointFromString( [ _sm_line_sin objectAtIndex:index ] );
            if ( inPoint.x < testPoint.x )
                max = index - 1;
            else if ( inPoint.x > testPoint.x )
                min = index + 1;
            else
                break;
        }
        if ( max == min )
            index = max;

        // Make a string with data point information.
        testPoint = NSPointFromString( [ _sm_line_sin objectAtIndex:index ] );

        tempString = [ NSString stringWithFormat:NSLocalizedString( @"Angle = %.0f; sin = %.3f; ", @"" ),
                    testPoint.x, testPoint.y ];

        testPoint = NSPointFromString( [ _sm_line_cos objectAtIndex:index ] );

        tempString = [ tempString stringByAppendingFormat:NSLocalizedString( @"cos = %.3f", @"" ),
                    testPoint.y ];

        // Set that string into the text field.
        [ _sm_clickInfo setStringValue:tempString ];
    }
}
*/

// ----------------------------------------------------------------------------------------
#pragma mark -
#pragma mark GROUPARRAY MAINTENANCE METHODS
// ----------------------------------------------------------------------------------------

/*
- (void)addGroupObject:(id)groupObject withCountObject:(id)countObject
{
    // Local access for convenience:
    NSMutableArray *groupNamesArray = [self groupNamesArray];
    NSMutableArray *valueLabelsArray = [self valueLabelsArray];
    NSMutableArray *groupedValuesArray = [self groupedValuesArray];
    NSMutableArray *valuesArray = nil;
   
    if (groupNamesArray && valueLabelsArray && groupedValuesArray && countObject)
    {
        // If not grouped, use a global group:
        if (!groupObject)
            groupObject = @"All";

        // Find the index of the group name:
        int groupIndex = [groupNamesArray indexOfObject:groupObject];

        // If not found, add it, and also add a group array for the value, with the necessary number of zeros:
        if (groupIndex == NSNotFound)
        {
            [groupNamesArray addObject:groupObject];
            valuesArray = [NSMutableArray array];

            int i;
            
            if ([valueLabelsArray count])
                for (i = 0; i < [valueLabelsArray count]; i++)
                    [valuesArray addObject:[NSNumber numberWithInt:0]];
            
            [groupedValuesArray addObject:valuesArray];
            
            groupIndex = [groupNamesArray indexOfObject:groupObject];
        }

        // Find the index of the value label:
        int valueIndex = [valueLabelsArray indexOfObject:countObject];

        // If not found, add it, and also add zero counts to each value array:
        if (valueIndex == NSNotFound)
        {
            [valueLabelsArray addObject:countObject];
            valueIndex = [valueLabelsArray indexOfObject:countObject];

            NSEnumerator *enumerator = [groupedValuesArray objectEnumerator];

            while (valuesArray = [enumerator nextObject])
                [valuesArray addObject:[NSNumber numberWithInt:0]];
        }

        // Grab the array for the group and increment the count:
        valuesArray = [groupedValuesArray objectAtIndex:groupIndex];
        int count = [[valuesArray objectAtIndex:valueIndex] intValue] + 1;

        [valuesArray replaceObjectAtIndex:valueIndex withObject:[NSNumber numberWithInt:count]];
    }
}
*/

- (BOOL)addGroupObject:(id)groupObject withCountObject:(id)countObject
{
    // Local access for convenience:
    NSMutableArray *groupNamesArray = [self groupNamesArray];
    NSMutableArray *valueLabelsArray = [self valueLabelsArray];
    NSMutableArray *groupedValuesArray = [self groupedValuesArray];
    NSMutableArray *valuesArray = nil;

    if (groupNamesArray && valueLabelsArray && groupedValuesArray && countObject)
    {
        // If not grouped, use a global group:
        if (!groupObject)
            groupObject = @"All";

        // Convert dates to strings (with just the date, not time), and other non-strings to strings:
        if ([countObject respondsToSelector:@selector(descriptionWithCalendarFormat:timeZone:locale:)])
            countObject = [countObject descriptionWithCalendarFormat:@"%Y-%m-%d" timeZone:nil locale:nil];
        else if (![countObject respondsToSelector:@selector(sizeWithAttributes:)])
            countObject = [countObject description];

        if ([groupObject respondsToSelector:@selector(descriptionWithCalendarFormat:timeZone:locale:)])
            groupObject = [groupObject descriptionWithCalendarFormat:@"%Y-%m-%d" timeZone:nil locale:nil];
        else if (![groupObject respondsToSelector:@selector(sizeWithAttributes:)])
            groupObject = [groupObject description];

        // Find the index of the group name:
        int groupIndex = [groupNamesArray indexOfObject:groupObject];

        // If not found, add it, and also add a group array for the value:
        if (groupIndex == NSNotFound)
        {
            if ([groupNamesArray count] > 20)
                return NO;
            
            [groupNamesArray addObject:groupObject];
            [groupedValuesArray addObject:[NSMutableArray array]];

            groupIndex = [groupNamesArray indexOfObject:groupObject];
        }

        // Find the index of the value label:
        int valueIndex = [valueLabelsArray indexOfObject:countObject];

        // If not found, add it, and also add zero counts to each value array:
        if (valueIndex == NSNotFound)
        {
            if ([valueLabelsArray count] > 20)
                return NO;

            [valueLabelsArray addObject:countObject];
            valueIndex = [valueLabelsArray indexOfObject:countObject];
        }

        // Grab the array for the group and increment the count:
        valuesArray = [groupedValuesArray objectAtIndex:groupIndex];

        // Make sure it has enough count values:
        while (valueIndex >= [valuesArray count])
            [valuesArray addObject:[NSNumber numberWithInt:0]];

        // Increment the count:
        int count = [[valuesArray objectAtIndex:valueIndex] intValue] + 1;

        [valuesArray replaceObjectAtIndex:valueIndex withObject:[NSNumber numberWithInt:count]];
    }

    return YES;
}

@end


// ----------------------------------------------------------------------------------------
#pragma mark -
// ----------------------------------------------------------------------------------------


@implementation DSGraphController (Private)

- (NSMutableArray *)groupNamesArray
{
    return _groupNamesArray;
}

- (void)setGroupNamesArray:(NSMutableArray *)value
{
    [value retain];
    [_groupNamesArray autorelease];
    _groupNamesArray = value;
}

- (NSMutableArray *)groupColorsArray
{
    return _groupColorsArray;
}

- (void)setGroupColorsArray:(NSMutableArray *)value
{
    [value retain];
    [_groupColorsArray autorelease];
    _groupColorsArray = value;
}

- (NSMutableArray *)valueLabelsArray
{
    return _valueLabelsArray;
}

- (void)setValueLabelsArray:(NSMutableArray *)value
{
    [value retain];
    [_valueLabelsArray autorelease];
    _valueLabelsArray = value;
}

- (NSMutableArray *)groupedValuesArray
{
    return _groupedValuesArray;
}

- (void)setGroupedValuesArray:(NSMutableArray *)value
{
    [value retain];
    [_groupedValuesArray autorelease];
    _groupedValuesArray = value;
}

- (NSMutableArray *)groupedLinesArray
{
    return _groupedLinesArray;
}

- (void)setGroupedLinesArray:(NSMutableArray *)value
{
    [value retain];
    [_groupedLinesArray autorelease];
    _groupedLinesArray = value;
}

@end
