//
//  DSPopupClusterHandler.m
//  Shared classes
//
//  Created by David Sinclair on Thu Jan 22 2004.
//  Copyright (c) 2004 Dejal Systems, LLC. All rights reserved.
//

#import "DSPopupClusterHandler.h"
#import "DSSharedDefines.h"


/*
 DSPopupClusterHandler
 
 This class is an abstract superclass for a DSViewsListView handler.  It should be used if there are multiple container views in multiple Nib files.  If there are multiple container views in one Nib file, use DSPopupHandler instead.  If there is only one container view in one Nib file, use DSPopupSingleHandler instead.  Do not change this class in a way that will break existing subclasses; any changes should probably go in a subclass instead.
*/

@implementation DSPopupClusterHandler

/*
 initForViewsListView:withSettings:userInfo:
 
 Initializes a handler for the ViewsListView, based on the specified settings.  Used to load a saved handler.  This is the designated initializer.  Subclasses in the class cluster should override this to set themselves up, calling super first.
 
 Written by DJS 2003-07.  Changed by DJS 2003-11 to implement as a class cluster.  Changed by DJS 2004-01 to add the user info param.  Changed by DJS 2004-03 to select the popup item based on the kind in the represented object.
*/

- (id)initForViewsListView:(id)viewsListView withSettings:(NSDictionary *)settings userInfo:(id)userInfo
{
    NSString *kind = [settings objectForKey:@"Kind"];
    Class clusterClass = [self clusterPublicClass];
    
    if ([self class] == clusterClass)                                                                   // If it's the abstract class....
    {
        // Release the abstract instance:
        [self release];
        
        // Determine the subclass to use:
        Class class = [clusterClass clusterSubclassForKind:kind];
        
        // Allocate and initialize it:
        self = [[class alloc] initForViewsListView:viewsListView withSettings:settings userInfo:userInfo];
    }
    else
    {
        self = [super initForViewsListView:viewsListView withSettings:settings userInfo:userInfo];	// Otherwise init as normal
        
        // If no mode set, get it from the kind:
        if (![self mode])
            [self setMode:[self modeForKind:kind]];
        
        int index = [modePopup indexOfItemWithRepresentedObject:kind];
        
        // If the represented object (kind string) was found, select that item (the selection in the superclass doesn't work when using kind-based popups):
        if (index >= 0)
            [modePopup selectItemAtIndex:index];
    }
    
    return self;
}

- (void)dealloc
{
    [self setKind:nil];
    
    [super dealloc];
}


/*
 completeInitializationWithSettings:
 
 Called just after initialization of the instance.  Set any ivars here, so they are available when needed during the initialization process.
 
 Written by DJS 2004-04.
*/

- (void)completeInitializationWithSettings:(NSDictionary *)settings
{
    NSString *kind = [settings objectForKey:@"Kind"];
    
    [self setKind:kind];
}


/*
 clusterPublicClass
 
 Returns the public class of the class cluster.  The class cluster subclass should override this and return it's own class.  Subclasses of that cluster should not override this, though.
 
 Written by DJS 2004-01.
*/

+ (Class)clusterPublicClass
{
    return [DSPopupClusterHandler class];
}

/*
 clusterPublicClass
 
 Returns the public class of the class cluster.  Subclasses shouldn't need to override this, as it is just an instance edition of the class method of the same name.
 
 Written by DJS 2004-01.
*/

- (Class)clusterPublicClass
{
    return [[self class] clusterPublicClass];
}

/*
 clusterSubclassForKind:
 
 Returns the class of the class cluster that will be used for the specified kind.  The class cluster subclass should override this.  Subclasses of that cluster should not need to override this.
 
 Written by DJS 2003-11.
*/

+ (Class)clusterSubclassForKind:(NSString *)kind
{
    NSAssert(NO, @"Forgot to override +clusterSubclassForKind: in the class cluster's abstract object.");
    
    return nil;
}

/*
 modeForKind:
 
 Returns the kind that corresponds to the specified mode number (as used for the mode popup).  The class cluster subclass may override this if the kind is not stored in the mode popup's represented object.  Subclasses of that cluster should not need to override this.
 
 Written by DJS 2004-03.
*/

- (int)modeForKind:(NSString *)kind
{
    int mode = 0;
    NSEnumerator *enumerator = [[modePopup itemArray] objectEnumerator];
    NSMenuItem *item;
    
    while (!mode && (item = [enumerator nextObject]))
    {
    	if ([[item representedObject] isEqualToString:kind])
            mode = [item tag];
    }
    
    return mode;
}

/*
 kindForMode:
 
 Returns the kind that corresponds to the specified mode number (as used for the mode popup).  The class cluster subclass may override this if the kind is not stored in the mode popup's represented object.  Subclasses of that cluster should not need to override this.
 
 Written by DJS 2004-01.  Changed by DJS 2004-03 to use the represented object for the kind string.
*/

- (NSString *)kindForMode:(int)mode
{
    NSString *kind = nil;
    NSEnumerator *enumerator = [[modePopup itemArray] objectEnumerator];
    NSMenuItem *item;
    
    while (!kind && (item = [enumerator nextObject]))
    {
    	if ([item tag] == mode)
            kind = [item representedObject];
    }
    
    return kind;
}


/*
 mutableSettings
 
 Returns a mutable dictionary representation of the handler's contents, for saving.  The subclass class cluster and its subclasses in the class cluster can override this to add any extra values.
 
 Written by DJS 2004-02.
*/

- (NSMutableDictionary *)mutableSettings
{
    NSMutableDictionary *dict = [super mutableSettings];
    
    [dict setObject:[self kind] forKey:@"Kind"];
    
    return dict;
}


/*
 didAddHandler:
 
 A message sent from the receiver to itself, to do any additional processing after adding another handler.  To clarify: self is the existing handler that had it's Add button clicked; newHandler is the one that was created and added above or below it (as per -shouldAddAfter).  See also -didAddFromHandler:, below.  Subclasses can override this if desired; no need to call super.
 
 Written by DJS 2004-01.
*/

- (void)didAddHandler:(id)newHandler
{
    // Does nothing by default
}

/*
 didAddFromHandler:
 
 A message sent from the handler that created the receiver this new handler, in case it wants to do additional processing.  To clarify: self is the newly-added handler, and otherHandler is the existing one that had it's Add button clicked.  See also -didAddHandler:, above.  Subclasses can override this if desired; no need to call super.
 
 Written by DJS 2004-01.
*/

- (void)didAddFromHandler:(id)otherHandler
{
    // Does nothing by default
}


/*
 appendValue:toString:
 
 Extracts the contents of the provided settings and appends to the output string.  Used for the Services menu item etc.  Subclasses in the class cluster should override this.
 
 Written by DJS 2003-11.
*/

+ (void)appendValue:(NSDictionary *)settings toString:(NSMutableString *)output
{
    // Only does anything for the abstract class, in case a subclass doesn't override this:
    if (self == [self clusterPublicClass])
    {
        NSString *kind = [settings objectForKey:@"Kind"];
        
        // Determine the subclass to use:
        Class class = [self clusterSubclassForKind:kind];
        
        // Call the method for that class:
        if (class)
            [class appendValue:settings toString:output];
    }
}


// ----------------------------------------------------------------------------------------
#pragma mark -
#pragma mark TARGET/ACTION METHODS
// ----------------------------------------------------------------------------------------

- (id)_handlerForKind:(NSString *)kind
{
    NSDictionary *settings = [NSDictionary dictionaryWithObject:kind forKey:@"Kind"];
    
    // Determine the subclass to use:
    Class class = [[self clusterPublicClass] clusterSubclassForKind:kind];
    
    // Allocate and initialize it:
    return [[[class alloc] initForViewsListView:[self viewsListView] withSettings:settings userInfo:[self userInfo]] autorelease];
}

/*
 changeMode:
 
 Action method for the popups.  Should not need overriding.
 
 Written 2004-01.
*/

- (IBAction)changeMode:(id)sender;
{
    // Retain ourselves so we don't go away just yet:
    [self retain];
    
    int newMode = [[sender selectedItem] tag];
    NSString *newKind = [self kindForMode:newMode];
    id newHandler = [self _handlerForKind:newKind];
    
    [self setMode:newMode];
    
    [self modeDidChange];
    
    // If the container view has changed, due to the mode change, replace it in the ViewsListView:
    if ([[newHandler containerView] isNotEqualTo:[self containerView]])
    {
        [self containerViewDidChange];
        
        [[self viewsListView] replaceHandler:self withHandler:newHandler];
    }
    
    [self notifyOfChange];
    
    // We're done with this instance, so balance the -retain above:
    [self release];
}

/*
 add:
 
 Action method for the add button.  Should not need overriding.
 
 Written 2004-01.
*/

- (IBAction)add:(id)sender;
{
    NSString *newKind = [self kindForMode:0];
    id newHandler = [self _handlerForKind:newKind];
    
    if (newHandler)
    {
        if ([self shouldAddAfter])
            [[self viewsListView] insertHandler:newHandler afterHandler:self];
        else
            [[self viewsListView] insertHandler:newHandler beforeHandler:self];
        
        // Send a message to myself to do any additional processing after adding another handler:
        [self didAddHandler:newHandler];
        
        // And send a message to the new handler telling it who added it, in case it wants to do additional processing:
        [newHandler didAddFromHandler:self];
    }
}


// ----------------------------------------------------------------------------------------
#pragma mark -
#pragma mark ACCESSOR METHODS
// ----------------------------------------------------------------------------------------

- (NSString *)kind
{
    return _kind;
}

- (void)setKind:(NSString *)value
{
    [value retain];
    [_kind autorelease];
    _kind = value;
}

@end
