//
//  DSPopupHandler.m
//  Shared classes
//
//  Created by David Sinclair on Mon Aug 04 2003.
//  Copyright (c) 2003 - 2004 Dejal Systems, LLC. All rights reserved.
//

#import "DSPopupHandler.h"
#import "DSSharedDefines.h"


/*
 DSPopupHandler
 
 This class is an abstract superclass for a DSViewsListView handler.  It should be used if there are multiple container views in one Nib file.  If there is only one container view in one Nib file, use DSPopupSingleHandler instead.  If there are multiple container views in multiple Nib files, use DSPopupClusterHandler instead.  Do not change this class in a way that will break existing subclasses; any changes should probably go in a subclass instead.
*/

@implementation DSPopupHandler

/*
 initForViewsListView:withSettings:userInfo:

 Initializes a handler for the ViewsListView, based on the specified settings.  Used to load a saved handler.  Subclasses should override this, call super, then set up their contents based on [self mode].
 
 Written by DJS 2003-07.
 Changed by DJS 2004-01 to add userInfo.
 */

- (id)initForViewsListView:(id)viewsListView withSettings:(NSDictionary *)settings userInfo:(id)userInfo
{
    self = [self init];

    [self setViewsListView:viewsListView];
    [self setUserInfo:userInfo];
    
    if ([settings objectForKey:@"Mode"])
        [self setMode:[[settings objectForKey:@"Mode"] intValue]];
    
    return self;
}

/*
 init

 Subclasses should override and call super, then load their nib and set up any relevant controls.  Also call -setChangeNotificationName: if you want to send notifications when values change.
 
 Written by DJS 2003-07.
*/

- (id)init
{
    self = [super init];
    
    [self setViewsListView:nil];
    [self setUserInfo:nil];
    [self setMode:[self defaultMode]];
    [self setChangeNotificationName:[self defaultChangeNotificationName]];
    
    [[NSNotificationCenter defaultCenter] addObserver:self selector:@selector(viewsListViewHandlersDidChange:)
    	name:@"DSViewsListViewHandlersDidChange" object:nil];
    
    return self;
}

- (void)dealloc
{
    [[NSNotificationCenter defaultCenter] removeObserver:self];

    [self setViewsListView:nil];
    [self setUserInfo:nil];
    [self setChangeNotificationName:nil];
    
    [super dealloc];
}


/*
 settings
 
 Returns a dictionary representation of the handler's contents, for saving.  Subclasses should override the -mutableSettings method to add their values; they shouldn't need to override this.
 
 Written by DJS 2003-07.
*/

- (NSDictionary *)settings
{
    return [self mutableSettings];
}

/*
 mutableSettings
 
 Returns a mutable dictionary representation of the handler's contents, for saving.  Subclasses should override this, call super, then add their values.
 
 Written by DJS 2003-07.
*/

- (NSMutableDictionary *)mutableSettings
{
    NSMutableDictionary *dict = [NSMutableDictionary dictionaryWithObject:[NSNumber numberWithInt:[self mode]]
                                                                   forKey:@"Mode"];
    
    return dict;
}


// ----------------------------------------------------------------------------------------
#pragma mark -
#pragma mark CONFIGURATION METHODS
// ----------------------------------------------------------------------------------------

/*
 shouldAddAfter
 
 Returns YES if an -add: operation should insert the new container view after the current one, or NO if it should go beforehand.  By default, returns YES.  Subclasses can override this if desired.
 
 Written by DJS 2004-02.
*/

- (BOOL)shouldAddAfter
{
    return YES;
}

/*
 modesAreUnique
 
 Returns YES if each handler must use a different mode.  Mode zero is excluded from this.  By default, returns NO.  Subclasses can override this if desired.
 
 Written by DJS 2004-02.
*/

- (BOOL)modesAreUnique
{
    return NO;
}

/*
 defaultMode
 
 Returns the mode to assign when adding a new handler.  By default, returns zero.  Subclasses can override this if desired.  A special value FIRST_UNUSED_MODE is available to automatically set the mode to the first unused one from those available in the mode popup menu.
 
 Written by DJS 2004-02.
*/

- (int)defaultMode
{
    return 0;
}

/*
 defaultChangeNotificationName
 
 Returns the notification name to use for the -notifyOfChange notification for this class.  Subclasses may override this if desired, or may call -setChangeNotificationName: to set the notification name dynamically.  Overriding this to return nil, or passing nil to the -setChangeNotificationName: method, will disable change notifications.
 
 Written by DJS 2004-02.
*/

+ (NSString *)defaultChangeNotificationName
{
    return @"DSPopupHandlerValueDidChange";
}

/*
 defaultChangeNotificationName
 
 Instance version of the class method of the same name, in case a subclass wants to refer to ivars to determine the default name.  See the class method for details.
 
 Written by DJS 2004-02.
*/

- (NSString *)defaultChangeNotificationName
{
    return [[self class] defaultChangeNotificationName];
}


// ----------------------------------------------------------------------------------------
#pragma mark -
#pragma mark UTILITY METHODS
// ----------------------------------------------------------------------------------------

/*
 selectDefaultItem
 
 Selects the default item in the view.  Subclasses may override this.
 
 Written by DJS 2004-02.
*/

- (void)selectDefaultItem
{
    // Does nothing by default
}

/*
 notifyOfChange
 
 Notify whoever cares that something changed.  Should not need overriding, but subclasses should call this when their controls change, if sending notifications of changes are desired.
 
 Written by DJS 2003-08.
*/

- (void)notifyOfChange
{
    if ([self changeNotificationName])
        [[NSNotificationCenter defaultCenter] postNotificationName:[self changeNotificationName] object:self];
}

/*
 viewsListViewHandlersDidChange:
 
 Notification received when the ViewsListView adds, replaces or removes a handler.  The object contains the ViewsListView.  Should not need overriding.
 
 Written by DJS 2003-07.
*/

- (void)viewsListViewHandlersDidChange:(NSNotification *)note
{
    DSViewsListView *vlv = [note object];

    if (vlv != [self viewsListView])
        return;
    
    BOOL ableToRemove = [vlv count] > 1;

    [self enableRemoveButtons:ableToRemove];

    [self notifyOfChange];
}

/*
 modeDidChange

 Subclasses may override this (without calling super), to do any extra work when the mode is changed.

 Written by DJS 2003-08.
*/

- (void)modeDidChange
{
    // Optional to override this
}

/*
 containerViewDidChange

 Subclasses must override this (without calling super), to update the popups when the view changes.

 Written by DJS 2003-08.
*/

- (void)containerViewDidChange
{
    NSAssert(NO, @"Forgot to override the -containerViewDidChange method!");
}

/*
 enableRemoveButtons:

 Subclasses must override this (without calling super), to enable or disable the remove buttons as indicated.
 
 Written by DJS 2003-08.
*/

- (void)enableRemoveButtons:(BOOL)enable
{
    NSAssert(NO, @"Forgot to override the -enableRemoveButtons: method!");
}


/*
 handlersForMode:includingSelf:
 
 Returns a mutable array of handlers that currently have the specified mode, or nil if there are none.  If inclSelf is YES, the current handler is included if it is that mode, otherwise it is omitted.  Should not need overriding.
 
 Written by DJS 2004-02.
*/

- (NSMutableArray *)handlersForMode:(int)mode includingSelf:(BOOL)inclSelf
{
    DSViewsListView *vlv = [self viewsListView];
    NSMutableArray *handlers = [NSMutableArray arrayWithCapacity:[vlv count]];
    NSEnumerator *enumerator = [[self viewsListView] handlerEnumerator];
    DSPopupHandler *handler = nil;
    
    while ((handler = [enumerator nextObject]))
    {
        if (mode == [handler mode] && (inclSelf || handler != self))
            [handlers addObject:handler];
    }
    
    if ([handlers count])
        return handlers;
    else
        return nil;
}

/*
 usedModesIncludingSelf:
 
 Returns a counted set of the mode numbers that are currently used, in order.  If inclSelf is YES, the current handler's mode is included, otherwise it is omitted.  Since it is a counted set, you can use -countForObject: on a set member to see how many times it is used.  Should not need overriding.
 
 Written by DJS 2004-02.
*/

- (NSCountedSet *)usedModesIncludingSelf:(BOOL)inclSelf
{
    DSViewsListView *vlv = [self viewsListView];
    NSCountedSet *usedModes = [NSCountedSet setWithCapacity:[vlv count]];
    NSEnumerator *enumerator = [[self viewsListView] handlerEnumerator];
    DSPopupHandler *handler = nil;
    
    while ((handler = [enumerator nextObject]))
    {
        if (inclSelf || handler != self)
            [usedModes addObject:[NSNumber numberWithInt:[handler mode]]];
    }
    
    return usedModes;
}

/*
 unusedModes
 
 Should return a mutable array of the mode numbers that aren't already used.  Defined in this abstract class, but not implemented here, since this class doesn't know what modes are available.  Subclasses should override it if needed.  (DSPopupSingleHandler implements it.)
 
 Written by DJS 2004-02.
*/

- (NSMutableArray *)unusedModes
{
    NSAssert(NO, @"Need to override -unusedModes, defined in DSPopupHandler.");
    
    return nil;
}

/*
 firstUnusedMode
 
 Returns the first mode value that isn't already used, or zero if all modes are in use.  This is automatically used to set the default mode if -defaultMode returns FIRST_UNUSED_MODE.  Defined in this abstract class, but always returns zero here, since this class doesn't know what modes are available.  Subclasses should override it if needed.  (DSPopupSingleHandler implements it.)
 
 Written by DJS 2004-02.
*/

- (int)firstUnusedMode
{
    return 0;
}
    
/*
 validateMenuItem:
 
 If the mode popup menu's -autoenablesItems is YES, and the handler is a delegate of the popup menu, this method will be called to validate the items.  If -modesAreUnique returns YES, this causes the items that correspond to modes that are used for other handlers to be disabled.  Mode zero is always allowed multiple times.  Subclasses may override this if desired.
 
 Written by DJS 2004-02.
*/

//  this is called for the target of the item; if having the popup as a whole targetting this object isn't enough, I could assign targets for each item when loading.

- (BOOL)validateMenuItem:(NSMenuItem *)menuItem
{
    int mode = [menuItem tag];
    
    if (!mode || mode == [self mode] || ![self modesAreUnique])
        return YES;
    
    return [self handlersForMode:mode includingSelf:NO] == nil;
}


/*
 appendValue:toString:
 
 Extracts the contents of the provided settings and appends to the output string.  Used for the Services menu item etc.  Subclasses should override this, if this method is used.
 
 Written by DJS 2004-02.
*/

+ (void)appendValue:(NSDictionary *)settings toString:(NSMutableString *)output
{
    // Does nothing by default
}


// ----------------------------------------------------------------------------------------
#pragma mark -
#pragma mark VIEWS LIST HANDLER PROTOCOL METHODS
// ----------------------------------------------------------------------------------------

/*
 containerView
 
 Subclasses must override this (without calling super), to return the current container view, based on [self mode].
 
 Written by DJS 2003-08.
*/

- (NSView *)containerView
{
    NSAssert(NO, @"Forgot to override the -containerView method!");
    
    return nil;
}


// ----------------------------------------------------------------------------------------
#pragma mark -
#pragma mark TARGET/ACTION METHODS
// ----------------------------------------------------------------------------------------

/*
 changeMode:
 
 Action method for the popups.  Should not need overriding.
 
 Written by DJS 2003-07.
*/

- (IBAction)changeMode:(id)sender;
{
    NSView *oldView = [self containerView];
    
    [self setMode:[[sender selectedItem] tag]];

    [self modeDidChange];

    // If the container view has changed, due to the mode change, replace it in the ViewsListView:
    if ([oldView isNotEqualTo:[self containerView]])
    {
        [self containerViewDidChange];
        
        [[self viewsListView] replaceView:oldView withView:[self containerView]];
    }

    [self notifyOfChange];
}

/*
 remove:
 
 Action method for the remove button.  Should not need overriding.
 
 Written by DJS 2003-07.
*/

- (IBAction)remove:(id)sender;
{
    [[self viewsListView] removeHandler:self];
}

/*
 add:
 
 Action method for the add button.  Should not need overriding.
 
 Written by DJS 2003-07.
 Changed by DJS 2004-02 to add FIRST_UNUSED_MODE support.
*/

- (IBAction)add:(id)sender;
{
    int mode = [self defaultMode];
    
    if (mode == FIRST_UNUSED_MODE)
        mode = [self firstUnusedMode];
    
    NSDictionary *settings = [NSDictionary dictionaryWithObject:[NSNumber numberWithInt:mode] forKey:@"Mode"];
    
    id newHandler = [[[[self class] alloc] initForViewsListView:[self viewsListView] withSettings:settings userInfo:[self userInfo]] autorelease];
    
    if (newHandler)
    {
        if ([self shouldAddAfter])
            [[self viewsListView] insertHandler:newHandler afterHandler:self];
        else
            [[self viewsListView] insertHandler:newHandler beforeHandler:self];
    }
}

/*
 hit:
 
 Generic action method for items in the container view.  Subclasses may override this to take any action required, then call super.
 
 Written by DJS 2004-02.
*/

- (IBAction)hit:(id)sender
{
    [self notifyOfChange];
}

/*
 textDidChange:
 
 If a field has the handler has its delegate, this will notify for each keypress.  Subclasses may override this to take any action required, then call super.
 
 Written by DJS 2004-02.
*/

- (void)textDidChange:(NSNotification *)aNotification
{
    [self notifyOfChange];
}


// ----------------------------------------------------------------------------------------
#pragma mark -
#pragma mark ACCESSOR METHODS
// ----------------------------------------------------------------------------------------

- (NSWindow *)window
{
    return [[self viewsListView] window];
}

- (id)viewsListView
{
    return _viewsListView;
}

- (void)setViewsListView:(id)value
{
    [value retain];
    [_viewsListView autorelease];
    _viewsListView = value;
}

- (id)userInfo
{
    return _userInfo;
}

- (void)setUserInfo:(id)value
{
    [value retain];
    [_userInfo autorelease];
    _userInfo = value;
}

- (int)mode
{
    return _mode;
}

- (void)setMode:(int)value
{
    _mode = value;
}

- (NSString *)changeNotificationName
{
    return _changeNotificationName;
}

- (void)setChangeNotificationName:(NSString *)value
{
    [value retain];
    [_changeNotificationName autorelease];
    _changeNotificationName = value;
}

@end
