//
//  DSPopupSingleHandler.m
//  Shared classes
//
//  Created by David Sinclair on Mon Feb 16 2004.
//  Copyright (c) 2004 Dejal Systems, LLC. All rights reserved.
//

#import "DSPopupSingleHandler.h"
#import "DSSharedDefines.h"


/*
 DSPopupSingleHandler
 
 This class is an abstract superclass for a DSViewsListView handler.  It should be used if there is only one container view in one Nib file.  If there are multiple container views in one Nib file, use DSPopupHandler instead.  If there are multiple container views in multiple Nib files, use DSPopupClusterHandler instead.  Do not change this class in a way that will break existing subclasses; any changes should probably go in a subclass instead.
*/

@implementation DSPopupSingleHandler

/*
 initForViewsListView:withSettings:userInfo:

 Initializes a handler for the ViewsListView, based on the specified settings.  Used to load a saved handler.  Subclasses should override this, call super, then set up their contents based on [self mode].
 
 Written by DJS 2004-02.
 */

- (id)initForViewsListView:(id)viewsListView withSettings:(NSDictionary *)settings userInfo:(id)userInfo
{
    self = [super initForViewsListView:viewsListView withSettings:settings userInfo:userInfo];
    
    [self completeInitializationWithSettings:settings];
    
    NSString *nibName = [self nibName];
    
    if (nibName)
        [NSBundle loadNibNamed:nibName owner:self];
    
    // Force the Nib to connect the outlets:
    [viewsListView window];
    
    // Build the mode menu, if needed:
    [self buildModePopup];
    
    // Select the appropriate item:
    [modePopup selectItemAtIndex:[modePopup indexOfItemWithTag:[self mode]]];
    
    /*
     [[NSNotificationCenter defaultCenter] addObserver:self 
                                              selector:@selector(willPopUpNotification:) 
                                                  name:@"NSPopUpButtonWillPopUpNotification"
                                                object:modePopup];
     */
    
    return self;
}

- (void)dealloc
{
    // We're responsible for the top-level objects in the Nib, since this is not a NSWindowController or NSDocument subclass.  So release the container view, which contains everything else:
    [containerView release];
    
    [super dealloc];
}


// ----------------------------------------------------------------------------------------
#pragma mark -
#pragma mark CONFIGURATION METHODS
// ----------------------------------------------------------------------------------------

/*
 completeInitializationWithSettings:
 
 Called just after initialization of the instance.  Set any ivars here, so they are available when needed during the initialization process.
 
 Written by DJS 2004-04.
*/

- (void)completeInitializationWithSettings:(NSDictionary *)settings
{
    // Nothing to do here
}

/*
 nibName
 
 Returns the name of the nib file for this class.  Subclasses should override this and return the appropriate name, referencing -mode if necessary.
 
 Written by DJS 2004-02.
*/

- (NSString *)nibName
{
    NSAssert(NO, @"Forgot to override -nibName, defined in DSPopupSingleHandler.");
    
    return nil;
}


// ----------------------------------------------------------------------------------------
#pragma mark -
#pragma mark UTILITY METHODS
// ----------------------------------------------------------------------------------------

/*
 buildModePopup
 
 Builds the mode popup menu.  Subclasses may override this if desired, and delete all existing items and add the appropriate ones.  If this isn't overridden, it uses the menu from the nib file intact.
 
 Written by DJS 2004-03.
*/

- (void)buildModePopup
{
    // Optional to override
}

/*
 willPopUpNotification:
 
 Notification that the mode popup is about to pop up -- filtered to just that popup.  If -modesAreUnique returns YES, this disables the items that correspond to modes that are used for other handlers.  Mode zero is always allowed multiple times.
 
 Written by DJS 2004-02.
*/

/*
- (void)willPopUpNotification:(NSNotification *)note
{
     if ([note object] != modePopup || ![self modesAreUnique])
         return;
     
     [modePopup setAutoenablesItems:NO];
     
     id handler;
     NSMenu *menu = [modePopup menu];
     NSMenuItem *item = nil;
     NSEnumerator *enumerator = [[menu itemArray] objectEnumerator];
     
     // Enable all items:
     while ((item = [enumerator nextObject]))
     {
         if (![item isSeparatorItem])
             [item setEnabled:YES];
     }
     
     enumerator = [[self viewsListView] handlerEnumerator];
     
     // Then disable the others that are in use:
     while ((handler = [enumerator nextObject]))
     {
         int mode = [handler mode];
         
         if (handler != self && mode)
             [[menu itemWithTag:mode] setEnabled:NO];
     }
}
*/

/*
 unusedModes
 
 Returns a mutable array of the mode numbers that aren't already used, or an empty array if all modes are in use.  Mode zero is excluded.  Should not need overriding.
 
 Written by DJS 2004-02.
*/

- (NSMutableArray *)unusedModes
{
    DSViewsListView *vlv = [self viewsListView];
    NSMutableArray *unusedModes = [NSMutableArray arrayWithCapacity:[vlv count]];
    NSCountedSet *usedModes = [self usedModesIncludingSelf:YES];
    NSMenu *menu = [modePopup menu];
    NSEnumerator *enumerator = [[menu itemArray] objectEnumerator];
    NSMenuItem *item = nil;
    int mode = 0;
    
    while ((item = [enumerator nextObject]))
    {
        mode = [item tag];
        
        if (mode && ![usedModes containsObject:[NSNumber numberWithInt:mode]])
            [unusedModes addObject:[NSNumber numberWithInt:mode]];
    }
    
    return unusedModes;
}

/*
 firstUnusedMode
 
 Returns the first mode value that isn't already used, or zero if all modes are in use.  The order is defined by the order in the popup menu, not numeric order.  This is automatically used to set the default mode if -defaultMode returns FIRST_UNUSED_MODE.  Should not need overriding.
 
 Written by DJS 2004-02.
*/

- (int)firstUnusedMode
{
    NSMutableArray *unusedModes = [self unusedModes];
    
    if ([unusedModes count])
        return [[[self unusedModes] firstObject] intValue];
    else
        return 0;
}


// ----------------------------------------------------------------------------------------
#pragma mark -
#pragma mark DSPOPUPHANDLER OVERRIDE METHODS
// ----------------------------------------------------------------------------------------

/*
 containerViewDidChange

 Subclasses may override this (without calling super), to update the popups when the view changes.  Not normally necessary for a single view handler, since the container view won't change.

 Written by DJS 2004-02.
*/

- (void)containerViewDidChange
{
    // Don't need to do anything here for a single view handler
}

/*
 enableRemoveButtons:
 
 Enables or disables the remove button as indicated.  Subclasses shouldn't need to override this.
 
 Written by DJS 2004-02.
*/

- (void)enableRemoveButtons:(BOOL)enable
{
    [removeButton setEnabled:enable];
}


// ----------------------------------------------------------------------------------------
#pragma mark -
#pragma mark VIEWS LIST HANDLER PROTOCOL METHODS
// ----------------------------------------------------------------------------------------

/*
 containerView
 
 Returns the container view.  Should be no need to override.
 
 Written by DJS 2004-02.
*/

- (NSView *)containerView
{
    return containerView;
}

@end
