//
//  DSChooserIntern.m
//  Shared classes
//
//  Created by David Sinclair on Tues Mar 02 2004.
//  Copyright (c) 2004 Dejal Systems, LLC. All rights reserved.
//
//  An "intern" is a component used by the DSAssistantController class -- the label was chosen as a description of a subordinate assistant.
//

#import "DSChooserIntern.h"
#import "DSSharedDefines.h"


@implementation DSChooserIntern

// ----------------------------------------------------------------------------------------
#pragma mark -
#pragma mark HELPER METHODS
// ----------------------------------------------------------------------------------------

/*
 settingsWithName:
 
 Helper method to construct the settings for this chooser intern (assistant component).  Call this to create the settings, then call the following method to add each line.
 
 Written by DJS 2004-03.
*/

+ (NSMutableDictionary *)settingsWithName:(NSString *)name
{
    NSMutableDictionary *dict = [NSMutableDictionary dictionary];
    
    [dict setObject:name forKey:@"Name" defaultValue:nil];
    [dict setObject:[NSMutableArray array] forKey:@"Items" defaultValue:nil];
    [dict setObject:@"YES" forKey:@"IsGroup"];
    
    return dict;
}

/*
 addLineToSettings:basedOnDictionary:withName:identifier:class:description:
 
 Helper method to construct the settings for this chooser intern.  Call the above method to create the settings, then call this one to add each line.  The dict can be nil, or contain some additional values to add to the settings for this line.
 
 Written by DJS 2004-03.
*/

+ (void)addLineToSettings:(NSMutableDictionary *)settings basedOnDictionary:(NSMutableDictionary *)dict withName:(NSString *)name identifier:(NSString *)identifier class:(Class)class description:(NSString *)description
{
    NSMutableArray *array = [settings objectForKey:@"Items"];
    
    if (dict)
        dict = [[dict mutableCopy] autorelease];
    else
        dict = [NSMutableDictionary dictionary];
    
    [dict setObject:name forKey:@"Name" defaultValue:nil];
    [dict setObject:identifier forKey:@"Identifier" defaultValue:nil];
    [dict setObject:[class description] forKey:@"Class" defaultValue:nil];
    [dict setObject:description forKey:@"Description" defaultValue:nil];
    
    [array addObject:dict];
}


// ----------------------------------------------------------------------------------------
#pragma mark -
#pragma mark INTERN PROTOCOL METHODS
// ----------------------------------------------------------------------------------------

/*
 initWithSettings:
 
 Initializes an instance of the intern (assistant component) using the specified settings.  If the settings is nil, defaults are assumed (though for this chooser one, that means a blank list... not very useful).  This is the designated initializer.
 
 Written by DJS 2004-03.
*/

- (id)initWithSettings:(NSDictionary *)settings
{
    self = [self init];
    
    NSArray *items = [settings objectForKey:@"Items"];
    
    if (!items)
        items = [NSArray array];
    
    [self setName:[settings objectForKey:@"Name"]];
    [self setItems:items];
    
    [NSBundle loadNibNamed:[self internNibName] owner:self];
    
    [listOutlineView setDoubleAction:@selector(doubleClicked:)];
    [listOutlineView reloadData];
    
    [self expandAllItems];
    
    return self;
}

- (void)dealloc
{
    // We're responsible for the top-level objects in the Nib, since this is not a NSWindowController or NSDocument subclass.  So release the container view, which contains everything else:
    [containerView release];
    
    [self setName:nil];
    [self setItems:nil];
    
    [super dealloc];
}

/*
 name
 
 Returns the name to display in the window prompt.
 
 Written by DJS 2004-03.
*/

- (NSString *)name
{
    return _name;
}

/*
 containerView
 
 Returns the view enclosing the intern's items.
 
 Written by DJS 2004-03.
*/

- (NSView *)containerView
{
    return containerView;
}

/*
 willFinish
 
 Returns YES if this is the last intern.  If so, the assistant button will be Finish, if not it will be Next.
 
 Written by DJS 2004-03.
*/

- (BOOL)willFinish
{
    // Will be Finish if a line is selected with no class associated with it:
    return ([listOutlineView selectedItem] && ![self nextInternClass]);
}

/*
 canProceed
 
 Returns YES if the Next or Finish button should be enabled.
 
 Written by DJS 2004-03.
*/

- (BOOL)canProceed
{
    NSDictionary *dict = [listOutlineView selectedItem];
    
    // Can proceed if a line is selected, and it isn't a parent line:
    return (dict && ![dict objectForKey:@"IsGroup"]);
}

/*
 nextInternClass
 
 Returns the class to use when creating the next intern (when the user clicks the Next button in the Assistant window).
 
 Written by DJS 2004-03.
*/

- (Class)nextInternClass
{
    NSDictionary *dict = [listOutlineView selectedItem];
    NSString *className = [dict objectForKey:@"Class"];
    Class class = NSClassFromString(className);
    
    return class;
}

/*
 operation
 
 Returns a string indicating what this intern represents.  The modal delegate can ask for this to figure out what to do next.
 
 Written by DJS 2004-03.
*/

- (NSString *)operation
{
    return @"DSChooser";
}

/*
 settings
 
 Returns a dictionary with information needed by the next intern.  For this chooser one, it outputs the dictionary for the selected item.
 
 Written by DJS 2004-03.
*/

- (NSDictionary *)settings
{
    return [listOutlineView selectedItem];
}


// ----------------------------------------------------------------------------------------
#pragma mark -
#pragma mark TARGET/ACTION METHODS
// ----------------------------------------------------------------------------------------

- (IBAction)outlineViewHit:(id)sender
{
    // Nothing to do
}

- (IBAction)doubleClicked:(id)sender
{
    [[NSNotificationCenter defaultCenter] postNotificationName:@"DSAssistantWantNextInternNotification" object:self];
}


// ----------------------------------------------------------------------------------------
#pragma mark -
#pragma mark NOTIFICATION METHODS
// ----------------------------------------------------------------------------------------

- (void)outlineViewSelectionDidChange:(NSNotification *)aNotification
{
    NSDictionary *dict = [listOutlineView selectedItem];
    NSString *description = [dict objectForKey:@"Description"];
    
    if (!description)
        description = @"";
    
    [descriptionTextView setString:description];
    
    [[NSNotificationCenter defaultCenter] postNotificationName:@"DSAssistantInternStateChangedNotification" object:self];
}


// ----------------------------------------------------------------------------------------
#pragma mark -
#pragma mark SPLIT VIEW DELEGATE METHODS
// ----------------------------------------------------------------------------------------

- (float)splitView:(NSSplitView *)sender constrainMaxCoordinate:(float)proposedMax ofSubviewAt:(int)offset
{
    return [sender frame].size.height - 75.0;
}

- (float)splitView:(NSSplitView *)sender constrainMinCoordinate:(float)proposedMin ofSubviewAt:(int)offset
{
    return 75.0;
}

- (BOOL)splitView:(NSSplitView *)sender canCollapseSubview:(NSView *)subview
{
    return YES;
}


// ----------------------------------------------------------------------------------------
#pragma mark -
#pragma mark OUTLINE VIEW DATA SOURCE METHODS
// ----------------------------------------------------------------------------------------

- (id)outlineView:(NSOutlineView *)olv child:(int)index ofItem:(id)item
{
    if (!item)
        return [[self items] objectAtIndex:index];
    else
        return [[item objectForKey:@"Items"] objectAtIndex:index];
}

- (BOOL)outlineView:(NSOutlineView *)olv isItemExpandable:(id)item
{
    return ([item objectForKey:@"IsGroup"] != nil);
}

- (int)outlineView:(NSOutlineView *)olv numberOfChildrenOfItem:(id)item
{
    if (!item)
        return [[self items] count];
    else
        return [[item objectForKey:@"Items"] count];
}

- (id)outlineView:(NSOutlineView *)olv objectValueForTableColumn:(NSTableColumn *)tableColumn byItem:(id)item
{
    id value = [item objectForKey:@"Name"];
    
    return value;
}


- (void)expandAllItems
{
    NSEnumerator *enumerator = [[self items] objectEnumerator];
    NSMutableDictionary *item;
    
    while ((item = [enumerator nextObject]))
    {
    	[listOutlineView expandItem:item expandChildren:YES];
    }
}


// ----------------------------------------------------------------------------------------
#pragma mark -
#pragma mark OUTLINE VIEW DELEGATE METHODS
// ----------------------------------------------------------------------------------------

- (void)outlineView:(NSOutlineView *)olv willDisplayCell:(NSCell *)cell
     forTableColumn:(NSTableColumn *)tableColumn item:(id)item
{
    NSFont *font = [cell font];
    
    if ([item objectForKey:@"IsGroup"])
        font = [[NSFontManager sharedFontManager] convertFont:font toHaveTrait:NSBoldFontMask];
    else
        font = [[NSFontManager sharedFontManager] convertFont:font toHaveTrait:NSUnboldFontMask];
    
    [cell setFont:font];
}


// ----------------------------------------------------------------------------------------
#pragma mark -
#pragma mark ACCESSOR METHODS
// ----------------------------------------------------------------------------------------

- (void)setName:(NSString *)value
{
    [value retain];
    [_name autorelease];
    _name = value;
}

- (NSArray *)items
{
    return _items;
}

- (void)setItems:(NSArray *)value
{
    [value retain];
    [_items autorelease];
    _items = value;
}

- (NSString *)internNibName
{
    return @"DSChooserIntern";
}

@end
