//
//  DSPluginManager.m
//  Shared classes
//
//  Created by David Sinclair on Thu Jun 19 2003.
//  Copyright (c) 2003 - 2007 Dejal Systems, LLC. All rights reserved.
//

#import "DSPluginManager.h"
#import "DSSharedController.h"
#import "DSSharedDefines.h"


@interface DSPluginManager (Private)

- (NSMutableArray *)allPluginPaths;
- (void)loadPlugins;
- (void)activatePlugin:(NSString *)path;
// - (void)instantiatePlugins:(Class)pluginClass;

@end


@implementation DSPluginManager

id sharedPluginManager = nil;

+ (id)sharedPluginManager;
{
    if (!sharedPluginManager)
        sharedPluginManager = [[self alloc] init];
    
    return sharedPluginManager;
}

- (id)init;
{
    self = [super init];

    _classes = [[NSMutableArray alloc] init];

    [self loadPlugins];

    [[NSNotificationCenter defaultCenter] addObserver:self selector:@selector(applicationWillTerminate:)
    	name:@"NSApplicationWillTerminateNotification" object:nil];

    return self;
}

- (void)dealloc
{
    [[NSNotificationCenter defaultCenter] removeObserver:self];
    
    [_classes release];

    [super dealloc];
}

- (void)applicationWillTerminate:(NSNotification *)notification
{
    NSEnumerator *enumerator;
    Class pclass;
    enumerator = [[self classes] objectEnumerator];

    while ((pclass = [enumerator nextObject]))
    {
        [pclass terminateClass];
    }

    [self autorelease];
}


- (NSEnumerator *)classEnumerator
{
    return [[self classes] objectEnumerator];
}

/*
 pluginForIdentifier:forType:protocol:withSettings:orDefaultIdentifier:orFirst:
 
 Finds the plugin class for the specified identifier value, and returns a new instance of that plugin, using the specified settings.  If that identifier is not found, but the default one is, it is returned instead, otherwise if firstIfNotFound is YES, an instance of the first encountered plugin is returned, otherwise nil is returned.  Pass nil for defaultIdentifier to fall back on the firstIfNotFound condition.
 
 Written 2004-06.
 Changed by DJS 2007-06 to support +pluginForType:withSettings:.
*/

- (id)pluginForIdentifier:(NSString *)identifier forType:(NSString *)type protocol:(Protocol *)protocol withSettings:(NSMutableDictionary *)settings orDefaultIdentifier:(NSString *)defaultIdentifier orFirst:(BOOL)firstIfNotFound;
{
    Class class = [self classForIdentifier:identifier withProtocol:protocol orDefaultIdentifier:defaultIdentifier orFirst:firstIfNotFound];
    
    // Note: once Leopard is the minimum, can displense with the type argument and just use NSStringFromProtocol:
   if ([class respondsToSelector:@selector(pluginForType:withSettings:)])
        return [class pluginForType:type withSettings:settings];
    else if ([class respondsToSelector:@selector(pluginWithSettings:)])
        return [class pluginWithSettings:settings];
    else
        return nil;
}

/*
 classForIdentifier:withProtocol:orFirst:
 
 Finds and returns the plugin class for the specified identifier value.  If that identifier is not found, and firstIfNotFound is YES, the class of the first encountered plugin is returned, otherwise nil is returned.
 
 Written 2004-06.
 Changed by DJS 2007-06 to add a protocol check.
*/

- (Class)classForIdentifier:(NSString *)identifier withProtocol:(Protocol *)protocol orFirst:(BOOL)firstIfNotFound;
{
    return [self classForIdentifier:identifier withProtocol:protocol orDefaultIdentifier:nil orFirst:firstIfNotFound];
}

/*
 classForIdentifier:withProtocol:orDefaultIdentifier:orFirst:
 
 Finds and returns the plugin class for the specified identifier value.  If that identifier is not found, but the default one is, it is returned instead, otherwise if firstIfNotFound is YES, the class of the first encountered plugin is returned, otherwise nil is returned.  Pass nil for defaultIdentifier to fall back on the firstIfNotFound condition.  It only considers the plugin a match if it conforms to the specified protocol, so a valid one should always be returned.
 
 Written 2004-06.
 Changed by DJS 2007-06 to add a protocol check.
*/

- (Class)classForIdentifier:(NSString *)identifier withProtocol:(Protocol *)protocol orDefaultIdentifier:(NSString *)defaultIdentifier orFirst:(BOOL)firstIfNotFound;
{
    NSEnumerator *enumerator = [self classEnumerator];
    Class class = nil;
    Class defaultClass = nil;
    Class firstClass = nil;
    BOOL found = NO;
    
    while (!found && (class = [enumerator nextObject]))
    {
        // Is it the identifier we want?
        if ([identifier isEqualToString:[class identifier]] && [class conformsToProtocol:protocol])
            found = YES;
        
        // Is a default identifier specified, and is it that one?
        if (defaultIdentifier && [defaultIdentifier isEqualToString:[class identifier]] && [class conformsToProtocol:protocol])
            defaultClass = class;
        
        // Have we not noted the first class yet, and do we care?
        if (!firstClass && firstIfNotFound && [class conformsToProtocol:protocol])
            firstClass = class;
    }
    
    if (found)
        return class;
    else if (defaultClass)
        return defaultClass;
    else if (firstClass)
        return firstClass;
    else
        return nil;
}


- (NSMutableArray *)classes
{
    return _classes;
}

@end


@implementation DSPluginManager (Private)

- (NSMutableArray *)allPluginPaths;
{
    NSArray *librarySearchPaths;
    NSEnumerator *searchPathEnum;
    NSString *currPath;
    NSMutableArray *bundleSearchPaths = [NSMutableArray array];
    NSMutableArray *allPluginPaths = [NSMutableArray array];
    
    librarySearchPaths = NSSearchPathForDirectoriesInDomains(NSLibraryDirectory, NSAllDomainsMask - NSSystemDomainMask, YES);
    
    searchPathEnum = [librarySearchPaths objectEnumerator];
    
    while ((currPath = [searchPathEnum nextObject]))
    {
        // Get the product data directory, and the plugins subdirectory:
        NSString *dataPath = [[[currPath stringByAppendingPathComponent:@"Application Support"] stringByAppendingPathComponent:COMPANY_NAME_BRIEF] stringByAppendingPathComponent:PRODUCT_NAME];
        NSString *pluginsPath = [dataPath stringByAppendingPathComponent:@"Plugins"];
        
        // If the product data directory exists but the plugin one doesn't, create it:
        if ([[NSFileManager defaultManager] fileExistsAtPath:dataPath] && ![[NSFileManager defaultManager] fileExistsAtPath:pluginsPath])
            [[NSFileManager defaultManager] createDirectoryAtPath:pluginsPath attributes:nil];
        
        [bundleSearchPaths addObject:pluginsPath];
    }
    
    [bundleSearchPaths addObject:[[NSBundle mainBundle] builtInPlugInsPath]];
    
    searchPathEnum = [bundleSearchPaths objectEnumerator];
    
    while ((currPath = [searchPathEnum nextObject]))
    {
        NSEnumerator *bundleEnum = [[[NSFileManager defaultManager] directoryContentsAtPath:currPath] objectEnumerator];
        NSString *currBundlePath;
        
        if (bundleEnum)
        {
            while ((currBundlePath = [bundleEnum nextObject]))
            {
                NSString *pluginPath = [currPath stringByAppendingPathComponent:currBundlePath];
                
                // Only add the bundle path if one with the same filename (without extension) isn't already there:
                if (![allPluginPaths objectMatching:[pluginPath lastPathComponentWithoutExtension] usingSelector:@selector(lastPathComponentWithoutExtension)])
                {
                    [allPluginPaths addObject:pluginPath];
                }
            }
        }
    }
    
    return allPluginPaths;
}

- (void)loadPlugins;
{
#if SHOW_SPLASH
    [[DSSharedController mainController] changeSplashProgress:NSLocalizedStringFromTable(@"Loading plug-ins...", @"DSShared", @"Splash: loading plugins")];
#endif
    
    NSEnumerator *enumerator = [[self allPluginPaths] objectEnumerator];
    NSString *pluginPath;

    while ((pluginPath = [enumerator nextObject]))
    {
        [self activatePlugin:pluginPath];

        [[DSSharedController mainController] animateDockIcon];
    }
}

//	This is called to activate each plug-in, meaning that each candidate bundle is checked,
//	loaded if it seems to contain a valid plug-in, and the plug-in's class' initiateClass
//	method is called. If this returns YES, it means that the plug-in agrees to run and the
//	class is added to the pluginClass array. Some plug-ins might refuse to be activated
//	depending on some external condition.

- (void)activatePlugin:(NSString *)path
{
    NSBundle *pluginBundle = [NSBundle bundleWithPath:path];

    if (pluginBundle)
    {
        NSDictionary *pluginDict = [pluginBundle infoDictionary];
        NSString *pluginName = [pluginDict objectForKey:@"NSPrincipalClass"];

        if (pluginName)
        {
            Class pluginClass = NSClassFromString(pluginName);

            if (!pluginClass)
            {
                pluginClass = [pluginBundle principalClass];
                
                if ([pluginClass conformsToProtocol:@protocol(DSPluginProtocol)] && [pluginClass isSubclassOfClass:[NSObject class]] && [pluginClass initializeClass:pluginBundle])
                {
                    [[self classes] addObject:pluginClass];
                }
            }
        }
    }
}

@end
