#import "DSPrefsAdvanced.h"
#import "DSSharedDefines.h"
#import "FilePathField.h"


@implementation DSPrefsAdvanced

// ----------------------------------------------------------------------------------------
#pragma mark -
#pragma mark MANAGEMENT METHODS
// ----------------------------------------------------------------------------------------

/*
 prepare
 
 Called after loading the preference view, and when restoring factory settings.  Advanced prefs subclasses should override and set the controls based on the preferences, then call super.
 
 Written by DJS 2005-05.
*/

- (BOOL)prepare
{
    [keepBackupCheck setIntValue:[PREFERENCES integerForKey:SHARED_DATA_KEEP_BACKUP]];
    [dataPathStat setObjectValue:[PREFERENCES objectForKey:SHARED_DATA_FOLDER_PATH]];
    
    return [super prepare];
}

/*
 maintainControls
 
 Called to maintain the enabled state of the controls.  Advanced prefs subclasses should override and enable or disable their own controls as appropriate, then call super.
 
 Written by DJS 2005-05.
*/

- (BOOL)maintainControls
{
    return [super maintainControls];
}


// ----------------------------------------------------------------------------------------
#pragma mark -
#pragma mark TARGET/ACTION METHODS
// ----------------------------------------------------------------------------------------

/*
 hit:
 
 Action method available for fields or other controls that are hit.  Advanced prefs subclasses can override this to perform any operations required, and should call super.
 
 Written by DJS 2005-05.
*/

- (IBAction)hit:(id)sender
{
    [super hit:sender];
}

/*
 keepBackupCheckHit:
 
 Action method for the keep backup checkbox.  Advanced prefs subclasses shouldn't need to override this.
 
 Written by DJS 2005-05.
*/

- (IBAction)keepBackupCheckHit:(id)sender
{
    [PREFERENCES setInteger:[keepBackupCheck intValue] forKey:SHARED_DATA_KEEP_BACKUP];

    [super hit:sender];
}

/*
 dataPathStatHit:
 
 Action method for the data path stat.  Advanced prefs subclasses shouldn't need to override this.
 
 Written by DJS 2005-05.
*/

- (IBAction)dataPathStatHit:(id)sender
{
    [self setDataPathStat:nil withPath:[dataPathStat stringValue]];
}

/*
 dataPathSetButtonHit:
 
 Action method for the data path Set button.  Advanced prefs subclasses shouldn't need to override this.
 
 Written by DJS 2005-05.
*/

- (IBAction)dataPathSetButtonHit:(id)sender
{
    NSOpenPanel *oPanel = [NSOpenPanel openPanel];

    [oPanel setAllowsMultipleSelection:NO];
    [oPanel setCanChooseFiles:NO];
    [oPanel setCanChooseDirectories:YES];

    // The following causes a leak, but the open panel autoreleases the path (really an Apple bug; reported to Apple):
    NSString *path = [[[PREFERENCES objectForKey:SHARED_DATA_FOLDER_PATH] stringByExpandingTildeInPath] retain];

    [oPanel beginSheetForDirectory:path file:nil types:nil modalForWindow:[self window]
                     modalDelegate:self didEndSelector:@selector(dataPathOpenPanelDidEnd:returnCode:contextInfo:)
                       contextInfo:nil];
}


// ----------------------------------------------------------------------------------------
#pragma mark -
#pragma mark ACCESSOR METHODS
// ----------------------------------------------------------------------------------------

/*
 displayName
 
 Class method to return the name to display as the toolbar label and window title for these preferences.  Advanced prefs subclasses do not need to override this.
 
 Written by DJS 2005-05.
*/

+ (NSString *)displayName
{
    return NSLocalizedStringFromTable(@"Advanced", @"DSShared", @"Advanced preference page title");
}

/*
 iconName
 
 Class method to return the name of the image to use as the toolbar icon.  Advanced prefs subclasses do not need to override this.
 
 Written by DJS 2005-05.
*/

+ (NSString *)iconName
{
    return @"ToolbarPrefsAdvanced";
}

/*
 helpURL
 
 Returns the URL of the User Guide page for these preferences, for use by the help (?) button.  Advanced prefs subclasses do not need to override this.
 
 Written by DJS 2005-05.
*/

- (NSString *)helpURL
{
    return URL_GUIDE_PREFS_ADVANCED;
}

/*
 mutablePreferenceKeys
 
 Returns a mutable array of strings, representing the preference keys controlled by this preference view.  Advanced prefs subclasses should override this and add their preference keys.
 
 Written by DJS 2005-05.
*/

- (NSMutableArray *)mutablePreferenceKeys
{
    NSMutableArray *array = [super mutablePreferenceKeys];
    
    [array addObject:SHARED_DATA_KEEP_BACKUP];
    [array addObject:SHARED_DATA_FOLDER_PATH];
    
    return array;
}


// ----------------------------------------------------------------------------------------
#pragma mark -
#pragma mark EXTRA METHODS
// ----------------------------------------------------------------------------------------

/*
 dataPathOpenPanelDidEnd:returnCode:contextInfo:
 
 Did-end method for the path open panel.  Advanced prefs subclasses do not need to call or override this.
 
 Written by DJS 2005-05.
*/

- (void)dataPathOpenPanelDidEnd:(NSOpenPanel *)sheet returnCode:(int)returnCode contextInfo:(void *)contextInfo
{
    if (returnCode == NSOKButton)
    {
        NSString *path = [[sheet filename] stringByAbbreviatingWithTildeInPath];

        [self setDataPathStat:dataPathStat withPath:path];
    }
}

/*
 setDataPathStat:withPath:
 
 Sets the path display and preference as specified.  Advanced prefs subclasses do not need to call or override this.
 
 Written by DJS 2005-05.
*/

- (void)setDataPathStat:(NSTextField *)stat withPath:(NSString *)path
{
    if (![path length])
        return;
    
    if (stat)
        [stat setStringValue:path];

    [PREFERENCES setObject:path forKey:SHARED_DATA_FOLDER_PATH];

    [self maintainControls];
}

@end
