//
//  DSProgressData.m
//  Shared classes
//
//  Created by David Sinclair on Wed Jun 11 2003.
//  Copyright (c) 2003 - 2005 Dejal Systems, LLC. All rights reserved.
//

#import "DSProgressData.h"
#import "DSSharedDefines.h"


#define PROGRESS_CURRENT_NAME @"Name"
#define PROGRESS_CURRENT_SIZE @"Size"
#define PROGRESS_CURRENT_PARENT_PATH @"ParentPath"
#define PROGRESS_CURRENT_LEVEL @"Level"
#define PROGRESS_CURRENT_OBJECT @"Object"


@interface DSProgressData (Private)

- (NSMutableArray *)items;
- (void)setItems:(NSMutableArray *)value;

- (NSMutableDictionary *)currentItem;
- (void)setCurrentItem:(NSMutableDictionary *)value;

@end


@implementation DSProgressData

- (id)init
{
    self = [super init];
    
    [self setItems:[NSMutableArray array]];
    
    return self;
}

- (void)dealloc
{
    [self setWindowTitle:nil];
    [self setOperation:nil];
    [self setSizeAction:nil];
    [self setSourceName:nil];
    [self setDestinationName:nil];
    [self setCurrentItem:nil];
    [self setButton1Title:nil];
    [self setButton2Title:nil];
    [self setButton3Title:nil];
    [self setMessage:nil];
    
    [self setCurrentObject:nil];
    [self setItems:nil];
    
    [super dealloc];
}

// -release is overrided to automatically set the mode to Finished when the client releases the progress data:

- (oneway void)release
{
    if ([self modeIsNormalOrCollapsed] && [self retainCount] <= 2)
        [self setMode:ProgressModeFinished];
    
    [super release];
}


// ----------------------------------------------------------------------------------------
#pragma mark -
#pragma mark CONVENIENCE METHODS
// ----------------------------------------------------------------------------------------

/*
 modeIsNormalOrCollapsed
 
 Returns YES if the mode is currently Normal or Collapsed, otherwise NO.
 
 Written 2003-06.
 */

- (BOOL)modeIsNormalOrCollapsed
{
    return ([self mode] == ProgressModeNormal || [self mode] == ProgressModeCollapsed);
}

/*
 setModeToDefaultAlways:
 
 Changes the mode to Normal or Collapsed, based on a preference.  If always is YES, it always changes the mode, otherwise it only does so if the mode isn't one of those already.  Client methods don't need to call this directly; it is called by the convenience methods, below.
 
 Written 2003-06.
 */

- (void)setModeToDefaultAlways:(BOOL)always
{
    if (always || ![self modeIsNormalOrCollapsed])
    {
        if ([PREFERENCES integerForKey:PROGRESS_START_COLLAPSED])
            [self setMode:ProgressModeCollapsed];
        else
            [self setMode:ProgressModeNormal];
    }
}

/*
 startOverWithWindowTitle:operation:sizeAction:sourceName:destinationName:buttonTitle:
 
 Call this to recycle a progress view for a very different operation: everything is reset; it's just like starting a new progress (via the similarly named method in ProgressWindowController), but the existing view is used.  Example values: windowTitle: @"Copy", operation: @"Preparing copy", sizeAction: @"Copied".
 
 Written 2003-06.
 */

- (void)startOverWithWindowTitle:(NSString *)windowTitle
                       operation:(NSString *)operation
                      sizeAction:(NSString *)sizeAction
                      sourceName:(NSString *)sourceName
                 destinationName:(NSString *)destinationName
                     buttonTitle:(NSString *)buttonTitle
{
    [self setWindowTitle:windowTitle];
    [self setOperation:operation];
    [self setSizeAction:sizeAction];
    [self setSourceName:sourceName];
    [self setDestinationName:destinationName];
    [self setButton1Title:buttonTitle];
    [self setButton2Title:nil];
    [self setButton3Title:nil];
    [self setMessage:nil];
    
    [self setCountNth:0];
    [self setSizeNth:0];
    [self setSizeTotal:0];
    
    [self setStopped:NO];
    
    [self setCurrentObject:nil];
    [self setItems:[NSMutableArray array]];
    
    [self setModeToDefaultAlways:NO];
}

/*
 reset
 
 Call this to reset the progress back to the beginning, i.e. as if -nextObject hadn't been called yet, but leave other aspects the same as before.  You may want to use -changeOperation:sizeAction:buttonTitle: instead.
 
 Written 2003-06.
 */

- (void)reset
{
    [self setCountNth:0];
    [self setSizeNth:0];
    
    [self setStopped:NO];
    
    [self setCurrentItem:nil];
}

/*
 donePreparing
 
 Returns YES if the progress has finished the preparing stage, i.e. -nextObject is being used, and the total size is known.
 
 Written 2003-06.
 */

- (BOOL)donePreparing
{
    return ([self countNth] && [self sizeTotal]);
}

/*
 preparingObject:withName:size:
 
 Call this during a preparing/scanning phase of an operation for each item to be counted.  It remembers the current object, name, and size, and adds the size to the total size value.  Note that it ignores the object if it has already been added with the same object, name, and size.  To also specify a parent path, call -preparingObject:withName:size:parentPath:, below.  Returns YES if the object was added, otherwise NO.  Poll -stopped each time around the loop, to see if the user clicked the Stop button.
 
 Written 2003-06.
 */

- (BOOL)preparingObject:(id)object withName:(NSString *)name size:(int)size
{
    return [self preparingObject:object withName:name size:size parentPath:nil level:0];
}

/*
 preparingObject:withName:size:parentPath:level:
 
 Call this during a preparing/scanning phase of an operation for each item to be counted.  It remembers the current object, name, size, and parent path, and adds the size to the total size value.  Note that it ignores the object if it has already been added with the same object, name, and size.  The parent path is for your use via -currentParentPath or -currentPath, below, to help maintain a hierarchy; it may be nil (or call -preparingObject:withName:size:, above).  Similarly, the level can help with a hierarchy, via -currentLevel, below.  Returns YES if the object was added, otherwise NO.  Poll -stopped each time around the loop, to see if the user clicked the Stop button.
 
 Written 2003-06.
 Changed by DJS 2005-01 to add parentPath and level.
 */

- (BOOL)preparingObject:(id)object withName:(NSString *)name size:(int)size parentPath:(NSString *)parentPath level:(int)level
{
    BOOL added = NO;
    NSMutableDictionary *oldCurrent = [self currentItem];
    
    // Create a new dictionary for the current item:
    [self setCurrentItem:[NSMutableDictionary dictionary]];
    
    // Set the values of the current (new) item:
    [self setCurrentObject:object];
    [self setCurrentName:name];
    [self setCurrentSize:size];
    [self setCurrentParentPath:parentPath];
    [self setCurrentLevel:level];
    
    // Make sure an identical item isn't already in the array (useful when preparing from an outline view with some parent and some child lines selected):
    if ([[self items] containsObject:[self currentItem]])
        [self setCurrentItem:oldCurrent];
    else
    {
        // Add the new dictionary to the array of items:
        [[self items] addObject:[self currentItem]];
        
        // Add the size to the total:
        [self setSizeTotal:[self sizeTotal] + size];
        
        added = YES;
    }
    
    [self setModeToDefaultAlways:NO];
    
    return added;
}

/*
 changeOperation:sizeAction:buttonTitle:
 
 Call this to reset the progress back to the beginning, i.e. as if -nextObject hadn't been called yet, plus also change the operation, action, and button titles.  Useful if performing two (or more) operations on the same set of items.
 
 Written 2003-06.
 */

- (void)changeOperation:(NSString *)operation sizeAction:(NSString *)sizeAction
            buttonTitle:(NSString *)buttonTitle
{
    [self setOperation:operation];
    [self setSizeAction:sizeAction];
    [self setButton1Title:[NSLocalizedStringFromTable(@"Stop", @"DSShared", @"Stop button title") or:buttonTitle]];
    [self setButton2Title:nil];
    [self setButton3Title:nil];
    [self setMessage:nil];
    
    [self setCountNth:0];
    [self setSizeNth:0];
    
    [self setCurrentItem:nil];
    
    [self setStopped:NO];
    
    [self setModeToDefaultAlways:NO];
}

/*
 nextObject
 
 Just like the nextObject for NSEnumerator, call this to obtain the next object for the progress.  The object is the one passed to -preparingObject:withName:size: during the preparing phase.  When -nextObject returns nil, all objects have been enumerated.  If the name or size are desired, they can be obtained via -currentName and -currentSize; also -currentObject can be used to get the current object again, if needed.  The progress view will maintain all of the user interface adjustments to indicate that this object is being processed.  Don't forget to poll -stopped each time around the loop too, to see if the user clicked the Stop button.  When done (successfully or if stopped), release the progress data to automatically dismiss the progress view.
 
 Written 2003-06.
 */

- (id)nextObject
{
    id object = nil;
    int nth = [self countNth];
    
    // If there are more to do:
    if (nth < [self countTotal])
    {
        // Increment the nth counter:
        nth++;
        [self setCountNth:nth];
        
        // Nth is 1-based, so need to subtract 1 to get the index in the array:
        [self setCurrentItem:[[self items] objectAtIndex:nth - 1]];
        
        // Get the current object to return:
        object = [self currentObject];
        
        // Add the current size to the size done:
        [self setSizeNth:[self sizeNth] + [self currentSize]];
        
        [self setModeToDefaultAlways:NO];
    }
    
    return object;
}

/*
 changeToAlertWithMessage:
 
 Changes the progress view to display the alert view with the specified message and a button title of "OK".  If the message is nil, a boring default message is used... but it is recommended to provide a helpful message.  The client method should release the receiver after calling this method; the progress methods will take care of displaying the alert and cleaning up.
 
 Written 2003-06.
 */

- (void)changeToAlertWithMessage:(NSString *)message
{
    [self changeToAlertWithMessage:message buttonTitle:nil];
}

/*
 changeToAlertWithMessage:buttonTitle:
 
 Changes the progress view to display the alert view with the specified message and button title.  If the message is nil, a boring default message is used... but it is recommended to provide a helpful message.  The button title can also be nil, in which case it will default to "OK".  The client method should release the receiver after calling this method; the progress methods will take care of displaying the alert and cleaning up.
 
 Written 2003-06.
 */

- (void)changeToAlertWithMessage:(NSString *)message buttonTitle:(NSString *)buttonTitle
{
    [self setMessage:message];
    
    if ([buttonTitle length])
        [self setButton1Title:buttonTitle];
    else
        [self setButton1Title:NSLocalizedStringFromTable(@"OK", @"DSShared", @"OK button title")];
    
    [self setMode:ProgressModeAlert];
}


// ----------------------------------------------------------------------------------------
#pragma mark -
#pragma mark ACCESSOR METHODS
// ----------------------------------------------------------------------------------------

- (NSString *)windowTitle
{
    return _windowTitle;
}

- (void)setWindowTitle:(NSString *)value
{
    [value retain];
    [_windowTitle autorelease];
    _windowTitle = value;
}

- (NSString *)operation
{
    return _operation;
}

- (void)setOperation:(NSString *)value
{
    [value retain];
    [_operation autorelease];
    _operation = value;
}

- (NSString *)sizeAction
{
    return _sizeAction;
}

- (void)setSizeAction:(NSString *)value
{
    [value retain];
    [_sizeAction autorelease];
    _sizeAction = value;
}

- (NSString *)sourceName
{
    return _sourceName;
}

- (void)setSourceName:(NSString *)value
{
    [value retain];
    [_sourceName autorelease];
    _sourceName = value;
}

- (NSString *)destinationName
{
    return _destinationName;
}

- (void)setDestinationName:(NSString *)value
{
    [value retain];
    [_destinationName autorelease];
    _destinationName = value;
}

- (NSString *)button1Title
{
    return _button1Title;
}

- (void)setButton1Title:(NSString *)value
{
    [value retain];
    [_button1Title autorelease];
    _button1Title = value;
}

- (NSString *)button2Title
{
    return _button2Title;
}

- (void)setButton2Title:(NSString *)value
{
    [value retain];
    [_button2Title autorelease];
    _button2Title = value;
}

- (NSString *)button3Title
{
    return _button3Title;
}

- (void)setButton3Title:(NSString *)value
{
    [value retain];
    [_button3Title autorelease];
    _button3Title = value;
}

- (NSString *)message
{
    return _message;
}

- (void)setMessage:(NSString *)value
{
    [value retain];
    [_message autorelease];
    _message = value;
}

- (ProgressModeType)mode
{
    return _mode;
}

- (void)setMode:(ProgressModeType)value
{
    _mode = value;
}

- (int)countRemaining
{
    return [self countTotal] - _countNth;
}

- (void)setCountRemaining:(int)value
{
    _countNth = [self countTotal] - value;
}

- (int)countNth
{
    return _countNth;
}

- (void)setCountNth:(int)value
{
    _countNth = value;
}

- (int)countTotal
{
    return [[self items] count];
}

- (int)sizeNth
{
    return _sizeNth;
}

- (void)setSizeNth:(int)value
{
    _sizeNth = value;
}

- (int)sizeTotal
{
    return _sizeTotal;
}

- (void)setSizeTotal:(int)value
{
    _sizeTotal = value;
}

- (BOOL)stopped
{
    return _stopped;
}

- (void)setStopped:(BOOL)value
{
    _stopped = value;
}


- (NSString *)currentName
{
    return [[self currentItem] objectForKey:PROGRESS_CURRENT_NAME];
}

- (void)setCurrentName:(NSString *)value
{
    if (!value)
        value = @"";
    
    [[self currentItem] setObject:value forKey:PROGRESS_CURRENT_NAME];
}

- (int)currentSize
{
    return [[[self currentItem] objectForKey:PROGRESS_CURRENT_SIZE] intValue];
}

- (void)setCurrentSize:(int)value
{
    [[self currentItem] setObject:[NSNumber numberWithInt:value] forKey:PROGRESS_CURRENT_SIZE];
}

- (NSString *)currentPath
{
    return [[self currentParentPath] stringByAppendingPathComponent:[self currentName]];
}

- (NSString *)currentParentPath
{
    return [[self currentItem] objectForKey:PROGRESS_CURRENT_PARENT_PATH];
}

- (void)setCurrentParentPath:(NSString *)parentPath
{
    [[self currentItem] setObject:parentPath forKey:PROGRESS_CURRENT_PARENT_PATH removeIfNil:YES];
}

- (int)currentLevel
{
    return [[[self currentItem] objectForKey:PROGRESS_CURRENT_LEVEL] intValue];
}

- (void)setCurrentLevel:(int)value
{
    [[self currentItem] setObject:[NSNumber numberWithInt:value] forKey:PROGRESS_CURRENT_LEVEL];
}

- (id)currentObject
{
    return [[self currentItem] objectForKey:PROGRESS_CURRENT_OBJECT];
}

- (void)setCurrentObject:(id)value
{
    if (value)
        [[self currentItem] setObject:value forKey:PROGRESS_CURRENT_OBJECT];
    else
        [[self currentItem] removeObjectForKey:PROGRESS_CURRENT_OBJECT];
}


- (NSString *)description
{
    return [NSString stringWithFormat:@"%@: %@ from '%@' to '%@'; %d of %d items; %d of %d bytes; current name '%@', size %d bytes.", [self className], [self operation], [self sourceName], [self destinationName], [self countNth], [self countTotal], [self sizeNth], [self sizeTotal], [self currentName], [self currentSize]];
}

@end


@implementation DSProgressData (Private)

- (NSMutableArray *)items
{
    return _items;
}

- (void)setItems:(NSMutableArray *)value
{
    [value retain];
    [_items autorelease];
    _items = value;
}

- (NSMutableDictionary *)currentItem
{
    return _currentItem;
}

- (void)setCurrentItem:(NSMutableDictionary *)value
{
    [value retain];
    [_currentItem autorelease];
    _currentItem = value;
}

@end
