#import "DSProgressWindowController.h"
#import "DSSharedDefines.h"
#import "DSLicenseController.h"
#import "DSProgressViewController.h"


@implementation DSProgressWindowController

id sharedProgressWindowController;

+ (id)sharedProgressWindowController
{
    if (!sharedProgressWindowController)
    {
        sharedProgressWindowController = [[self alloc] init];
    }
    
    return sharedProgressWindowController;
}

+ (DSProgressData *)progressWindowTitle:(NSString *)windowTitle
                              operation:(NSString *)operation
                             sourceName:(NSString *)sourceName
                        destinationName:(NSString *)destinationName
                            buttonTitle:(NSString *)buttonTitle
{
    DSProgressWindowController *controller = [self sharedProgressWindowController];
    
    DSProgressData *progressData = [controller addProgressWindowTitle:windowTitle
                                                            operation:operation
                                                           sourceName:sourceName
                                                      destinationName:destinationName
                                                          buttonTitle:buttonTitle];
    
    // An unusual case: the DSProgressWindowController creates a DSProgressViewController, which in turn creates a DSProgressData, and that is what is returned.  The DSProgressViewController is private to this class:
    return progressData;
}

- (id)init
{
    self = [super init];
    
    [PREFERENCES registerDefaults:[NSDictionary dictionaryWithObjectsAndKeys:
                                   [NSNumber numberWithFloat:0.2], PROGRESS_CYCLE_FREQUENCY,
                                   // to be implemented: also store the window position based on the Finder's, if possible
                                   nil]];
    
    [self setProgressViewArray:[NSMutableArray array]];
    
    [NSBundle loadNibNamed:@"DSProgressWindow" owner:self];
    
    [NSTimer scheduledTimerWithTimeInterval:[PREFERENCES floatForKey:PROGRESS_CYCLE_FREQUENCY]
                                     target:self
                                   selector:@selector(cycle)
                                   userInfo:nil repeats:YES];
    
    return self;
}

- (void)dealloc
{
    [self setProgressViewArray:nil];
    
    [super dealloc];
}


- (DSProgressData *)addProgressWindowTitle:(NSString *)windowTitle
                                 operation:(NSString *)operation
                                sourceName:(NSString *)sourceName
                           destinationName:(NSString *)destinationName
                               buttonTitle:(NSString *)buttonTitle
{
    DSProgressViewController *progressView = [[[DSProgressViewController alloc] init] autorelease];
    
    DSProgressData *progressData = [progressView progressData];
    
    if (progressView && progressData)
    {
        [progressData setModeToDefaultAlways:YES];
        
        [progressData startOverWithWindowTitle:windowTitle operation:operation sizeAction:nil sourceName:sourceName destinationName:destinationName buttonTitle:buttonTitle];
        
        [[self progressViewArray] addObject:progressView];
        
        [self maintainWindowTitle];
        [self maintainProductStat];
        
        if ([progressView adjustContentsViewChanged])
            [self changeViewFrom:nil forProgressView:progressView];
        
        [self showWindow:nil];
    }
    
    return progressData;
}

- (void)cycle
{
    if ([[self progressViewArray] count])
    {
        NSEnumerator *enumerator = [[self progressViewArray] objectEnumerator];
        DSProgressViewController *progressView;
        
        while ((progressView = [enumerator nextObject]))
        {
            NSView *oldView = [progressView containerView];
            
            if ([progressView adjustContentsViewChanged])
                [self changeViewFrom:oldView forProgressView:progressView];
        }
        
        [self maintainProductStat];
        
        [[self window] display];
    }
}

- (NSArray *)_adjustAutoresizingAroundPosition:(float)position
                            stickPositionToTop:(BOOL)stickPositionToTop
                                  forSuperview:(NSView *)superview
{
    // Code based on SNDisclosableView by Kurt Revis.
    
    // Adjust the autoresize masks of the superview's subviews, remembering the original masks.
    NSArray *subviews = [superview subviews];
    unsigned int subviewCount = [subviews count];
    unsigned int subviewIndex;
    NSMutableArray *subviewMasks = [NSMutableArray arrayWithCapacity:subviewCount];
    
    for (subviewIndex = 0; subviewIndex < subviewCount; subviewIndex++)
    {
        NSView *subview = [subviews objectAtIndex:subviewIndex];
        unsigned int mask = [subview autoresizingMask];
        BOOL stickToBottom = NSMaxY([subview frame]) < position;
        
        if (!stickPositionToTop && (NSMaxY([subview frame]) == position))
            stickToBottom = YES;
        
        [subviewMasks addObject:[NSNumber numberWithUnsignedInt:mask]];
        
        if (stickToBottom)
        {
            // This subview is below us, or is us. Make it stick to the bottom of the window.
            // It should not change height.
            mask &= ~NSViewHeightSizable;
            mask |= NSViewMaxYMargin;
            mask &= ~NSViewMinYMargin;
        }
        else
        {
            // This subview is above us. Make it stick to the top of the window.
            // It should not change height.
            mask &= ~NSViewHeightSizable;
            mask &= ~NSViewMaxYMargin;
            mask |= NSViewMinYMargin;
        }
        
        [subview setAutoresizingMask:mask];
    }
    
    return subviewMasks;
}

- (void)_restoreAutoresizeMasks:(NSArray *)masks forSuperview:(NSView *)superview
{
    NSArray *subviews = [superview subviews];
    unsigned int count, index;
    
    count = [masks count];
    
    for (index = 0; index < count; index++)
        [[subviews objectAtIndex:index] setAutoresizingMask:[[masks objectAtIndex:index] unsignedIntValue]];
}

- (void)changeViewFrom:(NSView *)oldView forProgressView:(DSProgressViewController *)progressView
{
    NSView *containerView = [progressView containerView];
    NSRect oldWindowFrame, newWindowFrame;
    int newWindowHeight;
    BOOL isMultiple = [[self progressViewArray] count] > 1;
    
    if (!oldView && !containerView)		// If neither of these are provided, there's nothing to do
        return;
    
    oldWindowFrame = [NSWindow contentRectForFrameRect:[[self window] frame] styleMask:[[self window] styleMask]];
    
    // Add the new container view; will be at the top of the window:
    NSRect containerFrame = [containerView frame];	// This rect is undefined if containerView is nil
    NSRect oldFrame = [oldView frame];			// This rect is undefined if oldView is nil
    float relevantPosition;
    
    // Position the new container view:
    if ((isMultiple || oldView) && containerView)
    {
        if (oldView)
            containerFrame.origin = oldFrame.origin;
        else
            containerFrame = NSOffsetRect(containerFrame, 0.0, NSHeight(oldWindowFrame) - 19);
        
        [containerView setFrame:containerFrame];
    }
    
    if (containerView && !oldView)
        relevantPosition = NSMaxY(containerFrame);
    else
        relevantPosition = NSMaxY(oldFrame);
    
    // Remove the old view or add a new one:
    if (oldView && !containerView)
        [oldView removeFromSuperview];
    else if (!oldView && containerView)
        [[scrollView documentView] addSubview:containerView];
    
    // Resize the window appropriately:
    if (!containerView)
        newWindowHeight = NSHeight(oldWindowFrame) - NSHeight(oldFrame) - 10;
    else if (!isMultiple)
        newWindowHeight = containerFrame.size.height + 29;
    else if (oldView)
        newWindowHeight = NSHeight(oldWindowFrame) + (NSHeight(containerFrame) - NSHeight(oldFrame));
    else
        newWindowHeight = NSHeight(oldWindowFrame) + containerFrame.size.height + 9;
    
    newWindowFrame = [NSWindow frameRectForContentRect:NSMakeRect(oldWindowFrame.origin.x, NSMaxY(oldWindowFrame) - newWindowHeight, oldWindowFrame.size.width, newWindowHeight) styleMask:[[self window] styleMask]];
    
    NSArray *subviewMasks = nil;
    
    if (oldView)
        subviewMasks = [self _adjustAutoresizingAroundPosition:relevantPosition
                                            stickPositionToTop:oldView && containerView
                                                  forSuperview:[scrollView documentView]];
    
    [[self window] setFrame:newWindowFrame display:YES animate:[[self window] isVisible]];
    
    if (oldView)
        [self _restoreAutoresizeMasks:subviewMasks forSuperview:[scrollView documentView]];
    
    // Replace the old view:
    if (oldView && containerView)
        [[scrollView documentView] replaceSubview:oldView with:containerView];
    
    // If removing the old view, remove the progress view controller too:
    if (!containerView)
        [[self progressViewArray] removeObject:progressView];
    
    // If only one view, and it's being removed, close the window:
    if (!isMultiple && !containerView)
        [[self window] close];
}

- (void)maintainWindowTitle
{
    NSString *combinedTitle = nil;
    NSEnumerator *enumerator = [[self progressViewArray] objectEnumerator];
    DSProgressViewController *progressView;
    
    while ((progressView = [enumerator nextObject]))
    {
        NSString *windowTitle = [[progressView progressData] windowTitle];
        
        if (![combinedTitle length])
            combinedTitle = windowTitle;
        else
        {
            if (![combinedTitle isEqualToString:windowTitle])
                combinedTitle = [NSString stringWithFormat:@"%@ & %@", combinedTitle, windowTitle];
        }
    }
    
    [[self window] setTitle:combinedTitle];
}

- (void)maintainProductStat
{
    NSString *string = [[DSLicenseController sharedLicenseController] progressString];
    
    if (string)
        [productStat setStringValue:string];
}


// ----------------------------------------------------------------------------------------
#pragma mark -
#pragma mark • TARGET/ACTION METHODS
// ----------------------------------------------------------------------------------------

- (IBAction)productStatHit:(id)sender
{
    [[NSWorkspace sharedWorkspace] openURL:[NSURL URLWithString:URL_PRODUCT_SITE]];
}


// ----------------------------------------------------------------------------------------
#pragma mark -
#pragma mark • ACCESSOR METHODS
// ----------------------------------------------------------------------------------------

- (NSMutableArray *)progressViewArray
{
    return _progressViewArray;
}

- (void)setProgressViewArray:(NSMutableArray *)value
{
    [value retain];
    [_progressViewArray autorelease];
    _progressViewArray = value;
}

@end
