//
//  DSModeTabSupervisor.m
//  Shared classes
//
//  Created by David Sinclair on Wed May 12 2004.
//  Copyright (c) 2004 Dejal Systems, LLC. All rights reserved.
//
//  A "supervisor" is a special kind of controller that manages a particular view object.
//  This is the supervisor for a NSTabView used to switch modes, e.g. between two or more tables.
//

#import "DSModeTabSupervisor.h"
#import "DSSharedDefines.h"


@implementation DSModeTabSupervisor

/*
 awakeFromNib
 
 Sets up the tab supervisor automatically.
 
 Written by DJS 2004-05.
*/

- (void)awakeFromNib
{
    [super awakeFromNib];
}


/*
 setup
 
 Sets up the tab view.  Subclasses may override this.
 
 Written by DJS 2004-05.
*/

- (void)setup
{
    [super setup];
}


/*
 setupWithSupervisors:
 
 Given an array of supervisors, asks each for its identifier and sets the tab supervisor's represented object to a dictionary of them.
 
 Written by DJS 2004-05.
*/

- (void)setupWithSupervisors:(NSArray *)supervisorArray
{
    NSMutableDictionary *dict = [NSMutableDictionary dictionaryWithCapacity:[supervisorArray count]];
    NSEnumerator *enumerator = [supervisorArray objectEnumerator];
    DSSupervisor *supervisor;
    
    while ((supervisor = [enumerator nextObject]))
    {
    	NSString *identifier = [supervisor identifier];
        
        if (identifier)
            [dict setObject:supervisor forKey:identifier];
    }
    
    [self setRepresentedObject:dict];
    
    [self prepare];
}

/*
 prepare
 
 Performs actions to re-syncronize the data after being inactive (e.g. while the app was in another mode).  Subclasses may override this.
 
 Written by DJS 2004-05.
*/

- (void)prepare
{
    [super prepare];
    
    NSString *preference = [self preferenceForSelectedTab];
    NSString *identifier = nil;
    
    if (preference)
        identifier = [PREFERENCES objectForKey:preference];
    
    if (!identifier)
        identifier = [[modeTabView tabViewItemAtIndex:0] identifier];
    
    [modeTabView selectTabViewItemWithIdentifier:identifier];
    
    [[self supervisorForIdentifier:identifier] prepare];
    
    [window makeFirstResponder:[[modeTabView selectedTabViewItem] view]];
}


// ----------------------------------------------------------------------------------------
#pragma mark -
#pragma mark CONFIGURATION METHODS
// ----------------------------------------------------------------------------------------

/*
 preferenceForSelectedTab
 
 Returns the preference name to remember and restore the selected tab.  By default returns a standard name.  Subclasses should override this if there is more than one mode tab in the application, or override to return nil to disable this functionality.
 
 Written by DJS 2004-05.
*/

- (NSString *)preferenceForSelectedTab
{
    return @"DSModeTab";
}


// ----------------------------------------------------------------------------------------
#pragma mark -
#pragma mark UTILITY METHODS
// ----------------------------------------------------------------------------------------

/*
 selectedSupervisor
 
 Returns the supervisor for the currently selected tab item.
 
 Written by DJS 2004-05.
*/

- (id)selectedSupervisor
{
    NSString *identifier = [self identifier];
    
    return [self supervisorForIdentifier:identifier];
}

/*
 supervisorForIdentifier:
 
 Given an identifier, finds and returns the corresponding supervisor, or nil if not found.  Subclasses may override this.
 
 Written by DJS 2004-05.
*/

- (id)supervisorForIdentifier:(NSString *)identifier
{
    return [[self representedObject] objectForKey:identifier];
}


// ----------------------------------------------------------------------------------------
#pragma mark -
#pragma mark NSTABVIEW DELEGATE METHODS
// ----------------------------------------------------------------------------------------

/*
 tabView:willSelectTabViewItem:
 
 Delegate method called when the tab view item is about to change.  Subclasses may override this.
 
 Written by DJS 2004-05.
*/

- (void)tabView:(NSTabView *)tabView willSelectTabViewItem:(NSTabViewItem *)tabViewItem
{
    NSString *identifier = [tabViewItem identifier];
    
    [[self supervisorForIdentifier:identifier] prepare];
    
    NSString *preference = [self preferenceForSelectedTab];
    
    if (preference && [[tabView window] isVisible])
        [PREFERENCES setObject:[tabViewItem identifier] forKey:preference];
}


// ----------------------------------------------------------------------------------------
#pragma mark -
#pragma mark VALIDATION METHODS
// ----------------------------------------------------------------------------------------

/*
 validateToolbarItem:
 
 Returns YES if the specified toolbar item should be enabled.  This method is just used to rename any toggled toolbar items; it calls -validateAction: to handle the activation decision.  By default just passes the method call through to the selected tab's supervisor.  Subclasses may override this (calling super) to rename any toggled items.
 
 Written by DJS 2004-05.
*/

- (BOOL)validateToolbarItem:(NSToolbarItem *)toolbarItem
{
    return [[self selectedSupervisor] validateToolbarItem:toolbarItem];
}

/*
 validateMenuItem:
 
 Returns YES if the specified menu item should be enabled.  This method is just used to rename any toggled menu items; it calls -validateAction: to handle the activation decision.  By default just passes the method call through to the selected tab's supervisor.  Subclasses may override this (calling super) to rename any toggled items.
 
 Written by DJS 2004-05.
*/

- (BOOL)validateMenuItem:(NSMenuItem *)anItem
{
    return [[self selectedSupervisor] validateMenuItem:anItem];
}

/*
 validateAction:
 
 Returns YES if the menu item or toolbar button corresponding to the action should be enabled currently.  By default just passes the method call through to the selected tab's supervisor.  Subclasses may override this (calling super) to enable items as appropriate.
 
 Written by DJS 2004-05.
*/

- (BOOL)validateAction:(SEL)action
{
    return [[self selectedSupervisor] validateAction:action];
}


// ----------------------------------------------------------------------------------------
#pragma mark -
#pragma mark ACCESSOR METHODS
// ----------------------------------------------------------------------------------------

/*
 identifier
 
 Rather than returning the tab supervisor's own identifier, this returns the identifier of the currently selected tab item.
 
 Written by DJS 2004-05.
*/

- (NSString *)identifier
{
    NSTabViewItem *item = [modeTabView selectedTabViewItem];
    
    return [item identifier];
}

/*
 setIdentifier:
 
 Rather than changing the tab supervisor's own identifier, this changes the currently selected tab item to the one with the specified identifier.
 
 Written by DJS 2004-05.
*/

- (void)setIdentifier:(NSString *)value
{
    [modeTabView selectTabViewItemWithIdentifier:value];
}

@end
