//
//  DSSplitSupervisor.m
//  Shared classes
//
//  Created by David Sinclair on Mon May 24 2004.
//  Copyright (c) 2004 Dejal Systems, LLC. All rights reserved.
//
//  A "supervisor" is a special kind of controller that manages a particular view object.
//  This is the supervisor for a NSSplitView.
//

#import "DSSplitSupervisor.h"
#import "DSSharedDefines.h"


@implementation DSSplitSupervisor

/*
 awakeFromNib
 
 Sets up the tab supervisor automatically.
 
 Written by DJS 2004-05.
*/

- (void)awakeFromNib
{
    [super awakeFromNib];
}


/*
 setup
 
 Sets up the tab view.  Subclasses may override this.
 
 Written by DJS 2004-05.
*/

- (void)setup
{
    [super setup];
    
    [self resizeSplitterViewFirstFixed:NO];
}


/*
 setupWithSupervisors:
 
 Given an array of supervisors, sets the split supervisor's represented object to that array.
 
 Written by DJS 2004-05.
*/

- (void)setupWithSupervisors:(NSArray *)supervisorArray
{
    [self setup];
    
    [self setRepresentedObject:supervisorArray];
}

/*
 prepare
 
 Performs actions to re-syncronize the data after being inactive (e.g. while the app was in another mode).  Subclasses may override this.
 
 Written by DJS 2004-05.
*/

- (void)prepare
{
    [super prepare];
}


// ----------------------------------------------------------------------------------------
#pragma mark -
#pragma mark CONFIGURATION METHODS
// ----------------------------------------------------------------------------------------

/*
 preferenceForSplitPosition
 
 Returns the preference name to remember and restore the split position.  By default returns a standard name.  Subclasses should override this if there is more than one splitter in the application, or override to return nil to disable this functionality.
 
 Written by DJS 2004-05.
*/

- (NSString *)preferenceForSplitPosition
{
    return @"DSSplitPosition";
}


// ----------------------------------------------------------------------------------------
#pragma mark -
#pragma mark UTILITY METHODS
// ----------------------------------------------------------------------------------------

/*
 supervisorForIdentifier:
 
 Given an identifier, finds and returns the corresponding supervisor, or nil if not found.  Subclasses may override this.
 
 Written by DJS 2004-05.
*/

- (id)supervisorForIdentifier:(NSString *)identifier
{
    return [[self representedObject] objectForKey:identifier];
}

/*
 resizeSplitterViewFirstFixed:
 
 Resizes the split view, optionally keeping the first subview a fixed size (though this is not fully implmented yet; see ).  You normally don't need to call this.  Subclasses may override this.
 
 Written by DJS 2004-05.
*/

- (void)resizeSplitterViewFirstFixed:(BOOL)firstIsFixed
{
    NSView *firstView = [self firstSubview];
    NSView *secondView = [self secondSubview];
    NSRect firstFrame = [firstView frame];
    NSRect secondFrame = [secondView frame];
    NSRect splitterFrame = [splitView frame];
    NSSize oldFirstSize = firstFrame.size;
    NSSize oldSecondSize = secondFrame.size;
    NSString *preference = [self preferenceForSplitPosition];
    BOOL firstIsCollapsed = [splitView isSubviewCollapsed:firstView];
    float delta = 0.0;
    
    if (preference)
        delta = [PREFERENCES floatForKey:preference];
    
    if ([splitView isVertical])
    {
        // Handle a vertical splitter:
        float height = NSHeight(splitterFrame);
        
        if (firstIsCollapsed)
            delta = 0.0;
        else if (!firstIsFixed || delta < 16.0)
            delta = NSWidth(firstFrame);
        
        if (!firstIsCollapsed)
        {
            firstFrame.size.width = delta;
            firstFrame.size.height = height;
        }
        
        secondFrame.origin.x = splitterFrame.origin.x + delta + [splitView dividerThickness];
        secondFrame.size.width = splitterFrame.size.width - [splitView dividerThickness] - delta;
        secondFrame.size.height = height;
    }
    else
    {
        // Handle a horizontal splitter:
        float width = NSWidth(splitterFrame);
        
        if (firstIsCollapsed)
            delta = 0.0;
        else if (!firstIsFixed || delta < 16.0)
            delta = NSHeight(firstFrame);
        
        if (!firstIsCollapsed)
        {
            firstFrame.size.width = width;
            firstFrame.size.height = delta;
        }
        
        secondFrame.origin.y = splitterFrame.origin.y + delta + [splitView dividerThickness];
        secondFrame.size.width = width;
        secondFrame.size.height = splitterFrame.size.height - [splitView dividerThickness] - delta;
    }
    
    [firstView setFrame:firstFrame];
    [firstView resizeSubviewsWithOldSize:oldFirstSize];
    [secondView setFrame:secondFrame];
    [secondView resizeSubviewsWithOldSize:oldSecondSize];
    [splitView setNeedsDisplay:YES];
}


// ----------------------------------------------------------------------------------------
#pragma mark -
#pragma mark NSSPLITVIEW DELEGATE METHODS
// ----------------------------------------------------------------------------------------

/*
 splitView:constrainMinCoordinate:ofSubviewAt:
 
 Split delegate method to return what the minimum size should be, i.e. the minimum width/height of the first subview.  Returns 100.0 by default, but subviews will probably want to override this.
 
 Written by DJS 2004-05.
*/

- (float)splitView:(NSSplitView *)sender constrainMinCoordinate:(float)proposedMin ofSubviewAt:(int)offset
{
    return 100.0;
}

/*
 splitView:constrainMaxCoordinate:ofSubviewAt:
 
 Split delegate method to return what the maximum size should be, i.e. the minimum width/height of the second subview.  Returns a value so the second subview is a minimum of 100.0 by default, but subviews will probably want to override this.
 
 Written by DJS 2004-05.
*/

- (float)splitView:(NSSplitView *)sender constrainMaxCoordinate:(float)proposedMax ofSubviewAt:(int)offset
{
    if ([sender isVertical])
        return [sender frame].size.width - 100.0;
    else
        return [sender frame].size.height - 100.0;
}

/*
 splitViewDidResizeSubviews:
 
 Split delegate method called when the split bar is moved, directly or indirectly.  Subclasses may override this.
 
 Written by DJS 2004-05.
*/

- (void)splitViewDidResizeSubviews:(NSNotification *)aNotification
{
    NSRect firstFrame = [[self firstSubview] frame];
    NSString *preference = [self preferenceForSplitPosition];
    
    if (preference)
    {
        if ([splitView isVertical])
            [PREFERENCES setFloat:firstFrame.size.width forKey:preference];
        else
            [PREFERENCES setFloat:firstFrame.size.height forKey:preference];
    }
}

/*
 splitView:resizeSubviewsWithOldSize:
 
 Split delegate method to handle resizing the subviews.  Subclasses may override this.
 
 Written by DJS 2004-05.
*/

/* ¶¶ disabled for now, as it isn't working quite right for some reason, and it isn't needed for Project Manager.  Maybe I do need the -isResizingWindow thing after all?  I can reenable this method and fix it some other time; make sure it is tested with that app, though.
- (void)splitView:(NSSplitView *)sender resizeSubviewsWithOldSize:(NSSize)oldSize
{
    [self resizeSplitterViewFirstFixed:NO];            // ¶¶ hack
    
    // ¶¶ the first view fixed feature isn't fully operational yet.  Need to have a configuration method to enable it, and implement some window delegate methods to set when the window is being resized.  See -[DSElementEditor windowWillResize:toSize:] and -[DSElementEditor windowDidResize:].  Can't implement those here, as this isn't the window delegate.  Perhaps have a couple of methods to indicating starting and ending resizing, or just set the ivar via the accessor directly.
    
//    [self resizeSplitterViewFirstFixed:[self isResizingWindow]];
}
*/

/*
 splitView:canCollapseSubview:
 
 Split delegate method to return whether a subview can be collapsed.  By default it returns YES.  Subclasses may override this.
 
 Written by DJS 2004-05.
*/

- (BOOL)splitView:(NSSplitView *)sender canCollapseSubview:(NSView *)subview
{
    return YES;
}


// ----------------------------------------------------------------------------------------
#pragma mark -
#pragma mark VALIDATION METHODS
// ----------------------------------------------------------------------------------------

/*
 validateToolbarItem:
 
 Returns YES if the specified toolbar item should be enabled.  This method is just used to rename any toggled toolbar items; it calls -validateAction: to handle the activation decision.  By default just passes the method call through to the subview supervisors.  Subclasses may override this (calling super) to rename any toggled items.
 
 Written by DJS 2004-05.
*/

- (BOOL)validateToolbarItem:(NSToolbarItem *)toolbarItem
{
    return [[self firstSupervisor] validateToolbarItem:toolbarItem] && [[self secondSupervisor] validateToolbarItem:toolbarItem];
}

/*
 validateMenuItem:
 
 Returns YES if the specified menu item should be enabled.  This method is just used to rename any toggled menu items; it calls -validateAction: to handle the activation decision.  By default just passes the method call through to the subview supervisors.  Subclasses may override this (calling super) to rename any toggled items.
 
 Written by DJS 2004-05.
*/

- (BOOL)validateMenuItem:(NSMenuItem *)anItem
{
    return [[self firstSupervisor] validateMenuItem:anItem] && [[self secondSupervisor] validateMenuItem:anItem];
}

/*
 validateAction:
 
 Returns YES if the menu item or toolbar button corresponding to the action should be enabled currently.  By default just passes the method call through to the subview supervisors.  Subclasses may override this to enable items appropriately.
 
 Written by DJS 2004-05.
*/

- (BOOL)validateAction:(SEL)action
{
    return [[self firstSupervisor] validateAction:action] && [[self secondSupervisor] validateAction:action];
}


// ----------------------------------------------------------------------------------------
#pragma mark -
#pragma mark ACCESSOR METHODS
// ----------------------------------------------------------------------------------------

/*
 firstSubview
 
 Returns the first subview of the splitter, i.e. the top or left view within it.  Subclasses shouldn't need to override this.
 
 Written by DJS 2004-05.
*/

- (NSView *)firstSubview
{
    return [[splitView subviews] firstObject];
}

/*
 secondSubview
 
 Returns the second subview of the splitter, i.e. the bottom or right view within it.  Note: currently assumes that there are only two subviews, but the implementation may change in the future if necessary!  Subclasses shouldn't need to override this.
 
 Written by DJS 2004-05.
*/

- (NSView *)secondSubview
{
    return [[splitView subviews] lastObject];
}

/*
 firstSupervisor
 
 Returns the first supervisor, i.e. the one corresponding to the first subview of the splitter.  Subclasses shouldn't need to override this.
 
 Written by DJS 2004-05.
*/

- (id)firstSupervisor
{
    return [[self representedObject] firstObject];
}

/*
 secondSupervisor
 
 Returns the second supervisor, i.e. the one corresponding to the second subview of the splitter.  Note: currently assumes that there are only two supervisors, but the implementation may change in the future if necessary!  Subclasses shouldn't need to override this.
 
 Written by DJS 2004-05.
*/

- (id)secondSupervisor
{
    return [[self representedObject] lastObject];
}

@end
