//
//  DSSupervisor.m
//  Shared classes
//
//  Created by David Sinclair on Wed May 12 2004.
//  Copyright (c) 2004 Dejal Systems, LLC. All rights reserved.
//
//  A "supervisor" is a special kind of controller that manages a particular view object.
//  This is the supervisor superclass, but it can be instantiated as a concrete class, e.g. for custom views.
//

#import "DSSupervisor.h"
#import "DSSharedDefines.h"


@implementation DSSupervisor

- (id)initWithRepresentedObject:(id)representedObject
{
    self = [self init];
    
    [self setRepresentedObject:representedObject];
    
    return self;
}

/*
 init
 
 Designated initializer of the supervisor classes.  Subclasses should override this to initialize other ivars, if any.
 
 Written by DJS 2004-05.
*/

- (id)init
{
    self = [super init];
    
    [self setIdentifier:[NSString universallyUniqueIdentifier]];
    [self setRepresentedObject:nil];
    [self setUserInfo:nil];
    
    return self;
}

/*
 dealloc
 
 Standard deallocator method.  Subclasses should override this to release their own ivars, if any.
 
 Written by DJS 2004-05.
*/

- (void)dealloc
{
    [self setUserInfo:nil];
    [self setRepresentedObject:nil];
    [self setIdentifier:nil];
    
    [super dealloc];
}


/*
 awakeFromNib
 
 Sets up the supervisor automatically when it has been loaded from a Nib file.
 
 Written by DJS 2004-05.
*/

- (void)awakeFromNib
{
    [self setup];
}


/*
 setup
 
 Sets up the supervisor.  By default this just calls -reload; subclasses may override this.
 
 Written by DJS 2004-05.
*/

- (void)setup
{
    [self reload];
}

/*
 reload
 
 Reloads and redisplays the supervisor contents, if appropriate.  Subclasses may override this.
 
 Written by DJS 2004-05.
*/

- (void)reload
{
    // Does nothing for this class
}

/*
 prepare
 
 Performs actions to re-syncronize the data after being inactive (e.g. while the app was in another mode).  By default this just calls -reload; subclasses may override this.
 
 Written by DJS 2004-05.
*/

- (void)prepare
{
    [self reload];
}


// ----------------------------------------------------------------------------------------
#pragma mark -
#pragma mark NOTIFICATION METHODS
// ----------------------------------------------------------------------------------------

/*
 postNotificationName:
 
 Posts two notifications: a specific one on the exact change, and a general one saying that the represented object was changed in some way.  Subclasses probably don't need to override this.  They or other classes may want to observe one of these notifications -- in most cases, they would probably only care that something has changed (e.g. so they can save the data), rather than exactly how.
 
 Written by DJS 2004-05.
*/

- (void)postNotificationName:(NSString *)name
{
    NSNotificationCenter *center = [NSNotificationCenter defaultCenter];
    
    [center postNotificationName:name object:self];
    [center postNotificationName:@"DSSupervisorRepresentedObjectDidChange" object:self];
}


// ----------------------------------------------------------------------------------------
#pragma mark -
#pragma mark VALIDATION METHODS
// ----------------------------------------------------------------------------------------

/*
 validateToolbarItem:
 
 Returns YES if the specified toolbar item should be enabled.  This method is just used to rename any toggled toolbar items; it calls -validateAction: to handle the activation decision.  By default just calls the -validateAction: method.  Subclasses may override this (calling super) to rename any toggled items.
 
 Written by DJS 2004-05.
*/

- (BOOL)validateToolbarItem:(NSToolbarItem *)toolbarItem
{
    return [self validateAction:[toolbarItem action]];
}

/*
 validateMenuItem:
 
 Returns YES if the specified menu item should be enabled.  This method is just used to rename any toggled menu items; it calls -validateAction: to handle the activation decision.  By default just calls the -validateAction: method.  Subclasses may override this (calling super) to rename any toggled items.
 
 Written by DJS 2004-05.
*/

- (BOOL)validateMenuItem:(NSMenuItem *)anItem
{
    return [self validateAction:[anItem action]];
}

/*
 validateAction:
 
 Returns YES if the menu item or toolbar button corresponding to the action should be enabled currently.  By default just returns YES.  Subclasses may override this to enable items appropriately.
 
 Written by DJS 2004-05.
*/

- (BOOL)validateAction:(SEL)action
{
    return YES;
}


// ----------------------------------------------------------------------------------------
#pragma mark -
#pragma mark DATA ACCESSOR METHODS
// ----------------------------------------------------------------------------------------

/*
 containerView
 
 Returns the container view, as connected in the Nib.
 
 Written by DJS 2004-05.
*/

- (NSView *)containerView
{
    return containerView;
}

/*
 identifier
 
 Convenience accessor available to subclasses to return an identifier string, if desired.  Initialized to a universally unique ID by this class.
 
 Written by DJS 2004-05.
*/

- (NSString *)identifier
{
    return _identifier;
}

/*
 setIdentifier:
 
 Convenience accessor available to subclasses to store an identifier string, if desired.  Initialized to a universally unique ID by this class.
 
 Written by DJS 2004-05.
*/

- (void)setIdentifier:(NSString *)value
{
    [value retain];
    [_identifier autorelease];
    _identifier = value;
}

/*
 representedObject
 
 Convenience accessor available to subclasses to return an arbitrary object, if desired.  Initialized to nil, but not used by this class.
 
 Written by DJS 2004-05.
*/

- (id)representedObject
{
    return _representedObject;
}

/*
 setRepresentedObject:
 
 Convenience accessor available to subclasses to store an arbitrary object, if desired.  Initialized to nil, but not used by this class.
 
 Written by DJS 2004-05.
*/

- (void)setRepresentedObject:(id)value
{
    [value retain];
    [_representedObject autorelease];
    _representedObject = value;
}

/*
 userInfo
 
 Convenience accessor available to subclasses to return arbitrary data, if desired.  Initialized to nil, but not used by this class.
 
 Written by DJS 2004-05.
*/

- (NSMutableDictionary *)userInfo
{
    return _userInfo;
}

/*
 setUserInfo:
 
 Convenience accessor available to subclasses to store arbitrary data, if desired.  Initialized to nil, but not used by this class.
 
 Written by DJS 2004-05.
*/

- (void)setUserInfo:(NSMutableDictionary *)value
{
    [value retain];
    [_userInfo autorelease];
    _userInfo = value;
}

@end
