//
//  DSTableSupervisor.m
//  Shared classes
//
//  Created by David Sinclair on Fri May 07 2004.
//  Copyright (c) 2004 - 2005 Dejal Systems, LLC. All rights reserved.
//
//  A "supervisor" is a special kind of controller that manages a particular view object.
//  This is the supervisor for a NSTableView.
//

#import <Carbon/Carbon.h>
#import "DSTableSupervisor.h"
#import "DSSharedDefines.h"


@implementation DSTableSupervisor

/*
 awakeFromNib
 
 Sets up the table supervisor automatically.
 
 Written by DJS 2004-05.
*/

- (void)awakeFromNib
{
    [super awakeFromNib];
}


/*
 setup
 
 Sets up the table.  Subclasses may override this.
 
 Written by DJS 2004-05.
*/

- (void)setup
{
/*
    // Only set the autosave name if one isn't already set:
    if (![[theTableView autosaveName] length])
    {
        // N.B.: the autosave will fail if the identifier isn't changed from its default value (a UUID):
        [theTableView setAutosaveName:[self identifier]];
        [theTableView setAutosaveTableColumns:YES];
    }
*/
    
    [theTableView setDoubleAction:@selector(doubleClicked:)];
    
    if ([self acceptsDragPasteboardTypes])
        [theTableView registerForDraggedTypes:[self acceptsDragPasteboardTypes]];
    
    [super setup];
}

/*
 reload
 
 Reloads and redisplays the current table.  Subclasses may override this.
 
 Written by DJS 2004-05.
*/

- (void)reload
{
    [theTableView reloadData];
    
    [super reload];
}

/*
 prepare
 
 Performs actions to re-syncronize the data after being inactive (e.g. while the app was in another mode).  Subclasses may override this.
 
 Written by DJS 2004-05.
*/

- (void)prepare
{
    [super prepare];
}

/*
 valueWithRowObject:forIdentifier:
 
 Given a row object and an identifier, returns the corresponding value, or nil if not found.  If the identifier includes " : " (or the old delimiter, " --> "), the part before that is the key of a dictionary in the rowObject, and the part after that is the key within that dictionary.  This can recurse as deep as needed.  Subclasses may override this.
 
 Written by DJS 2004-05.
 Changed by DJS 2004-11 to allow the " : " delimited instead, since " --> " breaks automatic sorting.
*/

- (id)valueWithRowObject:(id)rowObject forIdentifier:(NSString *)identifier
{
    NSString *subIdentifier = nil;
    NSRange range = [identifier rangeOfString:@"_"];
    
    // Old delimiter:
    if (range.location == NSNotFound)
        range = [identifier rangeOfString:@" --> "];
    
    if (range.location != NSNotFound)
    {
        subIdentifier = [identifier substringFromIndex:range.location + range.length];
        identifier = [identifier substringToIndex:range.location];
    }
    
    id value = [rowObject objectForKey:identifier];
    
    if (subIdentifier)
        value = [self valueWithRowObject:value forIdentifier:subIdentifier];
    
    return value;
}

/*
 setValue:withRowObject:forIdentifier:
 
 Sets the value within the row object, using the specified identifier as a key.  Note: this currently does not support the recursive dictionaries of the above method; this can be implemented if needed.  Subclasses may override this.
 
 Written by DJS 2004-05.
*/

- (void)setValue:(id)value withRowObject:(id)rowObject forIdentifier:(NSString *)identifier
{
    [rowObject setObject:value forKey:identifier];
}

/*
 objectForRow:
 
 Given a row number, finds and returns the corresponding object, or nil if not found.  Subclasses may override this.
 
 Written by DJS 2004-05.
*/

- (id)objectForRow:(int)row
{
    id object = nil;
    
    if ([[self representedObject] isValidIndex:row])
        object = [[self representedObject] objectAtIndex:row];
    
    return object;
}

/*
 rowForObject:
 
 Given an object, finds and returns the corresponding row index, or NSNotFound if not found.  Subclasses may override this.
 
 Written by DJS 2004-05.
*/

- (int)rowForObject:(id)object
{
    int row = [[self representedObject] indexOfObject:object];
    
    return row;
}

/*
 selectedObject
 
 Returns the first selected object, or nil if no row is selected.  Subclasses shouldn't need to override this.
 
 Written by DJS 2004-05.
*/

- (id)selectedObject
{
    return [self objectForRow:[theTableView selectedRow]];
}

/*
 selectedOrAllObjects:
 
 Returns an array containing just the selected objects.  If there are none selected and flag is YES, all objects are returned, otherwise an empty array is returned.  Subclasses shouldn't need to override this.
 
 Written by DJS 2004-05.
*/

- (NSMutableArray *)selectedOrAllObjects:(BOOL)flag
{
    NSMutableArray *selected = [NSMutableArray array];
    NSEnumerator *enumerator;
    NSNumber *rowNum;
    
    if (flag)
        enumerator = [theTableView selectedOrAllRowsEnumerator];
    else
        enumerator = [theTableView selectedRowEnumerator];
    
    while ((rowNum = [enumerator nextObject]))
    {
        [selected addObject:[self objectForRow:[rowNum intValue]]];
    }
    
    return selected;
}

/*
 selectObjects:byExtendingSelection:
 
 Given an array of objects, the corresponding rows in the table are selected.  Subclasses shouldn't need to override this.
 
 Written by DJS 2004-05.
*/

- (void)selectObjects:(NSArray *)selected byExtendingSelection:(BOOL)flag
{
    NSEnumerator *enumerator = [selected objectEnumerator];
    id object;
    
    [self beginSelection];
    
    if (!flag)
        [theTableView deselectAll:nil];
    
    while ((object = [enumerator nextObject]))
    {
        int row = [self rowForObject:object];
        
        if (row != NSNotFound)
            [theTableView selectRow:row byExtendingSelection:YES];
    }
    
    [self endSelection];
}

/*
 sort
 
 Sorts the currently selected column, in the current sort order.  Subclasses shouldn't need to override this.
 
 Written by DJS 2004-05.
*/

- (void)sort
{
    [self tableView:theTableView sortDescriptorsDidChange:nil];
}


// ----------------------------------------------------------------------------------------
#pragma mark -
#pragma mark CONFIGURATION METHODS
// ----------------------------------------------------------------------------------------

/*
 isSortable
 
 If YES is returned, sorting the table is handled automatically; if NO, the table is doesn't allow sorting.  Returns YES by default; subclasses may override this if desired.
 
 Written by DJS 2004-05.
*/

- (BOOL)isSortable
{
    return YES;
}

/*
 acceptsDragPasteboardTypes
 
 If an array of pasteboard names is returned, dragging into the table is supported; if nil, dragging into the table is not supported.  The array should contain one or more pasteboard names.  If dragging is only desired for internal rearranging use, not dragging to another table, you can use -internalDraggingPasteboardType in the array.  By default returns nil; subclasses may override this to enable dragging in.
 
 Written by DJS 2004-05.
*/

- (NSArray *)acceptsDragPasteboardTypes
{
    return nil;
}

/*
 dragWithPasteboardTypes
 
 If an array of pasteboard names is returned, dragging out of the table is supported; if nil, dragging out of the table is not supported.  The array should contain one or more pasteboard names.  If dragging is only desired for internal rearranging use, not dragging to another table, you can use -internalDraggingPasteboardType in the array.  By default returns nil; subclasses may override this to enable dragging out.
 
 Written by DJS 2004-05.
*/

- (NSArray *)dragWithPasteboardTypes
{
    return nil;
}

/*
 internalDraggingPasteboardType
 
 Returns a pasteboard name to use when dragging only within this table; it is guaranteed to be unique to this table only.  Subclasses shouldn't need to override this.
 
 Written by DJS 2004-05.
*/

- (NSString *)internalDraggingPasteboardType
{
    return [self identifier];
}

/*
 supportsDragDelete
 
 If YES is returned, the table is a DSDragDeleteTableView, and -dragWithPasteboardTypes doesn't return nil, drag deletion is supported.  Otherwise drag deleting is not supported.  By default returns NO; subclasses may override this to enable drag deletion.
 
 Written by DJS 2004-05.
*/

- (BOOL)supportsDragDelete
{
    return NO;
}

/*
 shouldWaitForDoubleClick
 
 If YES is returned, when a click occurs the supervisor will wait until it is sure it isn't the first click of a double click; if NO, the first click of a double-click will be treated as a single click.  Returns NO by default; subclasses may override this if desired.
 
 Written by DJS 2004-05.
*/

- (BOOL)shouldWaitForDoubleClick
{
    return NO;
}


// ----------------------------------------------------------------------------------------
#pragma mark -
#pragma mark TABLE VIEW DATA SOURCE METHODS
// ----------------------------------------------------------------------------------------

/*
 numberOfRowsInTableView:
 
 Table data source method to return the number of rows in the table.  By default returns the count of the represented array.  Subclasses may override this.
 
 Written by DJS 2004-05.
*/

- (int)numberOfRowsInTableView:(NSTableView *)tableView
{
    return [[self representedObject] count];
}

/*
 tableView:objectValueForTableColumn:row:
 
 Table data source method to return the value for the specified column and row.  By default returns the value from the represented array, assuming it is an NSArray or DSSortedData of dictionaries.  Subclasses may override this.
 
 Written by DJS 2004-05.
*/

- (id)tableView:(NSTableView *)tableView objectValueForTableColumn:(NSTableColumn *)tableColumn row:(int)row
{
    NSString *column = [tableColumn identifier];
    id rowObject = [self objectForRow:row];
    id value = [self valueWithRowObject:rowObject forIdentifier:column];
    
    if ([value isKindOfClass:[NSDate class]])
        value = [value descriptionWithShortDateAndOrTime];
    
    return value;
}

/*
 tableView:setObjectValue:forTableColumn:row:
 
 Table data source method to change the value of the specified column and row.  By default it changes the value in the corresponding dictionary of the represented array.  Subclasses may override this.
 
 Written by DJS 2004-05.
*/

- (void)tableView:(NSTableView *)tableView setObjectValue:(id)object forTableColumn:(NSTableColumn *)tableColumn row:(int)row
{
    NSString *column = [tableColumn identifier];
    id rowObject = [self objectForRow:row];
    
    if (object && column && rowObject)
    {
        [self setValue:object withRowObject:rowObject forIdentifier:column];
        
        [[NSNotificationCenter defaultCenter] postNotificationName:@"DSTableDidEditValueNotification" object:self];
    }
}

/*
 tableView:sortDesriptorsDidChange:
 
 Table data source method to support sorting of columns.  Subclasses shouldn't need to override this.
 
 Written by DJS 2004-05.
*/

- (void)tableView:(NSTableView *)tableView sortDescriptorsDidChange:(NSArray *)oldDescriptors
{
    if (![self isSortable])
        return;
    
    id object = [self representedObject];
    
//    NSAssert(object != nil, @"-[DSTableSupervisor tableView:sortDescriptorsDidChange:] - the represented object is nil");
    
    if (!object)
        return;
    
    NSAssert([tableView respondsToSelector:@selector(sortDescriptors)], @"-[DSTableSupervisor tableView:sortDescriptorsDidChange:] - tableView doesn't respond to -sortDescriptors (so you must be running 10.2)");
    NSAssert([object respondsToSelector:@selector(sortUsingDescriptors:)], @"-[DSTableSupervisor tableView:sortDescriptorsDidChange:] - representedObject doesn't respond to -sortUsingDescriptors:");
    
    if ([object respondsToSelector:@selector(sortUsingDescriptors:)] && [tableView respondsToSelector:@selector(sortDescriptors)])
    {
        NSArray *selected = [self selectedOrAllObjects:NO];
        
        [object sortUsingDescriptors:[tableView sortDescriptors]];
        
        [self selectObjects:selected byExtendingSelection:NO];
        
        [self reload];
    }
}


// ----------------------------------------------------------------------------------------
#pragma mark -
#pragma mark TABLE VIEW DRAGGING DATA SOURCE METHODS
// ----------------------------------------------------------------------------------------

/*
 tableView:writeRows:toPasteboard:
 
 Drag delegate method to put the dragged rows on the specified pasteboard.  Declares the -dragWithPasteboardTypes names, and puts the corresponding objects from the represented array in the first of them.  Subclasses may override this, call super, and add other values to the pboard, if desired.
 
 Written by DJS 2004-05.
*/

- (BOOL)tableView:(NSTableView *)tableView writeRows:(NSArray *)rows toPasteboard:(NSPasteboard *)pboard
{
    NSArray *types = [self dragWithPasteboardTypes];
    
    if (!types)
        return NO;
    
    NSEnumerator *enumerator = [rows objectEnumerator];
    NSMutableArray *array = [NSMutableArray arrayWithCapacity:[rows count]];
    NSNumber *rowNum;
    int row;
    id object;
    
    while ((rowNum = [enumerator nextObject]))
    {
        row = [rowNum intValue];
        object = [self objectForRow:row];
        
        if (object)
            [array addObject:object];
    }
    
    [pboard declareTypes:types owner:self];
    [pboard setPropertyList:array forType:[types firstObject]];
    
    return YES;
}

/*
 tableView:validateDrop:proposedRow:proposedDropOperation:
 
 Drag delegate method to make sure the drop is acceptable.  Subclasses may override this if desired.
 
 Written by DJS 2004-05.
*/

- (NSDragOperation)tableView:(NSTableView *)tableView validateDrop:(id <NSDraggingInfo>)info proposedRow:(int)row proposedDropOperation:(NSTableViewDropOperation)operation
{
    if (![self dragWithPasteboardTypes])
        return NSDragOperationNone;
    
    if ([[self representedObject] count])
        operation = NSTableViewDropAbove;
    else
        operation = NSTableViewDropOn;
    
    [tableView setDropRow:row dropOperation:operation];
    
    return NSDragOperationGeneric;
}

/*
 tableView:acceptDrop:row:dropOperation:
 
 Drag delegate method that is called when ready to accept the drop.  This implementation only supports drop-to operations, as drop-on ones don't make sense with the simple represented array.  Subclasses may override this if desired.
 
 Written by DJS 2004-05.
*/

- (BOOL)tableView:(NSTableView *)tableView acceptDrop:(id <NSDraggingInfo>)info row:(int)row dropOperation:(NSTableViewDropOperation)operation
{
    if (![self dragWithPasteboardTypes])
        return NO;
    
    NSPasteboard *pboard = [info draggingPasteboard];
    NSString *pasteboardName = [pboard availableTypeFromArray:[self acceptsDragPasteboardTypes]];
    
    if (pasteboardName)
    {
        NSArray *drag = [pboard propertyListForType:pasteboardName];
        
        [[self representedObject] insertOrMoveObjectsFromArray:drag atIndex:row];
    }
    
    [tableView reloadData];
    
    [self postNotificationName:@"DSTableDidAcceptDrop"];
    
    return YES;
}

/*
 tableView:deleteWithPasteboard:
 
 Special method called by the DSDragDeleteTableView table subclass.  The dragged objects are deleted.  Subclasses shouldn't need to override this; they should enable drag deletion by overriding -dragWithPasteboardTypes and -supportsDragDelete.
 
 Written by DJS 2004-05.
*/

- (BOOL)tableView:(NSTableView *)tableView deleteWithPasteboard:(NSPasteboard *)pboard
{
    //    NSLog(@"-tableView:%@ deleteWithPasteboard:%@", tableView, pboard);		// ¶¶ log
    
    NSString *pasteboardName = [[self dragWithPasteboardTypes] firstObject];
    
    if (!pasteboardName || ![self supportsDragDelete])
        return NO;
    
    NSArray *drag = [pboard propertyListForType:pasteboardName];
    
    [[self representedObject] removeObjectsInArray:drag];
    
    [tableView reloadData];
    
    [self postNotificationName:@"DSTableDidDragDelete"];
    
    return YES;
}


// ----------------------------------------------------------------------------------------
#pragma mark -
#pragma mark TABLE VIEW DELEGATE METHODS
// ----------------------------------------------------------------------------------------

/*
 tableView:didClickTableColumn:
 
 Table delegate method, called when the user clicks in a column title.  Initiates sorting.  Subclasses shouldn't need to override this.
 
 Written by DJS 2004-05.
*/

/*
- (void)tableView:(NSTableView *)tableView didClickTableColumn:(NSTableColumn *)tableColumn
{
    // This code based on sample at: <http://cocoadev.com/index.pl?SortingTableViewByColumn>
    
    int row = [self adjustRow:[theTableView selectedRow]];
    NSDictionary *dict = nil;
    
    if (row >= 0)
        dict = [statsArray objectAtIndex:row];
    
    if (lastColumn == tableColumn)
    {
        // User clicked same column, change sort order:
        sortDescending = !sortDescending;
        [PREFERENCES setBool:sortDescending forKey:STATS_SORT_DESCENDING];
    }
    else
    {
        // User clicked new column, change old/new column headers,
        // save new sorting selector, and re-sort the array:
        if (lastColumn)
        {
            [tableView setIndicatorImage:nil inTableColumn:lastColumn];
            [lastColumn release];
        }
        
        lastColumn = [tableColumn retain];
        [PREFERENCES setObject:[lastColumn identifier] forKey:STATS_SORT_COLUMN];
        
        [tableView setHighlightedTableColumn:tableColumn];
        
        [self sort];
    }
    
    // Set the graphic for the new column header:
    
    [tableView setIndicatorImage:[NSImage imageNamed:sortDescending ? @"NSAscendingSortIndicator" : @"NSDescendingSortIndicator"]
                   inTableColumn:tableColumn];
    [tableView setHighlightedTableColumn:tableColumn];
    
    [tableView reloadData];
    
    if (dict)
    {
        row = [self adjustRow:[statsArray indexOfObject:dict]];
        [theTableView selectRow:row byExtendingSelection:NO];
    }
}
*/

/*
 tableViewSelectionDidChange:
 
 Notification that the table selection changed.  Currently just posts a notification.  Subclasses may override this if desired.
 
 Written by DJS 2004-05.
*/

- (void)tableViewSelectionDidChange:(NSNotification *)aNotification
{
    if (![self selectionCount])
        [[NSNotificationCenter defaultCenter] postNotificationName:@"DSTableSelectionDidChange" object:self];
}

/*
 tableViewColumnDidResize:
 
 Notification that a column resized.  Currently just posts a notification.  Subclasses may override this if desired.
 
 Written by DJS 2004-05.
*/

- (void)tableViewColumnDidResize:(NSNotification *)aNotification
{
    [[NSNotificationCenter defaultCenter] postNotificationName:@"DSTableColumnDidResize" object:self];
}


// ----------------------------------------------------------------------------------------
#pragma mark -
#pragma mark TARGET / ACTION METHODS
// ----------------------------------------------------------------------------------------

/*
 tableViewHit:
 
 Action method for the table.  The table in the Nib should be connected to this.  A subclass should override -singleClicked: instead of this method.
 
 Written by DJS 2004-05.
*/

- (IBAction)tableViewHit:(id)sender
{
    // This may be the first click of a double click, so delay doing anything till we're sure, if so requested:
    if ([self shouldWaitForDoubleClick])
        [self performSelector:@selector(singleClicked:) withObject:sender afterDelay:DSDoubleClickWithSlopInterval];
    else
        [self singleClicked:sender];
}

/*
 singleClicked:
 
 The user clicked on the table.  Subclasses may override this, calling super, to perform any appropriate operation.  Note that if -shouldWaitForDoubleClick returns NO (as it does by default), this may be the first click of a double-click.
 
 Written by DJS 2004-05.
*/

- (void)singleClicked:(id)sender
{
    [[NSNotificationCenter defaultCenter] postNotificationName:@"DSTableSingleClicked" object:self];
}

/*
 doubleClicked:
 
 The user double-clicked on the table.  Subclasses may override this, calling super, to perform any appropriate operation.
 
 Written by DJS 2004-05.
*/

- (IBAction)doubleClicked:(id)sender
{
    // Cancel the single click perform request, if any, since this turned out to be a double-click:
    if ([self shouldWaitForDoubleClick])
        [NSObject cancelPreviousPerformRequestsWithTarget:self selector:@selector(singleClicked:) object:sender];
    
    [[NSNotificationCenter defaultCenter] postNotificationName:@"DSTableDoubleClicked" object:self];
}


// ----------------------------------------------------------------------------------------
#pragma mark -
#pragma mark ACCESSOR METHODS
// ----------------------------------------------------------------------------------------

/*
 currentTableView
 
 Returns the current table view controlled by this class.  Most useful when a subclass handles multiple tables; subclasses should override this if they support multiple tables.  It is recommended to have a separate table supervisor for each table, though, and use a mode tab supervisor to switch between them.  Other classes shouldn't manipulate the table directly; go through a method in this class or a subclass.
 
 Written by DJS 2004-05.
*/

- (NSTableView *)currentTableView
{
    return theTableView;
}


/*
 selectionCount
 
 Returns the current selection level of the table.  If greater than zero, table rows are being actively selected, so any actions based on the selection should be deferred.
 
 Written by DJS 2004-05.
*/

- (int)selectionCount
{
    return _selectionCount;
}

/*
 beginSelection
 
 This will increase the selection level of the table, returning the new level.  Normally no need to call this, as it is called automatically -- but it should be called if the table rows are selected manually.  It must be balanced by -endSelection, below.  Subclasses shouldn't need to override this.
 
 Written by DJS 2004-05.
*/

- (int)beginSelection
{
    _selectionCount++;
    
    // Paranoia is good:
    if (_selectionCount < 1)
        _selectionCount = 1;
    
    return _selectionCount;
}

/*
 endSelection
 
 This will decrease the selection level of the table, returning the new level.  Normally no need to call this, as it is called automatically -- but it should be called if the table rows are selected manually.  It must be called to balance -beginSelection, above.  Subclasses shouldn't need to override this.
 
 Written by DJS 2004-05.
*/

- (int)endSelection
{
    _selectionCount--;
    
    if (_selectionCount < 0)
        _selectionCount = 0;
    
    return _selectionCount;
}

@end

