//
//  DSIconTextCell.m
//  Shared classes
//
//  Created by David Sinclair on 2006-04-09.
//  Copyright 2006 Dejal Systems, LLC. All rights reserved.
//
//  DSIconTextCell based on Apple's ImageAndTextCell sample code.
//  DSSourceIconTextCell based on Scotland Software's SourceListTextCell sample code.
//

#import "DSIconTextCell.h"
#import "DSStringCategories.h"


#define DS_ICON_TEXT_MARGIN 3
#define DS_ICON_TEXT_GAP 3


@implementation DSIconTextCell

- (void)dealloc
{
    [image release];
    image = nil;
    
    [super dealloc];
}

- (id)copyWithZone:(NSZone *)zone
{
    DSIconTextCell *cell = (DSIconTextCell *)[super copyWithZone:zone];
    
    cell->image = [image retain];
    
    return cell;
}

- (void)setImage:(NSImage *)anImage
{
    if (anImage != image)
    {
        [image release];
        image = [anImage retain];
    }
}

- (NSImage *)image
{
    return image;
}

- (NSRect)imageFrameForCellFrame:(NSRect)cellFrame
{
    if (image)
    {
        NSRect imageFrame;
        
        imageFrame.size = [image size];
        imageFrame.origin = cellFrame.origin;
        imageFrame.origin.x += DS_ICON_TEXT_MARGIN;
        imageFrame.origin.y += ceil((cellFrame.size.height - imageFrame.size.height) / 2);
        
        return imageFrame;
    }
    else
        return NSZeroRect;
}

- (void)editWithFrame:(NSRect)aRect inView:(NSView *)controlView editor:(NSText *)textObj delegate:(id)anObject event:(NSEvent *)theEvent
{
    NSRect textFrame, imageFrame;
    
    if (image)
        NSDivideRect (aRect, &imageFrame, &textFrame, DS_ICON_TEXT_MARGIN + DS_ICON_TEXT_GAP + [image size].width, NSMinXEdge);
    else
        textFrame = aRect;
    
    [super editWithFrame: textFrame inView: controlView editor:textObj delegate:anObject event: theEvent];
}

- (void)selectWithFrame:(NSRect)aRect inView:(NSView *)controlView editor:(NSText *)textObj delegate:(id)anObject start:(int)selStart length:(int)selLength
{
    NSRect textFrame, imageFrame;
    
    if (image)
        NSDivideRect (aRect, &imageFrame, &textFrame, DS_ICON_TEXT_MARGIN + DS_ICON_TEXT_GAP + [image size].width, NSMinXEdge);
    else
        textFrame = aRect;
    
    [super selectWithFrame:textFrame inView:controlView editor:textObj delegate:anObject start:selStart length:selLength];
}

- (void)drawWithFrame:(NSRect)cellFrame inView:(NSView *)controlView
{
    NSRect	imageFrame;
    NSString *oldTitle = [self title];
    
    if (image)
    {
        NSSize	imageSize;
        
        imageSize = [image size];
        NSDivideRect(cellFrame, &imageFrame, &cellFrame, DS_ICON_TEXT_MARGIN + DS_ICON_TEXT_GAP + imageSize.width, NSMinXEdge);
        
        if ([self drawsBackground])
        {
            [[self backgroundColor] set];
            NSRectFill(imageFrame);
        }
        
        imageFrame.origin.x += DS_ICON_TEXT_GAP;
        imageFrame.size = imageSize;
        
        if ([controlView isFlipped])
            imageFrame.origin.y += ceil((cellFrame.size.height + imageFrame.size.height) / 2);
        else
            imageFrame.origin.y += ceil((cellFrame.size.height - imageFrame.size.height) / 2);
        
        int titleWidth = [self titleRectForBounds:cellFrame].size.width;
        
        [self setTitle:[oldTitle truncatedStringWithIndicator:@"..." forFont:[self font] withWidth:titleWidth]];
    }
    
    [super drawWithFrame:cellFrame inView:controlView];
    
    if (image)
        [image compositeToPoint:imageFrame.origin operation:NSCompositeSourceOver];
    
    [self setTitle:oldTitle];
}

- (NSSize)cellSize
{
    NSSize cellSize = [super cellSize];
    
    cellSize.width += (image ? [image size].width : 0) + DS_ICON_TEXT_MARGIN + DS_ICON_TEXT_GAP;
    
    return cellSize;
}

@end


// ----------------------------------------------------------------------------------------
#pragma mark -
// ----------------------------------------------------------------------------------------


@implementation DSSourceIconTextCell

- (void)drawInteriorWithFrame:(NSRect)cellFrame inView:(NSView *)controlView;
{
    if (![self isHighlighted])
    {
        [super drawInteriorWithFrame:cellFrame inView:controlView];
        
        return;
    }
    
/*
    NSImage *gradient;
    
    // Determine whether we should draw a blue or grey gradient.  We will automatically redraw when our parent view loses/gains focus, or when our parent window loses/gains main/key status:
    if (([[controlView window] firstResponder] == controlView) && 
        [[controlView window] isMainWindow] &&
        [[controlView window] isKeyWindow])
        gradient = [NSImage imageNamed:@"SourceCellEnabled.tiff"];
    else
        gradient = [NSImage imageNamed:@"SourceCellDisabled.tiff"];
    
    // Make sure we draw the gradient the correct way up:
    [gradient setFlipped:YES];
    
    [controlView lockFocus];
    
    // Draw the gradient background:
    NSSize gradientSize = [gradient size];
    int i = 0;
    
    for (i = cellFrame.origin.x; i < (cellFrame.origin.x + cellFrame.size.width); i += gradientSize.width)
    {
        [gradient drawInRect:NSMakeRect(i, cellFrame.origin.y, gradientSize.width, cellFrame.size.height)
                    fromRect:NSMakeRect(0, 0, gradientSize.width, gradientSize.height)
                   operation:NSCompositeSourceOver
                    fraction:1.0];
    }
*/
    
    if (![[self stringValue] length])
        return;
    
    [controlView lockFocus];
    
    // Now draw our text in white:
    NSRect inset = cellFrame;
    inset.origin.x += 2;
    
    NSMutableDictionary *attrs = [NSMutableDictionary dictionaryWithDictionary:[[self attributedStringValue] attributesAtIndex:0 effectiveRange:NULL]];
    [attrs setValue:[NSColor whiteColor] forKey:@"NSColor"];
    [[self stringValue] drawInRect:inset withAttributes:attrs];
    
    [controlView unlockFocus];
}

@end

