//
//  DSViewsListView.m
//  Shared classes
//
//  Created by David Sinclair on Fri Jun 20 2003.
//  Copyright (c) 2003 - 2005 Dejal Systems, LLC. All rights reserved.
//

#import "DSViewsListView.h"
#import "DSSharedDefines.h"


@interface DSViewsListView (Private)

- (void)_privateReplaceView:(NSView *)oldView withView:(NSView *)newView isEmpty:(BOOL)isEmpty;

- (NSArray *)adjustAutoresizingAroundPosition:(float)position
                           stickPositionToTop:(BOOL)stickPositionToTop
                                 forSuperview:(NSView *)superview;

- (void)restoreAutoresizeMasks:(NSArray *)masks forSuperview:(NSView *)superview;

- (void)_adjustNextKeyViews;

- (void)postBatchCleanup;

- (BOOL)badHandler:(id)handler;

- (NSMutableArray *)viewHandlersArray;
- (void)setViewHandlersArray:(NSMutableArray *)value;

@end


@implementation DSViewsListView

- (id)initWithFrame:(NSRect)frame
{
    self = [super initWithFrame:frame];
    
    if (self)
    {
        _batchCount = 0;
        
        [self setViewHandlersArray:[NSMutableArray array]];
        [self setResizesViaSplitView:nil isWithinTopSubview:NO];
        [self setResizesViaScrollView:nil];
        [self setOldNextKeyView:[self nextKeyView]];
        [self setMinimumHeight:0];
        [self setAddToBottom:YES];
        [self setDrawsBackground:YES];
        [self setDrawsBorder:YES];
        [self setAnimate:YES];
    }

    return self;
}

- (void)dealloc
{
    [self setViewHandlersArray:nil];
    [self setResizesViaSplitView:nil isWithinTopSubview:NO];
    [self setResizesViaScrollView:nil];
    [self setOldNextKeyView:nil];
    
    [super dealloc];
}

- (void)drawRect:(NSRect)rect
{
    if ([self drawsBackground])
    {
        NSArray *alternatingBackgroundColors;
        
        // NSColor +controlAlternatingRowBackgroundColors is only available in 10.3 and later:
        if ([NSColor respondsToSelector:@selector(controlAlternatingRowBackgroundColors)])
            alternatingBackgroundColors = [NSColor controlAlternatingRowBackgroundColors];
        else
            alternatingBackgroundColors = [NSArray arrayWithObjects:[NSColor whiteColor], [NSColor colorWithCalibratedRed:0.93 green:0.95 blue:0.99 alpha:1.0], nil];
        
        NSColor *bgColor;
        NSEnumerator *enumerator = [self handlerEnumerator];
        NSView *view;
        int index = 0;
        int count = [alternatingBackgroundColors count];
        
        // Firstly paint everything the window background color, in case the VLV is larger than the contained views (can happen, e.g. via a splitter):
        rect = [self frame];
        [[NSColor controlBackgroundColor] set];
        NSRectFillUsingOperation(rect, NSCompositeSourceOver);
        
        // Fill each container view's frame with alternating colors:
        while ((view = [[enumerator nextObject] containerView]))
        {
            rect = [view frame];
            
            bgColor = [alternatingBackgroundColors objectAtIndex:index];
            
            [bgColor set];
            NSRectFillUsingOperation(rect, NSCompositeSourceOver);
            
            index++;
            
            if (index >= count)
                index = 0;
        }
    }

    if ([self drawsBorder])
    {
        [[NSColor grayColor] set];
        NSFrameRectWithWidthUsingOperation([self bounds], 1.0, NSCompositeSourceOver);
    }
}

- (BOOL)isOpaque
{
    return [self drawsBackground];
}

- (BOOL)isFlipped
{
    return [self willAddToBottom];
}


// ----------------------------------------------------------------------------------------
#pragma mark -
#pragma mark CONTENT MANAGEMENT METHODS
// ----------------------------------------------------------------------------------------

/*
 addHandlersWithClass:settingsArray:defaultSettings:userInfo:

 Given a handler class, an array of settings that that class understands, and default settings, this clears the list and adds handlers for each settings instance in the array, or one handler for the default settings.  Of course, this assumes that one handler class is used for all items; if different settings require different handler classes, you'll need to add the handlers manually.  The user info can be any value you want to pass to all handlers.  Any of the settings array, default settings, and user info can be nil.  Each settings object is normally a dictionary, but can be any object.  These are setup by allocating a new instance of the specified class, then calling the -initForViewsListView:withSettings:userInfo: method of each handler (which is required by the protocol).
 
 Written by DJS 2003-07.
*/

- (void)addHandlersWithClass:(Class)handlerClass
               settingsArray:(NSArray *)settingsArray
             defaultSettings:(id)defaultSettings
                    userInfo:(id)userInfo
{
    [self beginBatch];
    
    [self removeAllHandlers];

    // If none in the array, use the default settings, if any:
    if (![settingsArray count] && defaultSettings)
        settingsArray = [NSArray arrayWithObject:defaultSettings];
    
    if ([settingsArray count])
    {
        id settings;
        
        for (settings in settingsArray)
        {
            // Allocate an instance of the specified class, and initialize it with the designated initializer specified in the protocol:
//            id handler = [[[handlerClass alloc] performSelector:@selector(initForViewsListView:withSettings:)
//                                                     withObject:self withObject:settings] autorelease];
            
            id handler = [[[handlerClass alloc] initForViewsListView:self withSettings:settings userInfo:userInfo] autorelease];
            
            if (handler)
                [self addHandler:handler];
        }
    }
    
//    NSLog(@"VLV add handlers for %@:\nhandlers:%@\nsettings: %@", NSStringFromClass(handlerClass), [self viewHandlersArray], settingsArray);  // 2007-10
    
    [self endBatch];
}

/*
 handlerSettings
 
 Returns an array containing settings from all current handlers (or an empty array if none).  These are obtained by calling the -settings method of each handler (which is required by the protocol).  The returned settings are immutable copies, to avoid possible issues when saving.
 
 Written by DJS 2003-07.
*/

- (NSArray *)handlerSettings
{
    NSMutableArray *array = [NSMutableArray arrayWithCapacity:[self count]];
    NSEnumerator *enumerator = [self handlerEnumerator];
    id handler;

    while ((handler = [enumerator nextObject]))
    {
        id settings = [handler performSelector:@selector(settings)];

        if ([settings conformsToProtocol:@protocol(NSCopying)])
            settings = [[settings copy] autorelease];

        if (settings)
            [array addObject:settings];
    }

    return array;
}


/*
 replaceView:withView:

 Replaces the old container view with the new one, resizing the DSViewsListView and window or split view appropriately.  Both views must be managed by the same handler, and both views must be specified.  To add or remove a view, call -addHandler: or -removeHandler: instead.

 Written by DJS 2003-07.
*/

- (void)replaceView:(NSView *)oldView withView:(NSView *)newView
{
    // If either of these are not provided, don't do anything:
    if (!oldView || !newView)
        return;

    [self _privateReplaceView:oldView withView:newView isEmpty:NO];
    
    [self _adjustNextKeyViews];

    [self display];
}

/*
 _replaceHandler:withHandler:otherHandler:after:

 Replaces the old handler and its containerView with the new one, optionally before or after the other handler (only applies when oldHandler is nil; if otherHandler is not found, it is added to the start or end based on the after parameter).  Private to this class.

 Written by DJS 2003-06.
 Changed by DJS 2004-01 to support inserting before or after an other handler.
*/

- (void)_replaceHandler:(id)oldHandler withHandler:(id)newHandler otherHandler:(id)otherHandler after:(BOOL)after
{
    if ([self badHandler:oldHandler] || [self badHandler:newHandler])
        return;

    NSView *oldView = [oldHandler containerView];
    NSView *newView = [newHandler containerView];
    BOOL isEmpty = ![[self viewHandlersArray] count];
    
    // If the view is either being removed or replaced, force editing to end to avoid crashing:
    if (oldView && oldView != newView)
        [[oldView window] forceEndEditingForView:oldView];
    
    // Replace or add before doing the work, so the -drawRect: method has the new view to paint the background while drawing the resizing animation:
    if (oldHandler && newHandler && oldHandler != newHandler)		// Replace handler
    {
        int index = [[self viewHandlersArray] indexOfObject:oldHandler];
        [[self viewHandlersArray] replaceObjectAtIndex:index withObject:newHandler];
    }
    else if (!oldHandler)						// Add handler
    {
        if (otherHandler && [self scrollView])            //  hack: inserting at the right place is only supported for scroll-based VLVs currently
        {
            int index = [[self viewHandlersArray] indexOfObject:otherHandler];
            
            if (after && index != NSNotFound)
                index++;
            
            [[self viewHandlersArray] insertObject:newHandler atIndex:index orEnd:after];
        }
        else
            [[self viewHandlersArray] addObject:newHandler];
    }
    
    if (!newHandler && [self scrollView])                               // Remove handler if scroll-based
        [[self viewHandlersArray] removeObject:oldHandler];
    
    [self _privateReplaceView:oldView withView:newView isEmpty:isEmpty];
    
    // Remove after doing the work if not scroll-based, so the -drawRect: method has the old view to paint the background while drawing the resizing animation:
    if (!newHandler && ![self scrollView])                              // Remove handler if not scroll-based
        [[self viewHandlersArray] removeObject:oldHandler];
    
    // Only do this stuff if not in batch mode:
    if (![self batchCount])
    {
        [self _adjustNextKeyViews];
        
        // Tell anyone who cares that the handlers have changed:
        [[NSNotificationCenter defaultCenter] postNotificationName:@"DSViewsListViewHandlersDidChange" object:self];
        
        [self display];
    }
}

/*
 replaceHandler:withHandler:
 
 Replaces the old handler and its containerView with the new one, resizing the DSViewsListView and window or split view appropriately.
 
 Written by DJS 2003-06.
*/

- (void)replaceHandler:(id)oldHandler withHandler:(id)newHandler
{
    [self _replaceHandler:oldHandler withHandler:newHandler otherHandler:nil after:YES];
}

/*
 addHandler:
 
 Adds the handler and its containerView to the DSViewsListView, resizing it and the window or split view appropriately.
 
 Written by DJS 2003-06.
*/

- (void)addHandler:(id)handler
{
    [self replaceHandler:nil withHandler:handler];
}

/*
 insertHandler:beforeHandler:
 
 Inserts the new handler and its containerView before the old handler in the DSViewsListView, resizing it and the window or split view appropriately.
 
 Written by DJS 2004-01.
*/

- (void)insertHandler:(id)newHandler beforeHandler:(id)oldHandler
{
    [self _replaceHandler:nil withHandler:newHandler otherHandler:oldHandler after:NO];
}

/*
 insertHandler:afterHandler:
 
 Inserts the new handler and its containerView after the old handler in the DSViewsListView, resizing it and the window or split view appropriately.
 
 Written by DJS 2004-01.
*/

- (void)insertHandler:(id)newHandler afterHandler:(id)oldHandler
{
    [self _replaceHandler:nil withHandler:newHandler otherHandler:oldHandler after:YES];
}

/*
 removeHandler:

 Removes the handler and its containerView from the DSViewsListView, resizing it and the window or split view appropriately.

 Written by DJS 2003-06.
 */

- (void)removeHandler:(id)handler
{
    [self replaceHandler:handler withHandler:nil];
}

/*
 removeAllHandlers
 
 Removes all handlers and their containerViews from the DSViewsListView, resizing it down to zero.
 
 Written by DJS 2003-07.
*/

- (void)removeAllHandlers
{
    [self beginBatch];
    
    if (![self count])	    // Special case: if none, ensure that the view is zero-sized; may not be initially
        [self _privateReplaceView:nil withView:nil isEmpty:YES];
    else
    {
        NSEnumerator *enumerator = [self reverseHandlerEnumerator];
        id handler;

        while ((handler = [enumerator nextObject]))
        {
            [self removeHandler:handler];
        }
    }
    
//    if (![self batchCount])
//        [self _adjustNextKeyViews];
    
    [self endBatch];
}


/*
 count
 
 Returns the number of handlers managed by the DSViewsListView, i.e. the number of "rows".
 
 Written by DJS 2003-06.
*/

- (int)count
{
    return [[self viewHandlersArray] count];
}

/*
 firstHandler
 
 Returns the first handler managed by the DSViewsListView, or nil if there are none.
 
 Written by DJS 2004-01.
*/

- (id)firstHandler
{
    return [[self viewHandlersArray] firstObject];
}

/*
 lastHandler
 
 Returns the last handler managed by the DSViewsListView, or nil if there are none.
 
 Written by DJS 2004-01.
*/

- (id)lastHandler
{
    return [[self viewHandlersArray] lastObject];
}

/*
 handlerEnumerator
 
 Returns an enumerator to all of the handlers managed by the DSViewsListView.
 
 Written by DJS 2003-06.
*/

- (NSEnumerator *)handlerEnumerator
{
    return [[self viewHandlersArray] objectEnumerator];
}

/*
 reverseHandlerEnumerator

 Returns an enumerator to all of the handlers managed by the DSViewsListView, in reverse order.

 Written by DJS 2003-07.
 */

- (NSEnumerator *)reverseHandlerEnumerator
{
    return [[self viewHandlersArray] reverseObjectEnumerator];
}


// ----------------------------------------------------------------------------------------
#pragma mark -
#pragma mark BATCH MODE METHODS
// ----------------------------------------------------------------------------------------

/*
 batchCount
 
 Returns the current batch level of the DSViewsListView.  If greater than zero, it is operating in batch mode, and multiple containers can be added or removed without drawing anything, for increased speed.
 
 Written by DJS 2004-01.
*/

- (int)batchCount
{
    return _batchCount;
}

/*
 beginBatch
 
 This will increase the batch level of the DSViewsListView, returning the new level.  It will then operate in batch mode, so multiple containers can be added or removed without drawing anything.  Normally no need to call this, as it is automatically called by -addHandlersWithClass::: and -removeAllHandlers.  It must be balanced by a call to -endBatch, below, and can be nested if desired.
 
 Written by DJS 2004-01.
*/

- (int)beginBatch
{
    _batchCount++;
    
    // Paranoia is good:
    if (_batchCount < 1)
        _batchCount = 1;
    
    return _batchCount;
}

/*
 endBatch:
 
 This will decrease the batch level of the DSViewsListView, returning the new level.  If the level (see -batchCount) is above zero, the DSViewsListView operates in batch mode, so multiple containers can be added or removed without drawing anything.  Normally no need to call this, as it is automatically called by -addHandlersWithClass::: and -removeAllHandlers.  It must be called to balance a prior call to -beginBatch, above, and can be nested if desired.
 
 Written by DJS 2004-01.
*/

- (int)endBatch
{
    _batchCount--;
    
    // Paranoia is good:
    if (_batchCount < 0)
        _batchCount = 0;
    
    if (!_batchCount)
    {
        // Do the deferred activities now:
        [self postBatchCleanup];
        
        [self _adjustNextKeyViews];
        
        // Tell anyone who cares that the handlers have changed:
        [[NSNotificationCenter defaultCenter] postNotificationName:@"DSViewsListViewHandlersDidChange" object:self];
        
        [self display];
    }
    
    return _batchCount;
}


// ----------------------------------------------------------------------------------------
#pragma mark -
#pragma mark ACCESSOR METHODS
// ----------------------------------------------------------------------------------------

/*
 splitView
 
 Returns the split view used to resize the receiver, if any.  If nil, the window is resized instead (unless a scroll view is set).
 
 Written by DJS 2003-11.
*/

- (NSSplitView *)splitView
{
    return _splitView;
}

/*
 isWithinTopSubview
 
 Returns YES if the receiver is above the splitter bar, otherwise NO.  Irrelevant if -splitView is nil.
 
 Written by DJS 2003-11.
*/

- (BOOL)isWithinTopSubview
{
    return _isWithinTopSubview;
}

/*
 setResizesViaSplitView:isWithinTopSubview:
 
 If a split view is provided, the receiver will resize itself by resizing that split view.  If nil, it will resize the window instead, unless a scroll view is set.  If the receiver is above the splitter bar pass YES for isWithinTopSubview, otherwise NO.
 
 Written by DJS 2003-11.
 Changed by DJS 2005-09 to automatically turn off animation.
*/

- (void)setResizesViaSplitView:(NSSplitView *)splitView isWithinTopSubview:(BOOL)isWithinTopSubview
{
    [splitView retain];
    [_splitView autorelease];
    _splitView = splitView;
    
    _isWithinTopSubview = isWithinTopSubview;
    
    [self setAnimate:NO];
}


/*
 scrollView
 
 Returns the scroll view used to resize the receiver, if any.  If nil, the window is resized instead (unless a split view is set).
 
 Written by DJS 2003-12.
*/

- (NSScrollView *)scrollView
{
    return _scrollView;
}

/*
 setResizesViaScrollView:
 
 If a scroll view is provided, the receiver will resize itself with that scroll view.  If nil, it will resize the window instead, unless a split view is set.
 
 Written by DJS 2003-12.
*/

- (void)setResizesViaScrollView:(NSScrollView *)scrollView
{
    [scrollView retain];
    [_scrollView autorelease];
    _scrollView = scrollView;
}


- (NSView *)oldNextKeyView
{
    return _oldNextKeyView;
}

- (void)setOldNextKeyView:(NSView *)value
{
    [value retain];
    [_oldNextKeyView autorelease];
    _oldNextKeyView = value;
}


/*
 willAddToBottom

 Returns YES if -addHandler: will add the new handler to the bottom of the DSViewsListView, or NO if it will appear at the top.  By default this will return YES.
 
 Written by DJS 2003-07.
*/

- (BOOL)willAddToBottom
{
    return _addToBottom;
}

/*
 setAddToBottom:

 Sets whether -addHandler: should add the new handler to the bottom of the DSViewsListView (if YES), or to the top (if NO).  By default this is set to YES, i.e. new handlers' views appear at the bottom.
 
 Written by DJS 2003-07.
*/

- (void)setAddToBottom:(BOOL)value
{
    _addToBottom = value;
}

/*
 drawsBackground

 Returns YES if the DSViewsListView draws an alternating color background, or NO if it doesn't draw a background.  By default this will return YES.
 
 Written by DJS 2003-07.
*/

- (BOOL)drawsBackground
{
    return _drawsBackground;
}

/*
 setDrawsBackground:

 Sets whether or not the DSViewsListView draws an alternating color background.  By default it does.
 
 Written by DJS 2003-07.
*/

- (void)setDrawsBackground:(BOOL)value
{
    _drawsBackground = value;
}

/*
 drawsBorder

 Returns YES if the DSViewsListView draws a gray border around the content.  By default this will return YES.
 
 Written by DJS 2003-07.
*/

- (BOOL)drawsBorder
{
    return _drawsBorder;
}

/*
 setDrawsBorder:

 Sets whether or not the DSViewsListView draws a gray border around the content.  By default it does.
 
 Written by DJS 2003-07.
*/

- (void)setDrawsBorder:(BOOL)value
{
    _drawsBorder = value;
}

/*
 willAnimate
 
 Returns YES if the DSViewsListView animates the resize.  Never animates when the window isn't visible.
 
 Written by DJS 2005-09.
*/

- (BOOL)willAnimate
{
    return _animate;
}

/*
 setAnimate:
 
 Sets whether or not the DSViewsListView should animate the resize.  Never animates when the window isn't visible.
 
 Written by DJS 2005-09.
*/

- (void)setAnimate:(BOOL)value
{
    _animate = value;
}

/*
 minimumHeight
 
 Returns the minimum height for the DSViewsListView, to avoid munching the contained views.  This is currently only supported in scroll-based views (others will always return zero).
 
 Written by DJS 2004-01.
*/

- (float)minimumHeight
{
    return _minimumHeight;
}

/*
 setMinimumHeight:
 
 Sets the minimum height for the DSViewsListView.  This is calculated when views are added or removed; you shouldn't need to call it.  It doesn't limit the DSViewsListView at all; this value is for external use, e.g. in sizing a splitter etc.
 
 Written by DJS 2004-01.
*/

- (void)setMinimumHeight:(float)value
{
    _minimumHeight = value;
}

@synthesize _batchCount;
@synthesize _oldNextKeyView;
@synthesize _scrollView;
@synthesize _isWithinTopSubview;
@synthesize _splitView;
@synthesize _viewHandlersArray;
@end


// ----------------------------------------------------------------------------------------
#pragma mark -
// ----------------------------------------------------------------------------------------

@implementation DSViewsListView (Private)

// ----------------------------------------------------------------------------------------
#pragma mark PRIVATE CONTENT MANAGEMENT METHODS
// ----------------------------------------------------------------------------------------

- (NSMutableArray *)_preserveAutoresizeMasks
{
    NSMutableArray *masks = [NSMutableArray arrayWithCapacity:[self count]];
    NSEnumerator *enumerator = [self handlerEnumerator];
    NSView *containerView;
    id handler;
    unsigned int mask;
    
    while ((handler = [enumerator nextObject]))
    {
        containerView = [handler containerView];
    	mask = [containerView autoresizingMask];
        
        [masks addObject:[NSNumber numberWithUnsignedInt:mask]];
        
        if (mask & NSViewHeightSizable)
        {
            mask &= ~NSViewHeightSizable;
            [containerView setAutoresizingMask:mask];
        }
    }
    
    return masks;
}

- (void)_restoreAutoresizeMasks:(NSMutableArray *)masks
{
    NSEnumerator *handlerEnumerator = [self handlerEnumerator];
    NSEnumerator *maskEnumerator = [masks objectEnumerator];
    NSView *containerView;
    id handler;
    unsigned int mask;
    
    while ((handler = [handlerEnumerator nextObject]))
    {
        containerView = [handler containerView];
    	mask = [[maskEnumerator nextObject] unsignedIntValue];
        
        [containerView setAutoresizingMask:mask];
    }
}

- (int)_repositionContainers
{
    NSScrollView *scrollView = [self scrollView];
    NSView *containerView;
    NSRect vlvFrame = [self frame];
    NSRect containerFrame;
    NSEnumerator *enumerator = [self handlerEnumerator];
    id handler;
    int vlvWidth = NSWidth([[scrollView contentView] frame]);     // NSWidth([scrollView frame]);     // vlvFrame
//    unsigned int oldMask = 0;
//    unsigned int tempMask = 0;
    unsigned int mask = 0;
    NSMutableArray *masks = [self _preserveAutoresizeMasks];
    NSEnumerator *maskEnumerator = [masks objectEnumerator];
    
//    vlvFrame.size.width = vlvWidth;
    vlvFrame.size.height = 0;
    int minimumHeight = 0;
    int remainder = NSHeight([scrollView frame]);      // NSHeight([[scrollView contentView] frame]);
    
    while ((handler = [enumerator nextObject]))
    {
        containerView = [handler containerView];
        containerFrame = [containerView frame];
        
        containerFrame.origin.x = 0;
        containerFrame.origin.y = 0;        // vlvFrame.size.height
        containerFrame.size.width = vlvWidth;
        
//        oldMask = [containerView autoresizingMask];
//        tempMask = oldMask;
        
        minimumHeight += containerFrame.size.height;
        
        mask = [[maskEnumerator nextObject] unsignedIntValue];
        
        if ((mask & NSViewHeightSizable))
        {
//            tempMask &= ~NSViewHeightSizable;
//            [containerView setAutoresizingMask:tempMask];
            
            if (containerFrame.size.height < remainder)
                containerFrame.size.height = remainder;
        }
        
//        NSLog(@"containerFrame = %@; remainder = %d", NSStringFromRect(containerFrame), remainder);		//  log
        
        vlvFrame.size.width = vlvWidth;
        vlvFrame.size.height += containerFrame.size.height;
        remainder -= containerFrame.size.height;
        
        [self setFrame:vlvFrame];
        [containerView setFrame:containerFrame];
        
//        [containerView setAutoresizingMask:oldMask];
    }
    
    // If no handlers, set the frame here so it is zeroed:
    if (![self count])
        [self setFrame:vlvFrame];
    
    [self _restoreAutoresizeMasks:masks];
    
    [self setMinimumHeight:minimumHeight];
    
    return vlvFrame.size.height;
}

- (void)_scrollResizingReplaceView:(NSView *)oldView withView:(NSView *)newView isEmpty:(BOOL)isEmpty
{
    BOOL lockedFocus = [self lockFocusIfCanDraw];
    
/*
    unsigned int oldMask = [newView autoresizingMask];
    unsigned int tempMask = oldMask;
    
    if (tempMask & NSViewHeightSizable)
    {
        tempMask &= ~NSViewHeightSizable;
        [newView setAutoresizingMask:tempMask];
    }
*/
    
    // Remove the old view, add a new one, or replace the old view:
    if (oldView && !newView)
        [oldView removeFromSuperview];
    else if (!oldView && newView)
        [self addSubview:newView];
    else if (oldView && newView)
        [self replaceSubview:oldView with:newView];
    
//    [newView setAutoresizingMask:oldMask];
    
//    NSScrollView *scrollView = [self scrollView];
//    NSRect vlvFrame = [self frame];
//    NSRect newFrame = [newView frame];		// This rect is undefined if newView is nil
    
/*
    NSLog(@"class = %@, selector = %@, responds = %d", NSStringFromClass(NSClassFromString(@"NSScrollView")), NSStringFromSelector(NSSelectorFromString(@"setAutohidesScrollers:")), [NSClassFromString(@"NSScrollView") respondsToSelector:NSSelectorFromString(@"setAutohidesScrollers:")]);		//  log
    
    if (DSClassSelectorAvailable(@"NSScrollView", @"setAutohidesScrollers:"))
    {
        NSLog(@"turning off autohiding");		//  log
        [scrollView setAutohidesScrollers:NO];
    }
*/
/*    
    if (isEmpty)
        vlvFrame.size.height = 0;
    
    int newViewHeight = NSHeight([scrollView frame]) - NSHeight(vlvFrame);
    
    // If the new container view is height-sizable and shorter than the DSViewsListView, resize it to the height of the DSViewsListView:
    if (newView && ([newView autoresizingMask] & NSViewHeightSizable) && newFrame.size.height < newViewHeight)
    {
        NSLog(@"before: newFrame.size.height = %f", newFrame.size.height);		//  log
        
//        newFrame.size.height = newViewHeight;
        
        NSLog(@"after:  newFrame.size.height = %f", newFrame.size.height);		//  log
    }
    
    [newView setFrame:newFrame];
*/
    
    if (![self batchCount])
        [self postBatchCleanup];
    
    // Slow-motion when Shift down:
    if ([[NSApp currentEvent] modifierFlags] & NSShiftKeyMask)
    {
        unsigned long finalTicks;
        Delay(30, &finalTicks);
    }
    
    if (lockedFocus)
        [self unlockFocus];
}

/*
- (void)_scrollResizingReplaceView:(NSView *)oldView withView:(NSView *)newView isEmpty:(BOOL)isEmpty
{
    BOOL lockedFocus = [self lockFocusIfCanDraw];
    
    NSScrollView *scrollView = [self scrollView];
    NSRect vlvFrame = [self frame];
    NSRect newFrame = [newView frame];		// This rect is undefined if newView is nil
    NSRect oldFrame = [oldView frame];		// This rect is undefined if oldView is nil
    int newVLVHeight;
    int newViewHeight = NSHeight([scrollView frame]) - NSHeight(vlvFrame);
    
    // If the new container view is height-sizable and shorter than the DSViewsListView, resize it to the height of the DSViewsListView:
    if (newView && ([newView autoresizingMask] & NSViewHeightSizable) && newFrame.size.height < newViewHeight)
        newFrame.size.height = newViewHeight;
    
    // Figure out the size of the DSViewsListView:
    if (!oldView && !newView)
        newVLVHeight = NSHeight(vlvFrame) - NSHeight(vlvFrame);
    else if (!newView)
        newVLVHeight = NSHeight(vlvFrame) - NSHeight(oldFrame) + 1;
    else if (isEmpty)
        newVLVHeight = NSHeight(vlvFrame) - NSHeight(vlvFrame) + newFrame.size.height - 1;
    else if (oldView)
        newVLVHeight = NSHeight(vlvFrame) + (NSHeight(newFrame) - NSHeight(oldFrame));
    else
        newVLVHeight = NSHeight(vlvFrame) + newFrame.size.height;
    
    // Resize the DSViewsListView appropriately:
    vlvFrame.size.height = newVLVHeight;
    
    [self setFrame:vlvFrame];
    
    [scrollView setNeedsDisplay:YES];
    
    // Position the new container view:
    if ((!isEmpty || oldView) && newView)
    {
        if (oldView)
            newFrame.origin = oldFrame.origin;
    }
    
    // Resize the new container view to the width of the DSViewsListView:
    newFrame.size.width = NSWidth(vlvFrame);
    
    [newView setFrame:newFrame];
    
    / *
     // If removing or replacing a view, move up the ones below it:
     if (oldView)
     {
         int delta = NSHeight(oldFrame);
         
         if (newView)
             delta -= NSHeight(newFrame);
         
         NSEnumerator *enumerator = [self handlerEnumerator];
         id handler;
         
         while ((handler = [enumerator nextObject]))
         {
             NSView *otherView = [handler containerView];
             NSRect otherFrame = [otherView frame];
             
             if (NSMaxY(otherFrame) < NSMaxY(oldFrame))
             {
                 otherFrame.origin.y += delta;
                 
                 [otherView setFrame:otherFrame];
             }
         }
     }
     * /
    
    // Remove the old view, add a new one, or replace the old view:
    if (oldView && !newView)
        [oldView removeFromSuperview];
    else if (!oldView && newView)
        [self addSubview:newView];
    else if (oldView && newView)
        [self replaceSubview:oldView with:newView];
    
    // Slow-motion when Shift down:
    if ([[NSApp currentEvent] modifierFlags] & NSShiftKeyMask)
    {
        unsigned long finalTicks;
        Delay(30, &finalTicks);
    }
    
    [self _recalculateSubviewFrames];
    
    if (lockedFocus)
        [self unlockFocus];
}
*/

- (void)_splitResizingReplaceView:(NSView *)oldView withView:(NSView *)newView isEmpty:(BOOL)isEmpty
{
    NSSplitView *splitView = [self splitView];
    NSBox *vlvBox;
    NSBox *otherBox;
    
    if ([self isWithinTopSubview])
    {
        vlvBox = [[splitView subviews] firstObject];
        otherBox = [[splitView subviews] lastObject];
    }
    else
    {
        otherBox = [[splitView subviews] firstObject];
        vlvBox = [[splitView subviews] lastObject];
    }
    
    NSRect splitFrame = [splitView frame];
    NSRect vlvFrame = [vlvBox frame];
    NSRect otherFrame = [otherBox frame];
    NSRect newFrame = [newView frame];		// This rect is undefined if newView is nil
    NSRect oldFrame = [oldView frame];		// This rect is undefined if oldView is nil
    int newVLVHeight;
    int newOtherHeight;
    
    // Figure out the sizes of the two parts of the split view:
    if (!oldView && !newView)
        newVLVHeight = NSHeight(vlvFrame) - NSHeight([self frame]);
    else if (!newView)
        newVLVHeight = NSHeight(vlvFrame) - NSHeight(oldFrame) + 1;
    else if (isEmpty)
        newVLVHeight = NSHeight(vlvFrame) - NSHeight([self frame]) + newFrame.size.height - 1;
    else if (oldView)
        newVLVHeight = NSHeight(vlvFrame) + (NSHeight(newFrame) - NSHeight(oldFrame));
    else
        newVLVHeight = NSHeight(vlvFrame) + newFrame.size.height;
    
    newOtherHeight = splitFrame.size.height - [splitView dividerThickness] - newVLVHeight;
    
    // Make sure other frame is not too small as a result:
    int otherFrameMinHeight = 50.0;
    id delegate = [splitView delegate];
    
    if ([delegate respondsToSelector:@selector(splitView:constrainMinCoordinate:ofSubviewAt:)])
        otherFrameMinHeight = [delegate splitView:splitView constrainMinCoordinate:newOtherHeight ofSubviewAt:0];
    
    if (newOtherHeight < otherFrameMinHeight)
    {
        NSRect oldWindowFrame = [NSWindow contentRectForFrameRect:[[self window] frame] styleMask:[[self window] styleMask]];
        int delta = otherFrameMinHeight - newOtherHeight;
        int oldWindowHeight = NSHeight(oldWindowFrame);
        int newWindowHeight = oldWindowHeight + delta;
        newOtherHeight += delta;
        NSRect newWindowFrame = [NSWindow frameRectForContentRect:NSMakeRect(oldWindowFrame.origin.x, NSMaxY(oldWindowFrame) - newWindowHeight, oldWindowFrame.size.width, newWindowHeight) styleMask:[[self window] styleMask]];
        
//        NSLog(@"newOtherHeight = %d; otherFrameMinHeight = %d; delta = %d; oldWindowHeight = %d; newWindowHeight = %d", newOtherHeight, otherFrameMinHeight, delta, oldWindowHeight, newWindowHeight);		//  log
        
        [[self window] setFrame:newWindowFrame display:YES animate:[self willAnimate]];
        
        // Need to reset these after the window is resized:
        vlvFrame = [vlvBox frame];
        otherFrame = [otherBox frame];
        newFrame = [newView frame];		// This rect is undefined if newView is nil
        oldFrame = [oldView frame];		// This rect is undefined if oldView is nil
    }
    
    // Resize the split view appropriately:
    vlvFrame.size.height = newVLVHeight;
    otherFrame.size.height = newOtherHeight;
    
    [vlvBox setFrame:vlvFrame];
    [otherBox setFrame:otherFrame];
    [splitView adjustSubviews];
    [splitView setNeedsDisplay:YES];
    
    // Position the new container view:
    if ((!isEmpty || oldView) && newView)
    {
        if (oldView)
            newFrame.origin = oldFrame.origin;
    }
    
    // Resize the new container view to the width of the DSViewsListView:
    newFrame.size.width = NSWidth([self frame]);
    
    [newView setFrame:newFrame];
    
    // Remove the old view, add a new one, or replace the old view:
    if (oldView && !newView)
        [oldView removeFromSuperview];
    else if (!oldView && newView)
        [self addSubview:newView];
    else if (oldView && newView)
        [self replaceSubview:oldView with:newView];
}

- (void)_windowResizingReplaceView:(NSView *)oldView withView:(NSView *)newView isEmpty:(BOOL)isEmpty
{
    NSRect oldWindowFrame, newWindowFrame;
    int newWindowHeight;
    
    // If neither of these are provided, there's nothing to do:
    //    if (!oldView && !newView)
    //        return;
    
    oldWindowFrame = [NSWindow contentRectForFrameRect:[[self window] frame] styleMask:[[self window] styleMask]];
    
    // Add the new container view; will be at the top of the DSViewsListView:
    NSRect newFrame = [newView frame];		// This rect is undefined if newView is nil
    NSRect oldFrame = [oldView frame];		// This rect is undefined if oldView is nil
    float relevantPosition;
    
    // Position the new container view:
    if ((!isEmpty || oldView) && newView)
    {
        if (oldView)
            newFrame.origin = oldFrame.origin;
        else
            newFrame = NSOffsetRect(newFrame, 0.0, NSHeight([self frame]));
        
        //        newFrame = NSOffsetRect(newFrame, 0.0, NSHeight(oldWindowFrame) - 19);
    }
    
    // Resize the new container view to the width of the DSViewsListView:
    newFrame.size.width = NSWidth([self frame]);
    
    [newView setFrame:newFrame];
    
    if (newView && !oldView)
        relevantPosition = NSMaxY(newFrame);
    else
        relevantPosition = NSMaxY(oldFrame);
    
    // Remove the old view or add a new one:
    if (oldView && !newView)
        [oldView removeFromSuperview];
    else if (!oldView && newView)
        [self addSubview:newView];
    
    // Resize the window appropriately:
    if (!oldView && !newView)
        newWindowHeight = NSHeight(oldWindowFrame) - NSHeight([self frame]);
    else if (!newView)
        newWindowHeight = NSHeight(oldWindowFrame) - NSHeight(oldFrame) + 1 /*- 10*/;
    else if (isEmpty)
        newWindowHeight = NSHeight(oldWindowFrame) - NSHeight([self frame]) + newFrame.size.height - 1 /*+ 29*/;
    else if (oldView)
        newWindowHeight = NSHeight(oldWindowFrame) + (NSHeight(newFrame) - NSHeight(oldFrame));
    else
        newWindowHeight = NSHeight(oldWindowFrame) + newFrame.size.height /*+ 9*/;
    
    newWindowFrame = [NSWindow frameRectForContentRect:NSMakeRect(oldWindowFrame.origin.x, NSMaxY(oldWindowFrame) - newWindowHeight, oldWindowFrame.size.width, newWindowHeight) styleMask:[[self window] styleMask]];
    
    NSArray *subviewMasks = nil;
    
    //    if (oldView)
    subviewMasks = [self adjustAutoresizingAroundPosition:relevantPosition
                                       stickPositionToTop:oldView && newView
                                             forSuperview:self];
    
    NSSize minWindowSize = [[self window] contentMinSize];
    NSSize maxWindowSize = [[self window] contentMaxSize];
    
    minWindowSize.height += (newWindowHeight - NSHeight(oldWindowFrame));
    maxWindowSize.height += (newWindowHeight - NSHeight(oldWindowFrame));
    
    [[self window] setContentMinSize:minWindowSize];
    [[self window] setContentMaxSize:maxWindowSize];
    
    [[self window] setFrame:newWindowFrame display:[self willAnimate] animate:[self willAnimate] && [[self window] isVisible]];
    
    //    if (oldView)
    [self restoreAutoresizeMasks:subviewMasks forSuperview:self];
    
    // Replace the old view:
    if (oldView && newView)
        [self replaceSubview:oldView with:newView];
}

/*
 _privateReplaceView:withView:isEmpty:
 
 Replaces the old container view with the new one, resizing the DSViewsListView and window or split view appropriately.  Private to this view; public equivalents are provided.
 
 Written by DJS 2003-06.
 Changed by DJS 2003-07.
 Changed by DJS 2003-11 to support resizing via a split view.
*/

- (void)_privateReplaceView:(NSView *)oldView withView:(NSView *)newView isEmpty:(BOOL)isEmpty
{
    // If the view is either being removed or replaced, force editing to end to avoid crashing:
    if (oldView && oldView != newView)
        [[self window] forceEndEditingForView:oldView];
    
    if ([self scrollView])
        [self _scrollResizingReplaceView:oldView withView:newView isEmpty:isEmpty];
    else if ([self splitView])
        [self _splitResizingReplaceView:oldView withView:newView isEmpty:isEmpty];
    else
        [self _windowResizingReplaceView:oldView withView:newView isEmpty:isEmpty];
}

- (NSArray *)adjustAutoresizingAroundPosition:(float)position
                           stickPositionToTop:(BOOL)stickPositionToTop
                                 forSuperview:(NSView *)superview
{
    // Code based on SNDisclosableView by Kurt Revis.

    // Adjust the autoresize masks of the superview's subviews, remembering the original masks.
    NSMutableArray *subviewMasks = [NSMutableArray array];
    NSView *oldSuperview = superview;
    
    while (superview)
    {
        // First adjust the superview's mask:
        unsigned int mask = [superview autoresizingMask];
        [subviewMasks addObject:[NSNumber numberWithUnsignedInt:mask]];

        // Make it stick to the top and bottom of the window, and change height.
        mask |= NSViewHeightSizable;
        mask &= ~NSViewMaxYMargin;
        mask &= ~NSViewMinYMargin;

        [superview setAutoresizingMask:mask];

        NSArray *subviews = [superview subviews];

//        NSLog(@"superview = %@; position = %f", superview, position);		//  log

        for (id subview in subviews)
        {

//            NSLog(@"..subview = %@; frame = %@", subview, NSStringFromRect([subview frame]));	//  log
            
            if (subview != oldSuperview)
            {
                mask = [subview autoresizingMask];
                BOOL stickToBottom = NSMaxY([subview frame]) <= position;

//                NSLog(@"...frame max = %f; position = %f; old mask = %d; stickToBottom = %d; NSViewHeightSizable = %d, NSViewMaxYMargin = %d, NSViewMinYMargin = %d", NSMaxY([subview frame]), position, mask, stickToBottom, mask, TestBit(mask, NSViewHeightSizable), TestBit(mask, NSViewMaxYMargin), TestBit(mask, NSViewMinYMargin));		//  log

                if (!stickPositionToTop && (NSMaxY([subview frame]) == position))
                    stickToBottom = YES;

//                if (reverseSticking)
//                    stickToBottom = !stickToBottom;

//                NSLog(@"...stickPositionToTop = %d; reverseSticking = %d; stickToBottom = %d", stickPositionToTop, reverseSticking, stickToBottom);		//  log

                [subviewMasks addObject:[NSNumber numberWithUnsignedInt:mask]];

/*
                if ([[subview class] isEqual:[NSClipView class]])
                {
                    // This subview is a clip view, so must be within the superview. Make it stick to the top and bottom of the window, and change height.
                    mask |= NSViewHeightSizable;
                    mask &= ~NSViewMaxYMargin;
                    mask &= ~NSViewMinYMargin;

//                    NSLog(@"....Sticking to the top and bottom; resizing; new mask = %d; NSViewHeightSizable = %d, NSViewMaxYMargin = %d, NSViewMinYMargin = %d", mask, TestBit(mask, NSViewHeightSizable), TestBit(mask, NSViewMaxYMargin), TestBit(mask, NSViewMinYMargin));		//  log
                }
                else
*/

                if (stickToBottom)
                {
                    // This subview is below us, or is us. Make it stick to the bottom of the window.
                    // It should not change height.
                    mask &= ~NSViewHeightSizable;
                    mask |= NSViewMaxYMargin;
                    mask &= ~NSViewMinYMargin;

//                    NSLog(@"....Sticking to the bottom; not resizing; new mask = %d; NSViewHeightSizable = %d, NSViewMaxYMargin = %d, NSViewMinYMargin = %d", mask, TestBit(mask, NSViewHeightSizable), TestBit(mask, NSViewMaxYMargin), TestBit(mask, NSViewMinYMargin));		//  log
                }
                else
                {
                    // This subview is above us. Make it stick to the top of the window.
                    // It should not change height.
                    mask &= ~NSViewHeightSizable;
                    mask &= ~NSViewMaxYMargin;
                    mask |= NSViewMinYMargin;

//                    NSLog(@"....Sticking to the top; not resizing; new mask = %d; NSViewHeightSizable = %d, NSViewMaxYMargin = %d, NSViewMinYMargin = %d", mask, TestBit(mask, NSViewHeightSizable), TestBit(mask, NSViewMaxYMargin), TestBit(mask, NSViewMinYMargin));		//  log
                }

                [subview setAutoresizingMask:mask];
            }
        }

        // Go to this superview's superview and repeat the process; note that the looping algorithm must be replicated exactly in the restore method, below.  Ideally both methods should use another method to get the next subview, but I can't be bothered refactoring it at this stage, so just be aware of the issue:
        oldSuperview = superview;
        position = NSMaxY([superview frame]);
        superview = [superview superview];
        
//        if (superview == [[self window] contentView])
//            superview = nil;
    }

    return subviewMasks;
}

- (void)restoreAutoresizeMasks:(NSArray *)masks forSuperview:(NSView *)superview
{
    NSView *oldSuperview = superview;
    NSEnumerator *enumerator = [masks objectEnumerator];

    while (superview)
    {
        // First item is the superview's mask:
        [superview setAutoresizingMask:[[enumerator nextObject] unsignedIntValue]];

        // Following items are the subview masks:
        NSArray *subviews = [superview subviews];

        for (id subview in subviews)
        {

            if (subview != oldSuperview)
                [subview setAutoresizingMask:[[enumerator nextObject] unsignedIntValue]];
        }

        // Go to this superview's superview and repeat the process; note that the looping algorithm must be replicated exactly in the adjust method, above.  Ideally both methods should use another method to get the next subview, but I can't be bothered refactoring it at this stage, so just be aware of the issue:
        oldSuperview = superview;
        superview = [superview superview];

//        if (superview == [[self window] contentView])
//            superview = nil;
    }
}

/*
 - (NSArray *)adjustAutoresizingAroundPosition:(float)position
 stickPositionToTop:(BOOL)stickPositionToTop
 forSuperview:(NSView *)superview
 {
     // Code based on SNDisclosableView by Kurt Revis.

     // Adjust the autoresize masks of the superview's subviews, remembering the original masks.
     NSArray *subviews = [superview subviews];
     unsigned int subviewCount = [subviews count];
     unsigned int subviewIndex;
     NSMutableArray *subviewMasks = [NSMutableArray arrayWithCapacity:subviewCount + 1];

     // First adjust the superview's mask:
     unsigned int mask = [[superview superview] autoresizingMask];
     [subviewMasks addObject:[NSNumber numberWithUnsignedInt:mask]];

     // Make it stick to the top and bottom of the window, and change height.
     mask |= NSViewHeightSizable;
     mask |= NSViewMaxYMargin;
     mask |= NSViewMinYMargin;

     [[superview superview] setAutoresizingMask:mask];

     for (subviewIndex = 0; subviewIndex < subviewCount; subviewIndex++)
     {
         NSView *subview = [subviews objectAtIndex:subviewIndex];
         mask = [subview autoresizingMask];
         BOOL stickToBottom = NSMaxY([subview frame]) < position;

         if (!stickPositionToTop && (NSMaxY([subview frame]) == position))
             stickToBottom = YES;

         [subviewMasks addObject:[NSNumber numberWithUnsignedInt:mask]];

         if (stickToBottom)
         {
             // This subview is below us, or is us. Make it stick to the bottom of the window.
             // It should not change height.
             mask &= ~NSViewHeightSizable;
             mask |= NSViewMaxYMargin;
             mask &= ~NSViewMinYMargin;
         }
         else
         {
             // This subview is above us. Make it stick to the top of the window.
             // It should not change height.
             mask &= ~NSViewHeightSizable;
             mask &= ~NSViewMaxYMargin;
             mask |= NSViewMinYMargin;
         }

         [subview setAutoresizingMask:mask];
     }

     return subviewMasks;
 }

 - (void)restoreAutoresizeMasks:(NSArray *)masks forSuperview:(NSView *)superview
 {
     NSArray *subviews = [superview subviews];
     unsigned int count, index;

     // First item is the superview's mask:
     [[superview superview] setAutoresizingMask:[[masks objectAtIndex:0] unsignedIntValue]];

     // Following items are the subview masks:
     count = [masks count] - 1;

     for (index = 0; index < count; index++)
         [[subviews objectAtIndex:index] setAutoresizingMask:[[masks objectAtIndex:index + 1] unsignedIntValue]];
 }
*/

- (NSView *)_findExitKeyViewForView:(NSView *)superview
{
    NSArray *subviews = [superview subviews];
    NSEnumerator *enumerator = [subviews objectEnumerator];
    NSView *view = nil;
    NSView *lastView = nil;
    
    while ((view = [enumerator nextObject]) && [view nextKeyView] != superview)
        lastView = view;
    
    if (!view)
        view = lastView;
    
    return view;
}

- (void)_adjustNextKeyViews
{
    NSEnumerator *enumerator = [self handlerEnumerator];
    id handler;
    NSView *previousView = nil;
    NSView *currentView = nil;
    
//    NSLog(@"self = %@", self);		//  log
    
    while ((handler = [enumerator nextObject]))
    {
        currentView = [handler containerView];
        
        if (!previousView)
            [self setNextKeyView:currentView];
        else if (currentView)
            [[self _findExitKeyViewForView:previousView] setNextKeyView:currentView];
        
//        NSLog(@"...previousView = %@; currentView = %@", previousView, currentView);		//  log
        
        previousView = currentView;
    }
    
    if ([self oldNextKeyView])
        [previousView setNextKeyView:[self oldNextKeyView]];
    
//    NSLog(@"...previousView = %@; oldNextKeyView = %@", previousView, [self oldNextKeyView]);		//  log
}

- (void)postBatchCleanup
{
    if ([self scrollView])
    {
        BOOL lockedFocus = [self lockFocusIfCanDraw];
        
        [self _repositionContainers];
        
        BOOL needsScroller = NSHeight([self frame]) > NSHeight([[self scrollView] frame]);
        
        [[self scrollView] setHasVerticalScroller:needsScroller];
        [[self scrollView] setNeedsDisplay:YES];
        
        if (lockedFocus)
            [self unlockFocus];
    }
}

- (BOOL)badHandler:(id)handler
{
    BOOL bad = NO;

    if (handler)
    {
        if ((bad = ![handler conformsToProtocol:@protocol(DSViewsListHandlerProtocol)]))
            NSLog(@"DSViewsListView: %@ doesn't conform to DSViewsListHandlerProtocol", handler);
        else if ((bad = ![handler containerView]))
            NSLog(@"DSViewsListView: %@'s container view failed to load", handler);
    }

    return bad;
}


// ----------------------------------------------------------------------------------------
#pragma mark -
#pragma mark PRIVATE ACCESSOR METHODS
// ----------------------------------------------------------------------------------------

- (NSMutableArray *)viewHandlersArray
{
    return _viewHandlersArray;
}

- (void)setViewHandlersArray:(NSMutableArray *)value
{
    [value retain];
    [_viewHandlersArray autorelease];
    _viewHandlersArray = value;
}

@end
