//
//  NarratorDocument.m
//  Narrator
//
//  Created by David Sinclair on 2007-10-08.
//  Copyright (c) 2002 - 2008 Dejal Systems, LLC. All rights reserved.
//

#import "NarratorDocument.h"
#import "MainController.h"
#import "DocumentEntity.h"
#import "ChapterEntity.h"
#import "TextEntity.h"
#import "CharacterEntity.h"
#import "VoiceEntity.h"
#import "MainDefines.h"
#import "MTCDUtilities.h"
#import "DSTableView.h"
#import "DSArrayControllerCategories.h"
#import "Version1Characters.h"
#import "CharacterAttachmentCell.h"


@implementation NarratorDocument

@synthesize speechWindow;

/*
 initWithType:error:
 
 Designated initializer for the document.  Invoked once per document when creating a new document, not when opening an existing one.
 
 Written by DJS 2007-10.
*/

- (id)initWithType:(NSString *)typeName error:(NSError **)outError;
{
    self = [super initWithType:typeName error:outError];
    
    if (self == nil)
        return nil;
    
    [self documentEntity];
    [self setupSpeechTextView];
    [self setupDefaultChapter];
    [self setupDefaultCharacters];
    [self clearChangeIndications];
    
    return self;
}

/*
 windowNibName
 
 Returns the name of the nib file to use for the document.
 
 Written by DJS 2007-10.
*/

- (NSString *)windowNibName;
{
    return @"NarratorDocument";
}

/*
 windowControllerDidLoadNib:
 
 Sets up the user interface after loading the nib contents.
 
 Written by DJS 2007-10.
*/

- (void)windowControllerDidLoadNib:(NSWindowController *)windowController 
{
    [super windowControllerDidLoadNib:windowController];
    
    [chapterArrayController setSortDescriptors:[NSArray arrayWithObject:[[NSSortDescriptor alloc] initWithKey:@"name" ascending:YES]]];
    
    [characterArrayController setSortDescriptors:[NSArray arrayWithObject:[[NSSortDescriptor alloc] initWithKey:@"name" ascending:YES]]];
    [charactersTableView setDoubleAction:@selector(assignCharacter:)];
    
    NSFont *font = [[[NSUserDefaults standardUserDefaults] objectForKey:DOCUMENT_FONT] object];
    
    [[speechTextView textStorage] setDelegate:self];
    [speechTextView setColorWell:characterColorWell];
    [speechTextView setFont:font];
    [speechTextView setTypingAttributes:[NSDictionary dictionaryWithObject:font forKey:NSFontAttributeName]];       // doesn't help
    
    [[NSNotificationCenter defaultCenter] addObserver:self
                                             selector:@selector(characterNameChangedNotification:)
                                                 name:@"CharacterNameChanged"
                                               object:nil];
    
    [NSTimer scheduledTimerWithTimeInterval:0.1 target:self selector:@selector(selectCurrentChapterSelection) userInfo:nil repeats:NO];
}

- (void)selectCurrentChapterSelection;
{
    ChapterEntity *currentChapter = [self documentEntity].currentChapter;
    
    if (currentChapter)
        [chapterArrayController setSelectedObjects:[NSArray arrayWithObject:currentChapter]];
}

- (void)windowWillClose:(NSNotification *)notification;
{
    [[NSNotificationCenter defaultCenter] removeObserver:self];
}

- (DocumentEntity *)documentEntity;
{
    if (!documentEntity)
    {
        if ([MTCDUtilities countObjectsForEntityNamed:@"Document" inContext:[self managedObjectContext]])
            documentEntity = (DocumentEntity *)[[MTCDUtilities objectsForEntityNamed:@"Document" inContext:[self managedObjectContext]] firstObject];
        else
            documentEntity = [NSEntityDescription insertNewObjectForEntityForName:@"Document" inManagedObjectContext:[self managedObjectContext]];
    }
    
    return documentEntity;
}

- (ChapterEntity *)setupDefaultChapter;
{
    ChapterEntity *chapter = [NSEntityDescription insertNewObjectForEntityForName:@"Chapter" inManagedObjectContext:[self managedObjectContext]];
    chapter.name = NSLocalizedString(@"Chapter 1", @"Default chapter name for source list");
    
    return chapter;
}

- (void)setupDefaultCharacters;
{
    NSManagedObjectContext *managedObjectContext = [self managedObjectContext];
    CharacterEntity *character = nil;
    
    character = [CharacterEntity insertCharacterWithIdentifier:@"SilentReadAlong"
                                                          name:NSLocalizedString(@"You", @"Silent (read along) character name for source list")
                                                         color:[NSColor colorWithCalibratedHue:0.34 saturation:0.25 brightness:1.0 alpha:1.0]
                                               voiceIdentifier:@"com.dejal.narrator.SilentReadAlong"
                                                     inContext:managedObjectContext];
    character.comment = NSLocalizedString(@"This is a placeholder for you to participate in the story.", @"Silent (read along) description");
    
    character = [CharacterEntity insertCharacterWithIdentifier:@"SilentSkipOver"
                                                          name:NSLocalizedString(@"Note", @"Silent (skip) character name for source list")
                                                         color:[NSColor colorWithCalibratedHue:0.18 saturation:0.25 brightness:1.0 alpha:1.0]
                                               voiceIdentifier:@"com.dejal.narrator.SilentSkipOver"
                                                     inContext:managedObjectContext];
    character.comment = NSLocalizedString(@"Stage directions, annotations, and other silent comments; will be skipped when reading.", @"Silent (read along) description");
    
    character = [CharacterEntity insertCharacterWithIdentifier:@"Default"
                                                          name:NSLocalizedString(@"Narrator", @"Default character name for source list")
                                                         color:[NSColor colorWithCalibratedHue:0.65 saturation:0.25 brightness:1.0 alpha:1.0]
                                               voiceIdentifier:@"com.apple.speech.synthesis.voice.Alex"
                                                     inContext:managedObjectContext];
    character.comment = NSLocalizedString(@"I'm the storyteller.  Let my dulcet tones guide you through the story.", @"Silent (read along) description");
}

- (void)clearChangeIndications;
{
    // Clear the undo manager and change count for the document such that untitled documents start with zero unsaved changes:
    [[self managedObjectContext] processPendingChanges];
    [[[self managedObjectContext] undoManager] removeAllActions];
    [self updateChangeCount:NSChangeCleared];
}

- (void)setupSpeechTextView;
{
    [speechTextView setContinuousSpellCheckingEnabled:[[self documentEntity].checkSpelling boolValue]];
    [speechTextView setGrammarCheckingEnabled:[[self documentEntity].checkGrammar boolValue]];
    [speechTextView setRulerVisible:[[self documentEntity].showRuler boolValue]];
    [speechTextView setSmartInsertDeleteEnabled:[[self documentEntity].smartCopyPaste boolValue]];
    [speechTextView setAutomaticQuoteSubstitutionEnabled:[[self documentEntity].smartQuotes boolValue]];
    [speechTextView setAutomaticLinkDetectionEnabled:[[self documentEntity].smartLinks boolValue]];
}

- (NSFileWrapper *)placeholderFileWrapperWithIdentifier:(NSString *)identifier name:(NSString *)name color:(NSColor *)color;
{
    NSString *wrapName = [[identifier stringByAppendingPathExtension:@"narratorcharacter"] stringByAppendingPathExtension:@"tiff"];
    NSData *data = [[CharacterAttachmentCell imageWithName:name color:color] TIFFRepresentation];
    NSFileWrapper *wrapper = [[NSFileWrapper alloc] initRegularFileWithContents:data];
    
    [wrapper setFilename:wrapName];
    [wrapper setPreferredFilename:wrapName];
    
    return wrapper;
}

- (void)convertVersion1MarkersInText:(NSMutableAttributedString *)text
                    fromOldCharacter:(Character *)oldCharacter
                      toNewCharacter:(CharacterEntity *)newCharacter;
{
    NSColor *color = [oldCharacter marker];
    NSColor *lighterColor = [NSColor colorWithCalibratedHue:[color hueComponent] saturation:0.25 brightness:1.0 alpha:1.0];
    NSFileWrapper *wrapper = [self placeholderFileWrapperWithIdentifier:[newCharacter identifier] name:[newCharacter name] color:lighterColor];
    NSTextAttachment *attachment = [[NSTextAttachment alloc] initWithFileWrapper:wrapper];
    NSAttributedString *attrib = [NSAttributedString attributedStringWithAttachment:attachment];
    NSRange effectiveRange = NSMakeRange(0, 0);
    
    while (NSMaxRange(effectiveRange) < [text length])
    {
        // Look at all foreground colors (looking for the continuious range); we've found the one for this character if it's the right color:
        if ([[text attribute:NSForegroundColorAttributeName atIndex:NSMaxRange(effectiveRange) longestEffectiveRange:&effectiveRange inRange:[text allRange]] isEqual:color])
        {
            [text removeAttribute:NSForegroundColorAttributeName range:effectiveRange];
            [text insertAttributedString:attrib atIndex:effectiveRange.location];
        }
    }
}

- (BOOL)readVersion1DocumentFromURL:(NSURL *)absoluteURL;
{
    [self documentEntity];
    [self setupSpeechTextView];
    
    NSData *data = [NSData dataWithContentsOfURL:absoluteURL];
    Cast *cast = [NSUnarchiver unarchiveObjectWithData:data];
    
    if (!cast)
        return NO;
    
    NSMutableAttributedString *text = [[cast nativeText] mutableCopy];
    NSManagedObjectContext *managedObjectContext = [self managedObjectContext];
    
    for (Character *oldCharacter in [cast characters])
    {
        NSString *voiceIdentifier = [oldCharacter voice];
        
        if ([voiceIdentifier isEqualToString:@"Silent (read along)"])
            voiceIdentifier = @"com.dejal.narrator.SilentReadAlong";
        else if ([voiceIdentifier isEqualToString:@"Silent (skip over)"])
            voiceIdentifier = @"com.dejal.narrator.SilentSkipOver";
        else
            voiceIdentifier = [@"com.apple.speech.synthesis.voice." stringByAppendingString:voiceIdentifier];
        
        NSColor *lighterColor = [NSColor colorWithCalibratedHue:[[oldCharacter marker] hueComponent] saturation:0.25 brightness:1.0 alpha:1.0];
        
        CharacterEntity *newCharacter = [CharacterEntity insertCharacterWithIdentifier:[oldCharacter name] name:[oldCharacter name] color:lighterColor voiceIdentifier:voiceIdentifier inContext:managedObjectContext];
        
        if ([voiceIdentifier hasSuffix:@"SilentSkipOver"])
            newCharacter.voice.rate = [NSNumber numberWithInt:0];
        else
            newCharacter.voice.rate = [oldCharacter rate];
        
        newCharacter.voice.pitch = [oldCharacter pitch];
        newCharacter.voice.inflection = [oldCharacter inflection];
        newCharacter.voice.volume = [oldCharacter volume];
        
        [self convertVersion1MarkersInText:text fromOldCharacter:oldCharacter toNewCharacter:newCharacter];
    }
    
    ChapterEntity *chapter = [self setupDefaultChapter];
    chapter.text.richText = text; //[text textViewBindingsValue];
    
    [self clearChangeIndications];
    
    return YES;
}

- (void)replaceCurrentChapterTextWithAttributedString:(NSAttributedString *)text;
{
    if (text)
    {
        ChapterEntity *chapter = (ChapterEntity *)[MTCDUtilities objectForEntityNamed:@"Chapter" inContext:[self managedObjectContext]];
        
        chapter.text.richText = text; //[text textViewBindingsValue];
    }
}

- (BOOL)readOtherFormatFromURL:(NSURL *)absoluteURL error:(NSError **)outError;
{
    NSData *data = [NSData dataWithContentsOfURL:absoluteURL options:NSUncachedRead error:outError];
    
    if (data)
    {
        [self documentEntity];
        [self setupSpeechTextView];
        
        NSAttributedString *text = [[NSAttributedString alloc] initWithData:data options:nil documentAttributes:nil error:outError];
        
        if (text)
        {
            ChapterEntity *chapter = [self setupDefaultChapter];
            chapter.text.richText = text; //[text textViewBindingsValue];
        }
        
        [self setupDefaultCharacters];
        [self clearChangeIndications];
        
        return (text != nil);
    }
    
    return NO;
}

- (BOOL)readFromURL:(NSURL *)absoluteURL ofType:(NSString *)typeName error:(NSError **)outError;
{
    if ([typeName isEqualToString:@"Narrator 2"])
    {
        BOOL okay = [super readFromURL:absoluteURL ofType:typeName error:outError];
        
        // will probably make more sense to auto-play the welcome document after loading it manually, not here:
        
        if (okay && ([[NSUserDefaults standardUserDefaults] boolForKey:AUTO_READ] || [[[absoluteURL path] lastPathComponentWithoutExtension] isEqualToString:NSLocalizedString(@"Welcome to Narrator!", @"Filename of the welcome document")]))
            [NSTimer scheduledTimerWithTimeInterval:1.0 target:self selector:@selector(playStopSpeech:) userInfo:nil repeats:NO];
        
        return okay;
    }
    else if ([typeName isEqualToString:@"Narrator 1"])
        return [self readVersion1DocumentFromURL:absoluteURL];
    else
        return [self readOtherFormatFromURL:absoluteURL error:outError];
}

// Code based on <http://developer.apple.com/documentation/Cocoa/Conceptual/NSPersistentDocumentTutorial/>:

- (BOOL)setMetadataForStoreAtURL:(NSURL *)url;
{
    NSPersistentStoreCoordinator *psc = [[self managedObjectContext] persistentStoreCoordinator];
    NSPersistentStore *store = [psc persistentStoreForURL:url];
    
    if (store != nil)
    {
        NSMutableDictionary *metadata = [[psc metadataForPersistentStore:store] mutableCopy];
        
        if (metadata == nil)
            metadata = [NSMutableDictionary dictionary];
        
        NSMutableArray *chapterNames = [NSMutableArray array];
        NSMutableString *text = [NSMutableString string];
        
        for (ChapterEntity *chapter in [chapterArrayController arrangedObjects])
        {
            [chapterNames addObject:chapter.name];
            [text appendString:[chapter.text.richText string]];
        }
        
        [text replaceAllOccurrencesOf:[NSString stringWithFormat:@"%C", NSAttachmentCharacter] with:@" "];
        
        [metadata setObject:chapterNames forKey:(NSString *)@"com_dejal_narrator_chapters"];
        [metadata setObject:text forKey:(NSString *)kMDItemTextContent];
        
        [psc setMetadata:metadata forPersistentStore:store];
        
        return YES;
    }
    return NO;
}

- (BOOL)configurePersistentStoreCoordinatorForURL:(NSURL *)url ofType:(NSString *)fileType modelConfiguration:(NSString *)configuration storeOptions:(NSDictionary *)storeOptions error:(NSError **)error;
{
    BOOL okay = [super configurePersistentStoreCoordinatorForURL:url ofType:fileType modelConfiguration:configuration storeOptions:storeOptions error:error];
    
    if (okay)
    {
        NSPersistentStoreCoordinator *psc = [[self managedObjectContext] persistentStoreCoordinator];
        NSPersistentStore *store = [psc persistentStoreForURL:url];
        id existingMetadata = [[psc metadataForPersistentStore:store] objectForKey:(NSString *)kMDItemTextContent];
        
        if (existingMetadata == nil)
        {
            okay = [self setMetadataForStoreAtURL:url];
        }
    }
    return okay;
}

- (BOOL)writeToURL:(NSURL *)absoluteURL ofType:(NSString *)typeName forSaveOperation:(NSSaveOperationType)saveOperation originalContentsURL:(NSURL *)absoluteOriginalContentsURL error:(NSError **)error;
{
    // Don't allow saving into the app bundle, to avoid saving edits to the welcome document:
    if ([[absoluteOriginalContentsURL path] hasPrefix:[[NSBundle mainBundle] bundlePath]])
    {
        *error = [NSError errorWithDomain:@"DejalNarratorDomain" code:712 userInfo:[NSDictionary dictionaryWithObject:NSLocalizedString(@"This document is read-only; use Save As to save a copy elsewhere.", @"Error when trying to save into the app bundle") forKey:NSLocalizedFailureReasonErrorKey]];
        return NO;
    }
    
    // If there isn't a store for this document (e.g. it was opened from a v1 document or third-party document), treat it as a new document:
    if (![[[[self managedObjectContext] persistentStoreCoordinator] persistentStores] count])
        absoluteOriginalContentsURL = nil;
    
    ChapterEntity *chapter = [[chapterArrayController selectedObjects] firstObject];
    NSRange range = [[[speechTextView selectedRanges] firstObject] rangeValue];
    
    chapter.text.selectionLocation = [NSNumber numberWithInt:range.location];
    chapter.text.selectionLength = [NSNumber numberWithInt:range.length];
    
    [self documentEntity].currentChapter = chapter;
    
    [self clearChangeIndications];
    
    if ([self fileURL])
        [self setMetadataForStoreAtURL:[self fileURL]];
    
    return [super writeToURL:absoluteURL ofType:typeName forSaveOperation:saveOperation originalContentsURL:absoluteOriginalContentsURL error:error];
}

- (NSError *)willPresentError:(NSError *)inError;
{
    // The error is a Core Data validation error if its domain is NSCocoaErrorDomain and it is between the minimum and maximum for Core Data validation error codes:
    if (!([[inError domain] isEqualToString:NSCocoaErrorDomain]))
        return inError;
    
    int errorCode = [inError code];
    
    if ((errorCode < NSValidationErrorMinimum) || (errorCode > NSValidationErrorMaximum))
        return inError;
    
    // If there are multiple validation errors, inError is an NSValidationMultipleErrorsError. If it's not, return it:
    if (errorCode != NSValidationMultipleErrorsError)
        return inError;
    
    // For an NSValidationMultipleErrorsError, the original errors are in an array in the userInfo dictionary for key NSDetailedErrorsKey:
    NSArray *detailedErrors = [[inError userInfo] objectForKey:NSDetailedErrorsKey];
    
    // We only present error messages for up to 3 validation errors at a time:
    NSUInteger numErrors = [detailedErrors count];
    NSMutableString *errorString = [NSMutableString stringWithFormat:NSLocalizedString(@"%u validation errors occurred", @""), numErrors];
    
    if (numErrors > 3)
        [errorString appendFormat:NSLocalizedString(@".  The first 3 are:\n", @"")];
    else
        [errorString appendFormat:@":\n"];
    
    NSUInteger i, displayErrors = numErrors > 3 ? 3 : numErrors;
    
    for (i = 0; i < displayErrors; i++)
        [errorString appendFormat:@"\n\u2022 %@", [[detailedErrors objectAtIndex:i] localizedDescription]];
    
    // Create a new error with the new userInfo:
    NSMutableDictionary *newUserInfo = [NSMutableDictionary dictionaryWithDictionary:[inError userInfo]];
    [newUserInfo setObject:errorString forKey:NSLocalizedDescriptionKey];
    
    return [NSError errorWithDomain:[inError domain] code:[inError code] userInfo:newUserInfo];
}


// ----------------------------------------------------------------------------------------
#pragma mark SPLIT VIEW DELEGATE METHODS
// ----------------------------------------------------------------------------------------

- (BOOL)splitView:(NSSplitView *)splitView canCollapseSubview:(NSView *)subview;
{
    return YES;
}

- (BOOL)splitView:(NSSplitView *)splitView shouldCollapseSubview:(NSView *)subview forDoubleClickOnDividerAtIndex:(NSInteger)dividerIndex;
{
    if (splitView == documentSplitView)
        return (subview == [[splitView subviews] firstObject]);
    else
        return ((dividerIndex == 0 && subview == [[splitView subviews] firstObject]) || (dividerIndex == 1 && subview == [[splitView subviews] lastObject]));
}

- (float)splitView:(NSSplitView *)splitView constrainMaxCoordinate:(CGFloat)proposedMax ofSubviewAt:(NSInteger)offset;
{
    if (splitView == documentSplitView)
        return [splitView bounds].size.width - 400.0;
    else
        return [splitView bounds].size.height - 224.0;
}

- (float)splitView:(NSSplitView *)splitView constrainMinCoordinate:(CGFloat)proposedMin ofSubviewAt:(NSInteger)offset;
{
    if (splitView == documentSplitView)
        return 230.0;
    else
    {
        if (offset == 0)
            return 100.0;
        else
            return 200.0;
    }
}


// #pragma mark OUTLINE VIEW DELEGATE METHODS


/*
- (BOOL)outlineView:(NSOutlineView *)outlineView isGroupItem:(id)item;
{
    return ([[[item representedObject] valueForKeyPath:@"parent.name"] isEqualToString:@"Root"]);
}
*/


#pragma mark TABLE VIEW DELEGATE METHODS


- (NSIndexSet *)tableView:(DSTableView *)tableView canDeleteRowIndexes:(NSIndexSet *)indexes;
{
    return indexes;
}

- (BOOL)tableView:(DSTableView *)tableView deleteRowIndexes:(NSIndexSet *)indexes;
{
    if (tableView == chaptersTableView)
            [chapterArrayController remove:nil];
    else
            [characterArrayController remove:nil];
    
    return YES;
}

- (void)tableViewSelectionDidChange:(NSNotification *)aNotification;
{
    if ([aNotification object] == chaptersTableView)
    {
        [self setupSpeechTextView];
    }
}


#pragma mark TABLE VIEW DRAGGING DATA SOURCE METHODS

- (NSImage *)tableView:(DSTableView *)tableView dragImageForRowsWithIndexes:(NSIndexSet *)dragRows
          tableColumns:(NSArray *)tableColumns event:(NSEvent *)dragEvent offset:(NSPointPointer)dragImageOffset;
{
    CharacterEntity *character = [[characterArrayController arrangedObjects] objectAtIndex:[dragRows firstIndex]];
    
    return [CharacterAttachmentCell imageWithName:character.name color:character.color];
}


#pragma mark TEXT VIEW DELEGATE METHODS

- (NSArray *)textView:(NSTextView *)aTextView writablePasteboardTypesForCell:(id <NSTextAttachmentCell>)cell
              atIndex:(NSUInteger)charIndex;
{
//    return [NSArray arrayWithObject:kCharacterMarkerPboardType];
    return [NSArray arrayWithObjects:NSFileContentsPboardType, /*NSFilenamesPboardType,*/ nil];
}

- (BOOL)textView:(NSTextView *)aTextView writeCell:(id <NSTextAttachmentCell>)cell
         atIndex:(NSUInteger)charIndex toPasteboard:(NSPasteboard *)pboard type:(NSString *)type;
{
    if (type == NSFileContentsPboardType)
        [pboard writeFileWrapper:[[cell attachment] fileWrapper]];
    
    return YES;
}

- (void)textStorageWillProcessEditing:(NSNotification *)note;
{
    NSAttributedString *text = [speechTextView textStorage];
    
    if ([note object] != text)
        return;
    
    NSUInteger length = [text length];
    NSRange effectiveRange = NSMakeRange(0, 0);
    id attachment;
    
    while (NSMaxRange(effectiveRange) < length)
    {
        attachment = [text attribute:NSAttachmentAttributeName atIndex:NSMaxRange(effectiveRange) effectiveRange:&effectiveRange];
        
        if (attachment)
        {
            if ([attachment isKindOfClass:[NSTextAttachment class]] &&
                (![[attachment attachmentCell] isKindOfClass:[CharacterAttachmentCell class]] || ![(CharacterAttachmentCell *)[attachment attachmentCell] identifier]) &&
                [[[[attachment fileWrapper] preferredFilename] pathExtension] isEqualToString:@"tiff"] &&
                [[[[[attachment fileWrapper] preferredFilename] stringByDeletingPathExtension] pathExtension] isEqualToString:@"narratorcharacter"])
            {
                CharacterAttachmentCell *cell = [CharacterAttachmentCell new];
                [cell setIdentifier:[[[[attachment fileWrapper] preferredFilename] stringByDeletingPathExtension] stringByDeletingPathExtension]];
                [attachment setAttachmentCell:cell];
            }
        }
    }        
}

- (void)textView:(NarratorTextView *)textView willPerformAction:(SEL)action;
{
    DocumentEntity *document = [self documentEntity];
    
    document.checkSpelling = [NSNumber numberWithBool:[speechTextView isContinuousSpellCheckingEnabled]];
    document.checkGrammar = [NSNumber numberWithBool:[speechTextView isGrammarCheckingEnabled]];
    document.showRuler = [NSNumber numberWithBool:[speechTextView isRulerVisible]];
    document.smartCopyPaste = [NSNumber numberWithBool:[speechTextView smartInsertDeleteEnabled]];
    document.smartQuotes = [NSNumber numberWithBool:[speechTextView isAutomaticQuoteSubstitutionEnabled]];
    document.smartLinks = [NSNumber numberWithBool:[speechTextView isAutomaticLinkDetectionEnabled]];
}


#pragma mark NOTIFICATION METHODS


- (void)characterNameChangedNotification:(NSNotification *)notification;
{
    NSAttributedString *text = [speechTextView textStorage];
    NSUInteger length = [text length];
    NSRange effectiveRange = NSMakeRange(0, 0);
    id attachment;
    
    while (NSMaxRange(effectiveRange) < length)
    {
        attachment = [text attribute:NSAttachmentAttributeName atIndex:NSMaxRange(effectiveRange) effectiveRange:&effectiveRange];
        
        if (attachment)
        {
            if ([attachment isKindOfClass:[NSTextAttachment class]] && [[attachment attachmentCell] isKindOfClass:[CharacterAttachmentCell class]])
            {
                CharacterAttachmentCell *cell = (CharacterAttachmentCell *)[attachment attachmentCell];
                
                [cell setName:nil];
            }
        }
    }
    
    [[speechTextView textStorage] edited:NSTextStorageEditedAttributes range:[speechTextView allRange] changeInLength:0];
}


#pragma mark ACTION METHODS


- (void)editNameInTable:(NSTimer *)timer;
{
    NSTableView *tableView = [timer userInfo];
    
    [tableView editColumnWithIdentifier:@"name" row:[tableView selectedRow]];
}

- (IBAction)saveDocument:(id)sender;
{
    [[self speechWindow] forceEndEditingForView:nil];
    
    [super saveDocument:sender];
}

- (IBAction)saveDocumentAs:(id)sender;
{
    [[self speechWindow] forceEndEditingForView:nil];
    
    [super saveDocumentAs:sender];
}

- (IBAction)newChapter:(id)sender;
{
    [chapterArrayController add:sender];
    [NSTimer scheduledTimerWithTimeInterval:0.0 target:self selector:@selector(editNameInTable:) userInfo:chaptersTableView repeats:NO];
}

- (IBAction)newCharacter:(id)sender;
{
    [characterArrayController add:sender];
    [NSTimer scheduledTimerWithTimeInterval:0.0 target:self selector:@selector(editNameInTable:) userInfo:charactersTableView repeats:NO];
}

- (IBAction)playPauseNextCapsule:(id)sender;
{
    switch ([[sender cell] tagForSegment:[sender selectedSegment]])
    {
        case 1:
            [self pauseResumeSpeech:sender];
            break;
        case 2:
            [self nextSpeech:sender];
            break;
        default:
            [self playStopSpeech:sender];
            break;
    }
}

- (IBAction)playStopSpeech:(id)sender;
{
}

- (IBAction)pauseResumeSpeech:(id)sender;
{
}

- (IBAction)nextSpeech:(id)sender;
{
}

- (IBAction)assignCharacter:(id)sender;
{
}

- (IBAction)resetRate:(id)sender;
{
}

- (IBAction)resetPitch:(id)sender;
{
}

- (IBAction)resetInflection:(id)sender;
{
}

- (IBAction)resetVolume:(id)sender;
{
}

- (IBAction)deselectAll:(id)sender;
{
    [speechTextView setSelectedRange:NSMakeRange(0, 0)];
}

- (IBAction)castingAssistant:(id)sender;
{
}

- (IBAction)search:(id)sender;
{
}

- (IBAction)quickLook:(id)sender;
{
}

- (IBAction)tellJoke:(id)sender;
{
}

- (void)exportProgressSheetDidEnd:(NSWindow *)sheet returnCode:(NSInteger)returnCode contextInfo:(void *)contextInfo;
{
}

- (id)exportProgressSheetWithPrompt:(NSString *)prompt andIconName:(NSString *)iconName;
{
    return nil;
}

- (void)exportAsAACPanelDidEnd:(NSSavePanel *)savePanel returnCode:(NSInteger)returnCode contextInfo:(void *)contextInfo;
{
}

- (NSString *)exportFinalFilename;
{
    return nil;
}

- (IBAction)exportAsAAC:(id)sender;
{
}

- (IBAction)exportToITunes:(id)sender;
{
}

@end

